NewsTab.Extension.IsBackgroundScript = true;

chrome.runtime.onInstalled.addListener(function (details) {
    if (details.reason === "install") {
        NewsTab.Helpers.SetLocalStorageItem("InstallTime", Date.now());

        NewsTab.Helpers.GetSyncStorage("NId", function (nid) {
            NewsTab.Extension.NId = nid;
            NewsTab.Extension.Init({
                Source: "install",
                OnFinished: function () {
                    NewsTab.Extension.OpenWelcomePage();
                    NewsTab.TrackAnalyticsEvent("Installed[BP]", chrome.runtime.getManifest().version , NewsTab.Extension.NameId);
                }
            });
        });
    } else if (details.reason === "update") {
        NewsTab.Helpers.SetLocalStorageItem("UpdateTime", Date.now());

        NewsTab.Helpers.GetSyncStorage("NId", function (nid) {
            NewsTab.Extension.NId = nid;
            NewsTab.Extension.Init({
                Source: "update",
                OnFinished: function () {
                    NewsTab.TrackAnalyticsEvent("Updated[BP]", details.previousVersion, NewsTab.Extension.NameId);
                }
            });
        });
    }
});

chrome.runtime.onStartup.addListener(function () {
    NewsTab.Helpers.GetSyncStorage("NId", function (nid) {
        NewsTab.Extension.NId = nid;
        NewsTab.Extension.Init({
            Source: "startup",
            OnFinished: function () {
                NewsTab.TrackAnalyticsEvent("Startup[BP]", chrome.runtime.getManifest().version, NewsTab.Extension.NameId);
            }
        });
    });
});

chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    if (request.dataType === "ExchangeExtension") {
        NewsTab.Helpers.GetSyncStorage("NId", function (nid) {
            var oldId = nid;
            NewsTab.Extension.NId = nid;
            NewsTab.Extension.Init({
                Source: "change",
                Url: request.value.href,
                OnFinished: function () {
                    NewsTab.Extension.OpenWelcomePage();
                    NewsTab.TrackAnalyticsEvent("ExchangeExtension[BP]", "Old NId:" + NewsTab.Helpers.PadStart(oldId, 6), NewsTab.Extension.NameId);
                    sendResponse({});
                }
            });
        });
    }
    else if (request.dataType === "OpenNewTab") {
        if (request.value) {
            NewsTab.Helpers.SetLocalStorageItem("TokenKey", request.value);
        }
        if (sender.tab) {
            chrome.tabs.update(sender.tab.id, { url: "chrome-extension://" + window.chrome.runtime.id + "/newtab.html" });
        } else {
            window.open("chrome-extension://" + window.chrome.runtime.id + "/newtab.html");
        }
        sendResponse({});
    }
    else if (request.dataType === "OpenSharedArticleInstall") {
        NewsTab.Helpers.SetLocalStorageItem("NewsTab_SharedArticle", request.value);
        NewsTab.TrackAnalyticsEvent("Share Interstitial", "Install", request.value.Url);
        window.chrome.tabs.update(sender.tab.id, { active: true, url: "chrome-extension://" + window.chrome.runtime.id + "/newtab.html" });
        sendResponse({});
    }
    else if (request.dataType === "RemoveFromStorage") {
        NewsTab.Helpers.RemoveLocalStorageItem(request.value);
        sendResponse({});
    }
    else if (request.dataType === "FetchSearchSuggestions") {
        var url = "https://api.bing.com/osjson.aspx?query=SEARCH_TERM&market=en_us";
        url = url.replace("SEARCH_TERM", request.term);
        var xhr = new XMLHttpRequest();
        xhr.open("GET", url, true);
        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                var resp = JSON.parse(xhr.responseText);
                sendResponse(resp);
            }
        };
        xhr.send();
        //fetch(url)
        //    .then(response => console.log(response.text()))
        //    .catch(error => console.log(error));
        return true;  // Will respond asynchronously.
    }
});

chrome.runtime.onMessageExternal.addListener(function (request, sender, sendResponse) {
    if (request.dataType === "OpenSharedArticle") {
        NewsTab.Helpers.SetLocalStorageItem("NewsTab_SharedArticle", request.value);
        window.chrome.tabs.update(sender.tab.id, { active: true, url: "chrome-extension://" + window.chrome.runtime.id + "/newtab.html" });
        sendResponse({});
    } else if (request.dataType === "IsInstalled") {
        var nid = request.value;
        var oldNid = NewsTab.Helpers.GetLocalStorageItem("NId");
        if (oldNid !== undefined) {
            if (nid === oldNid) {
                sendResponse({ isInstalled: true });
            } else {
                sendResponse({ isInstalled: false });
            }
        } else {
            sendResponse({ isInstalled: false });
        }
    }
});

chrome.tabs.onUpdated.addListener(function (tabId, changeInfo, tab) {
    if (tab.status === "complete" && tab.active) {
        //var timeWeatherView = NewsTab.Helpers.GetLocalStorageItem("Show_Time_Weather");
        //if (timeWeatherView !== false) {
        //    NewsTab.Modules.TimeAndWeatherWidget.UpdateWeather();
        //}

        //var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
        //if (collage !== undefined && collage !== null && collage.Image !== undefined && collage.Image !== null) {
        //    NewsTab.Extension.GenerateColage();
        //}
    }
});