﻿var NewsTab = NewsTab || {};

NewsTab.Extension = {
    DEFAULT_COLORS: ["#e91e63", "#9761a9", "#03a9f4", "#009688", "#00bfa5", "#ebc24e", "#fb7243", "#795548"],

    LOCAL_EXTENSION_ID: "klfanecickcomfbhkdjkjnmllpgffhnh",
    DEFAULT_SECTION_ID: "00000000-EEEE-1111-EEEE-000000000000",
    ADDITIONAL_SECTION_ID: "00000000-EEEE-1111-EEEE-000000000001",

    S3_URL: "https://s3.newstab.com",
    VERSION: chrome.runtime.getManifest().version,

    NId: null,
    Config: null,
    NameId: null,

    ClickId: null,

    _regionCode: null,

    IsBackgroundScript: false,
    BackgroundUpdatePeriod: null,

    Theme: "Light",
    ShowNewsFeed: true,
    RegionCode: "US",

    LocationRepeatCounter: 0,

    LogedIn: null,

    OnFinishedCallback: null,

    Init: function (options) {
        var self = this;

        var opt = options !== undefined ? options : {};
        if (opt.OnFinished) {
            self.OnFinishedCallback = opt.OnFinished;
        }

        NewsTab.IsExtension = true;
        NewsTab.SetDeveloperMode(NewsTab.Helpers.GetLocalStorageItem("DeveloperMode"));

        if (self.NId === null) {
            NewsTab.Helpers.RemoveLocalStorageItem("LastConfigUpdate");
            NewsTab.Helpers.RemoveLocalStorageItem("Config");
        } else {
            self.NId = parseInt(self.NId);
            if (self.NId === 2) {
                self.NId = 0;
            } else if (self.NId > 0 && self.NId < 100000) {
                self.NId = self.NId + 200000;
            }
        }

        chrome.runtime.getPackageDirectoryEntry(function (dirEntry) {
            //StandAlone Config
            dirEntry.getFile("Configs/config.json", undefined, function (fileEntry) {
                fileEntry.file(function (file) {
                    var reader = new FileReader();
                    reader.addEventListener("load", function () {
                        self.Config = JSON.parse(reader.result);
                        self.NId = self.Config.Id;
                        self.UpdateConfig();
                    });
                    reader.readAsText(file);
                });
            }, function () {
                //Generic Config
                dirEntry.getFile("Configs/fallback-config.json", undefined, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var reader = new FileReader();
                        reader.addEventListener("load", function () {
                            self.Config = JSON.parse(reader.result);
                            if (self.NId === null) {
                                self.NId = self.Config.Id;
                            }
                            if (opt.Source === "install") {
                                self.CheckForCookieNId();
                            } else if (opt.Source === "change") {
                                self.CheckForNIdExchange(opt);
                            } else {
                                self.UpdateConfig();
                            }
                        });
                        reader.readAsText(file);
                    });
                }, function (erorr) {
                    NewsTab.TrackAnalyticsEvent("Error", "Init [Nid=null]", chrome.runtime.id);
                    console.error("Can't access local config file.", erorr);
                });
            });
        });
    },

    CheckForCookieNId: function () {
        var self = this;
        window.chrome.cookies.get({ url: NewsTab.EnviromentUrl, name: "ExtensionConfigUrl" }, function (cookie) {
            if (cookie !== null && cookie !== undefined) {
                if (cookie.value !== null && cookie.value !== undefined) {
                    var url = new URL(cookie.value);
                    self.NId = parseInt(NewsTab.Helpers.GetParameterFromUrl("nid", url.href));
                    self.ClickId = NewsTab.Helpers.GetParameterFromUrl("clickid", url.href);
                    window.chrome.cookies.remove({ url: NewsTab.EnviromentUrl, name: cookie.name });
                }
            }
            self.UpdateConfig();
        });
    },

    CheckForNIdExchange: function (options) {
        var self = this;

        localStorage.clear();

        var url = new URL(options.Url);
        self.NId = parseInt(NewsTab.Helpers.GetParameterFromUrl("nid", url.href));
        self.ClickId = NewsTab.Helpers.GetParameterFromUrl("clickid", url.href);

        self.UpdateConfig();
    },

    UpdateConfig: function () {
        var self = this;

        var lastConfigUpdate = NewsTab.Helpers.GetLocalStorageItem("LastConfigUpdate");
        if (lastConfigUpdate !== null) {
            var newDate = Date.now();
            if ((newDate - lastConfigUpdate) / NewsTab.Helpers.HOUR < 6) {
                var storedConfig = NewsTab.Helpers.GetLocalStorageItem("Config");
                if (storedConfig && storedConfig.Id === self.NId) {
                    self.Config = storedConfig;
                    self.SetupExtension();
                    return;
                }
            }
        }

        var xhr = new XMLHttpRequest();
        xhr.addEventListener("load", function (ev) {
            var webConfig = JSON.parse(xhr.responseText);
            if (webConfig && webConfig.Id === self.NId) {
                self.Config = webConfig;

                NewsTab.Helpers.SetSyncStorage("NId", self.NId);
                NewsTab.Helpers.SetLocalStorageItem("Config", self.Config);
                NewsTab.Helpers.SetLocalStorageItem("LastConfigUpdate", Date.now());
                self.SetupExtension(true);
            } else {
                self.SetupExtension();
            }
        });
        xhr.addEventListener("error", function (ev) {
            console.error("Failed to load config from web.", ev);
            NewsTab.TrackAnalyticsEvent("Error", "UpdateConfig [Nid=null]", self.NId);

            var storedConfig = NewsTab.Helpers.GetLocalStorageItem("Config");
            if (storedConfig && storedConfig.Id === self.NId) {
                self.Config = storedConfig;
            }
            self.SetupExtension();
        });
        xhr.open("GET", self.S3_URL + "/ExtensionConfigs/V1/" + NewsTab.Helpers.PadStart(self.NId, 6) + ".json?ver=" + chrome.runtime.getManifest().version, true);
        xhr.send();
    },

    SetupExtension: function (freshConfig) {
        var self = this;

        self.NameId = NewsTab.Helpers.FirstLetterCapitalize(self.Config.Name.toLowerCase()) + "_" + NewsTab.Helpers.PadStart(self.NId, 6);

        window.chrome.storage.sync.get(["CustomLinks"], function (data) {
            if (data !== undefined && data.CustomLinks !== undefined) {
                var customLinkIsChangedByUser = data.CustomLinks.filter(function (x) { return x.Custom === true; });
                if (customLinkIsChangedByUser.length === 0) {
                    NewsTab.Helpers.SetSyncStorage("CustomLinks", self.Config.CustomLinks);
                }
            } else {
                NewsTab.Helpers.SetSyncStorage("CustomLinks", self.Config.CustomLinks);
            }
        });

        var showSimplified = NewsTab.Helpers.GetLocalStorageItem("Show_Simplified_Article");
        if (showSimplified !== true && showSimplified !== false) {
            NewsTab.Helpers.SetLocalStorageItem("Show_Simplified_Article", self.Config.ShowSimplifieldView === undefined ? true : self.Config.ShowSimplifieldView);
        }

        if (freshConfig === true) {
            self.SetUninstallPage();
        }
        self.UIInit();
        self.API_GetRegionData(function () { });

        if (!self.IsBackgroundScript) {
           
            
            //var feedInterval = setInterval(function () {
            //    if (NewsTab.Helpers.GetLocalStorageItem("ExtensionSections") !== null) {
            //        clearInterval(feedInterval);
                    
            //    }
            //}, 20);
        }

        if (self.OnFinishedCallback !== null) {
            self.OnFinishedCallback();
        }
    },

    UIInit: function () {
        var self = this;

        // Priority #1 UI Related Init - Not needed for Background Page
        if (!self.IsBackgroundScript) {
            NewsTab.Modules.ArticleView.MASTER_LAYOUT = NewsTab.Helpers.GetLocalStorageItem("Layout");
            if (NewsTab.Modules.ArticleView.MASTER_LAYOUT === null) {
                NewsTab.Modules.ArticleView.MASTER_LAYOUT = "layout-cascading-grid";
            }

            self.LogedIn = NewsTab.Helpers.GetLocalStorageItem("LogedIn");

            //Override DEFAULT_SETTINGS for ArticleView Module
            NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.ContainerSelector = "#articles_grid";
            NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.ActiveBoxSize = function () {
                var gridWidth = $(NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.ContainerSelector).width();
                if (gridWidth < 500) {
                    return 1;
                } else if (gridWidth >= 500 && gridWidth < 844) {
                    return 2;
                } else if (gridWidth >= 844 && gridWidth < 1188) {
                    return 3;
                } else if (gridWidth >= 1188 && gridWidth < 1532) {
                    return 4;
                } else if (gridWidth >= 1532 && gridWidth < 1876) {
                    return 5;
                } else if (gridWidth >= 1876 && gridWidth < 2048) {
                    return 6;
                } else {
                    return 7;
                }
            };
            NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.ShowSimplifiedArticle = function () {
                var simplifiedArticleView = NewsTab.Helpers.GetLocalStorageItem("Show_Simplified_Article");
                return simplifiedArticleView !== false;
            };

            self.Get_RegionCode();

            self.Theme = NewsTab.Helpers.GetLocalStorageItem("Theme");
            self.SetUp_Theme();

            var hideArticles = NewsTab.Helpers.GetLocalStorageItem("Hide_Articles");
            if (hideArticles === true) {
                self.ShowNewsFeed = false;
                $(document.body).addClass("hideArticles");
            } else {
                self.ShowNewsFeed = true;
                $(document.body).removeClass("hideArticles");
                $(NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.ContainerSelector).html('<div id="feedLoader" class="loader0">Loading...</div>');
            }
            NewsTab.Modules.NotificationsManager.Init();
            NewsTab.Modules.TimeAndWeatherWidget.Init();
            NewsTab.Modules.SearchWidget.Init();
            NewsTab.Modules.TopSitesWidget.Init();
        }

        // START - Init of Data Requierd by Content and Background Page
        self.BackgroundUpdatePeriod = NewsTab.Helpers.GetLocalStorageItem("Update_Period");


        NewsTab.Token = NewsTab.Helpers.GetLocalStorageItem("TokenKey");
        NewsTab.Modules.SectionsManager.ExtensionSections = [];

        var sections = NewsTab.Helpers.GetLocalStorageItem("ExtensionSections");
        if (sections !== null) {
            NewsTab.Modules.SectionsManager.ExtensionSections = sections;
        }
        else {
            var defaultSection = new Section();
            defaultSection.Id = this.DEFAULT_SECTION_ID;
            defaultSection.Name = "Default News Feeds";
            defaultSection.ClassName = "googlechrome";
            defaultSection.Layout = "layout-grid";
            defaultSection.LayoutType = 1;
            defaultSection.Feeds = [];
            defaultSection.Color = "#333333";

            var additionalSection = new Section();
            additionalSection.Id = this.ADDITIONAL_SECTION_ID;
            additionalSection.Name = "Popular Topics";
            additionalSection.ClassName = "googlechrome";
            additionalSection.Layout = "layout-grid";
            additionalSection.LayoutType = 1;
            additionalSection.Feeds = [];
            additionalSection.Color = "#333333";

            NewsTab.Modules.SectionsManager.ExtensionSections.push(defaultSection);
            NewsTab.Modules.SectionsManager.ExtensionSections.push(additionalSection);
        }

        if (NewsTab.Token) {
            //if (self.LogedIn === true) {
            var userData = NewsTab.Helpers.GetLocalStorageItem("UserData");
            // Use the localy stored data for fast display and update with fresh data once API_GetLightUserData is done
            if (userData) {
                NewsTab.UseUserData(userData);
            }
            //}
            NewsTab.API_GetLightUserData();
            if (!self.IsBackgroundScript) {
                NewsTab.Modules.RegionManager.API_GetCountries("");
            }
        }
        else {
            localStorage.removeItem("UserData");
            localStorage.removeItem("LogedIn");
        }
        // END - Init of Data Requierd by Content and Background Page

        if (!self.IsBackgroundScript) {
            NewsTab.API_GetInitialDataForPublicExtension();

            NewsTab.Modules.SideMenu.Init();
            NewsTab.Modules.TopMenu.Init();

            NewsTab.Modules.SectionsManager.STORE_CURRENT_SECTION_FEED_ID = true;
            NewsTab.Modules.SectionsManager.DEFAULT_FEED_PARSING_OPTIONS.StoreFeedItemsLocaly = true;

            if (NewsTab.Token === null || userData !== null) {

                var currentSectionId = NewsTab.Helpers.GetLocalStorageItem("CurrentSectionId");
                var currentFeedId = NewsTab.Helpers.GetLocalStorageItem("CurrentFeedId");

                if (NewsTab.Modules.SectionsManager.SetCurrentSectionAndFeed(currentSectionId, currentFeedId) === true) {
                    NewsTab.Modules.SectionsManager.LoadCurrentFeed();
                    self.SetUp_NewTabFeedTitle();
                }
            }

            NewsTab.Modules.SectionsManager.Init();
            NewsTab.Modules.AddNewContent.Init();

            NewsTab.Modules.ArticleView.API_ReadHistory();

            $(document).ready(function () {
                self.DocumentReadyInit();
            });
        }

        var showTimeWeatherWidget = NewsTab.Helpers.GetLocalStorageItem("Show_Time_Weather");
        if (showTimeWeatherWidget !== false) {
            NewsTab.Modules.TimeAndWeatherWidget.API_GetWeatherDetailsByCity();
        }
    },

    DocumentReadyInit: function () {
        var self = this;

        setTimeout(function () {
            var sharedArticle = NewsTab.Helpers.GetLocalStorageItem("NewsTab_SharedArticle");
            if (sharedArticle !== null) {
                NewsTab.Helpers.ClearLocalStorageByKey("NewsTab_SharedArticle");

                sharedArticle._sourceFeed = NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed;
                sharedArticle.SourceFeedId = sharedArticle._sourceFeed.Id;
                NewsTab.Modules.ArticleView.SetUp_ExpandView(new Article().Init(sharedArticle), true);
            }
        }, 100);


        //SocialTab.Menu_AttacheEventHandlers();
        NewsTab.Modules.ImageCollage.Init();
        if (self.LogedIn === null && NewsTab.Token) {
            NewsTab.Modules.AccountManager.OpenSignInSuccessfull();
        }

        // GLOBAL EVENTS
        $(document.body).on("click", function (e) {
            var target = $(e.target);

            if (!target.hasClass(".dropdown") &&
                !target.hasClass(".customDropdown") &&
                target.closest(".dropdown").length === 0 &&
                target.closest(".customDropdown").length === 0) {
                $(".dropdown").removeClass("open");
                $(".customDropdown").remove();
            }
        });

        $(document.body).on("error", ".imgWithOnError", function (e) {
            $(this).attr("src", $(this).data("error-src"));
        });

        // Google Analytics Reporting
        $(document.body).on("click", ".gAbalyticsReport", function () {
            var category = $(this).data("report-category");
            var categories = category.split("|");

            var action = $(this).data("report-action");
            if (categories.length > 1) {
                $.each(categories, function (index, c) {
                    NewsTab.TrackAnalyticsEvent(c, action, NewsTab.Extension.NameId);
                });
            }
            else {
                NewsTab.TrackAnalyticsEvent(category, action, NewsTab.Extension.NameId);
            }
        });

        $(document.body).on("click", ".clickable", function () {
            var url = $(this).data("url");
            if (url === undefined) {
                url = $(this).attr("href");
            }

            var target = $(this).data("url-target");
            if (target === undefined) {
                target = $(this).attr("target");
            }

            if (target === "_blank") {
                window.open(url);
            }
            else {
                window.location.assign(url);
            }
        });

        // CLEARABLE INPUT
        function tog(v) { return v ? 'addClass' : 'removeClass'; }

        $(document.body).on('input', '.clearable', function () {
            $(this)[tog(this.value)]('x');
        }).on('mousemove', '.x', function (e) {
            var pr = parseInt($(this).css("padding-right"));
            $(this)[tog(this.offsetWidth - pr < e.clientX - this.getBoundingClientRect().left)]('onX');
        }).on('touchstart click', '.onX', function (e) {
            e.preventDefault();
            $(this).removeClass('x onX').val('').change();
        });
        // Tooltips
        $(document.body).tooltip({
            selector: "[data-tooltip=\"tooltip\"]"
        });

        $('input,textarea').blur(function () {
            $(this).attr('placeholder', $(this).data('placeholder'));
        });

        $("#newstab_submit_input").focus();

        var res;

        $(window).on('resize.main', function () {
            if (res) { clearTimeout(res); };
            res = setTimeout(function () {
                NewsTab.Modules.TopSitesWidget.Prepare_TopSites(true);
                self.UpdateArticleGridWidth();
            }, 100);
        });

      
    },

    Get_RegionCode: function () {
        if (this._regionCode === null) {
            var rc = NewsTab.Helpers.GetLocalStorageItem("Region_Code");
            if (rc === null) {
                this._regionCode = null;
            }
            else {
                this._regionCode = rc;
            }
        }
        return this._regionCode;
    },

    Set_RegionCode: function (rc) {
        this._regionCode = rc;
        NewsTab.Helpers.SetLocalStorageItem("Region_Code", this._regionCode);
    },


    Get_ArticleGridWidth: function () {
        var w = $(".most_visited_container_wrapper").innerWidth();
        var nbr = parseInt(w / (170));
        if (nbr > 7) {
            nbr = 7;
        }
        return (170 * nbr);
    },


    SetUp_NewTabFeedTitle: function () {
        var self = this;
        var $container = $("#show_hide");
        $container.html("");

        if (NewsTab.Extension.ShowNewsFeed) {
            var html = [];
            html.push('<span>');
            if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.ALL) {
                html.push('<span class="iconPreview" style="background-color:' + NewsTab.Modules.SectionsManager.CurrentSection.Color + '">' + NewsTab.Modules.SectionsManager.CurrentSection.Name.substring(0, 2) + '</span>');
                html.push('<span class="publisherTitle">' + NewsTab.Modules.SectionsManager.CurrentSection.Name.replace(" - Google News", "").replace(" (Google News)", "") + '</span>');
            }
            else if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Icon != undefined) {
                if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Icon.indexOf("http") > -1) {
                    html.push('<img class="publisherFavIcon" src="' + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Icon + '" />');
                } else {
                    html.push('<span class="listIcon ' + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Icon + '"></span>');
                }
                html.push('<span class="publisherTitle">' + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name.replace(" - Google News", "").replace(" (Google News)", "") + '</span>');
            } else if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type.Id === Feed.TYPES.TWITTER.Id) {
                html.push('<span class="icon-twitter"></span>');
                html.push('<span class="publisherTitle">' + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name.replace(" - Google News", "").replace(" (Google News)", "") + '</span>');
            }

            html.push('</span>');
            html.push('<span class="line-separator"> | </span>');
            html.push('<span class="show-hide-title">Hide news');
            html.push('</span>');
            html.push('<span class="line-separator"> | </span>');
            html.push('<span class="refresh mainRefrash"><i class="icon-refresh"></i></span>');
            $container.append(html.join(""));

            $(".publisherFavIcon",$container).on("error", function (e) {
                this.onerror = null;
                this.src = "/Media/ExtensionImages/rss.png";
            });
            //NewsTab.Extension.Show_Tooltip({
            //    Destination: $(".show-hide-title"),
            //    Id: "Tooltip_ShowHideNews_hideNews",
            //    Text: "You can hide the news feed and return to a classic Chrome tab look",
            //    BtnText: "GOT IT",
            //    Position: "bottom"
            //});

            $(".publisherFavIcon, .publisherTitle", $container).on("click", function (e) {
                e.stopPropagation();
                NewsTab.Modules.SideMenu.ToggleSideMenu();
            });

            $(".refresh", $container).on("click", function () {
                $(".refresh.mainRefrash .icon-refresh").removeClass("newArticles");
                $(".refresh.mainRefrash .icon-refresh").addClass("spin");

                NewsTab.Modules.SectionsManager.LoadCurrentFeed({
                    Refresh: true,
                    CompleteCallback: function () {
                        $(".refresh.mainRefrash .icon-refresh").removeClass("spin");
                    }
                });
            });
        } else {
            var $showArticles = $("<span/>", {
                "class": "show-hide-title hide-title",
                "title": "Show articles",
                "html": "<i class=\"icon-showsource\"></i>"
            });

            $container.append($showArticles);

            NewsTab.Extension.Show_Tooltip({
                Destination: $(".show-hide-title"),
                Id: "Tooltip_ShowHideNews_showNews",
                Text: "Click to show news on your New Tab",
                BtnText: "GOT IT",
                Position: "right",
                Offset: { X: 0, Y: 12 }
            });
        }

        $(".show-hide-title", $container).on("click", function (e) {
            NewsTab.Extension.ToggleNewsFeed();
        });

        $(".customeTooltip > i,.customeTooltip > span", $container).on("click", function (e) {
            e.stopPropagation();
            NewsTab.Helpers.SetLocalStorageItem("Tooltip_ShowHideNews_" + $(this).parent().attr("data-tooltip-id"), true);
            $(this).parent().remove();
        });
    },

    SetUp_Theme: function () {
        var self = this;

        switch (self.Theme) {
            case "Dark":
                $("body").removeClass("light");
                $("body").removeClass("white");
                $("body").removeClass("black");
                $("body").addClass("dark");
                break;
            case "White":
                $("body").removeClass("dark");
                $("body").removeClass("black");
                $("body").addClass("light");
                $("body").addClass("white");
                break;
            case "Black":
                $("body").removeClass("light");
                $("body").removeClass("white");
                $("body").addClass("dark");
                $("body").addClass("black");
                break;
            default:
                $("body").removeClass("dark");
                $("body").removeClass("white");
                $("body").removeClass("black");
                $("body").addClass("light");
        }
    },

    Set_SportDate: function () {
        NewsTab.Modules.SportWidget.API_GetSportData(true, function () {
            chrome.windows.getCurrent(function (currentWindow) {
                chrome.tabs.query({ title: "New Tab", windowId: currentWindow.id }, function (newTabs) {
                    newTabs.map(function (tab) {
                        chrome.tabs.sendMessage(tab.id, { command: "update_sport_data" });
                    });
                });
            });
        });
    },

    GenerateColage: function () {
        var interval = 1000 * 60 * 60 * 6; // Update interval
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
        var lastTimeGenerated = collage != undefined ? collage.Generated_At : undefined;
        if (lastTimeGenerated) {
            lastTimeGenerated = parseInt(lastTimeGenerated);
            var currentTime = NewsTab.Helpers.GetTime();
            var nextTimeToGenerate = currentTime - lastTimeGenerated;
            var newTimeOut = lastTimeGenerated + interval - currentTime;

            if (nextTimeToGenerate > 0 && nextTimeToGenerate < interval) {
                setTimeout(function () {
                    NewsTab.Extension.GenerateColage();
                }, newTimeOut);

                return;
            }
        }

        if (collage != undefined) {
            //collage.Generated_At = NewsTab.Helpers.GetTime();
            //NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
            //NewsTab.Modules.Wallpapers.Create_Collage(collage.FeedId, {
            //    Refresh: true,
            //    LoadInbackground: true,
            //    ShowCollage: false,
            //    CompleteCallback: function () {
            //        NewsTab.Extension.GenerateColage();
            //    }
            //});
        }
    },

    ForceHideNewsFeed: function () {
        var self = this;
        self.ShowNewsFeed = false;
        self.UpdateNewsFeedLayout();
    },

    ToggleNewsFeed: function () {
        var self = this;
        self.ShowNewsFeed = !self.ShowNewsFeed;
        self.UpdateNewsFeedLayout();
    },

    UpdateNewsFeedLayout: function () {
        var self = this;

        NewsTab.Modules.TopSitesWidget.Prepare_TopSites(true);
        self.SetUp_NewTabFeedTitle();

        if (self.ShowNewsFeed) {
            NewsTab.Helpers.SetLocalStorageItem("Hide_Articles", false);

            self.UpdateArticleGridWidth();
            NewsTab.Modules.SectionsManager.LoadCurrentFeed({ Refresh: true });
            $(document.body).removeClass("hideArticles");
        } else {
            $(document.body).addClass("hideArticles");
            NewsTab.Helpers.SetLocalStorageItem("Hide_Articles", true);
        }
        NewsTab.Modules.Wallpapers.Set_FullScreen(!self.ShowNewsFeed);
    },

    UpdateArticleGridWidth: function () {
        var self = this;

        if (self.ShowNewsFeed) {
            var $container = $("#articles_grid");
            var containerSize = self.Get_ArticleGridWidth();

            $("#show_hide").width(containerSize - 30);

            if ($container.hasClass("layout-cascading-grid")) {
                $container.width(containerSize + 10); // margin on cards 13px * 2 = 26, margin on top sites 8px * 2 = 16, 26 - 16 = 10 for lining up verticaly

                NewsTab.Modules.ArticleView.UpdatePreviewItemsSize();

                var $mcontainer = $container.masonry({
                    itemSelector: ".socialTabItem",
                    columnWidth: $container.find(".socialTabItem")[0],
                    transitionDuration: 0,
                    stagger: 0,
                    isAnimated: false
                });

                $container.masonry("reloadItems").masonry("layout");
                $mcontainer.masonry();

                setTimeout(function () {
                    $container.masonry("reloadItems").masonry("layout");
                    $mcontainer.masonry();
                }, 200);
            } else {
                $container.width(containerSize - 16);
            }
        }
    },

    Show_Tooltip: function (options) {
        var self = this;

        var htmlContent = "";
        htmlContent += "<i class='icon-close tlp-close'></i>";
        htmlContent += "<p>" + options.Text + "</p>";

        htmlContent += "<div class='bottomButtons'>";
        if (options.Step) {
            htmlContent += "<i class='step'>" + options.Step + "</i>";
        }
        htmlContent += "<i class='tlp-button'>" + options.BtnText + "</i>";
        htmlContent += "</div>";

        var tooltip = NewsTab.Helpers.GetLocalStorageItem("Tooltip-" + options.Id);
        if (tooltip !== true) {
            setTimeout(function () {
                options.Destination.showBalloon({
                    contents: htmlContent,
                    html: true,
                    classname: "customeTooltip",
                    position: options.Position || "bottom",
                    offsetX: options.Offset ? options.Offset.X : 0,
                    offsetY: options.Offset ? options.Offset.Y : 0,
                    tipPosition: options.TipPosition
                });

                setTimeout(function () {
                    $(".tlp-button").on("click", function (e) {
                        e.preventDefault();
                        $(".customeTooltip").remove();

                        if (options.NextStep != undefined) {
                            self.Show_Tooltip(options.NextStep);
                        }
                    });

                    $(".tlp-close").on("click", function (e) {
                        e.preventDefault();
                        $(".customeTooltip").remove();
                    });
                }, options.Animation || 500);

                NewsTab.Helpers.SetLocalStorageItem("Tooltip-" + options.Id, true);
            }, options.Animation || 500);
        }
    },


    OpenWelcomePage: function () {
        var self = this;

        var additional = "";
        if (NewsTab.Extension.ClickId !== null) {
            additional = "?clickid=" + NewsTab.Extension.ClickId;
        }

        var landingPage = NewsTab.EnviromentUrl + "/extensions/welcome/" + self.NId + additional;
        if (self.Config.OverrideWelcomePageUrl !== undefined && self.Config.OverrideWelcomePageUrl.length > 0) {
            landingPage = self.Config.OverrideWelcomePageUrl + additional;
        }

        window.chrome.tabs.create({ url: landingPage });
    },

    SetUninstallPage: function () {
        var self = this;

        var uninstallPage = NewsTab.EnviromentUrl + "/extensions/uninstall/" + self.NId;

        if (self.Config.OverrideUninstallPageUrl !== undefined && self.Config.OverrideUninstallPageUrl.length > 0) {
            uninstallPage = self.Config.OverrideUninstallPageUrl;
        }

        window.chrome.runtime.setUninstallURL(uninstallPage);
    },


    SignOut: function () {
        NewsTab.Token = null;

        localStorage.removeItem("TokenKey");
        localStorage.removeItem("UserData");
        localStorage.removeItem("LogedIn");

        window.location.reload();
    },

    /* API */

    API_GetRegionData: function () {
        var city = NewsTab.Helpers.GetLocalStorageItem("City");

        if (NewsTab.Extension.Get_RegionCode() !== null && city != null && city != undefined) {
            NewsTab.Modules.SectionsManager.API_GetFeedsByRegion(NewsTab.Extension.Get_RegionCode());

            chrome.windows.getCurrent(function (currentWindow) {
                chrome.tabs.query({ title: "New Tab", windowId: currentWindow.id }, function (newTabs) {
                    newTabs.map(function (tab) {
                        chrome.tabs.sendMessage(tab.id, { command: "update_weather" });
                    });
                });
            });
        } else {
            $.ajax({
                url: "https://geoip-db.com/json/",
                method: "GET",
                dataType: "json"
            }).done(function (data) {
                NewsTab.Extension.LocationRepeatCounter = 0;

                NewsTab.Extension.Set_RegionCode(data.country_code);
                NewsTab.Helpers.SetLocalStorageItem("City", data.city);

                var timeFormat = NewsTab.Helpers.GetLocalStorageItem("TimeFormat");
                if (timeFormat == undefined) {
                    NewsTab.Helpers.SetLocalStorageItem("TimeFormat", "h:mm");
                }
                var degreeFormat = NewsTab.Helpers.GetLocalStorageItem("DegreeFormat");
                if (degreeFormat == undefined) {
                    degreeFormat = "c";
                    NewsTab.Helpers.SetLocalStorageItem("DegreeFormat", degreeFormat);
                }

                chrome.windows.getCurrent(function (currentWindow) {
                    chrome.tabs.query({ title: "New Tab", windowId: currentWindow.id }, function (newTabs) {
                        newTabs.map(function (tab) {
                            chrome.tabs.sendMessage(tab.id, { command: "update_weather", type: "full_update" });
                        });
                    });
                });

                NewsTab.Modules.SectionsManager.API_GetFeedsByRegion(NewsTab.Extension.Get_RegionCode());
            }).fail(function (jqXHR, textStatus) {
                NewsTab.Extension.LocationRepeatCounter++;

                if (NewsTab.Extension.LocationRepeatCounter <= 10) {
                    setTimeout(function () {
                        NewsTab.Extension.API_GetRegionData();
                    }, 10 * 1000);
                } else {
                    NewsTab.Extension.Set_RegionCode("US");

                    var timeFormat = NewsTab.Helpers.GetLocalStorageItem("TimeFormat");
                    if (timeFormat == undefined) {
                        NewsTab.Helpers.SetLocalStorageItem("TimeFormat", "h:mm");
                    }
                    var degreeFormat = NewsTab.Helpers.GetLocalStorageItem("DegreeFormat");
                    if (degreeFormat == undefined) {
                        degreeFormat = "c";
                        NewsTab.Helpers.SetLocalStorageItem("DegreeFormat", degreeFormat);
                    }

                    chrome.windows.getCurrent(function (currentWindow) {
                        chrome.tabs.query({ title: "New Tab", windowId: currentWindow.id }, function (newTabs) {
                            newTabs.map(function (tab) {
                                chrome.tabs.sendMessage(tab.id, { command: "update_weather" });
                            });
                        });
                    });

                    NewsTab.Modules.SectionsManager.API_GetFeedsByRegion(NewsTab.Extension.Get_RegionCode());
                }
            });
        }
    },

    API_SendEmailFromExtension: function (toEmail, extensionName, link, callback) {
        $.ajax({
            url: NewsTab.ApiUrl_Global + "post/sendemailfromextension/" + toEmail + "/" + extensionName,
            type: "POST",
            headers: {
                Link: link
            },
            success: function () {
                if (typeof callback === "function") {
                    callback();
                }
            }
        });
    }
};