var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.SportWidget = {
    Init: function () {
        var self = this;
        self.SportRepeatCounter = 0;

        self.SetUp_SportWidgetPlaceholder();
    },

    SetUp_SportWidgetPlaceholder: function () {
        $(".sport-widget-placeholder").remove();

        var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");
        if (sportData == undefined || (sportData != undefined && sportData.VisibleMode == "hide")) {
            NewsTab.Modules.SportWidget.API_GetSportData();
            return;
        }

        var visibleMode = (sportData != undefined && sportData.VisibleMode != undefined) ? sportData.VisibleMode + "-widget" : "";

        $("body").append("<div class=\"sport-widget-placeholder " + visibleMode + "\"><div class=\"content\"></div></div>");
        var placeholder = $(".sport-widget-placeholder");
        var content = placeholder.find(".content");

        var lastMatchDate = sportData != undefined && sportData.Matches.length > 1 ? new Date(sportData.Matches[1].Date) : new Date();
        var daysDiff = NewsTab.Helpers.TimeDifferenceInDays(lastMatchDate, new Date());
        if (daysDiff <= 14) {
            // Inject data from storage if exist
            if (sportData != undefined && sportData.Matches.length > 0) {
                if (sportData.VisibleMode === "maxi") {
                    NewsTab.Modules.SportWidget.SetUp_SportWidgetHtml(sportData);
                } else if (sportData.VisibleMode === "mini") {
                    NewsTab.Modules.SportWidget.SetUp_MiniSportWidgetHtml(sportData);
                } else {
                    $(".sport-widget-placeholder").remove();
                }
            }

            // Update from API
            NewsTab.Modules.SportWidget.API_GetSportData();
        } else {
            $(".sport-widget-placeholder").addClass("minimized");
            NewsTab.Modules.SportWidget.SetUp_SportWidgetHtml(sportData);
        }

        return placeholder;
    },

    SetUp_SportWidgetHtml: function (data) {
        var placeholder = $(".sport-widget-placeholder");
        var content = placeholder.find(".content");
        content.html("");
        var item = "<div class=\"sw-sport-icon-wrapper\">";
        item += "<div class=\"resize-sport-widget sw-more-icon-wrapper\">";
        item += "<a href=\"" + NewsTab.Helpers.GetLocalStorageItem("Config").SportWidget.GoogleTeamUrl + "\"><img src=\"" + data.Logo + "\" /></a>";
        item += "<span><i class=\"icon icon-more\"></i></span>";
        item += "</div>";
        item += "</div>";
        item += "<a class=\"sw-match-data\" href=\"" + NewsTab.Helpers.GetLocalStorageItem("Config").SportWidget.GoogleTeamUrl + "\">";
        item += NewsTab.Modules.SportWidget.Get_SporWidgetData();
        item += "</a>";

        content.append(item);

        NewsTab.Modules.SportWidget.Events(placeholder, data);

        NewsTab.Modules.SportWidget.RefreshSportWidget();
    },

    SetUp_MiniSportWidgetHtml: function (data, isEmpty) {
        var placeholder = $(".sport-widget-placeholder");
        placeholder.addClass("mini-widget");
        var content = placeholder.find(".content");

        content.html("");
        var item = "<a class=\"sw-match-data\" href=\"" + NewsTab.Helpers.GetLocalStorageItem("Config").SportWidget.GoogleTeamUrl + "\">";
        item += NewsTab.Modules.SportWidget.Get_MiniSporWidgettData();
        item += "</a>";
        item += "<div class=\"sw-more-icon-wrapper\">";
        item += "<i class=\"icon icon-more\"></i>";
        item += "</div>";

        content.append(item);

        NewsTab.Modules.SportWidget.Events(placeholder, data);

        NewsTab.Modules.SportWidget.RefreshSportWidget();
    },

    Get_SporWidgetData: function () {
        var result = "";
        var col = 1;

        var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");

        sportData.Matches.forEach(function (match) {
            if (match == null) return;

            var inProgClass = "";
            if (match.Status == "InProgress") {
                $(".sport-widget-placeholder").addClass("inProgress");
            }

            result += "<div class=\"sw-col-" + col + "\">";
            result += "<div class=\"sw-match-date\">"
            result += "<span>" + moment(match.Date).format('MMM D') + "</span> | ";
            if (match.GameMinute != "-1") {
                result += "<span class=\"sw-game-minute\">(" + match.GameMinute + ")</span>";
            } else {
                result += "<span>" + moment(match.Date).format('HH:mm') + "</span>";
            }

            var homeResult = parseInt(match.Result.split('-')[0]);
            var awayResult = parseInt(match.Result.split('-')[1]);
            var homeTeamIsWinner = match.Status === "Finished" && (homeResult > awayResult || match.HomeAdditionalResult !== "");
            var awayTeamIsWinner = match.Status === "Finished" && (homeResult < awayResult || match.AwayAdditionalResult !== "");

            result += "<br /><span class=\"sw-competition\">" + match.Competition + "</span>";
            result += "</div>";
            result += "<div class=\"sw-teams\">";
            result += "<div class=\"sw-home-team " + (homeTeamIsWinner ? "sw-team-winner" : "") + "\">";
            result += "<div class=\"sw-logo\">";
            result += "<img src=\"" + match.HomeTeamLogo + "\" />";
            result += "</div>";
            result += "<div class=\"sw-name\">";
            result += "<span>" + match.HomeTeam + "</span>";
            result += "</div>";
            result += "<div class=\"sw-result\">";
            if (match.Status === "InComing") {
                result += "<span> - </span>";
            } else {
                if (match.HomeAdditionalResult === "") {
                    result += "<span>" + homeResult + "</span>";
                } else {
                    result += "<span>" + homeResult + " (" + match.HomeAdditionalResult + ")</span>";
                }
                if (homeTeamIsWinner) {
                    result += "<span><i class=\"icon icon-play\"></i></span>";
                }
            }
            result += "</div>";
            result += "</div>";
            result += "<div class=\"sw-away-team " + (awayTeamIsWinner ? "sw-team-winner" : "") + "\">";
            result += "<div class=\"sw-logo\">";
            result += "<img src=\"" + match.AwayTeamLogo + "\" />";
            result += "</div>";
            result += "<div class=\"sw-name\">";
            result += "<span>" + match.AwayTeam + "</span>";
            result += "</div>";
            result += "<div class=\"sw-result\">";
            if (match.Status === "InComing") {
                result += "<span> - </span>";
            } else {
                if (match.AwayAdditionalResult === "") {
                    result += "<span>" + awayResult + "</span>";
                } else {
                    result += "<span>" + awayResult + " (" + match.AwayAdditionalResult + ")</span>";
                }
                if (awayTeamIsWinner) {
                    result += "<i class=\"icon icon-play\"></i>";
                }
            }
            result += "</div>";
            result += "</div>";
            result += "</div>";

            result += "</div>";

            col++;
        });

        return result;
    },

    Get_MiniSporWidgettData: function () {
        var result = "";
        var match;
        var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");

        if (sportData.Matches[0] == null) return;

        if (sportData.Matches.length > 1) {
            match = sportData.Matches[1];
        } else {
            match = sportData.Matches[0];
        }

        if (match.Status == "InProgress") {
            $(".sport-widget-placeholder").addClass("inProgress");
        }

        var homeResult = parseInt(match.Result.split('-')[0]);
        var awayResult = parseInt(match.Result.split('-')[1]);
        var homeTeamIsWinner = match.Status === "Finished" && (homeResult > awayResult || match.HomeAdditionalResult !== "");
        var awayTeamIsWinner = match.Status === "Finished" && (homeResult < awayResult || match.AwayAdditionalResult !== "");

        result += "<div class=\"sw-match-date\" data-tooltip=\"tooltip\" data-placement=\"left\" title=\"" + match.Competition + "\">"
        result += "<span>" + moment(match.Date).format('MMM D') + "</span><br/>";
        if (match.GameMinute != "-1") {
            result += "<span class=\"sw-game-minute\">(" + match.GameMinute + ")</span>";
        } else {
            result += "<span>" + moment(match.Date).format('HH:mm') + "</span>";
        }
        result += "</div>";
        result += "<div>";
        result += "<div class=\"sw-logo home-team-logo\" data-tooltip=\"tooltip\" data-placement=\"left\" title=\"" + match.HomeTeam + "\">";
        result += "<img src=\"" + match.HomeTeamLogo + "\" />";
        result += "</div>";
        result += "<div class=\"sw-result\">";
        if (match.Status === "InComing") {
            result += "<span> - </span>";
        } else {
            if (match.HomeAdditionalResult === "") {
                result += "<span class=\"" + (homeTeamIsWinner ? "sw-team-winner" : "") + "\">" + homeResult + "</span>";
            } else {
                result += "<span class=\"" + (homeTeamIsWinner ? "sw-team-winner" : "") + "\">" + homeResult + " (" + match.HomeAdditionalResult + ")</span>";
            }
        }
        result += "</div>";
        result += "<div class=\"sw-result\">";
        if (match.Status === "InComing") {
            result += "<span> - </span>";
        } else {
            if (match.AwayAdditionalResult === "") {
                result += "<span class=\"" + (awayTeamIsWinner ? "sw-team-winner" : "") + "\">" + awayResult + "</span>";
            } else {
                result += "<span class=\"" + (awayTeamIsWinner ? "sw-team-winner" : "") + "\">" + awayResult + " (" + match.AwayAdditionalResult + ")</span>";
            }
        }
        result += "</div>";
        result += "<div class=\"sw-logo away-team-logo\" data-tooltip=\"tooltip\" data-placement=\"left\" title=\"" + match.AwayTeam + "\">";
        result += "<img src=\"" + match.AwayTeamLogo + "\" />";
        result += "</div>";
        result += "</div>";

        return result;
    },

    RefreshSportWidget: function () {
        var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");
        var matchInProgressInterval = 0;

        if (sportData != null && sportData.Matches[1] != null && sportData.Matches[1].Status == "InProgress" && sportData.VisibleMode !== "hide") {
            matchInProgressInterval = setInterval(function () {
                NewsTab.Modules.SportWidget.API_GetSportData(function (data) {
                    if (sportData.VisibleMode === "maxi") {
                        $(".sport-widget-placeholder").find(".content .sw-match-data").html(NewsTab.Modules.SportWidget.Get_SporWidgetData());
                    } else {
                        $(".sport-widget-placeholder").find(".content .sw-match-data").html(NewsTab.Modules.SportWidget.Get_MiniSporWidgettData());
                    }

                    if (data != undefined && data.Matches[1] != undefined && data.Matches[1].Status != "InProgress") {
                        clearInterval(matchInProgressInterval);
                    }
                });
            }, 60 * 1000);
        } else {
            clearInterval(matchInProgressInterval);
        }
    },

    Events: function (placeholder, data) {
        $(".sw-more-icon-wrapper .icon-more").click(function () {
            placeholder.find(".sw-more-icon-wrapper").append(NewsTab.Modules.SportWidget.Get_Menu(data.VisibleMode));

            $(".sw-more-icon-wrapper .items").find("li").click(function () {
                var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");
                var value = $(this).data("val");
                if (value === "hide") {
                    NewsTab.Modules.NotificationsManager.Open_Modal({
                        ShowTitle: true,
                        Title: "Hide Sport Widget",
                        Text: "<div class=\"sw-hide-widget\">You can add the sport widget back from settings</div>",
                        Width: "450px",
                        Height: "165px",
                        FullscreenOption: false,
                        Buttons: [{
                            Name: "Cancel",
                            Classes: ["gray", "closeNotification", "float-left", "sw-no-margin"]
                        },
                        {
                            Name: "OK",
                            Classes: ["closeNotification"],
                            Action: function (e, $btn) {
                                sportData.VisibleMode = value;
                                NewsTab.Helpers.SetLocalStorageItem("SportWidgetData", sportData);
                                NewsTab.Modules.SportWidget.SetUp_SportWidgetPlaceholder();
                            }
                        }]
                    });
                    NewsTab.TrackAnalyticsEvent("Matches Widget", "Hide", NewsTab.Extension.NameId);
                } else {
                    sportData.VisibleMode = value;
                    NewsTab.Helpers.SetLocalStorageItem("SportWidgetData", sportData);
                    NewsTab.Modules.SportWidget.SetUp_SportWidgetPlaceholder();

                    if (value === "maxi") {
                        NewsTab.TrackAnalyticsEvent("Matches Widget", "Expand", NewsTab.Extension.NameId);
                    } else if (value === "mini") {
                        NewsTab.TrackAnalyticsEvent("Matches Widget", "Minimize", NewsTab.Extension.NameId);
                    }
                }
            });
        });

        $("body").click(function (e) {
            if ($(e.target).closest(".sw-links-popup").length === 0 && $(e.target).closest(".icon.icon-more").length === 0) {
                $(".sw-links-popup").remove();
            }
        });

        $(".sport-widget-placeholder").click(function (e) {
            if ($(e.target).closest(".icon.icon-more").length === 0) {
                NewsTab.TrackAnalyticsEvent("Matches Widget", "Clicked", NewsTab.Extension.NameId);
            }
        });
    },

    Get_Menu: function (mode) {
        var placeholder = $(".sport-widget-placeholder");
        var result = "<div class=\"sw-links-popup extension-link-popup\">";
        result += "<ul class=\"items\">";

        if (mode === "mini" || $(".sport-widget-placeholder").hasClass("minimized")) {
            result += "<li class=\"option-maxi-widget\" data-val=\"maxi\">Expand Widget</li>";
        }
        if (mode === "maxi" || $(".sport-widget-placeholder").hasClass("minimized")) {
            result += "<li class=\"option-mini-widget\" data-val=\"mini\">Minimize Widget</li>";
        }
        result += "<li class=\"option-hide-widget\" data-val=\"hide\">Hide Widget</li>";
        result += "</ul>";
        result += "</div>";

        return result;
    },

    API_GetSportData: function (ignoreDownloadSettings, callback) {
        var sportData = NewsTab.Helpers.GetLocalStorageItem("SportWidgetData");
        var config = NewsTab.Helpers.GetLocalStorageItem("Config");

        if (config != undefined && config.SportWidget != undefined) {
            if (ignoreDownloadSettings == true || sportData == undefined || sportData.Matches.length === 0 || (sportData != undefined && sportData.Matches[1] != undefined && new Date().getTime() > moment(sportData.Matches[1].Date).valueOf())) {
                var teamUrl = config.SportWidget.OperaTeamUrl + "&vch=" + new Date().getTime();
                var url = NewsTab.ApiUrl_Global + "get/sportdata?url=" + encodeURIComponent(teamUrl);
                $.ajax({
                    url: url,
                    method: "GET",
                    dataType: "json",
                    cache: false
                }).done(function (data) {
                    NewsTab.Modules.SportWidget.SportRepeatCounter = 0;

                    if (data != undefined) {
                        if (sportData == undefined || sportData.VisibleMode == undefined) {
                            data.VisibleMode = "mini";
                        } else {
                            data.VisibleMode = sportData.VisibleMode;
                        }
                        NewsTab.Helpers.SetLocalStorageItem("SportWidgetData", data);
                    }

                    if (typeof callback === "function") {
                        callback(data);
                    }
                }).fail(function () {
                    NewsTab.Modules.SportWidget.SportRepeatCounter++;

                    if (NewsTab.Modules.SportWidget.SportRepeatCounter <= 10) {
                        setTimeout(function () {
                            NewsTab.Modules.SportWidget.SportRepeatCounter.API_GetSportData(ignoreDownloadSettings, callback);
                        }, 10 * 1000);
                    } else {
                        callback(sportData);
                    }
                });
            }
        } else {
            callback(sportData);
        }
    }
}