var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.TopMenu = {
    Init: function () {
        var self = this;

        if ($(".extension-links").length === 0) {
            $(document.body).append("<div class=\"extension-links\"></div>");
        }

        window.chrome.storage.sync.get(["CustomLinks"], function (data) {
            self.AppendCustomLinks(data);
            self.AppendTodoLink();
            self.AppendRateUsLink();
            self.AppendMobileLink();

            self.AppendWallpapersSettings();

            self.AppendSportWidgetLink(); // Sport widget on the right side

            NewsTab.Setup_DeveloperMode();
        });
    },

    AppendCustomLinks: function (data) {
        if (data.length && data.CustomLinks !== null) {
            data.CustomLinks.sort(function (a, b) { return b.Order - a.Order; }).forEach(function (customLink) {
                if (customLink.Name === "MyLinks" && chrome.runtime.getManifest().homepage_url !== "http://staging.newstab.com/") return; // don't render 'My Links' for production

                if ($(".custom-link_" + customLink.Order).length === 0) {
                    if (customLink.Links.length > 0) {
                        $("<a class=\"extension-link custom-links custom-link_" + customLink.Order + "\">" + customLink.Name + "</a>")
                            .prependTo(".extension-links").off("click").on("click", function (e) {
                                if (e.target !== $(this).get(0)) return;

                                var openedSublinks = $(".custom_sublink_" + customLink.Order).length;
                                $(".custom-sublinks").remove();
                                if (openedSublinks === 0) {
                                    $("#newtab").removeClass("show_menu");
                                    $(".extension-link-popup").remove();
                                    $(".extension-popup").removeClass("show");
                                    NewsTab.Extensions.Common.SetUpCustomSublinksMenu(customLink);
                                } else {
                                    $(".custom-sublinks").remove();
                                }

                                return false;
                            });
                    } else if (NewsTab.Extension.Config.CustomLinks[customLink.Order].AddShortcut) {
                        $("<a class=\"extension-link custom-links custom-link_" + customLink.Order + "\"><i class=\"icon icon-plus\"></i></a>")
                            .prependTo(".extension-links").off("click").on("click", function (e) {
                                NewsTab.Extensions.Common.AddSubmenuPopup();
                            });
                    }

                    $("body").click(function (e) {
                        if ($(e.target).closest(".custom-sublinks").length === 0) {
                            $(".custom-sublinks").remove();
                            $(".icon-more").removeClass("active");
                        }
                    });
                }
            });
        }
    },

    AppendRateUsLink: function () {
        if ($(".rateUs-link").length === 0) {
            var showRateLink = NewsTab.Helpers.GetLocalStorageItem("Show_Rate_Link");
            if (showRateLink !== false) {
                $('<a class=\"extension-link rateUs-link\">Rate</a>')
                    .appendTo(".extension-links").click(function () {
                        var ratePh = $(".rateContent").closest(".rate-overlay").length;

                        $("#newtab").removeClass("show_menu");
                        $(".extension-link-popup").remove();

                        if ($(".rate-overlay").length > 0) {
                            $(".rate-overlay").remove();
                        }

                        if (ratePh === 0) {
                            $("#newtab").removeClass("show_menu");
                            $(".extension-link-popup").remove();
                            $(".extension-popup").removeClass("show");
                            NewsTab.Extensions.Common.SetUpRateArticleItem();
                        } else {
                            $(".rateContent").closest(".rate-overlay").remove();
                        }

                        return false;
                    });

                var count = NewsTab.Helpers.GetLocalStorageItem("NewsTab_RateCount");
                if (count !== undefined) {
                    count++;

                    if (count > NewsTab.Extensions.Config.ExtensionRate.NumberForShow) {
                        NewsTab.Extensions.Config.ExtensionRate.ShouldRate = true;
                        var showRateArtice = NewsTab.Helpers.GetLocalStorageItem("Show_Rate_Article");
                        if (showRateArtice !== false) {
                            NewsTab.Extensions.Common.SetUpRateArticleItem();
                        }
                    }
                    else {
                        NewsTab.Helpers.SetLocalStorageItem("NewsTab_RateCount", count);
                    }
                } else {
                    NewsTab.Helpers.SetLocalStorageItem("NewsTab_RateCount", 1);
                }
            }
        }
    },

    AppendTodoLink: function () {
        if ($(".todo-link").length === 0) {
            $('<a class=\"extension-link todo-link\">Todo</a>')
                .appendTo(".extension-links").click(function () {
                    var todoPh = $(".todo-placeholder").length;

                    $(".todo-placeholder").remove();
                    if (todoPh === 0) {
                        $("#newtab").removeClass("show_menu");
                        $(".extension-link-popup").remove();
                        $(".extension-popup").removeClass("show");
                        NewsTab.Extensions.Common.SetUpTodoList();
                        NewsTab.TrackAnalyticsEvent("Todo", "Open", NewsTab.Extension.NameId);
                    } else {
                        $(".todo-placeholder").remove();
                        NewsTab.TrackAnalyticsEvent("Todo", "Close", NewsTab.Extension.NameId);
                    }

                    $(".todo-form .todo-input").focus();

                    $("body").click(function (e) {
                        if ($(e.target).closest(".todo-placeholder").length === 0 && $(e.target).closest(".todo-confirm-message").length === 0 && (e.target !== undefined && e.target.className !== "icon-close remove")) {
                            $(".todo-placeholder").remove();
                        }
                    });

                    return false;
                });
        }
    },

    AppendSportWidgetLink: function () {
        if (NewsTab.Extension.Config.SportWidget !== null &&
            NewsTab.Extension.Config.SportWidget !== undefined &&
            $(".sport-widget-link").length === 0) {
            NewsTab.Modules.SportWidget.Init();
        }
    },

    AppendMobileLink: function () {
        if ($(".mobileApp-link").length === 0) {
            var showRateLink = NewsTab.Helpers.GetLocalStorageItem("Show_Mobile_Link");
            if (showRateLink !== false) {
                $('<a class=\"extension-link mobileApp-link\">Mobile App</a>')
                    .appendTo(".extension-links").click(function () {
                        var mobileLinkPh = $(".mobileApp-placeholder").length;

                        $("#newtab").removeClass("show_menu");
                        $(".mobileApp-placeholder").remove();

                        if (mobileLinkPh === 0) {
                            $("#newtab").removeClass("show_menu");
                            $(".extension-link-popup").remove();
                            $(".extension-popup").removeClass("show");
                            NewsTab.Extensions.Common.SetUpMobileAppPopup();
                        } else {
                            $(".mobileApp-placeholder").remove();
                        }

                        $("body").click(function (e) {
                            if ($(e.target).closest(".mobileApp-placeholder").length === 0) {
                                $(".mobileApp-placeholder").remove();
                            }
                        });

                        return false;
                    });
            }
        }
    },

    AppendWallpapersSettings: function () {
        NewsTab.Modules.Wallpapers.Init();
    }
}