var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.Wallpapers = {

    DefaultSrcImageData: {
        Source: {
            Name: "NewsTab",
            Url: "https://www.newstab.com"
        },
        Artist: {
            Name: "Dynamic News Collage"
        }
    },

    Type: {
        THEME: 0,
        WALLPAPER: 1,
        COLLAGE: 2
    },

    CollageType: {
        NORMAL: 0,
        FULLPAGE: 1
    },

    DefaultColors: [
        { Name: "black-color", Color: "#00000066" },
        { Name: "red", Color: "#e00002" },
        { Name: "blue", Color: "#013b8e" },
        { Name: "newstab-blue", Color: "#00b8f5" },
        { Name: "amaranth", Color: "#e91e63" },
        { Name: "cerulean", Color: "#9761A9" },
        { Name: "caribbeanGreen", Color: "#00bfa5" },
        { Name: "ronchi", Color: "#ebc24e" },
        { Name: "crusta", Color: "#fb7243" },
        { Name: "romanCoffee", Color: "#795548" }
    ],

    Init: function () {
        var self = this;

        chrome.runtime.getPackageDirectoryEntry(function (dirEntry) {
            //StandAlone Config
            dirEntry.getFile("Configs/wallpapers-config.json", undefined, function (fileEntry) {
                fileEntry.file(function (file) {
                    var reader = new FileReader();
                    reader.addEventListener("load", function () {
                        var config = JSON.parse(reader.result);
                        NewsTab.Helpers.SetLocalStorageItem("Wallpapers-Collections", config);
                        self.AppendWallpapersLink();
                    });
                    reader.readAsText(file);
                });
            }, function () {
                NewsTab.TrackAnalyticsEvent("Error", "Wallpapers Config", chrome.runtime.id);
                console.error("Can't access local wallpaper config file.", erorr);
            });
        });
    },

    AppendWallpapersLink: function () {
        var self = this;
        var showWallpapersLink = NewsTab.Helpers.GetLocalStorageItem("Wallpapers");
        if (showWallpapersLink !== false) {
            if ($(".wallpapers-link").length === 0) {
                $('<a class="extension-link wallpapers-link">Wallpaper</a>')
                    .appendTo(".extension-links").click(function () {
                        var wallpapersPh = $(".wallpapers-placeholder");

                        $("#newtab").removeClass("show_menu");

                        if (!wallpapersPh.is(":visible")) {
                            $("#newtab").removeClass("show_menu");
                            $(".extension-link-popup").remove();
                            self.SetUp_WallpapersPopup();
                        } else {
                            $(".extension-link-popup").remove();
                            wallpapersPh.removeClass("show");
                        }

                        $("body").click(function (e) {
                            if ($(e.target).closest(".wallpapers-placeholder").length === 0 && $(e.target).closest(".notificationModal").length === 0) {
                                $(".wallpapers-placeholder").removeClass("show");
                            }
                        });

                        return false;
                    });

                self.Set_Wallpaper(self.Get_PinnedWallpaper());
            }

            self.SetUp_Tooltips();
        } else {
            $(".wallpapers-link").remove();
            self.SetUp_Background();
        }
    },

    SetUp_WallpapersPopup: function () {
        var self = this;
        var $placeholder = $(".wallpapers-placeholder");
        $placeholder.addClass("show");

        var wlpType = NewsTab.Helpers.GetLocalStorageItem("Wlp-Type");
        switch (wlpType) {
            case self.Type.WALLPAPER:
                self.Select_Tab($(".tabMenu > a[data-name=wallpapersTab]"), { ChangeImage: false });
                break;
            case self.Type.COLLAGE:
                self.Select_Tab($(".tabMenu > a[data-name=collageTab]"), { ChangeImage: false });
                break;
            case self.Type.THEME:
            default:
                self.Select_Tab($(".tabMenu > a[data-name=blankTab]"), { ChangeImage: false });
                break;
        }
    },

    SetUp_WallpaperTab: function ($placeholder, options) {
        var self = this;
        var opt = $.extend({}, options);
        var shuffleAllImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-All-Images");
        var shuffleFavoriteImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-Favorite-Images");
        var collections = self.Get_WallpaperCollections();
        $placeholder.find(".wallpapers-bg-images").html(self.Get_AllWallpapers());
        $placeholder.find(".collections ul").html(self.Get_AllCollections(collections));
        NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.WALLPAPER);
        $("#newtab").removeClass("collage-bg");

        if (shuffleAllImages) {
            $placeholder.find("#shuffle-all").prop("checked", true);
            $placeholder.find("#shuffle-favorite").prop("checked", false);
        }
        if (shuffleFavoriteImages) {
            $placeholder.find("#shuffle-favorite").prop("checked", true);
            $placeholder.find("#shuffle-all").prop("checked", false);
        }

        var pinnedWallpaper = self.Get_PinnedWallpaper();
        if (pinnedWallpaper != undefined) {
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Pinned-Idx", pinnedWallpaper);
            $("#newtab").find(".wallpapers-image[data-collection-index=" + pinnedWallpaper.split("_")[0] + "][data-image-index=" + pinnedWallpaper.split("_")[1] + "]");
        }

        if (opt.ChangeImage !== false) {
            self.Set_Wallpaper(pinnedWallpaper);
        }

        self.Select_WallpaperItem();
        self.SetUp_Footer();
        NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();

        NewsTab.TrackAnalyticsEvent("New Tab", "WallPaper", NewsTab.Extension.NameId);
    },

    SetUp_CollageTab: function ($placeholder, options) {
        var self = this;
        var opt = $.extend({}, options);
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
        NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.COLLAGE);

        self.SetUp_Feeds();

        if (collage != undefined) {
            if (collage.Shuffle_All_Colors) {
                $placeholder.find("#shuffle-all").prop("checked", true);
                $placeholder.find("#shuffle-favorite").prop("checked", false);
            }
            if (collage.Shuffle_Favorite_Colors) {
                $placeholder.find("#shuffle-favorite").prop("checked", true);
                $placeholder.find("#shuffle-all").prop("checked", false);
            }

            if (collage.Type === self.CollageType.FULLPAGE) {
                $placeholder.find("#full-page").prop("checked", true);
            } else {
                $placeholder.find("#full-page").prop("checked", false);
            }

            if (collage.Images != undefined) {
                var image = self.Get_CollageImageByType();
                if (image == undefined) return;
                if (opt.ChangeImage !== false) {
                    self.SetUp_Background({ ImageUrl: image, IgnoreShuffleSettings: true, SetColor: false });
                    self.SetUp_BgImageColor();
                }
                self.SetUp_Colors();
                self.SetUp_Footer({ ImageData: self.DefaultSrcImageData, ShowDynamicMsg: true, ShowFullPage: true });
            } else {
                self.Create_Collage(collage.FeedId, {
                    ShowCollage: true,
                    CompleteCallback: function () {
                        self.SetUp_Colors();
                    }
                });
                self.SetUp_Footer({ ShowDynamicMsg: true, ShowFullPage: true });
            }
        } else {
            self.Create_Collage($("#newtab").find("#collageTab").find(".wp-feeds").find("li").eq(0).data("feed-id"), {
                ShowCollage: true,
                CompleteCallback: function () {
                    self.SetUp_Colors();
                }
            });
            self.SetUp_Footer({ ShowDynamicMsg: true, ShowFullPage: true });
        }

        NewsTab.TrackAnalyticsEvent("New Tab", "Collage", NewsTab.Extension.NameId);
    },

    SetUp_ThemeTab: function ($placeholder, options) {
        var self = this;
        var opt = $.extend({}, options);

        var $item;
        if (NewsTab.Extension.Theme != undefined) {
            $item = $("#newtab").find(".wallpapers-content").find(".themeItem.theme-" + NewsTab.Extension.Theme);
            $item.addClass("selected");
        } else {
            $item = $("#newtab").find(".wallpapers-content").find("li.themeItem").eq(0);
            $item.addClass("selected");
        }

        if ($item.attr("value") === "theme-light") {
            NewsTab.Extension.Theme = "Light";
        } else if ($item.attr("value") === "theme-white") {
            NewsTab.Extension.Theme = "White";
        } else if ($item.attr("value") === "theme-black") {
            NewsTab.Extension.Theme = "Black";
        } else {
            NewsTab.Extension.Theme = "Dark";
        }

        if (opt.ChangeImage !== false) {
            self.SetUp_Background({ IgnoreShuffleSettings: true });
        }

        NewsTab.Helpers.SetLocalStorageItem("Theme", NewsTab.Extension.Theme);
        NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.THEME);

        self.Select_ThemeItem($item);
        NewsTab.Extension.SetUp_Theme();
        self.SetUp_Footer({ HideButtons: true, HideFooter: true });
        NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();

        NewsTab.TrackAnalyticsEvent("New Tab", NewsTab.Extension.Theme, NewsTab.Extension.NameId);
    },

    Set_Wallpaper: function (indexes) {
        var self = this;
        var image,
            collection,
            wallpapers;

        var wlpType = NewsTab.Helpers.GetLocalStorageItem("Wlp-Type");

        switch (wlpType) {
            case self.Type.WALLPAPER:
                if (indexes == undefined) {
                    wallpapers = self.Get_Wallpaper();
                    collection = wallpapers.Collection;
                    image = wallpapers.Image;
                } else {
                    var collectionId = parseInt(indexes.split("_")[0]);
                    var imageIndex = parseInt(indexes.split("_")[1]);
                    var collections = self.Get_WallpaperCollections();
                    collection = $.grep(collections, function (x) { return x.Id === collectionId; });
                    if (collection.length) {
                        collection = collection[0];
                    }
                    image = collection.Images[imageIndex];
                }
                break;

            case self.Type.COLLAGE:
                var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
                if (collage != undefined && collage.Images != undefined) {
                    var collageImage = self.Get_CollageImageByType();
                    self.SetUp_Background({ ImageUrl: collageImage });
                    self.SetUp_Footer({ ImageData: self.DefaultSrcImageData, ShowDynamicMsg: true, ShowFullPage: true });

                    // Generate new collage in background
                    self.Create_Collage(collage.FeedId, {
                        Refresh: true,
                        LoadInbackground: true,
                        ShowCollage: false,
                        CompleteCallback: function () {
                            NewsTab.Extension.GenerateColage();
                        }
                    });
                    return;
                } else {
                    wallpapers = self.Get_Wallpaper();
                    collection = wallpapers.Collection;
                    image = wallpapers.Image;
                }

                // Generate new collage in background
                self.Create_Collage(null, {
                    Refresh: true,
                    LoadInbackground: true,
                    ShowCollage: false,
                    CompleteCallback: function () {
                        NewsTab.Extension.GenerateColage();
                    }
                });

                break;
            case self.Type.THEME:
            default:
                self.SetUp_Footer({ HideFooter: true });
                return;
        }

        self.SetUp_Background({ ImageUrl: collection.Path + image.Name });
        self.SetUp_Footer({ ImageData: image });
    },

    SetUp_Feeds: function () {
        var self = this;
        var feeds = NewsTab.Modules.SectionsManager.ExtensionSections[0].Feeds;
        var html = [];
        feeds.forEach(function (feed) {
            html.push('<li data-feed-id="' + feed.Id + '">');
            html.push('<div><span>' + feed.Name + '</span></div>');
            html.push('</li>');
        });

        //html.push('<li data-name="add-new" class="add-new"><span>+ New Feed</span></li>');

        $("#newtab").find("#feedsTab").find("ul").html("").append(html.join(""));
        self.Select_CollageItem();
    },

    SetUp_Colors: function () {
        var self = this;
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage") || [];

        if (collage.FavoriteColors == undefined) {
            collage.FavoriteColors = [];
        }

        var collageImage = self.Get_CollageImageByType(true);

        var html = "";
        self.DefaultColors.forEach(function (item) {
            var colorIsFavorite = $.grep(collage.FavoriteColors, function (x) { return x === item.Color; });
            html += "<li data-color=\"" + item.Color + "\">";
            html += "<div class=\"color " + item.Name + "\" style=\"background-image: linear-gradient(0deg, " + item.Color + ", " + item.Color + "), url('" + collageImage + "')\"></div>";
            html += "<span><i class=\"icon icon-star" + (colorIsFavorite.length === 0 ? "-o" : "") + " add-img-to-bg-favorite\"></i></span>";
            html += "</li>";
        });

        $("#newtab").find("#colorsTab").find("ul.wp-colors").html("").append(html);
        self.Select_CollageItem();
    },

    SetUp_Background: function (options) {
        var self = this;
        var opt = $.extend({}, options);
        var $newtab = $("#newtab");

        if (opt.SetColor !== false) {
            self.SetUp_BgImageColor();
        }

        if (opt.ImageUrl != undefined) {
            $newtab.addClass("nt-bg-img");
            var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
            var color = collage != undefined ? collage.Color : "transparent";

            var wlpType = NewsTab.Helpers.GetLocalStorageItem("Wlp-Type");
            if (wlpType === self.Type.COLLAGE) {
                collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
                if (collage != undefined) {
                    $newtab.addClass("collage-bg");
                    if (collage.Type === self.CollageType.FULLPAGE) {
                        $newtab.addClass("collage-bg").addClass("full-screen-collage");
                    }
                    $newtab.find(".nt-background-image").css({
                        "background-image": "linear-gradient(0deg, " + color + ", " + color + "), url('" + opt.ImageUrl + "')"
                    });
                }
            } else {
                $newtab.removeClass("collage-bg").removeClass("full-screen-collage");;
                $newtab.find(".nt-background-image").css({ "background-image": "url('" + opt.ImageUrl + "')" });
            }

            if (color === "transparent") {
                $newtab.find(".nt-background-image").addClass("transparent");
            } else {
                $newtab.find(".nt-background-image").removeClass("transparent");
            }
        } else {
            $newtab.removeClass("nt-bg-img").removeClass("collage-bg").removeClass("full-screen-collage");
            $newtab.find(".nt-background-image").css({ "background-image": "none" });
        }
    },

    SetUp_BgImageColor: function (options) {
        var self = this;
        var opt = $.extend({}, options);
        var $newtab = $("#newtab");
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
        var color = collage != undefined ? collage.Color : "transparent";
        var randomColorIdx;

        if (collage != undefined) {
            if (opt.IgnoreShuffleSettings !== true && collage.Shuffle_All_Colors) {
                randomColorIdx = Math.floor(Math.random() * (NewsTab.Modules.Wallpapers.DefaultColors.length));
                collage.Color = color = NewsTab.Modules.Wallpapers.DefaultColors[randomColorIdx].Color;
            }

            if (opt.IgnoreShuffleSettings !== true && collage.Shuffle_Favorite_Colors && collage.FavoriteColors != undefined && collage.FavoriteColors.length) {
                randomColorIdx = Math.floor(Math.random() * (collage.FavoriteColors.length));
                collage.Color = color = collage.FavoriteColors[randomColorIdx];
            }

            NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
        }

        if (color === "transparent") {
            $newtab.find(".nt-background-image").addClass("transparent");
        } else {
            $newtab.find(".nt-background-image").removeClass("transparent");
        }

        self.SetUp_Colors();
    },

    SetUp_Footer: function (options) {
        var self = this;
        var opt = $.extend({}, options);
        var $footer = $(".wallpapers-footer");
        var wlpType = NewsTab.Helpers.GetLocalStorageItem("Wlp-Type");

        if (wlpType === self.Type.THEME || opt.HideFooter) {
            $footer.addClass("hide");
        } else {
            $footer.removeClass("hide");

            if (opt.HideButtons) {
                $footer.find(".wallpapers-buttons").addClass("hide");
            } else {
                $footer.find(".wallpapers-buttons").removeClass("hide");
            }

            if (opt.HideSource) {
                $footer.find(".wallpapers-copyright").addClass("hide");
            } else {
                $footer.find(".wallpapers-copyright").removeClass("hide");
                if (opt.ImageData != undefined) {
                    $footer.find(".wallpapers-copyright .wallpapers-source-link").html("<a href=\"" + opt.ImageData.Source.Url + "\">" +
                        opt.ImageData.Source.Name + "</a> / " + opt.ImageData.Artist.Name);
                }
            }

            if (wlpType === self.Type.COLLAGE) {
                $footer.find(".wallpapers-description").addClass("hide");
                $footer.find(".wallpapers-buttons").addClass("wps-btn-3");
            } else {
                $footer.find(".wallpapers-description").removeClass("hide");
                if (opt.ImageData != undefined) {
                    $footer.find(".wallpapers-copyright .wallpapers-description-text").html("<span>" + opt.ImageData.Description + "</span>");
                }
                if (opt.ShowFullPage) {
                    $footer.find(".wallpapers-buttons").addClass("wps-btn-3");
                } else {
                    $footer.find(".wallpapers-buttons").removeClass("wps-btn-3");
                }
            }

            if (opt.ShowDynamicMsg) {
                $footer.find(".wallpapers-dynamic").removeClass("hide");
            } else {
                $footer.find(".wallpapers-dynamic").addClass("hide");
            }
        }

        self.Events();
    },

    SetUp_CollageWallpaper: function (feed, options) {
        var self = this;
        var opt = $.extend({}, options);

        var items = feed.Items;
        if (items == undefined) {
            items = NewsTab.Helpers.GetLocalStorageItem("FeedArticles_" + feed.Id) || [];
        }

        var images = [];
        items.forEach(function (article) {
            var image = article.ImageUrl;
            if (image != undefined && images.indexOf(image) === -1) {
                images.push(image);
            }
        });

        var normalImgLength = images.length;
        if (normalImgLength > 3) {
            normalImgLength = 10;
        }
        var n = normalImgLength - 3; // Get random template beetween 3 closest templates

        var templateIndexNormal = Math.floor(Math.random() * (normalImgLength - n)) + n + 1;
        var templateIndexFullPage = Math.floor(Math.random() * 2); // Get random number beetween 0 and 1 because we had only 2 full page templates

        var opts = {
            Images: images,
            GetFullPageImage: true,
            TemplateIndexNormal: templateIndexNormal,
            TemplateIndexFullPage: templateIndexFullPage,
            TemplateScale: 4,
            Callback: images => self.GenerateCollageCallback(feed, images, opt)
        };
        NewsTab.Modules.ImageCollage.GenerateCollage(opts);
    },

    Set_FullScreen: function (fullScreen) {
        var self = this;
        var wlpType = NewsTab.Helpers.GetLocalStorageItem("Wlp-Type");

        if (wlpType === self.Type.COLLAGE) {
            var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
            collage.Type = fullScreen ? self.CollageType.FULLPAGE : self.CollageType.NORMAL;
            NewsTab.Helpers.SetLocalStorageItem("Collage", collage);

            if (fullScreen) {
                $("#newtab").addClass("full-screen-collage");
                self.SetUp_Background({ ImageUrl: collage.Images.FullPage });
            } else {
                $("#newtab").removeClass("full-screen-collage");
                self.SetUp_Background({ ImageUrl: collage.Images.Normal });
            }
        }
    },

    GenerateCollageCallback: function (feed, images, opt) {
        var self = this;
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");

        if (collage == undefined) {
            collage = {
                FeedId: feed.Id,
                Images: {
                    Normal: images.NormalImage,
                    FullPage: images.FullPageImage
                },
                Color: "transparent",
                Generated_At: NewsTab.Helpers.GetTime(),
                Shuffle_All_Colors: true,
                Type: self.CollageType.NORMAL
            };
        } else {
            collage.FeedId = feed.Id;
            collage.Images = {
                Normal: images.NormalImage,
                FullPage: images.FullPageImage
            };
            collage.Color = collage.Color != undefined ? collage.Color : "none";
            collage.Generated_At = NewsTab.Helpers.GetTime();
        }

        NewsTab.Helpers.SetLocalStorageItem("Collage", collage);

        if (opt.ShowCollage) {
            if (collage.Type === self.CollageType.FULLPAGE) {
                self.SetUp_Background({ ImageUrl: images.FullPageImage });
            } else {
                self.SetUp_Background({ ImageUrl: images.NormalImage });
            }

            self.SetUp_Footer({ ImageData: self.DefaultSrcImageData });
        }

        if (opt.CompleteCallback) {
            opt.CompleteCallback();
        }

        NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();
        self.Set_Toast({ Done: true });
    },

    SetUp_AllFeedsPopup: function () {
        var feeds = NewsTab.Modules.SectionsManager.ExtensionSections[0].Feeds.map(i => { return i });
        var html = '<ul class="wp-feeds">';
        feeds.forEach(function (feed) {
            html += '<li data-feed-id="' + feed.Id + '">';
            html += '<div><span>' + feed.Name + '</span></div>';
            html += '</li>';
        });
        html += '</ul>';

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: "<div class=\"title\"><h2>Select feed</h2></div>",
            Height: "auto",
            MaxHeight: "285",
            Text: html,
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Add Feed",
                    Classes: ["blue", "save"]
                }
            ]
        });
    },

    Get_AllWallpapers: function () {
        var self = this;

        $("#newtab").find(".wallpapers-bg-images").html();

        var collections = self.Get_WallpaperCollections();
        var favorites = NewsTab.Helpers.GetLocalStorageItem("Wlp-Favorites-Idx") != undefined ? NewsTab.Helpers.GetLocalStorageItem("Wlp-Favorites-Idx") : [];
        var pinnedWallpaper = NewsTab.Helpers.GetLocalStorageItem("Wlp-Pinned-Idx");
        var shuffleAllImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-All-Images");
        var shuffleFavoriteImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-Favorite-Images");
        var selectedWlpIndexes = $("#newtab").attr("wallpaper-indexes");
        if (selectedWlpIndexes == undefined) selectedWlpIndexes = "1_0";
        selectedWlpIndexes = (pinnedWallpaper == undefined || shuffleAllImages === true || shuffleFavoriteImages === true) ?
            selectedWlpIndexes : pinnedWallpaper.split("/")[pinnedWallpaper.split("/").length - 1];

        var item = "";
        //collections.sort(NewsTab.Helpers.Sort("Id", false, parseInt));
        collections.forEach(function (collection) {
            collection.Images.forEach(function (image) {
                var imageIdx = collection.Images.indexOf(image);
                var imageIsFavorite = $.grep(favorites, function (x) { return x === (collection.Id + "_" + imageIdx); });
                var collectionId = parseInt(selectedWlpIndexes.split("_")[0]);
                var imageIndex = parseInt(selectedWlpIndexes.split("_")[1]);

                if (collectionId === collection.Id && imageIndex === imageIdx) {
                    item += "<li class=\"wallpapers-image selected\" data-collection-index=\"" + collection.Id + "\" data-image-index=\"" + imageIdx + "\">";
                } else {
                    item += "<li class=\"wallpapers-image\" data-collection-index=\"" + collection.Id + "\" data-image-index=\"" + imageIdx + "\">";
                }

                item += "<img src=\"" + (collection.Path + image.Name) + "\"/>";
                item += "<span><i class=\"icon icon-star" + (imageIsFavorite.length === 0 ? "-o" : "") + " add-img-to-bg-favorite\"></i></span>";
                item += "</li>";
            });
        });

        return item;
    },

    Get_AllCollections: function (collections) {
        var self = this;

        var collectionItem = "";
        NewsTab.Helpers.GetLocalStorageItem("Wallpapers-Collections").Collections.forEach(function (collection) {
            var collAdded = $.grep(collections, function (x) { return x.Name === collection.Name });
            collectionItem += "<li data-name=\"" + collection.Name + "\" class=\"select-collections " + (collAdded.length ? "selected" : "") + "\">";
            for (var i = 0; i < 4; i++) {
                collectionItem += "<img src=\"" + collection.Path + collection.Images[i].Name + "\" />";
            }
            collectionItem += "<div class=\"collectionName\">";
            collectionItem += collection.Name;
            collectionItem += "</div>";
            collectionItem += "</li>";
        });

        //collectionItem += "<li data-name=\"add-new\" class=\"add-new\">";
        //collectionItem += "<div class=\"plus\">+</div>";
        //collectionItem += "<div class=\"label\">New Collection</div>";
        //collectionItem += "</li>";

        return collectionItem;
    },

    Get_Wallpaper: function () {
        var self = this;
        var collections = self.Get_WallpaperCollections(); // List of image collections
        var favoritImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Favorites-Idx");
        var indexes = self.Get_CollectionAndImageIndex();

        var collectionId = indexes.collectionId;
        var imageIndex = indexes.imageIndex;

        var collection = $.grep(collections, function (x) { return x.Id === collectionId; });
        if (collection.length) {
            collection = collection[0];
        }

        var image = collection.Images[imageIndex];

        if (NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-All-Images") === true) {
            var ids = $.map(collections, function (x) { return x.Id });
            var randomIdx = Math.floor(Math.random() * ids.length);
            collectionId = ids[randomIdx];
            collection = $.grep(collections, function (x) { return x.Id === collectionId; });
            if (collection.length) {
                collection = collection[0];
            }
            imageIndex = Math.floor(Math.random() * collection.Images.length);
            image = collection.Images[imageIndex];
        }

        if (favoritImages != undefined && favoritImages.length > 0 && NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-Favorite-Images") === true) {
            var favoritePos = Math.floor(Math.random() * favoritImages.length);
            var favoriteIdx = favoritImages[favoritePos];
            collectionId = parseInt(favoriteIdx.split("_")[0]);
            collection = $.grep(collections, function (x) { return x.Id === collectionId; });
            if (collection.length) {
                collection = collection[0];
                imageIndex = parseInt(favoriteIdx.split("_")[1]);
            }
            image = collection.Images[imageIndex];
        }

        NewsTab.Helpers.SetLocalStorageItem("Wlp-Position-Idx", collectionId + "_" + imageIndex);
        $("#newtab").attr("wallpaper-indexes", collectionId + "_" + imageIndex);

        return { Collection: collection, Image: image };
    },

    Get_WallpaperCollections: function () {
        var collections = NewsTab.Helpers.GetLocalStorageItem("Wlp-Collections");
        if (collections == undefined) {
            collections = [];
            collections.push(NewsTab.Helpers.GetLocalStorageItem("Wallpapers-Collections").Collections[0]);
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Collections", collections);
        }

        return collections;
    },

    Get_CollectionAndImageIndex: function () {
        var self = this;
        var collections = self.Get_WallpaperCollections(); // List of image collections
        var indexes = NewsTab.Helpers.GetLocalStorageItem("Wlp-Position-Idx"); // Index of selected collection and image
        var collectionId = 0;
        var imageIndex = 0;

        if (indexes != undefined) {
            collectionId = parseInt(indexes.split("_")[0]);
            imageIndex = parseInt(indexes.split("_")[1]);
        }

        var collection = $.grep(collections, function (x) { return x.Id === collectionId; });
        if (collection.length) {
            collection = collection[0];
            if (collections.indexOf(collection) === -1) {
                collectionId = 1;
                imageIndex = 0;
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Position-Idx", collectionId + "_" + imageIndex);
                self.Get_Wallpaper();
            }

            imageIndex += 1; // Next image

            if (imageIndex > collection.Images.length) {
                imageIndex = 0;

                if (collections.indexOf(collection) === -1) {
                    collectionId += 1;
                } else {
                    collectionId = collections[0].Id;
                }

                NewsTab.Helpers.SetLocalStorageItem("Wlp-Position-Idx", collectionId + "_" + imageIndex);
                self.Get_Wallpaper();
            }
        } else {
            collectionId = collections[0].Id;
            imageIndex = 0;
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Position-Idx", collectionId + "_" + imageIndex);
        }

        return { collectionId, imageIndex };
    },

    Get_PinnedWallpaper: function () {
        var shuffleAllImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-All-Images");
        var shuffleFavoriteImages = NewsTab.Helpers.GetLocalStorageItem("Wlp-Shuffle-Favorite-Images");
        var pinnedImage = NewsTab.Helpers.GetLocalStorageItem("Wlp-Pinned-Idx");
        return (shuffleAllImages === true || shuffleFavoriteImages === true) ? undefined : pinnedImage;
    },

    Get_CollageImageByType: function (takeBox) {
        var self = this;
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
        if (collage == undefined) return null;

        if (takeBox && collage.Images.FullPage != undefined) {
            return collage.Images.FullPage;
        } else if (collage.Type === self.CollageType.NORMAL && collage.Images.Normal != undefined) {
            return collage.Images.Normal;
        } else if (collage.Type === self.CollageType.FULLPAGE && collage.Images.FullPage != undefined) {
            return collage.Images.FullPage;
        }

        return collage.Images.Normal;
    },

    Set_Toast: function (options) {
        var opt = $.extend({}, options);

        if (opt.Done) {
            var $toast = $("#newtab").find("#collageTab").find(".wps-toast");
            $toast.addClass("done");
            setTimeout(function () {
                $toast.fadeOut(500);
            }, 2000);
        } else {
            var html = '<div class="wps-toast"><span>Creating new collage - this may take a moment</span><div class="spinner"></div><i class="icon icon-check"></i></div>';
            $("#newtab").find("#collageTab").append(html);
        }
    },

    Create_Collage: function (feedId, options) {
        var self = this;
        var opt = $.extend({}, options);

        if (feedId == undefined) {
            feedId = NewsTab.Helpers.GetLocalStorageItem("CurrentFeedId");
        }

        var feed = $.grep(NewsTab.Modules.SectionsManager.ExtensionSections[0].Feeds, function (x) { return x.Id === feedId });
        if (feed.length) {
            feed = feed[0];

            if (feed.Items.length === 0) {
                if (!opt.LoadInbackground) {
                    self.Set_Toast();
                }

                var feedArticles = NewsTab.Helpers.GetLocalStorageItem("FeedArticles_" + feedId);
                if (feedArticles != undefined && feedArticles.length) {
                    feed.Items = feedArticles;
                    self.SetUp_CollageWallpaper(feed, opt);
                } else {
                    NewsTab.Modules.SectionsManager.LoadFeed(feed, {
                        Refresh: true,
                        LoadInbackground: true,
                        CompleteCallback: function () {
                            var itemsProcessed = 0;
                            feed.Items.forEach((article, index, array) => {
                                NewsTab.Modules.ArticleView.API_ParseUnparsedArticle(article, {
                                    LoadInbackground: true,
                                    CompleteCallback: function () {
                                        itemsProcessed++;
                                        if (itemsProcessed === array.length) {
                                            self.SetUp_CollageWallpaper(feed, opt);
                                        }
                                    }
                                });
                            });
                        }
                    });
                }
            } else {
                self.SetUp_CollageWallpaper(feed, opt);
            }
        }
    },

    Select_WallpaperItem: function ($selector) {
        var self = this;
        var collectionId,
            imageIndex;
        $("#newtab").find(".wallpapers-image.selected").removeClass("selected");

        if ($selector == undefined) {
            var pinnedImage = self.Get_PinnedWallpaper();
            var indexes = NewsTab.Helpers.GetLocalStorageItem("Wlp-Position-Idx");
            if (pinnedImage) {
                collectionId = pinnedImage.split('_')[0];
                imageIndex = pinnedImage.split('_')[1];
                $selector = $("#newtab").find(".wallpapers-image[data-collection-index=" +
                    collectionId +
                    "][data-image-index=" +
                    imageIndex +
                    "]");
            } else if (indexes) {
                collectionId = indexes.split('_')[0];
                imageIndex = indexes.split('_')[1];
                $selector = $("#newtab").find(".wallpapers-image[data-collection-index=" +
                    collectionId +
                    "][data-image-index=" +
                    imageIndex +
                    "]");
            } else {
                $selector = $("#newtab").find(".wallpapers-image").eq(0);
            }
        }

        if ($selector != undefined) {
            $selector.addClass("selected");
        }
    },

    Select_CollageItem: function ($selector) {
        var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");

        var $feedsTab = $("#newtab").find("#feedsTab");
        var $colorsTab = $("#newtab").find("#colorsTab");

        if ($selector == undefined) {
            if (collage != undefined) {
                $feedsTab.find("li[data-feed-id='" + collage.FeedId + "']").addClass("selected");
                $colorsTab.find("li[data-color='" + collage.Color + "']").addClass("selected");
            } else {
                $feedsTab.find("li[data-feed-id]").eq(0).addClass("selected");
                $colorsTab.find("li[data-color]").eq(0).addClass("selected");
            }
        } else {
            $selector.closest(".wps-content").find("li.selected").removeClass("selected");
            $selector.addClass("selected");
        }
    },

    Select_ThemeItem: function ($selector) {
        var $themesTab = $("#newtab").find("#blankTab");

        $themesTab.find("li.selected").removeClass("selected");

        if ($selector == undefined) {
            $selector = $themesTab.find(".theme-" + NewsTab.Helpers.GetLocalStorageItem("Theme"));
        }

        if ($selector != undefined) {
            $selector.addClass("selected");
        }
    },

    Select_Tab: function ($tab, options) {
        var self = this;
        var opt = $.extend({}, options);
        var $placeholder = $(".wallpapers-placeholder");
        $tab.closest(".tabMenu").find(">a").removeClass("active");
        $tab.addClass("active");

        $placeholder.find(".wpsTabs").find("> div").removeClass("active");
        $placeholder.find(".wpsTabs").find("#" + $tab.data("name")).addClass("active");

        if ($tab.data("name") === "wallpapersTab") {
            self.SetUp_WallpaperTab($placeholder, opt);
        } else if ($tab.data("name") === "collageTab") {
            self.SetUp_CollageTab($placeholder, opt);
        } else {
            self.SetUp_ThemeTab($placeholder, opt);
        }

        self.Events();
    },

    Events: function () {
        var self = this;
        var arrowPosition = $(".wallpapers-link").offset().left;
        if (arrowPosition > $(".wallpapers-placeholder").outerWidth()) {
            arrowPosition = $(".wallpapers-placeholder").outerWidth() / 2 - $(".wallpapers-link").width() / 2;

            $(".extension-popup-arrow-1").css({ "left": arrowPosition + "px" });
            $(".extension-popup-arrow-2").css({ "left": arrowPosition + "px" });

            $(".wallpapers-placeholder").css({ left: $(".wallpapers-link").offset().left - 150 });
        } else {
            $(".extension-popup-arrow-1").css({ "left": arrowPosition + 25 + "px" });
            $(".extension-popup-arrow-2").css({ "left": arrowPosition + 25 + "px" });
        }

        $("#newtab").find(".tabMenu > a").off("click").on("click", function (e) {
            e.preventDefault();
            var $self = $(this);

            self.Select_Tab($self);
        });

        $("#newtab").find(".wallpapers-image img").off("click").on("click", function () {
            var $self = $(this);

            var colIdx = $self.closest(".wallpapers-image").data("collection-index");
            var imgIdx = $self.closest(".wallpapers-image").data("image-index");

            NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.WALLPAPER);
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Pinned-Idx", colIdx + "_" + imgIdx);
            self.Set_Wallpaper(colIdx + "_" + imgIdx);
            self.Select_WallpaperItem($self.closest(".wallpapers-image"));

            $("#newtab").removeClass("collage-bg");

            NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();
        });

        $("#newtab").find(".add-img-to-bg-favorite").off("click").on("click", function () {
            var $self = $(this);
            var activeTab = $("#newtab").find(".wallpapers-content > .tabMenu a.active").data("name");
            if (activeTab === "wallpapersTab") {
                var colIdx = $self.closest(".wallpapers-image").data("collection-index");
                var imgIdx = $self.closest(".wallpapers-image").data("image-index");
                var favorites = NewsTab.Helpers.GetLocalStorageItem("Wlp-Favorites-Idx");
                var image = colIdx + "_" + imgIdx;

                if (favorites == undefined) {
                    favorites = [];
                }

                var imageIsFavorite = $.grep(favorites, function (x) { return x === image; });
                if (imageIsFavorite.length > 0) {
                    $self.removeClass("icon-star").addClass("icon-star-o");
                    favorites.splice($.inArray(image, favorites), 1);
                } else {
                    $self.removeClass("icon-star-o").addClass("icon-star");
                    favorites.push(image);
                }

                NewsTab.Helpers.SetLocalStorageItem("Wlp-Favorites-Idx", favorites);
            } else if (activeTab === "collageTab") {
                var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
                if (collage != undefined) {
                    var color = $self.closest("li").data("color");
                    if (collage.FavoriteColors == undefined) {
                        collage.FavoriteColors = [];
                    }

                    var colorIsFavorite = $.grep(collage.FavoriteColors, function (x) { return x === color; });
                    if (colorIsFavorite.length > 0) {
                        $self.removeClass("icon-star").addClass("icon-star-o");
                        collage.FavoriteColors.splice($.inArray(color, collage.FavoriteColors), 1);
                    } else {
                        $self.removeClass("icon-star-o").addClass("icon-star");
                        collage.FavoriteColors.push(color);
                    }

                    NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
                }
            }
        });

        $("#newtab").find("#shuffle-all").off("click").on("click", function () {
            var $self = $(this);
            $("#newtab").find("#shuffle-favorite").prop("checked", false);

            var activeTab = $("#newtab").find(".wallpapers-content > .tabMenu a.active").data("name");
            if (activeTab === "wallpapersTab") {
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Shuffle-All-Images", $self.is(":checked"));
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Shuffle-Favorite-Images", false);
                NewsTab.Helpers.ClearLocalStorageByKey("Wlp-Pinned-Idx"); // TODO: Check it
            } else if (activeTab === "collageTab") {
                var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
                if (collage != undefined) {
                    collage.Shuffle_All_Colors = $self.is(":checked");
                    collage.Shuffle_Favorite_Colors = false;
                    NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
                }
            }
        });

        $("#newtab").find("#shuffle-favorite").off("click").on("click", function () {
            var $self = $(this);
            $("#newtab").find("#shuffle-all").prop("checked", false);

            var activeTab = $("#newtab").find(".wallpapers-content > .tabMenu a.active").data("name");
            if (activeTab === "wallpapersTab") {
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Shuffle-All-Images", false);
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Shuffle-Favorite-Images", $self.is(":checked"));
                NewsTab.Helpers.ClearLocalStorageByKey("Wlp-Pinned-Idx"); // TODO: Check it
            } else if (activeTab === "collageTab") {
                var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");
                if (collage != undefined) {
                    collage.Shuffle_All_Colors = false;
                    collage.Shuffle_Favorite_Colors = $self.is(":checked");
                    NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
                }
            }
        });

        $("#newtab").find("#full-page").off("click").on("click", function () {
            var $self = $(this);
            self.Set_FullScreen($self.is(":checked"));
            self.Events();
        });

        $("#newtab").find(".select-collections").off("click").on("click", function () {
            var $self = $(this);

            var selectedCollection = $.grep(NewsTab.Helpers.GetLocalStorageItem("Wallpapers-Collections").Collections, function (x) { return x.Name === $self.closest("li").data("name"); });
            if (selectedCollection.length) {
                selectedCollection = selectedCollection[0];

                var collections = self.Get_WallpaperCollections();
                var collectionExist = $.grep(collections, function (x) { return x.Name === $self.closest("li").data("name"); });

                if (collectionExist.length > 0) {
                    $self.removeClass("selected");

                    NewsTab.Helpers.RemoveItemFromArray(collections, collectionExist[0]);
                    NewsTab.Helpers.ClearLocalStorageByKey("Wlp-Position-Idx");
                    NewsTab.Helpers.ClearLocalStorageByKey("Wlp-Pinned-Idx");
                    NewsTab.Helpers.ClearLocalStorageByKey("Wlp-Favorites-Idx");
                    NewsTab.Helpers.ClearLocalStorageByKey("Collage");
                } else {
                    $self.addClass("selected");
                    collections.unshift(selectedCollection);
                }

                NewsTab.Helpers.SetLocalStorageItem("Wlp-Collections", collections);
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.WALLPAPER);

                $("#newtab").find(".wallpapers-bg-images").html(self.Get_AllWallpapers());

                self.Events();
                NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();
            }
        });

        $("#newtab").find("#feedsTab").find("li").off("click").on("click", function () {
            var $self = $(this);
            var feedId = $self.data("feed-id");

            if (feedId != undefined) {
                NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.COLLAGE);
                self.Select_CollageItem($self);
                self.Create_Collage(feedId, { ShowCollage: true });
            } else {
                self.SetUp_AllFeedsPopup();
            }
        });

        $("#newtab").find("#colorsTab").find("li .color").off("click").on("click", function () {
            var $self = $(this);
            var color = $self.closest("li").data("color");
            var collage = NewsTab.Helpers.GetLocalStorageItem("Collage");

            if (collage == undefined) return false;

            collage.Color = color;

            var imageUrl = collage.Type === self.CollageType.FULLPAGE ? collage.Images.FullPage : collage.Images.Normal;
            $("#newtab").find(".nt-background-image").css({
                "background-image": "linear-gradient(0deg, " + color + ", " + color + "), url('" + imageUrl + "')"
            });

            if (color === "transparent") {
                $("#newtab").find(".nt-background-image").addClass("transparent");
            } else {
                $("#newtab").find(".nt-background-image").removeClass("transparent");
            }

            NewsTab.Helpers.SetLocalStorageItem("Collage", collage);
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.COLLAGE);
            self.Select_CollageItem($self.closest("li"));
            self.SetUp_BgImageColor({ IgnoreShuffleSettings: true });

            self.Events();
            return false;
        });

        $("#newtab").find(".wallpapers-content").find(".themeItem").off("click").on("click", function () {
            var $self = $(this);
            self.SetUp_Background();

            if ($self.attr("value") === "theme-light") {
                NewsTab.Extension.Theme = "Light";
            } else if ($self.attr("value") === "theme-white") {
                NewsTab.Extension.Theme = "White";
            } else if ($self.attr("value") === "theme-black") {
                NewsTab.Extension.Theme = "Black";
            } else {
                NewsTab.Extension.Theme = "Dark";
            }

            NewsTab.Helpers.SetLocalStorageItem("Theme", NewsTab.Extension.Theme);
            NewsTab.Helpers.SetLocalStorageItem("Wlp-Type", self.Type.THEME);

            NewsTab.Extension.SetUp_Theme();
            NewsTab.Modules.TimeAndWeatherWidget.SetUp_TimeWeatherWidget();

            self.Select_ThemeItem($self);
            self.SetUp_Footer({ HideFooter: true });

            NewsTab.TrackAnalyticsEvent("New Tab", NewsTab.Extension.Theme, NewsTab.Extension.NameId);
        });
    },

    SetUp_Tooltips: function () {
        NewsTab.Extension.Show_Tooltip({
            Destination: $(".wallpapers-link"),
            Id: "fts-wallpaper",
            Text: "Set and modify wallpaper",
            Step: "1/3",
            BtnText: "NEXT",
            Position: "bottom",
            Offset: { X: 10, Y: -5 },
            NextStep: {
                Destination: $("#settingsButton"),
                Id: "fts-settings",
                Text: "Personalize news feed",
                Step: "2/3",
                BtnText: "NEXT",
                Position: "right",
                Offset: { X: 0, Y: -12 },
                TipPosition: 3,
                NextStep: {
                    Destination: $(".show-hide-title"),
                    Id: "fts-showHideNews",
                    Text: "Hide news feed",
                    Step: "3/3",
                    BtnText: "FINISH",
                    Position: $("body").hasClass("hideArticles") ? "right" : "top",
                    Offset: { X: 0, Y: $("body").hasClass("hideArticles") ? 12 : 0 }
                }
            }
        });
    }
}