﻿var NewsTab = NewsTab || {};

NewsTab.Helpers = {
    RTL_languageCodes: ["ar", "arc", "dv", "ha", "he", "he-il", "ko", "ks", "ku", "ps", "ur", "yi"],

    HOUR: 1000 * 60 * 60,

    GetTodayDate: function () {
        var today = new Date();
        var sec = today.getSeconds();
        var min = today.getMinutes();
        var hh = today.getHours();
        var dd = today.getDate();
        var mm = today.getMonth() + 1; //January is 0!

        var yyyy = today.getFullYear();
        if (dd < 10) {
            dd = "0" + dd;
        }
        if (mm < 10) {
            mm = "0" + mm;
        }
        today = dd + "/" + mm + "/" + yyyy + "/" + hh + "/" + min + "/" + sec;
        return today;
    },

    GetTime: function () {
        return new Date().getTime();
    },

    LeapYear: function (year) {
        return ((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0);
    },

    GetPasedTime: function (datetime, rtl) {
        var timeDiff = Math.round((new Date().getTime() - Date.parse(datetime)) / (1000 * 3600 / 60));
        if (timeDiff < 0) {
            return rtl ? "m0" : "0m";
        } else if (timeDiff <= 60) {
            return rtl ? "m" + timeDiff : timeDiff + "m";
        } else {
            timeDiff = Math.round(timeDiff / 60);
            if (timeDiff <= 24) {
                return rtl ? "h" + timeDiff : timeDiff + "h";
            } else {
                timeDiff = Math.round(timeDiff / 24);
                if (timeDiff <= 30) {
                    return rtl ? "d" + timeDiff : timeDiff + "d";
                } else {
                    timeDiff = Math.round(timeDiff / 30);
                    if (timeDiff <= 12) {
                        return rtl ? "M" + timeDiff : timeDiff + "M";
                    } else {
                        timeDiff = Math.round(timeDiff / 12);
                        return rtl ? "y" + timeDiff : timeDiff + "y";
                    }
                }
            }
        }
    },

    GetShortCount: function (number) {
        if (number < 1000) {
            return number;
        } else {
            return Math.round((number / 1000) * 10) / 10 + "K";
        }
    },

    PadStart: function (str, max) {
        str = str.toString();
        return str.length < max ? NewsTab.Helpers.PadStart("0" + str, max) : str;
    },

    CreateCookie: function (name, value, days) {
        var expires;
        if (days) {
            var date = new Date();
            date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
            expires = "; expires=" + date.toGMTString();
        } else {
            expires = "";
        }
        document.cookie = name + "=" + value + expires + "; path=/";
    },

    ReadCookie: function (name) {
        var nameEq = name + "=";
        var ca = document.cookie.split(";");
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) === " ") c = c.substring(1, c.length);
            if (c.indexOf(nameEq) === 0) return c.substring(nameEq.length, c.length);
        }
        return null;
    },

    EraseCookie: function (name) {
        var self = this;
        self.CreateCookie(name, "", -1);
    },

    GetQueryVariable: function (variable) {
        var query = window.location.search.substring(1);
        var vars = query.split("&");
        for (var i = 0, len = vars.length; i < len; i++) {
            var pair = vars[i].split("=");
            if (pair[0] === variable) {
                return pair[1];
            }
        }
        return (false);
    },

    ValidateURL: function (textval) {
        var urlregex = /^(http|https|ftp):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/i;
        return urlregex.test(textval);
    },

    ValidateEmail: function (email) {
        var re = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
        return re.test(email);
    },

    IsUrlImage: function (url) {
        url = url.toLowerCase();
        if ((url.indexOf(".jpeg") > -1) || (url.indexOf(".jpg") > -1) || (url.indexOf(".gif") > -1) || (url.indexOf(".png") > -1)) {
            return true;
        }
        return false;
    },

    GetRandomInt: function (min, max) {
        return Math.floor(Math.random() * (max - min)) + min;
    },

    LimitText: function (text, limit) {
        if (text.length > limit) {
            return text.substring(0, limit);
        }
        return text;
    },

    GetFilename: function (url) {
        if (url) {
            return url.substring(url.lastIndexOf("/") + 1, url.lastIndexOf("."));
        }
        return "";
    },

    GetFriendlyTitle: function (title, remapToAscii, maxlength) {
        if (title === null) {
            return "";
        }

        if (maxlength === undefined) {
            maxlength = 80;
        }

        var length = title.length;
        var prevdash = false;
        var stringBuilder = [];
        var c;

        for (var i = 0; i < length; ++i) {
            c = title[i];
            if ((c >= "a" && c <= "z") || (c >= "0" && c <= "9")) {
                stringBuilder.push(c);
                prevdash = false;
            }
            else if (c >= "A" && c <= "Z") {
                // tricky way to convert to lower-case
                stringBuilder.push(c.toLowerCase());
                prevdash = false;
            }
            else if ((c === " ") || (c === ",") || (c === ".") || (c === "/") ||
                (c === "\\") || (c === "-") || (c === "_") || (c === "=")) {
                if (!prevdash && (stringBuilder.length > 0)) {
                    stringBuilder.push("-");
                    prevdash = true;
                }
            }
            else if (c >= 128) {
                var previousLength = stringBuilder.length;

                if (remapToAscii === true) {
                    stringBuilder.push(NewsTab.Helpers.RemapInternationalCharToAscii(c));
                }
                else {
                    stringBuilder.push(c);
                }

                if (previousLength !== stringBuilder.length) {
                    prevdash = false;
                }
            }

            if (i === maxlength) {
                break;
            }
        }

        if (prevdash) {
            return stringBuilder.join("").substring(0, stringBuilder.length - 1);
        }
        else {
            return stringBuilder.join("");
        }
    },

    RemapInternationalCharToAscii: function (character) {
        var s = character.toLowerCase();
        if ("àåáâäãåąā".indexOf(s) !== -1) {
            return "a";
        }
        else if ("èéêëę".indexOf(s) !== -1) {
            return "e";
        }
        else if ("ìíîïı".indexOf(s) !== -1) {
            return "i";
        }
        else if ("òóôõöøőð".indexOf(s) !== -1) {
            return "o";
        }
        else if ("ùúûüŭů".indexOf(s) !== -1) {
            return "u";
        }
        else if ("çćčĉ".indexOf(s) !== -1) {
            return "c";
        }
        else if ("żźž".indexOf(s) !== -1) {
            return "z";
        }
        else if ("śşšŝ".indexOf(s) !== -1) {
            return "s";
        }
        else if ("ñń".indexOf(s) !== -1) {
            return "n";
        }
        else if ("ýÿ".indexOf(s) !== -1) {
            return "y";
        }
        else if ("ğĝ".indexOf(s) !== -1) {
            return "g";
        }
        else if (character === "ř") {
            return "r";
        }
        else if (character === "ł") {
            return "l";
        }
        else if (character === "đ") {
            return "d";
        }
        else if (character === "ß") {
            return "ss";
        }
        else if (character === "Þ") {
            return "th";
        }
        else if (character === "ĥ") {
            return "h";
        }
        else if (character === "ĵ") {
            return "j";
        }
        else {
            return "";
        }
    },

    IsMobile: function () {
        var check = false;
        (function (a) { if (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4))) check = true })(navigator.userAgent || navigator.vendor || window.opera);
        return check || window.screen.width <= 800;
    },

    IsChrome: function () {
        var isChromium = window.chrome,
            winNav = window.navigator,
            vendorName = winNav.vendor,
            isOpera = winNav.userAgent.indexOf("OPR") > -1,
            isIEedge = winNav.userAgent.indexOf("Edge") > -1,
            isIOSChrome = winNav.userAgent.match("CriOS");

        if (isIOSChrome) {
            return true;
        } else if (isChromium !== null && isChromium !== undefined && vendorName === "Google Inc." && isOpera == false && isIEedge == false) {
            return true;
        } else {
            return false;
        }
    },

    RemoveItemFromArray: function (arr) {
        var what, a = arguments, l = a.length, ax;
        while (l > 1 && arr.length) {
            what = a[--l];
            while ((ax = arr.indexOf(what)) !== -1) {
                arr.splice(ax, 1);
            }
        }
        return arr;
    },

    Sort: function (field, reverse, primer) {
        var key = primer ?
            function (x) { return primer(x[field]) } :
            function (x) { return x[field] };

        reverse = !reverse ? 1 : -1;

        return function (a, b) {
            return a = key(a), b = key(b), reverse * ((a > b) - (b > a));
        };
    },

    SortF: function (field1, field2, reverse) {
        reverse = !reverse ? 1 : -1;
        return function (a, b) {
            return reverse * (a[field1] - b[field1]) || reverse * (a[field2] - b[field2]);
        };
    },

    SortByGuid: function (a, b) {
        var aGuid = a.Id.toLowerCase();
        var bGuid = b.Id.toLowerCase();
        return ((aGuid < bGuid) ? -1 : ((aGuid > bGuid) ? 1 : 0));
    },

    ArrayUnique: function (array) {
        var a = array.concat();
        for (var i = 0; i < a.length; ++i) {
            for (var j = i + 1; j < a.length; ++j) {
                if (a[i] === a[j])
                    a.splice(j--, 1);
            }
        }
        return a;
    },

    CloneObject: function (obj) {
        var self = this;
        if (obj === null || typeof obj !== "object") {
            return obj;
        }

        var temp = obj.constructor(); // give temp the original obj's constructor
        for (var key in obj) {
            temp[key] = self.CloneObject(obj[key]);
        }

        return temp;
    },

    HtmlEnDeCode: (function () {
        var charToEntityRegex,
            entityToCharRegex,
            charToEntity,
            entityToChar;

        function resetCharacterEntities() {
            charToEntity = {};
            entityToChar = {};
            // add the default set
            addCharacterEntities({
                "&amp;": "&",
                "&gt;": ">",
                "&lt;": "<",
                "&quot;": "\"",
                "&#39;": "'"
            });
        }

        function addCharacterEntities(newEntities) {
            var charKeys = [],
                entityKeys = [],
                key,
                echar;
            for (key in newEntities) {
                echar = newEntities[key];
                entityToChar[key] = echar;
                charToEntity[echar] = key;
                charKeys.push(echar);
                entityKeys.push(key);
            }
            charToEntityRegex = new RegExp("(" + charKeys.join("|") + ")", "g");
            entityToCharRegex = new RegExp("(" + entityKeys.join("|") + "|&#[0-9]{1,5};" + ")", "g");
        }

        function htmlEncode(value) {
            var htmlEncodeReplaceFn = function (match, capture) {
                return charToEntity[capture];
            };

            return (!value) ? value : String(value).replace(charToEntityRegex, htmlEncodeReplaceFn);
        }

        function htmlDecode(value) {
            var htmlDecodeReplaceFn = function (match, capture) {
                return (capture in entityToChar) ? entityToChar[capture] : String.fromCharCode(parseInt(capture.substr(2), 10));
            };

            return (!value) ? value : String(value).replace(entityToCharRegex, htmlDecodeReplaceFn);
        }

        resetCharacterEntities();

        return {
            htmlEncode: htmlEncode,
            htmlDecode: htmlDecode
        };
    })(),

    CssSizeValue: function (value, suffix) {
        if (value === "auto") {
            return value;
        }
        else if (value.toString().indexOf("px") < 0 &&
            value.toString().indexOf("%") < 0 &&
            value.toString().indexOf("vh") < 0 &&
            value.toString().indexOf("vw") < 0) {
            return value.toString() + suffix;
        }
        return value.toString();
    },

    OpenPopUp: function (url, name, w, h) {
        w += 32;
        //h += 96;
        wleft = (screen.width - w) / 2;
        wtop = (screen.height - h) / 2;
        if (wleft < 0) {
            w = screen.width;
            wleft = 0;
        }
        if (wtop < 0) {
            h = screen.height;
            wtop = 0;
        }
        var win = window.open(url,
            name,
            "width=" + w + ", height=" + h + ", " +
            "left=" + wleft + ", top=" + wtop + ", " +
            "location=no, menubar=no, " +
            "status=no, toolbar=no, scrollbars=no, resizable=yes");
        win.resizeTo(w, h);
        win.moveTo(wleft, wtop);
        win.focus();
    },

    IsLanguageRTL: function (languageCode) {
        return RTL_languageCodes.indexOf(languageCode.toLowerCase()) >= 0;
    },

    ShouldBeRTL: function (plainText) {
        if (plainText === undefined || plainText === null || plainText === "") return false;

        var rtlChar = /[\u0590-\u083F]|[\u08A0-\u08FF]|[\uFB1D-\uFDFF]|[\uFE70-\uFEFF]/mg;
        var trimRegex = /^\s+|\s+$/g;
        var rtlThreshold = 0.3;

        var matchedRtlChars = plainText.match(rtlChar);

        // Remove original placeholder text from this
        var urlMentionsLength = 0;
        var trimmedText = plainText.replace(trimRegex, "");

        if (!matchedRtlChars) {
            return false; // No RTL chars, use LTR
        }

        if (plainText) {
            var mentions = twttr.txt.extractMentionsWithIndices(plainText);
            var mentionsLength = mentions.length;
            var i;

            for (i = 0; i < mentionsLength; i++) {
                urlMentionsLength += mentions[i].screenName.length + 1;
            }

            var urls = twttr.txt.extractUrlsWithIndices(plainText);
            var urlsLength = urls.length;

            for (i = 0; i < urlsLength; i++) {
                urlMentionsLength += urls[i].url.length;
            }

        }
        var length = trimmedText.length - urlMentionsLength;
        return length > 0 && matchedRtlChars.length / length > rtlThreshold;
    },

    GetParameterFromUrl: function (name, url) {
        name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
        var regexS = "[\\?&]" + name + "=([^&#]*)";
        var regex = new RegExp(regexS);
        var results = regex.exec(url);
        return results === null || results === undefined ? null : results[1];
    },

    CenterContent: function (field, parent) {
        if (parent) {
            parent = field.parent();
        } else {
            parent = window;
        }
        field.css({
            "position": "absolute",
            "top": ($(parent).height() - field.outerHeight()) / 2 + $(parent).scrollTop() + "px",
            "left": ($(parent).width() - field.outerWidth()) / 2 + $(parent).scrollLeft() + "px"
        });
    },

    FindClosestItem: function (arr, num) {
        var curr = arr[0];
        var diff = Math.abs(num - curr);
        for (var val = 0; val < arr.length; val++) {
            var newdiff = Math.abs(num - arr[val]);
            if (newdiff < diff) {
                diff = newdiff;
                curr = arr[val];
            }
        }
        return curr;
    },

    GetImage: function (url) {
        return new Promise(function (resolve, reject) {
            var img = new Image();
            img.onload = function () {
                resolve(url);
            };
            img.onerror = function () {
                reject(url);
            };
            img.src = url;
        });
    },

    GetBase64Image: function (img, callback) {
        img.onload = function () {
            // Create an empty canvas element
            var c = document.createElement("canvas");
            c.width = img.width;
            c.height = img.height;

            // Copy the image contents to the canvas
            var ctx = c.getContext("2d");
            ctx.drawImage(img, 0, 0);

            // Get the data-URL formatted image
            var dataURL = c.toDataURL();
            callback(dataURL);
        }
    },

    DrawImage: function (canvas, imgSrc, urlToSave, cacheMapName, fullUrl) {
        var self = this;
        canvas.src = imgSrc;

        // If "urlToSave" is specified, save the snapshot for future use.
        if (urlToSave !== undefined) {
            var cacheMap = self.GetLocalStorageItem(cacheMapName);
            if (cacheMap === null) {
                cacheMap = {};
            }
            self.GetBase64Image(canvas, function (data) {
                var _save = {
                    data: data,
                    time: new Date(),
                    active: false,
                    full_url: fullUrl
                };
                cacheMap[urlToSave] = _save;
                self.SetLocalStorageItem(cacheMapName, cacheMap);
            });
        }
    },


    GetCookie: function (name) {
        var value = "; " + document.cookie;
        var parts = value.split("; " + name + "=");
        if (parts.length === 2) {
            return parts.pop().split(";").shift();
        }
        return null;
    },

    CapitalizeText: function (txt) {
        return txt.charAt(0).toUpperCase() + txt.slice(1);
    },

    GetAllUrlParameters: function (url) {
        var params = {};

        var paramsArray = url.substring(url.indexOf("?") + 1).split("&");

        for (var i = 0; i < paramsArray.length; ++i) {
            var param = paramsArray[i]
                .split("=", 2);

            if (param.length !== 2)
                continue;

            params[param[0]] = decodeURIComponent(param[1].replace(/\+/g, " "));
        }

        return params;
    },

    /* Local Storage */
    StringifyReplacer: function (name, val) {
        // convert RegExp to string
        if (val && val.constructor === RegExp) {
            return val.toString();
        } else if (name === "_sourceFeed") { // 
            return undefined; // remove from result
        } else {
            return val; // return as is
        }
    },

    GetSyncStorage: function (key, callback) {
        window.chrome.storage.sync.get(key, function (pair) {
            if (pair[key] === undefined) {
                callback(null);
            } else {
                callback(pair[key]);
            }
        });
    },

    SetSyncStorage: function (key, value, callback) {
        var objToStore = {};
        objToStore[key] = value;
        window.chrome.storage.sync.set(objToStore, callback);
    },

    GetLocalStorageItem: function (itemName) {
        var item = localStorage.getItem(itemName);
        if (item !== null && item !== undefined && item !== "undefined") {
            return JSON.parse(item);
        }
        return null;
    },

    SetLocalStorageItem: function (itemName, itemValue, secondtry) {
        var self = this;
        try {
            localStorage.setItem(itemName, JSON.stringify(itemValue, self.StringifyReplacer));
        } catch (e) {
            if (secondtry !== true) {
                if (e.code === 22) { // Storage Full - Clear It
                    self.ClearLocalStorageByKey("FeedArticles_");
                    self.SetLocalStorageItem(itemName, itemValue, true);
                }
            }
        }
    },

    SetLocalStorageItemNoStringify: function (itemName, itemValue, secondtry) {
        var self = this;
        try {
            localStorage.setItem(itemName, itemValue);
        } catch (e) {
            if (secondtry !== true) {
                if (e.code === 22) {
                    self.ClearLocalStorageByKey("FeedArticles_");
                    self.SetLocalStorageItemNoStringify(itemName, itemValue, true);
                }
            }
        }
    },

    RemoveLocalStorageItem: function (itemName) {
        localStorage.removeItem(itemName);
    },

    ClearLocalStorageByKey: function (keypefix) {
        for (var i = 0, il = localStorage.length; i < il; i++) {
            var key = localStorage.key(i);
            if (key !== null && key.indexOf(keypefix) === 0) {
                localStorage.removeItem(localStorage.key(i));
            }
        }
    },

    GetLocalStorageMaxSize: function (error) {
        if (localStorage) {
            var max = 10 * 1024 * 1024,
                i = 64,
                string1024 = "",
                string = "",
                // generate a random key
                testKey = "size-test-" + Math.random().toString(),
                minimalFound = 0,
                error = error || 25e4;

            // fill a string with 1024 symbols / bytes    
            while (i--) string1024 += 1e16;

            i = max / 1024;

            // fill a string with "max" amount of symbols / bytes    
            while (i--) string += string1024;

            i = max;

            // binary search implementation
            while (i > 1) {
                try {
                    localStorage.setItem(testKey, string.substr(0, i));
                    localStorage.removeItem(testKey);

                    if (minimalFound < i - error) {
                        minimalFound = i;
                        i = i * 1.5;
                    }
                    else break;
                } catch (e) {
                    localStorage.removeItem(testKey);
                    i = minimalFound + (i - minimalFound) / 2;
                }
            }

            return minimalFound;
        }
    },

    /* Image Color */
    GetDominantColors: function (img, succesCallback, paletteSize) {
        var self = this;
        this.GetImageData(img, function (data) {
            var length = data.length,
                colorCounts = {},
                rgbString = "",
                rgb = [],
                colors = {
                    dominant: { name: "", count: 0 },
                    palette: Array.apply(null, Array(paletteSize || 10)).map(Boolean).map(function (a) { return { name: "0,0,0", count: 0 }; }),
                    image: ""
                };

            // Loop over all pixels, in BlockSize iterations.
            var i = 0;
            while (i < length) {
                rgb[0] = data[i];
                rgb[1] = data[i + 1];
                rgb[2] = data[i + 2];
                rgbString = rgb.join(",");

                // Keep track of counts.
                if (rgbString in colorCounts) {
                    colorCounts[rgbString] = colorCounts[rgbString] + 1;
                }
                else {
                    colorCounts[rgbString] = 1;
                }

                // Find dominant and palette, ignoring black/white pixels.
                if (rgbString !== "0,0,0" && rgbString !== "255,255,255") {
                    var colorCount = colorCounts[rgbString];

                    if (colorCount > colors.dominant.count) {
                        colors.dominant.name = rgbString;
                        colors.dominant.count = colorCount;
                    } else {
                        colors.palette.some(function (c) {
                            if (colorCount > c.count) {
                                c.name = rgbString;
                                c.count = colorCount;
                                return true;
                            }
                        });
                    }
                }

                // Increment!
                i += 5 * 4;
            }

            succesCallback && succesCallback({
                dominant: self.MakeRGB(colors.dominant.name),
                palette: self.MapPalette(colors.palette),
                image: img
            });
        });
    },

    MakeRGB: function (name) {
        return ["rgb(", name, ")"].join("");
    },

    HexToRgbA: function (hex, alpha) {
        var c;
        if (/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)) {
            c = hex.substring(1).split("");
            if (c.length === 3) {
                c = [c[0], c[0], c[1], c[1], c[2], c[2]];
            }
            c = "0x" + c.join("");
            return "rgba(" + [(c >> 16) & 255, (c >> 8) & 255, c & 255].join(",") + "," + alpha + ")";
        }
        throw new Error("Bad Hex");
    },

    AddAlphaToColor: function (color, alpha) {
        if (color.indexOf("#") === 0) {
            if (color.length === 9) {
                color = color.slice(0, 7);
            }
            else if (color.length === 5) {
                color = color.slice(0, 4);
            }
            return NewsTab.Helpers.HexToRgbA(color, alpha);
        } else {
            if (color.indexOf("rgb(") === 0) {
                color = color.replace("rgb(", "rgba(");
                return color.slice(0, color.length - 1) + ("," + alpha) + ")";
            } else if (color.indexOf("rgba(") === 0) {
                return color.slice(0, color.lastIndexOf(",")) + ("," + alpha) + ")";
            }
        }
        return color;
    },

    MapPalette: function (palette) {
        var self = this;
        return palette.map(function (c) { return self.MakeRGB(c.name); });
    },

    GetImageData: function (img, loadedCallback) {
        var imgObj = new Image();
        var imgSrc = img.src || img;

        if (imgSrc.substring(0, 5) !== "data:")
            imgObj.crossOrigin = "Anonymous";

        imgObj.onload = function () {
            var context = document.createElement("canvas").getContext("2d");
            context.drawImage(imgObj, 0, 0);

            var imageData = context.getImageData(0, 0, imgObj.width, imgObj.height);
            loadedCallback && loadedCallback(imageData.data);
        };

        imgObj.src = imgSrc;
    },

    FirstLetterCapitalize: function (str) {
        return str && str.charAt(0).toUpperCase() + str.slice(1);
    },

    Guid: function () {
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }
        return s4() + s4() + "-" + s4() + "-" + s4() + "-" +
            s4() + "-" + s4() + s4() + s4();
    },

    TimeDifferenceInDays: function (date1, date2) {
        var timeDiff = Math.abs(date2.getTime() - date1.getTime());
        var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));

        return diffDays;
    }
};