﻿var Feed = function () {
    this.Id = null;
    this.Name = null;
    this.Icon = null;
    this.Type = Feed.TYPES.NONE;
    this.MaxId = "X";
    this.MinId = "X";
    this.Count = this.DEFAULT_COUNT;
    this.Items = [];
    this.EditMenuIcon = null;
    this.EditMenuClass = null;

    this.IsLoading = false;
    this.IsAtEnd = false;
    this.IsList = false;
    this.IsFixed = false;

    this.ShowInList = false;
    this.Url = null;
    this.ApiUrl = NewsTab.ApiUrl_Base;
    this.SpecialParameter = null;
    this.PageNbr = 1;
    this.UpdateTimeout = 300; //sec
    this.LastUpdate = 0;
    this.UserTagLinkBase = null;
    this.HashTagLinkBase = null;
    this.FeedUrlHash = null;

    this.SectionId = null;

    this.TextLinker = (typeof Autolinker === 'undefined' || Autolinker === null) ? null : new Autolinker({ truncate: 20 });

    this.Reset = function () {
        this.MaxId = "X";
        this.MinId = "X";
        this.IsLoading = false;
        this.IsAtEnd = false,
            this.Items = [];
        this.PageNbr = 1;
    };

    this.Init = function (feed) {
        this.Id = feed.Id;
        this.SectionId = feed.SectionId;
        this.IsList = feed.IsList;
        this.Name = feed.Name;
        this.Url = feed.Url;
        this.SiteUrl = feed.SiteUrl;
        this.Icon = feed.Icon;
        this.Type = feed.Type;
        this.Language = feed.Language;
        this.Count = feed.Count;
        this.OrderNumber = feed.OrderNumber;
        this.Layout = feed.Layout === "null" || feed.Layout === undefined ? "layout-grid" : feed.Layout;
        this.LayoutType = feed.LayoutType;
        this.Selected = feed.Selected;
        this.UsageCount = feed.UsageCount;
        this.TopFeed = feed.TopFeed;
        this.LastTimeUsed = feed.LastTimeUsed;
        this.PopularityRank = feed.PopularityRank;
        if (feed.ShowInList !== "undefined") {
            this.ShowInList = feed.ShowInList;
        }
        else {
            this.ShowInList = false;
        }
        if (feed.ApiUrl !== "undefined") {
            this.ApiUrl = feed.ApiUrl;
        }

        if (feed.EditMenuIcon !== "undefined") {
            this.EditMenuIcon = feed.EditMenuIcon;
        }

        if (feed.EditMenuClass !== "undefined") {
            this.EditMenuClass = feed.EditMenuClass;
        }

        if (feed.SpecialParameter !== "undefined") {
            this.SpecialParameter = feed.SpecialParameter;
        }
        switch (this.Type) {
            case Feed.TYPES.TWITTER:
            case "twitter":
            case "twitter/list":
                {
                    this.UserTagLinkBase = "https://twitter.com/";
                    this.HashTagLinkBase = "https://twitter.com/hashtag/";
                    this.TextLinker = (typeof Autolinker === 'undefined' || Autolinker === null) ? null : new Autolinker({
                        twitter: true,
                        hashtag: true,
                        truncate: 20,
                        replaceFn: function (autolinker, match) {
                            switch (match.getType()) {
                                case "twitter":
                                    var twitterHandle = match.getTwitterHandle();
                                    return "&nbsp;<a href=\"https://twitter.com/" + twitterHandle + "\" target=\"_blank\" class=\"add-to-filter\">@" + twitterHandle + "</a>&nbsp;";
                                case "hashtag":
                                    var hashtagHandle = match.getHashtagHandle();
                                    return "&nbsp;<a href=\"https://twitter.com/hashtag/" + hashtagHandle + "\" target=\"_blank\" class=\"add-to-filter\">#" + hashtagHandle + "</a>&nbsp;";
                                default:
                                    return true;
                            }
                        }
                    });
                }
                break;
        }
        return this;
    };

    this.NewInit = function (feed) {
        this.Init(feed);

        this.Id = feed.Id;
        this.SectionId = feed.SectionId;
        this.Name = feed.Name;
        this.Url = feed.Url;
        this.SiteUrl = feed.SiteUrl;
        this.Icon = feed.IconUrl;
        this.Description = feed.Description;
        this.OriginalFeedName = feed.OriginalFeedName;
        this.TotalSubscribers = feed.TotalSubscribers;

        this.IsList = false;

        this.Language = feed.Language;
        this.Count = Feed.DEFAULT_COUNT;
        this.OrderNumber = feed.OrderNumber;
        this.Layout = feed.Layout === "null" || feed.Layout === undefined ? "layout-grid" : feed.Layout;
        this.LayoutType = feed.LayoutType;
        this.ShowInList = true;
        this.UsageCount = feed.UsageCount;
        this.TopFeed = feed.TopFeed;
        this.LastTimeUsed = feed.LastTimeUsed;
        this.PopularityRank = feed.PopularityRank;
        this.FeedUrlHash = feed.FeedUrlHash;
        switch (feed.Type) {
            case "rss":
                {
                    this.Type = Feed.TYPES.RSS;
                    this.ApiUrl = NewsTab.ApiUrl_Base + "api/global/";
                    this.EditMenuIcon = "icon-angle-down";
                    this.EditMenuClass = "editRssFeed";
                }
                break;
            case "twitter":
            case "twitter/list":
                {
                    this.Type = Feed.TYPES.TWITTER;
                    this.ApiUrl = NewsTab.ApiUrl_Base + feed.Url;
                    this.SpecialParameter = feed.AdditionalInfo;

                    if (feed.Type === "twitter/list") {
                        this.IsList = true;
                        this.EditMenuIcon = "icon-angle-down";
                        this.EditMenuClass = "hideSpecialFeed";
                    }
                    else if (this.SpecialParameter !== null) {
                        this.EditMenuIcon = "icon-angle-down";
                        this.EditMenuClass = "deleteSpecialFeed";
                    }

                    if (this.Name === "Timeline") {
                        this.IsFixed = true;
                        this.OrderNumber = 0;
                        this.EditMenuIcon = null;
                        this.EditMenuClass = null;
                    }
                }
                break;
            case "st":
                {
                    this.Type = Feed.TYPES.SOCIALTRENDING;
                    this.ApiUrl = NewsTab.ApiUrl_Base + feed.Url;
                    this.EditMenuIcon = "icon-angle-down";
                    this.EditMenuClass = "sortSocialTrendingNews";
                }
                break;
        }
        return this;
    };
};

Feed.DEFAULT_COUNT = 20;

Feed.TYPES = {
    NONE: { Id: "-99", Linked: false, CssClass: null },
    ALL: { Id: "-100", Linked: true, CssClass: "rss" },
    HOME: { Id: "-101", Linked: true, CssClass: "rss" },
    TRENDING: { Id: "-102", Linked: true, CssClass: "rss" },
    SOCIALTRENDING: { Id: "-103", Linked: true, CssClass: "rss" },
    PUBLICPINWALL: { Id: "-3", Linked: true, CssClass: "rss" },
    PINWALL: { Id: "-2", Linked: true, CssClass: "rss" },
    CHROME: { Id: "-1", Linked: true, CssClass: "chrome" },
    RSS: { Id: "0", Linked: true, CssClass: "rss" },
    TWITTER: { Id: "2", Linked: false, CssClass: "twitter" },
    PUBLICSHARE: { Id: "3", Linked: true, CssClass: "rss" }
};

Feed.Linked = function (type) {
    switch (type) {
        case Feed.TYPES.NONE: return Feed.TYPES.NONE.Account !== null;
        case Feed.TYPES.TWITTER: return Feed.TYPES.TWITTER.Account !== null;
        default: return true;
    }
};

Feed.Account = function (type) {
    switch (type) {
        case Feed.TYPES.TWITTER: return Feed.TYPES.TWITTER.Account;
        default: return null;
    }
};

Feed.Reset = function (feed) {
    feed.MaxId = "X";
    feed.MinId = "X";
    feed.IsLoading = false;
    feed.IsAtEnd = false,
        feed.Items = [];
    feed.PageNbr = 1;
    return feed;
};

Feed.ChangeFeedReset = function (feed) {
    feed.MaxId = "X";
    feed.MinId = "X";
    feed.IsLoading = false;
    feed.IsAtEnd = false,
        feed.PageNbr = 1;
    return feed;
};