﻿var Section = function () {
    this.Id = null;
    this.Name = null;
    this.ClassName = null;
    this.Connected = false;
    this.ShowInSourceList = true;

    this.SectionTypeId = Section.TYPES.Normal;

    this.User = null;

    this.IconUrl = null;
    this.Color = null;
    this.OrderNumber = null;
    this.Layout = "layout-grid";
    this.LayoutType = 0;

    this.EditCssClass = null;

    this.HiddenLists = [];

    this.Feeds = [new Feed().Init({
        Id: "-0",
        Name: "None",
        Icon: "icon-question",
        Type: Feed.TYPES.NONE,
        Count: Feed.DEFAULT_COUNT,
        OrderNumber: "-99",
        Layout: Section.Layout,
        LayoutType: Section.LayoutType,
        ShowInList: false,
        SectionId: this.Id
    })];

    this.CurrentFeed = null;
    this.DefaultFeed = null;

    this.Delete = function () {
        this.User.Resete();
        this.Connected = false;
    };

    this.Refresh = function (list) {
        if (list == null) {
            list = this.CurrentFeed;
        }
        if (list && !list.IsLoading) {
            if (NewsTab.Token) {
                SocialTab.ParseFeed(list);
            }
        }
        return false;
    };

    this.GetRandomColor = function (itemColor, tryNumber) {
        if (tryNumber === undefined) {
            tryNumber = 0;
        }
        else {
            tryNumber++;
        }

        var color = itemColor !== undefined ? itemColor.slice(1) : this.Color.slice(1);
        var rgbValues = [parseInt(color.slice(0, 2), 16), parseInt(color.slice(2, 4), 16), parseInt(color.slice(4, 6), 16)];

        var random = NewsTab.Helpers.GetRandomInt(1, 64);
        var operation = Math.random() < 0.5 ? -1 : 1;

        var max = rgbValues[0] > rgbValues[1] ? (rgbValues[0] > rgbValues[2] ? rgbValues[0] : rgbValues[2]) : (rgbValues[1] > rgbValues[2] ? rgbValues[1] : rgbValues[2]);

        rgbValues[2] = rgbValues[2] + (operation * (random - Math.floor((max - rgbValues[2]) / random)));
        rgbValues[1] = rgbValues[1] + (operation * (random - Math.floor((max - rgbValues[1]) / random)));
        rgbValues[0] = rgbValues[0] + (operation * (random - Math.floor((max - rgbValues[0]) / random)));

        for (var i = 0; i < 3; i++) {

            if (rgbValues[i] > 255) {
                rgbValues[i] = 255;
            }
            else if (rgbValues[i] < 0) {
                rgbValues[i] = 0;
            }
            rgbValues[i] = rgbValues[i].toString(16);
            if (rgbValues[i].length === 1) {
                rgbValues[i] = "0" + rgbValues[i];
            }
        }

        var returnColor = "#" + rgbValues[0] + rgbValues[1] + rgbValues[2];
        if (returnColor === color && tryNumber < 4) { // avoid article color same like dashboard
            return this.GetRandomColor(itemColor, tryNumber);
        }
        return returnColor;
    };
};

Section.prototype.Init = function (section) {
    if (section.Layout === "null") {
        section.Layout = "layout-grid";
    }

    switch (section.SectionTypeId) {
        case Section.TYPES.Normal:
            return this.Init_Normal(section);
        case Section.TYPES.Social:
            switch (section.ClassName) {
                case "twitter":
                    return this.Init_Twitter(section);
            }
            return null;
        case Section.TYPES.Home:
            return this.Init_Home(section);
        case Section.TYPES.Pinned:
            return this.Init_PinWall(section);
        case Section.TYPES.Chrome:
            return this.Init_Chrome(section);
        default:
            return this.Init_Normal(section);
    }
};

Section.prototype.Init_Normal = function (section) {
    this.Id = section.Id;

    this.Name = section.Name;
    this.EnglishName = section.EnglishName;
    this.ClassName = section.ClassName;
    this.Connected = section.Connected;

    this.SectionTypeId = Section.TYPES.Normal;

    this.Color = section.Color;
    this.OrderNumber = section.OrderNumber;
    this.Layout = section.Layout;
    this.LayoutType = section.LayoutType;

    this.EditCssClass = "editRssSection";

    this.Feeds.push(new Feed().Init(
        {
            Id: this.Id + "-100",
            Name: "All Feeds",
            Icon: "icon-question",
            Type: Feed.TYPES.ALL,
            Count: Feed.DEFAULT_COUNT,
            OrderNumber: "-1",
            Layout: this.Layout,
            LayoutType: this.LayoutType,
            ShowInList: false,
            SectionId: this.Id
        }));

    var hiddenLists = localStorage.getItem("ST_" + NewsTab.User.Id + "_" + this.Name + "_HiddenLists");
    if (hiddenLists) {
        this.HiddenLists = JSON.parse(hiddenLists);
    }

    if (section.Feeds) {
        for (var i = 0, len = section.Feeds.length; i < len; i++) {
            var tempFeed = new Feed().NewInit(section.Feeds[i]);
            tempFeed.ShowInList = this.HiddenLists.indexOf(tempFeed.Id) < 0;
            tempFeed.SectionId = this.Id;
            tempFeed.Layout = this.Layout;
            this.Feeds.push(tempFeed);
        }
    }
    this.DefaultFeed = this.Feeds[0];

    var selectedFeed = this.Feeds.filter(function (feed) {
        return feed.Id === section.SelectedFeed;
    });
    if (selectedFeed.length) {
        this.CurrentFeed = selectedFeed[0];
    } else if (this.Feeds.length > 1) {
        this.CurrentFeed = this.Feeds[1];
    } else {
        this.CurrentFeed = this.DefaultFeed;
    }

    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];

    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    return this;
};

Section.prototype.Init_Social = function () {
    this.SectionTypeId = Section.TYPES.Social;

    this.EditCssClass = "editSocialSection";

    this.Disconnect = function (removeForSections) {
        var callUrl = NewsTab.ApiUrl_Base + "api/" + this.ClassName + "/disconnect";
        var self = this;
        $.ajax({
            url: callUrl,
            headers: { Authorization: 'Bearer ' + NewsTab.Token },
            type: "POST",
            contentType: "application/json",
            dataType: "json",
            processdata: true,
            error: function (error) {
                console.error("ERROR - " + self.Name + "_Disconnect: ", error);
            }
        }).then(function (response) {
            if (response) {
                if (self.ClassName === "twitter") {
                    Feed.TYPES.TWITTER.Account = null;
                }

                if (removeForSections) {
                    $("." + self.ClassName).removeClass("connected");
                    SocialTab.RemoveSection(self.Id);
                }
                else {
                    SocialTab.SetCurrentFeed(NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Id, true);
                }
            }
        });
    };
};

Section.prototype.Init_Twitter = function (section) {
    this.Init_Social();

    this.Id = section.Id;
    this.Name = section.Name;
    this.EnglishName = section.EnglishName;
    this.ClassName = section.ClassName;
    this.Connected = false;
    this.OrderNumber = section.OrderNumber;
    this.Layout = section.Layout;
    this.LayoutType = section.LayoutType;

    this.User = {
        Username: "",
        FullName: "",
        ProfilePictureUrl: "",
        Resete: function () {
            this.Username = "";
            this.FullName = "";
            this.ProfilePictureUrl = "";
        }
    };

    this.IconUrl = null;
    this.Color = section.Color;

    this.Url = "https://twitter.com/";

    this.HiddenLists = [];

    var hiddenLists = localStorage.getItem("ST_" + NewsTab.User.Id + "_" + this.Name + "_HiddenLists");
    if (hiddenLists) {
        this.HiddenLists = JSON.parse(hiddenLists);
    }

    this.UpdateList = function () {

        var callUrl = NewsTab.ApiUrl_Base + "api/twitter/get/mylists/" + this.Id;
        var self = this;
        $.ajax({
            url: callUrl,
            headers: { Authorization: 'Bearer ' + NewsTab.Token },
            error: function (error) {
                if (self.ShowLoadErrors) {
                    console.error("ERROR - GetLists:");
                    console.error(error);
                }
            }
        }).then(function (data) {
            if (data) {
                for (var i = 0, len = self.Feeds.length; i < len; i++) {
                    if (self.Feeds[i].IsList == true) {
                        self.Feeds.splice(i, 1);
                        len--;
                        i--;
                    }
                }
                for (var i = 0, len = data.length; i < len; i++) {
                    var tempFeed = new Feed().NewInit(data[i]);
                    tempFeed.ShowInList = self.HiddenLists.indexOf(data[i].Id) < 0;
                    tempFeed.SectionId = self.Id;
                    tempFeed.Layout = self.Layout;
                    self.Feeds.push(tempFeed);
                }
            }
        });
    };

    if (section.Feeds) {
        for (var i = 0, len = section.Feeds.length; i < len; i++) {
            var tempFeed = new Feed().NewInit(section.Feeds[i]);
            if (tempFeed.Name == "Timeline" && !tempFeed.IsList) {
                tempFeed.ShowInList = true;
                this.HiddenLists.pop(tempFeed.Id);
            }
            else {
                tempFeed.ShowInList = this.HiddenLists.indexOf(tempFeed.Id) < 0;
            }
            tempFeed.SectionId = this.Id;
            tempFeed.Layout = this.Layout;
            this.Feeds.push(tempFeed);
        }
        this.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt))
        this.UpdateList();
    }

    this.DefaultFeed = this.Feeds[0];
    if (this.Feeds.length > 0) {
        this.DefaultFeed = this.Feeds[1];
    }

    this.CurrentFeed = this.DefaultFeed;

    this.ReadFeeds = [];
    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];
    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    this.ShowLoadErrors = true;

    return this;
};

Section.prototype.Init_Home = function (section) {
    this.Id = section.Id;
    this.Name = section.Name;
    this.EnglishName = section.EnglishName;
    this.ClassName = section.ClassName;
    this.Connected = true;

    this.SectionTypeId = Section.TYPES.Home;

    this.ShowInSourceList = false;

    this.IconUrl = null;
    this.Color = section.Color;
    this.OrderNumber = section.OrderNumber;
    this.Layout = section.Layout;
    this.LayoutType = section.LayoutType;

    this.EditCssClass = "editRssSection";

    this.ApiUrl = NewsTab.ApiUrl_Global;

    this.Feeds.push(new Feed().Init({
        Id: section.Feeds[0] ? section.Feeds[0].Id : "11111111-0000-1001-0000-000000000000",
        Name: section.Feeds[0] ? section.Feeds[0].Name : "Best",
        Icon: "icon-trending0",
        Type: Feed.TYPES.TRENDING,
        Count: Feed.DEFAULT_COUNT,
        OrderNumber: 1,
        Layout: this.Layout,
        LayoutType: this.LayoutType,
        ShowInList: true,
        SectionId: this.Id,
        EditMenuIcon: "icon-angle-down",
        EditMenuClass: "sortTrendingNews"
    }));

    this.Feeds.push(new Feed().Init({
        Id: this.Id + "-100",
        Name: "All",
        Icon: "icon-home",
        Type: Feed.TYPES.HOME,
        Count: Feed.DEFAULT_COUNT,
        OrderNumber: 0,
        Layout: this.Layout,
        LayoutType: this.LayoutType,
        ShowInList: true,
        SectionId: this.Id
    }));

    this.DefaultFeed = this.Feeds[1];

    if (this.Feeds.length > 1) {
        this.CurrentFeed = this.Feeds[1];
    } else {
        this.CurrentFeed = this.DefaultFeed;
    }

    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];

    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    var list = document.getElementsByClassName("homeSourceItem");
    for (var i = 0, len = list.length; i < len; i++) {
        list[i].setAttribute("data-section-id", this.Id);
    }

    return this;
};

Section.prototype.Init_Chrome = function (section) {
    return this.Init_Normal(section);
};

Section.prototype.Init_PinWall = function (section) {
    this.Id = section.Id;
    this.Name = section.Name;
    this.EnglishName = section.EnglishName;
    this.ClassName = section.ClassName;
    this.Connected = true;

    this.SectionTypeId = Section.TYPES.Pinned;

    this.ShowInSourceList = false;

    this.IconUrl = null;
    this.Color = section.Color;
    this.OrderNumber = section.OrderNumber;
    this.Layout = section.Layout;

    this.ApiUrl = NewsTab.ApiUrl_Global;

    this.Feeds.push(new Feed().Init({
        Id: this.Id + "-1",
        Name: "All Pins",
        Icon: "icon-pin",
        Type: Feed.TYPES.PINWALL,
        Count: Feed.DEFAULT_COUNT,
        OrderNumber: "1",
        Layout: this.Layout,
        LayoutType: this.LayoutType,
        ShowInList: true,
        SectionId: this.Id,
        ApiUrl: NewsTab.ApiUrl_Global
    }));

    this.DefaultFeed = this.Feeds[0];

    if (this.Feeds.length > 1) {
        this.CurrentFeed = this.Feeds[1];
    } else {
        this.CurrentFeed = this.DefaultFeed;
    }

    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];

    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    var list = document.getElementsByClassName("pinnedSourceItem");
    for (var i = 0, len = list.length; i < len; i++) {
        list[i].setAttribute("data-section-id", this.Id);
    }
    return this;
};

Section.prototype.Init_PublicShare = function (feedToShow) {
    this.Id = SocialTab.NewId;
    this.Name = "Tranding News";
    this.EnglishName = "Tranding News";
    this.ClassName = "public-share";
    this.Connected = true;

    this.SectionTypeId = Section.TYPES.PublicShare;

    this.ShowInSourceList = true;

    this.IconUrl = null;
    this.Color = "#3bc1f1";
    this.OrderNumber = 0;
    this.Layout = "layout-grid";
    this.LayoutType = 1;

    this.ApiUrl = NewsTab.ApiUrl_Global;

    if (feedToShow) {
        var tempFeed = new Feed().NewInit(feedToShow);
        tempFeed.Type = Feed.TYPES.PUBLICSHARE;
        tempFeed.SectionId = this.Id,
            tempFeed.Layout = this.Layout;
        this.Feeds.push(tempFeed);
    }

    this.DefaultFeed = this.Feeds[0];

    if (this.Feeds.length > 1) {
        this.CurrentFeed = this.Feeds[1];
    } else {
        this.CurrentFeed = this.DefaultFeed;
    }

    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];

    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    return this;
};

Section.prototype.Init_PublicPinn = function () {
    this.Id = SocialTab.NewId;
    this.Name = "Public News";
    this.EnglishName = "Public News";
    this.ClassName = "public-share";
    this.Connected = true;

    this.SectionTypeId = Section.TYPES.PublicShare;

    this.ShowInSourceList = true;

    this.IconUrl = null;
    this.Color = "#3bc1f1";
    this.OrderNumber = 0;
    this.Layout = "layout-grid";
    this.LayoutType = 1;

    this.ApiUrl = NewsTab.ApiUrl_Global;

    this.Feeds.push(new Feed().Init(
        {
            Id: 0,
            Url: null,
            Name: "Shared",
            Icon: "icon-share",
            Type: Feed.TYPES.PUBLICPINWALL,
            Count: Feed.DEFAULT_COUNT,
            OrderNumber: "1",
            Layout: this.Layout,
            LayoutType: this.LayoutType,
            ShowInList: true,
            SectionId: this.Id,
            ApiUrl: NewsTab.ApiUrl_Global
        }));

    this.DefaultFeed = this.Feeds[0];

    if (this.Feeds.length > 1) {
        this.CurrentFeed = this.Feeds[1];
    } else {
        this.CurrentFeed = this.DefaultFeed;
    }

    var readFeeds = localStorage["ST_" + NewsTab.User.Id + "_" + this.Name + "_ReadFeeds"];

    if (readFeeds) {
        this.ReadFeeds = JSON.parse(readFeeds);
    }

    return this;
};

Section.TYPES = {
    Default: "aaaaaaaa-0000-1001-0000-000000000000",
    Normal: "aaaaaaaa-0000-1001-0000-000000000001",
    Social: "aaaaaaaa-0000-1001-0000-000000000002",
    Pinned: "aaaaaaaa-0000-1001-0000-000000000003",
    Home: "aaaaaaaa-0000-1001-0000-000000000004",
    Chrome: "aaaaaaaa-0000-1001-0000-000000000005",
    PublicShare: "aaaaaaaa-0000-1001-0000-000000000006"
};