﻿var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.ArticleView = {
    _defaultFeedIcon: "/Media/ExtensionImages/rss.png",
    _mp3Icon: "/Media/DashboardImages/mp3_icon.png",
    _mp3IconSmall: "/Media/DashboardImages/mp3_icon_small.png",
    _videoIcon: "/Media/DashboardImages/video_icon.png",
    _videoIconSmall: "/Media/DashboardImages/video_icon_small.png",
    _flipColorDark: "#2E2D2D",
    _flipColorLight: "#E2E1E1",

    _maxArticleToDisplay: 0,

    MASTER_LAYOUT: null,

    DEFAULT_SETTINGS: {
        IsPreview: false,
        ShowOptions: true,
        ContainerSelector: "#magicGrid",
        Layout: "layout-grid",
        FlipAutomaticlyOnImageLoad: true,
        InsertAtBeginning: false,
        FullHtmlParse: true,
        MaxNbrOfArticlesPerSet: 20,
        ActiveBoxSize: function () { return NewsTab.Modules.ArticleView.ItemSettings.ActiveBoxSize; },
        ActualBoxSize: function () { return NewsTab.Modules.ArticleView.ItemSettings.ActualBoxSize; },
        ShowSimplifiedArticle: function () { return true; },
        FormMaxWidth: 1024,
        LoadImages: true,
        FontSizes: [10, 12, 14, 16, 18],
        ActiveFontSize: 2,
        BorderStyle: "border-none"
    },

    PREVIEW_SETTINGS: {
        IsPreview: true,
        ShowOptions: false,
        ContainerSelector: "#articles_preview_grid",
        Layout: "layout-grid",
        FlipAutomaticlyOnImageLoad: true,
        InsertAtBeginning: false,
        MaxNbrOfArticlesPerSet: 20,
        ActiveBoxSize: function () { return NewsTab.Modules.ArticleView.ItemSettings.ActiveBoxSize_Preview; },
        ActualBoxSize: function () { return NewsTab.Modules.ArticleView.ItemSettings.ActualBoxSize_Preview; },
        ShowSimplifiedArticle: function () { return false; }
    },


    ForceParsButton: false,

    ListedArticles: [],
    UnparsedArticles: [],
    ReadArticles: [],

    PinnedItemAliases: [],

    ItemSettings: {
        BoxSizes: [250, 275, 300, 325, 350, 375, 400],
        ActiveBoxSize: 2,
        ActualBoxSize: 300,
        ActiveBoxSize_Preview: 2,
        ActualBoxSize_Preview: 300
    },

    ExpandFormSettings: {
        Font: 1,
        FontSize: 2,
        LineHeight: 4,
        Brightness: 7,
        Background: 0
    },

    BackUpSourceFeed: {
        Id: "None",
        Name: "NewsTab",
        SiteUrl: "newstab.com",
        Type: {
            Id: "0",
            Linked: true,
            CssClass: "rss"
        }
    },


    _$getArticleContainer: function (options) {
        if (options === undefined) {
            options = this.DEFAULT_SETTINGS;
        }
        return $(options.ContainerSelector);
    },

    _getTitle: function (article) {
        if (article) {
            var title = article.Title === null ? article.HtmlTitle : article.Title;
            if (article.SourceFeed() && article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                title = scEmoji(title);
            }
            return title;
        }
        return null;
    },

    _getSummary: function (article) {
        var summary = "";
        if (article.Summary === undefined || article.Summary === null || article.Summary === "") {
            summary = article.HtmlSummary
        }
        else {
            var $s = $("<div>" + article.Summary + "</div>");
            if ($("a.googleFullCoverage", $s).length > 0) {
                summary = article.HtmlSummary
                //$("a.googleFullCoverage", $s).remove();
            }
            else {
                summary = $s[0].innerHTML;
            }
        }

        if (summary === undefined || summary === null || summary === "" || summary.match(/^\s*$/) !== null) {
            var tmp = document.createElement("DIV");
            tmp.innerHTML = article.Text;
            summary = tmp.textContent || tmp.innerText || "";
            if (summary.length > 512) {
                summary = summary.substring(0, 512);
            }
        }
        return (summary === "undefined" || summary === undefined || summary === null) ? "" : summary;
    },

    _getImageUrl: function (article) {
        var imgUrl = article.ImageUrl ? article.ImageUrl : article.Thumbnail;
        if (imgUrl !== undefined && imgUrl !== null && imgUrl.indexOf("http") === -1) {
            imgUrl = "http:" + imgUrl;
        }
        if (imgUrl !== undefined && imgUrl !== null) {
            return NewsTab.Helpers.HtmlEnDeCode.htmlDecode(imgUrl);
        }
        return imgUrl;
    },

    _getSourceName: function (article) {
        return article.SourceFeed().Type.CssClass === "rss" ?
            (article.Author.Username ? article.Author.Username : (article.SourceFeed().SiteUrl ? article.SourceFeed().SiteUrl : article.SourceFeed().Name)) :
            article.Author.FullName;
    },

    _getStatusClasses: function (feed, article, excludeAsRead) {
        var self = this;

        var feedStateCLasses = [];

        if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
            if (article.HasFavorited) {
                feedStateCLasses.push("favorited");
            }
            if (article.HasRetweeted) {
                feedStateCLasses.push("retweeted");
            }
            if (article.Author.Following === 1) {
                feedStateCLasses.push("following");
            }
            if (article.Author.FollowingYou === 1) {
                feedStateCLasses.push("following-you");
            }
        }

        if (!excludeAsRead) {
            var asread = $.grep(self.ReadArticles, function (e) { return e === article.Alias; });
            if (asread.length) {
                feedStateCLasses.push("readX");
            }
        }

        if ((feed !== null && feed.Type === Feed.TYPES.PINWALL) || article.Pinned) {
            feedStateCLasses.push("pinned");
        }

        if (article.Rsr >= 3) {
            feedStateCLasses.push("trending");
        }

        return feedStateCLasses.join(" ");
    },

    _getContentBlockClasses: function () {
        return "font" + this.ExpandFormSettings.Font + " fontSize" + this.ExpandFormSettings.FontSize + " lineHeight" + this.ExpandFormSettings.LineHeight;
    },


    GetDefaultSettings: function (settings) {
        return $.extend({}, this.DEFAULT_SETTINGS);
    },

    GetPreviewSettings: function (settings) {
        return $.extend({}, this.PREVIEW_SETTINGS);
    },

    GenerateCustomSettings: function (settings) {
        return $.extend({}, this.DEFAULT_SETTINGS, settings);
    },


    ParseViewItemToArticleObject: function (item) {
        if (item instanceof jQuery) {
            var id = item.attr("data-article-id");
            var article = $.grep(NewsTab.Modules.ArticleView.ListedArticles, function (e) { return e.Id === id });
            if (article.length > 0) {
                article = article[0];
            }
            else {
                article = null;
            }
            return article;
        }
        return item;
    },

    ResetArticleView: function (options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, options);

        var $container = self._$getArticleContainer(opt);

        if (self.MASTER_LAYOUT !== null) {
            opt.Layout = self.MASTER_LAYOUT;
        }

        if (NewsTab.IsExtension) {
            $container.html('<div id="feedLoader" class="loader0">Loading...</div>');
        } else {
            $container.html('<div class="gap"></div>');
        }

        self.ListedArticles = [];
    },

    DisplayFeedArticles: function (feed, options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, { Layout: feed.Layout }, options);

        $("#feedLoader").remove();
        $(".feedsListMessage").remove();

        if (self.MASTER_LAYOUT !== null) {
            opt.Layout = self.MASTER_LAYOUT;
        }

        var newArticles = [];

        for (var i = 0, j = 0, l = feed.Items.length; i < l && j < opt.MaxNbrOfArticlesPerSet; i++) {
            var existingArticle = [];
            if (self.ListedArticles.length) {
                existingArticle = $.grep(self.ListedArticles, function (f) { return f.Alias === feed.Items[i].Alias; });
            }
            if (existingArticle.length === 0) {

                if (self.ListedArticles.length > j) {
                    self.ListedArticles = self.ListedArticles.splice(j, 0, feed.Items[i]);
                } else {
                    self.ListedArticles.unshift(feed.Items[i]);
                }
                
                newArticles.push(feed.Items[i]);
                j++;
            }
        }


        if (newArticles.length > 0) {
            if ($(".socialTabItem.empty:first", opt.ContainerSelector).length === 0) {
                self.SetUp_PreviewItem_Placeholders(newArticles.length, opt);
            }

            $(opt.ContainerSelector).removeClass(function (index, className) {
                return (className.match(/(^|\s)layout-\S+/g) || []).join(' ');
            });

            $(opt.ContainerSelector).addClass(opt.Layout);



            switch (opt.Layout) {
                case "layout-compact":
                    newArticles.forEach(function (article) {
                        self.SetUp_CompactPreviewItem(feed, article, opt);
                    });
                    break;
                case "layout-list":
                    newArticles.forEach(function (article) {
                        self.SetUp_ListPreviewItem(feed, article, opt);
                    });
                    break;
                case "layout-cascading-grid":
                    newArticles.forEach(function (article) {
                        self.SetUp_CascadingGridPreviewItem(feed, article, opt);
                    });
                    NewsTab.Extension.UpdateArticleGridWidth();
                    break;
                default:
                    newArticles.forEach(function (article) {
                        self.SetUp_GridPreviewItem(feed, article, opt);
                    });
            }
        }

        self.ClearPlaceholders(options);
    },

    DisplayFeedEmpty: function (feed, options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, { Layout: feed.Layout }, options);

        self.ClearPlaceholders(opt);

        $("#noFeeds").remove();
        $("#errorFeeds").remove();
        $("#feedLoader").remove();

        var section = NewsTab.Modules.SectionsManager.CurrentSection;

        var html = "<div id=\"noFeeds\" class=\"feedsListMessage\" style=\"color:" + section.Color + "\"><div>";

        if (section.Id === Feed.TYPES.TWITTER.Id) {
            html += "<span class=\"iconContainer\"><i class=\"icon-" + section.ClassName + "\"></i></span>";
        } else {
            html += "<span class=\"iconContainer\">" + (section.ClassName ? "<i class=\"icon-" + section.ClassName + "\"></i>" : section.Name.substring(0, 2)) + "</span>";
        }

        if (section.CurrentFeed != undefined && section.CurrentFeed.Type != undefined && section.CurrentFeed.Type.Id === Feed.TYPES.CHROME.Id) {
            html += "<h1>Get Chrome Extension</h1>";
            html += "<p>Set NewsTab as your default new tab on Chrome and get beautiful speed dial of your top sites and history.</p>";
            html += "<a href=\"#\" class=\"btn border\">Get Extension</a>";
        } else if (section.CurrentFeed != undefined && section.CurrentFeed.Type != undefined && section.CurrentFeed.Type.Id === Feed.TYPES.TRENDING.Id) {
            html += "<h1>No stories available</h1>";
        } else if (section.CurrentFeed != undefined && section.CurrentFeed.Type != undefined && section.CurrentFeed.Type.Id === Feed.TYPES.HOME.Id) {
            html += "<h1>Home is empty</h1>";
        } else if (section.CurrentFeed != undefined && section.CurrentFeed.Type != undefined && section.CurrentFeed.Type.Id === Feed.TYPES.PINWALL.Id) {
            html += "<h1>Pinned is empty</h1>";
            html += "<p>You can save and tag news to read later or archive, by pinning it. Anything you pin shows up here.</p>";
        } else if (section.CurrentFeed != undefined && section.CurrentFeed.Type === Feed.TYPES.TWITTER && !Feed.Linked(section.CurrentFeed.Type)) {
            html += "<h1>" + section.Name + "</h1>";
            html += "<a href=\"#\" class=\"btn border blue connectTwitter\">Connect to " + section.Name + "</a>";
        } else if (section.CurrentFeed && section.CurrentFeed.RateLimit === true) {
            html += "<h1>" + section.Name + " API</h1>";
            html += "<p>Rate limit exceeded. Please try again later.</p>";
        } else if (section.CurrentFeed && section.CurrentFeed.Id === "FeedPreview") {
            html += "<h1>" + section.Name + "</h1>";
            html += "<p>Feed is empty</p>";
        } else if (section.Feeds.length > 2) {
            html += "<h1>" + section.Name + "</h1>";
            html += "<p>Feed is empty</p>";
        } else if (!opt.IsPreview) {
            html += "<h1>Add feeds to " + section.Name + "</h1>";
            html += "<p>Feeds are the news sources \"feeding\" your sections. Click the button below to add feeds to this section.</p><br/>";
            html += "<button class=\"btn border addContentItem\" data-tab-select=\"#tabFeeds\" tabindex=\"551\" style=\"color:" + section.Color + ";border:3px solid " + section.Color+";\">+ Add Feeds</button>";
        } else if (opt.IsPreview) {
            html += "<h1>" + section.Name + "</h1>";
            html += "<p>Feed is empty</p>";
        }
        html += "</div></div>";

        $(opt.ContainerSelector).prepend(html);
        $(".connectTwitter", opt.ContainerSelector).on("click", function () {
            NewsTab.Modules.TwitterManager.Connect();
        });
    },

    DisplayFeedError: function (feed, options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, { Layout: feed.Layout }, options);

        self.ResetArticleView(opt);
        $("#feedLoader").remove();

        var section = NewsTab.Modules.SectionsManager.CurrentSection;

        var html = "<div id=\"errorFeeds\" class=\"feedsListMessage\" ";

        if (NewsTab.IsExtension !== true) {
            html += "style=\"background-color:" + section.Color + "\"><div class=\"verticalCenter\">";
        } else {
            html += ">";
        }

        if (opt.ErrorCode === 0) {
            html += "<img class=\"icon-nointernet\" src=\"/Media/ExtensionImages/cloud.svg\" />";
            //html += "<h1>Oops...</h1>";
            html += "<p>No Internet connection...</p><br/>";
        } else {
            if (section.Id === Feed.TYPES.TWITTER.Id) {
                html += "<span class=\"iconContainer\" style=\"color:" + section.Color + "\"><i class=\"icon-" + section.ClassName + "\"></i></span>";
            } else {
                html += "<span class=\"iconContainer\" style=\"color:" + section.Color + "\">" + (section.ClassName ? "<i class=\"icon-" + section.ClassName + "\"></i>" : section.Name.substring(0, 2)) + "</span>";
            }
            html += "<h1>Oops...</h1>";
            html += "<p>There was a problem!</p><br/>";
        }
        
       


        html += "<button class=\"btn border refreshFeed\">Try Again</button>";
        if (NewsTab.IsExtension !== true) {
            html += "</div>";
        }
        html += "</div>";
        $(opt.ContainerSelector).prepend(html);
        $(".refreshFeed", opt.ContainerSelector).on("click", function () {
            NewsTab.Modules.SectionsManager.LoadCurrentFeed(opt);
        });
    },

    DisplaySlowNet: function (options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, options);

        var $feedLoader = $("#feedLoader");

        var html = "<div id=\"slowInternet\" class=\"feedsListMessage\" style=\"display:none\">";
        html += "<img class=\"icon-nointernet\" src=\"/Media/ExtensionImages/snail.svg\" />";
        html += "<p>Slow connection...</p></div>";

        $feedLoader.after(html);
        $("#slowInternet").fadeIn();
    },
    // Placeholders & PreviewItems
    ClearPlaceholders: function (options) {
        $(".socialTabItem.empty", this._$getArticleContainer(options)).remove();
    },

    SetUp_PreviewItem_Placeholders: function (numberOfPlaceholders, options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_SETTINGS, options);

        if (self.MASTER_LAYOUT !== null) {
            opt.Layout = self.MASTER_LAYOUT;
        }

        var $container = this._$getArticleContainer(opt);
        if (NewsTab.Modules.SectionsManager.CurrentSection !== null && !NewsTab.IsExtension) {
            $container.css("background-color", NewsTab.Modules.SectionsManager.CurrentSection.Color);
        }

        $(".feedsListMessage", $container).remove();

        var existingPlaceholder = $(".socialTabItem.empty:first", $container);
        if (existingPlaceholder.length === 0) {
            var $placeHolderItem;
            switch (opt.Layout) {
                case "layout-list": $placeHolderItem = this.$Generate_ListPreviewItem_Placeholder(); break;
                case "layout-compact": $placeHolderItem = this.$Generate_CompactPreviewItem_Placeholder(); break;
                case "layout-cascading-grid": $placeHolderItem = this.$Generate_CascadingGridPreviewItem_Placeholder(opt); break;
                default: $placeHolderItem = this.$Generate_GridPreviewItem_Placeholder(opt); break;
            }

            for (var i = 0; i < numberOfPlaceholders; i++) {
                if (opt.InsertAtBeginning) {
                    $container.prepend($placeHolderItem.prop("outerHTML"));
                } else {
                    $container.append($placeHolderItem.prop("outerHTML"));
                }
            }
            $placeHolderItem.remove();
            return true;
        }
        return false;
    },

    $Generate_CompactPreviewItem_Placeholder: function () {
        var placeholder = $("<section/>", {
            "class": "socialTabItem empty",
            "tabindex": 0,
            "style": "border-right: 5px solid " + NewsTab.Modules.SectionsManager.CurrentSection.Color,
            "data-space": true
        });

        return placeholder;
    },

    $Generate_ListPreviewItem_Placeholder: function () {
        var placeholder = $("<section/>", {
            "class": "socialTabItem empty",
            "tabindex": 0,
            "style": "border-right: 5px solid " + NewsTab.Modules.SectionsManager.CurrentSection.Color,
            "data-space": true
        });

        return placeholder;
    },

    $Generate_GridPreviewItem_Placeholder: function (options) {
        var width = options.ActualBoxSize();
        var flipColor = NewsTab.IsLightTheme ? this._flipColorLight : this._flipColorDark;

        var placeholder = $("<section/>", {
            "class": "socialTabItem empty flipItemWrap",
            "tabindex": 0,
            "style": "height:0; width:" + width + "px; padding-bottom:" + width + "px;",
            "data-space": true
        });

        var flipItem = $("<div/>", { "class": "flipItem" });

        var front = $("<figure/>", { "class": "front" });
        var phImage = $("<img/>", { "class": "img-responsive", "alt": "", "style": "background-color:" + flipColor, "src": NewsTab.PlaceholderImage });
        front.append(phImage);
        flipItem.append(front);

        placeholder.append(flipItem);
        return placeholder;
    },

    $Generate_CascadingGridPreviewItem_Placeholder: function (options) {
        var placeholder = $("<section/>", {
            "class": "socialTabItem empty",
            "tabindex": 0,
            "style": "width: calc(" + 100 / options.ActiveBoxSize() + "% - 16px)!important;",
            "data-space": true
        });

        var item = $("<div/>");
        var figure = $("<figure/>", { "class": "back" });
        item.append(figure);

        placeholder.append(item);
        return placeholder;
    },


    _$initialSetUp_PreviewItem: function (feed, article, isRtl, settings) {
        var self = this;
        if (article.SourceFeed() === undefined || article.SourceFeed() === null) {
            article._sourceFeed = self.BackUpSourceFeed;
            if (article._sourceFeed) {
                article.SourceFeedId = article._sourceFeed.Id;
            }
        }

        article.Pinned = NewsTab.Modules.ArticleView.PinnedItemAliases.indexOf(article.Alias) > -1 ? true : false;

        var $container = self._$getArticleContainer(settings);

        var $placeHolderItem = $(".socialTabItem[data-article-id='" + article.Id + "']", $container);

        if ($placeHolderItem.length === 0) {
            $placeHolderItem = $(".socialTabItem.empty:first", $container);
        } else {
            var $temp;

            switch (settings.Layout) {
                case "layout-list": $temp = self.$Generate_ListPreviewItem_Placeholder(); break;
                case "layout-compact": $temp = self.$Generate_CompactPreviewItem_Placeholder(); break;
                case "layout-cascading-grid": $temp = self.$Generate_CascadingGridPreviewItem_Placeholder(settings); break;
                default: $temp = self.$Generate_GridPreviewItem_Placeholder(settings); break;
            }

            $placeHolderItem.replaceWith($temp);
            $placeHolderItem = $temp;
            
        }

        if (isRtl) {
            $placeHolderItem.attr("dir", "rtl");
        }

        $placeHolderItem.removeClass("empty");
        $placeHolderItem.addClass("new");

        $placeHolderItem.addClass(article.SourceFeed().Type.CssClass);
        $placeHolderItem.addClass(self._getStatusClasses(feed, article));


        $placeHolderItem.attr("data-feed-id", article.SourceFeedId);
        $placeHolderItem.attr("data-article-id", article.Id);
        $placeHolderItem.attr("data-article-alias", article.Alias);

        $placeHolderItem.click(function (e) {
            if (!settings.ShowSimplifiedArticle() || (article.OtherParams && article.OtherParams.indexOf("AdArticle") !== -1)) {
                if (NewsTab.IsExtension) {
                    NewsTab.TrackAnalyticsEvent("Web View All (Including Preview)", "Preview Article", NewsTab.Extension.NameId);
                    NewsTab.TrackAnalyticsEvent("Article", "Web View (Original)", NewsTab.Extension.NameId);
                }
                window.open(article.Url, "_blank");
            }
            else {
                if (e.target.tagName !== "A") {
                    self.SetUp_ExpandView(article);
                    setTimeout(function () {
                        $(".socialTabItem.selected", settings.ContainerSelector).removeClass("selected");
                        $placeHolderItem.addClass("selected");
                        $(".back", $placeHolderItem).addClass("jsHover");
                    }, 100);
                }
            }
        });

        return $placeHolderItem;
    },


    SetUp_CompactPreviewItem: function (feed, article, settings) {
        var self = this;

        var title = self._getTitle(article);
        var summary = self._getSummary(article);

        var isRtl = NewsTab.Helpers.ShouldBeRTL(title);

        var placeholder = self._$initialSetUp_PreviewItem(feed, article, false, settings);

        var back = $("<figure/>", { "class": "back" });

        // Options
        var feedItem = "";
        var position = placeholder.is(":first-child") ? "bottom" : "top";
        if (settings.ShowOptions) {
            var options = $("<div/>", { "class": "options" });

            options.append(self.$Generate_ArrowButton({ tooltipPosition: position }));

            if (NewsTab.User.HasAccessToAdvancedPinned) {
                options.append(self.$Generate_AdvancedPinSettingsButton(article, { tooltipPosition: position }));
            }

            options.append(self.$Generate_ViewOrginalButton(article, { tooltipPosition: position }));
            options.append(self.$Generate_ShareButton(article, { tooltipPosition: position }));
            options.append(self.$Generate_PinButton(article, { tooltipPosition: position }));
            options.append(self.$Generate_MarkAsReadButton(article, { tooltipPosition: position }));

            if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                options.append(self.$Generate_RetweetButton(article, { tooltipPosition: position }));
                options.append(self.$Generate_LikeButton(article, { tooltipPosition: position }));
            }
            back.append(options);
        }

        var listBody = $("<div/>", { "class": "compact-layout-body" });

        var listInfo = [];
        listInfo.push("<div class=\"compact-layout-info\" " + (isRtl ? "dir=\"rtl\"" : "") + ">");
        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            listInfo.push("<span class=\"compact-layout-title grayText\">" + (article.Author.FullName ? article.Author.FullName : (article.Author.Username ? article.Author.Username : article.SourceFeed().Name)) + "</span>");
        } else {
            listInfo.push("<span class=\"compact-layout-title grayText\">" + title + "</span>");
        }

        if (summary) {
            listInfo.push("<span class=\"compact-layout-details grayText\"> - " + scEmoji(summary.replace(/<\/?[^>]+(>|$)/g, "")) + "</span>");
        } else {
            listInfo.push("<span class=\"compact-layout-details grayText\">\u00a0</span>");
        }
        listInfo.push("</div>");
        listBody.append(listInfo.join(""));

        var listPubisher = $("<div/>", { "class": "compact-layout-pubisher grayFilter" });
        if (isRtl) {
            listPubisher.attr("rtl", "rtl");
        }

        if (article.SourceFeed().Type === Feed.TYPES.TWITTER) {
            if (article.Author && article.Author.ProfilePictureUrl) {
                listPubisher.append("<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"img-circle profile-picture\"></div>");
            }
            listPubisher.append("<span class=\"fullname\">@" + (article.Author.Username ? article.Author.Username : (article.Author.FullName ? article.Author.FullName : article.SourceFeed().Name)) + "</span>");
        } else {
            if (article.FavIcon) {
                listPubisher.append("<img src=\"" + article.FavIcon + "\" class=\"profile-picture\" onError=\"this.onerror=null;this.src='" + self._defaultFeedIcon + "';\" />");
            }
            else if (article.SourceFeed() && article.SourceFeed().Icon != null && article.SourceFeed().Icon.indexOf(".") > 0) {
                listPubisher.append("<img src=\"" + article.SourceFeed().Icon + "\" class=\"profile-picture\" onError=\"this.onerror=null;this.src='" + self._defaultFeedIcon + "';\" />");
            } else {
                listPubisher.append("<img src=\"" + self._defaultFeedIcon + "\" class=\"profile-picture\" onError=\"this.onerror=null;this.remove();\"/>");
            }
            listPubisher.append("<span class=\"siteUrl grayText\">" + ((article.SourceFeed().Type.CssClass === "rss") ? (article.Author.Username ? article.Author.Username : (article.SourceFeed().SiteUrl ? article.SourceFeed().SiteUrl : article.SourceFeed().Name)) : article.Author.FullName) + "</span>");
        }
        listPubisher.append(self.$Indicator_Time(article, settings));

        if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type.Id === Feed.TYPES.SOCIALTRENDING.Id) {
            listPubisher.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
        } else {
            listPubisher.append(self.$Indicator_Trending(article, settings));
        }

        listBody.append(listPubisher);
        back.append(listBody);

        var desc = $("<div/>", { "class": "desc" });
        if (isRtl) {
            desc.attr("dir", "rtl");
        }

        if (settings.ShowOptions) {
            var optionsClone = options.clone(true, true);
            optionsClone.addClass("nextLine");
            $("button:first-child", optionsClone).remove();
            desc.append(optionsClone);
        }

        var image = "";
        if (article.ImageUrl) {
            image = article.ImageUrl;
        }
        else if (article.Thumbnail) {
            image = article.Thumbnail;
        }
        else if (article.Author != null && article.Author.ProfilePictureUrl != null) {
            image = article.Author.ProfilePictureUrl;
        }
        else if (article.SourceFeed().Icon) {
            image = article.SourceFeed().Icon;
        }

        var thumbnail_noContent = $("<div/>", {
            "class": "compact-layout-thumbnail grayFilter" + (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass ? " compact-layout-social-thumbnail" : ""),
            "style": "background:url(\"" + image + "\"),url(\"/Media/Images/Backgrounds/gray-background.jpg\");background-position:center;background-size:cover;"
        })
        desc.append(thumbnail_noContent);

        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            desc.append("<span class=\"compact-layout-title grayText\">" + NewsTab.Modules.SectionsManager.CurrentSection.Name + "</span>");
        } else {
            desc.append("<span class=\"compact-layout-title grayText\">" + title + "</span>");
        }

        if (summary) {
            desc.append("<span class=\"compact-layout-details grayText\">" + scEmoji(summary.replace(/<(?!\s*\/?(i|b|strong)\b)[^>]+(>|$)/ig, "")) + "</span>");
        }

        back.append(desc);

        back.addClass("fadeInAnimation");
        placeholder.append(back);

        self.Set_DefaultBackgroundImage(placeholder, article, settings);
    },

    SetUp_ListPreviewItem: function (feed, article, settings) {
        var self = this;

        var title = self._getTitle(article);
        var summary = self._getSummary(article);

        var isRtl = NewsTab.Helpers.ShouldBeRTL(title);

        var placeholder = self._$initialSetUp_PreviewItem(feed, article, false, settings);

        var back = $("<figure/>", { "class": "back" });

        var thumbnail = $("<div/>", {
            "class": "list-layout-thumbnail grayFilter" + (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass ? " list-layout-social-thumbnail" : ""),
            "style": "background-image:none;background-color:" + NewsTab.Modules.SectionsManager.GetRandomColorForCurrentSection()
        })
        back.append(thumbnail);

        var image = "";
        if (article.ImageUrl) {
            image = article.ImageUrl;
        }
        else if (article.Thumbnail) {
            image = article.Thumbnail;
        }
        else if (article.Author != null && article.Author.ProfilePictureUrl != null) {
            image = article.Author.ProfilePictureUrl;
        }
        else if (article.SourceFeed().Icon) {
            image = article.SourceFeed().Icon;
        }

        var thumbnail_noContent = $("<div/>", {
            "class": "list-layout-thumbnail list-layout-thumbnail-noContent grayFilter",
            "style": "background:url(\"" + image + "\"),url(\"/Media/Images/Backgrounds/gray-background.jpg\");background-position:center;background-size:cover;"
        })
        back.append(thumbnail_noContent);


        var listBody = $("<div/>", { "class": "list-layout-body" });

        var listInfo = [];
        listInfo.push("<div class=\"list-layout-info\" " + (isRtl ? "dir=\"rtl\"" : "") + ">");
        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            listInfo.push("<span class=\"list-layout-title grayText\">" + (article.Author.FullName ? article.Author.FullName : (article.Author.Username ? article.Author.Username : article.SourceFeed().Name)) + "</span>");
        } else {
            listInfo.push("<span class=\"list-layout-title grayText\">" + title + "</span>");
        }

        if (summary) {
            listInfo.push("<span class=\"list-layout-shortdesc grayText\">" + scEmoji(summary.replace(/<\/?[^>]+(>|$)/g, "")) + "</span>");
        } else {
            listInfo.push("<span class=\"list-layout-shortdesc grayText\">\u00a0</span>");
        }
        listInfo.push("</div>");
        listBody.append(listInfo.join(""));

        var listPubisher = $("<div/>", { "class": "list-layout-pubisher" });
        if (article.SourceFeed().Type === Feed.TYPES.TWITTER) {
            if (article.Author && article.Author.ProfilePictureUrl) {
                listPubisher.append("<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"img-circle profile-picture grayFilter\"></div>");
            }
            listPubisher.append("<span class=\"fullname grayText\">@" + (article.Author.Username ? article.Author.Username : (article.Author.FullName ? article.Author.FullName : article.SourceFeed().Name)) + "</span>");
        } else {
            if (article.FavIcon) {
                listPubisher.append("<img src=\"" + article.FavIcon + "\" class=\"profile-picture grayFilter\" onError=\"this.onerror=null;this.src='" + self._defaultFeedIcon + "';\" />");
            }
            else if (article.SourceFeed() && article.SourceFeed().Icon != null && article.SourceFeed().Icon.indexOf(".") > 0) {
                listPubisher.append("<img src=\"" + article.SourceFeed().Icon + "\" class=\"profile-picture grayFilter\" onError=\"this.onerror=null;this.src='" + self._defaultFeedIcon + "';\" />");
            } else {
                listPubisher.append("<img src=\"" + self._defaultFeedIcon + "\" class=\"profile-picture grayFilter\" onError=\"this.onerror=null;this.remove();\"/>");
            }
            listPubisher.append("<span class=\"siteUrl grayText\">" + ((article.SourceFeed().Type.CssClass === "rss") ? (article.Author.Username ? article.Author.Username : (article.SourceFeed().SiteUrl ? article.SourceFeed().SiteUrl : article.SourceFeed().Name)) : article.Author.FullName) + "</span>");
        }
        listPubisher.append(self.$Indicator_Time(article, settings));
        if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type.Id === Feed.TYPES.SOCIALTRENDING.Id) {
            listPubisher.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
        }
        else {
            listPubisher.append(self.$Indicator_Trending(article, settings));
        }

        listBody.append(listPubisher);
        back.append(listBody);


        var listPubisherClone = listPubisher.clone(true, true);
        listPubisherClone.addClass("nextLine");
        back.append(listPubisherClone);

        // Options
        if (settings.ShowOptions) {
            var options = $("<div/>", { "class": "options" });

            options.append(self.$Generate_MarkAsReadButton(article, { tooltipPosition: "top" }));
            options.append(self.$Generate_PinButton(article, { tooltipPosition: "top" }));
            options.append(self.$Generate_ShareButton(article, { tooltipPosition: "top" }));
            options.append(self.$Generate_ViewOrginalButton(article, { tooltipPosition: "top" }));

            if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                options.append(self.$Generate_RetweetButton(article, { tooltipPosition: "top" }));
                options.append(self.$Generate_LikeButton(article, { tooltipPosition: "top" }));
            }

            if (NewsTab.User.HasAccessToAdvancedPinned) {
                options.append(self.$Generate_AdvancedPinSettingsButton(article, { tooltipPosition: "top" }));
            }

            options.append(self.$Generate_ArrowButton({ tooltipPosition: "top" }));
            back.append(options);
        }

        var desc = $("<div/>", { "class": "desc" });
        if (isRtl) {
            desc.attr("dir", "rtl");
        }
        if (settings.ShowOptions) {
            var optionsClone = options.clone(true, true);
            optionsClone.addClass("nextLine");
            $("button:last-child", optionsClone).remove();
            desc.append(optionsClone);
        }
        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            desc.append("<span class=\"list-layout-title grayText\">" + NewsTab.Modules.SectionsManager.CurrentSection.Name + "</span>");
        } else {
            desc.append("<span class=\"list-layout-title grayText\">" + title + "</span>");
        }

        if (summary) {
            desc.append("<span class=\"list-layout-details grayText\">" + scEmoji(summary.replace(/<(?!\s*\/?(i|b|strong)\b)[^>]+(>|$)/ig, "")) + "</span>");
        }
        back.append(desc);

        back.addClass("fadeInAnimation");
        placeholder.append(back);

        self.Set_DefaultBackgroundImage(placeholder, article, settings);
    },

    SetUp_GridPreviewItem: function (feed, article, settings) {
        var self = this;

        var title = self._getTitle(article);
        var summary = self._getSummary(article);

        var isRtl = NewsTab.Helpers.ShouldBeRTL(title);

        var placeholder = self._$initialSetUp_PreviewItem(feed, article, isRtl, settings);

        var back = $("<figure/>", { "class": "back" });

        // OPTIONS
        if (settings.ShowOptions) {
            var options = $("<div/>", { "class": "options" });

            options.append(self.$Generate_MarkAsReadButton(article, { tooltipPosition: "bottom" }));
            options.append(self.$Generate_PinButton(article, { tooltipPosition: "bottom" }));
            options.append(self.$Generate_ShareButton(article, { tooltipPosition: "bottom" }));
            options.append(self.$Generate_ViewOrginalButton(article, { tooltipPosition: "bottom" }));

            if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                options.append(self.$Generate_RetweetButton(article, { tooltipPosition: "bottom" }));
                options.append(self.$Generate_LikeButton(article, { tooltipPosition: "bottom" }));
            }

            if (NewsTab.User.HasAccessToAdvancedPinned && NewsTab.Modules.SectionsManager.CurrentSection.SectionTypeId === Section.TYPES.Pinned) {
                options.append(self.$Generate_AdvancedPinSettingsButton(article, { tooltipPosition: "bottom" }));
            }

            back.append(options);
        }
        // END OPTIONS

        // SHORT DESC
        var shortDesc = $("<div/>", { "class": "shortdesc grayFilter" });

        var imgUrlToUse = self._getImageUrl(article);

        if (imgUrlToUse) {
            var smallArticleImage = $("<div/>", {
                "class": "smallArticleImage",
                "style": "background-image:url('" + imgUrlToUse + "'),url('" + self._defaultFeedIcon + "')"
            });

            if (article.VideoUrl) {
                if (article.VideoUrl.indexOf(".mp3") > 0) {
                    smallArticleImage.html("<img src=\"" + self._mp3IconSmall + "\" class=\"videoIcon\"/>");
                }
                else {
                    smallArticleImage.html("<img src=\"" + self._videoIconSmall + "\" class=\"videoIcon\"/>");
                }
            }
            shortDesc.append(smallArticleImage);
        }
        else if (article.Author != null && article.Author.ProfilePictureUrl != null) {
            var smallArticleImage = $("<div/>", {
                "class": "smallArticleImage"
            });
            if (article.VideoUrl) {
                if (article.VideoUrl.indexOf(".mp3") > 0) {
                    smallArticleImage.html("<img src=\"" + self._mp3IconSmall + "\" class=\"videoIcon\"/>");
                }
                else {
                    smallArticleImage.html("<img src=\"" + self._videoIconSmall + "\" class=\"videoIcon\"/>");
                }
            }
            smallArticleImage.append("<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"profile-picture\"></div>")
            shortDesc.append(smallArticleImage);
        }
        else {
            if (article.VideoUrl) {
                var smallArticleImage = $("<div/>", {
                    "class": "smallArticleImage"
                });
                if (article.VideoUrl) {
                    if (article.VideoUrl.indexOf(".mp3") > 0) {
                        smallArticleImage.html("<img src=\"" + self._mp3IconSmall + "\" class=\"videoIcon\"/>");
                    }
                    else {
                        smallArticleImage.html("<img src=\"" + self._videoIconSmall + "\" class=\"videoIcon\"/>");
                    }
                }
                shortDesc.append(smallArticleImage);
            }
        }

        shortDesc.append("<p class=\"shortDescTitle grayText\">" + title + "</p>");
        // END SHORT DESC

        // IMAGE CONTENT
        back.append("<div class=\"thumbnail fadeInAnimation0 grayFilter\" style=\"background-image:none; background-color:" + NewsTab.Modules.SectionsManager.GetRandomColorForCurrentSection() + ";\"></div>");
        // END IMAGE CONTENT

        // UPPER BAR
        var upperBar = $("<div/>", { "class": "upperbar" });
        if (article.SourceFeed().Type.CssClass === "rss") {
            var iconSources = [];

            if (article.FavIcon) { iconSources.push(article.FavIcon); }
            if (article.SourceFeed().Icon) { iconSources.push(article.SourceFeed().Icon); }
            iconSources.push(self._defaultFeedIcon);

            if (iconSources.length > 0) {
                var icon = $("<img/>", {
                    "class": "grayFilter",
                    "data-src-count": 1
                });

                icon.on("error", function () {
                    var counter = $(this).attr("data-src-count");
                    var src = $(this).attr("data-src-" + counter);
                    if (src.length) {
                        counter++;
                        $(this).attr("data-src-count", counter);
                        $(this).attr("src", src);
                    }
                    else {
                        $(this).remove();
                    }
                });

                for (var i = iconSources.length - 1; i >= 0; i--) {
                    if (i === 0) {
                        icon.attr("src", iconSources[i]);
                    }
                    else {
                        icon.attr("data-src-" + i, iconSources[i]);
                    }
                }

                upperBar.append(icon);
            }
        }
        else if (article.Author && article.Author.ProfilePictureUrl) {
            upperBar.append("<div class=\"img-circle profile-picture grayFilter\" style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\"></div>");
        }
        if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
            var indicators = $("<div/>", { "class": "indicators grayText" });
            indicators.append(self.$Indicator_Likes(article, settings));
            indicators.append("\u00a0");
            indicators.append(self.$Indicator_Retweets(article, settings));
            upperBar.append(indicators);
        }
        var userInfo = $("<div/>", { "class": "userInfo" });

        userInfo.append("<p class=\"fullname grayText grayFilter\">" + self._getSourceName(article) + "</p>");

        var userIndicators = $("<div/>", { "class": "userIndicators grayText" }).html("\u00a0/\u00a0");
        userIndicators.append(self.$Indicator_Time(article, settings));

        if (NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.SOCIALTRENDING) {
            userIndicators.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
        }
        else {
            userIndicators.append(self.$Indicator_Trending(article, settings));
        }

        userIndicators.append("\u00a0<span class=\"icon-markasread\"></span>\u00a0<span class=\"icon-pin\"></span>");
        userInfo.append(userIndicators);

        upperBar.append(userInfo);
        // END UPPER BAR

        // INFO
        var info = $("<div/>", { "class": "info" });

        var relatedArticleLink = null;

        var d = summary !== undefined ? summary.replace(/<(?!\s*\/?(i|b|strong)\b)[^>]+(>|$)/ig, "") : "";

        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            d = scEmoji(d);
        }

        if (d === undefined) {
            d = "";
        }
        var desc = "<p class=\"desc grayText\">" + d + "</p>"; // CLEANUP

        if (article.SourceFeed().Type.CssClass === "rss") {
            info.append(shortDesc);
            info.append(self.$Generate_MobileHoverButton());
            info.append(upperBar);
            info.append(desc);
        } else {
            info.append(upperBar);
            info.append(shortDesc);
        }

        back.append(info);
        // END INFO

        // HOVER INFO
        var hoverInfo = $("<div/>", { "class": "hoverInfo" });

        if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
            hoverInfo.append(desc);
            var userInfo = "";
            if (article.Author.ProfilePictureUrl) {
                hoverInfo.append("<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"img-circle profile-picture grayFilter\" ></div><br />");
            }
            hoverInfo.append("<p class=\"fullname grayText\">" + article.Author.FullName + "</p>");

            var usernameBox = $("<p/>", { "class": "usernameBox" });
            usernameBox.append("<a class=\"username\" href=\"" + article.SourceFeed().UserTagLinkBase + article.Author.Username + "\" target=\"_blank\">@" + article.Author.Username + "</a> ");
            usernameBox.append(self.$Indicator_TwitterAction(article, settings));
            usernameBox.append("\u00a0/\u00a0");
            usernameBox.append(self.$Indicator_Time(article, settings));

            hoverInfo.append(usernameBox);
            hoverInfo.append(self.$Indicator_Loaction(article, settings));
        } else {
            var topInfo = $("<div/>", { "class": "topInfo" });
            topInfo.append(shortDesc.clone(true, true));
            topInfo.append(upperBar.clone(true, true));
            hoverInfo.append(topInfo);
            hoverInfo.append(desc);

            if (relatedArticleLink != null) {
                var ral = $(relatedArticleLink);
                ral.addClass("relatedArticle");
                hoverInfo.append(ral);
            }

            hoverInfo.append(self.$Generate_MobileHoverButton());
        }
        back.append(hoverInfo);
        $(".flipItem", placeholder).append(back);
        // END HOVER INFO

        self.Set_DefaultBackgroundImage(placeholder, article, settings);
    },

    SetUp_CascadingGridPreviewItem: function (feed, article, settings) {
        var self = this;

        if (NewsTab.Modules.SectionsManager.CurrentSection != null) {
            var showExpandBtn = true;

            var title = self._getTitle(article);
            var summary = self._getSummary(article);

            var isRtl = NewsTab.Helpers.ShouldBeRTL(title);

            var placeholder = self._$initialSetUp_PreviewItem(feed, article, isRtl, settings);

            var back = $(".back", placeholder);

            // SHORT DESC
            var shortDesc = $("<div/>", { "class": "shortdesc" });

            var imgUrlToUse = self._getImageUrl(article);

            if (imgUrlToUse) {
                var smallArticleImage = $("<div/>", {
                    "class": "smallArticleImage",
                    "style": "background-image:url('" + imgUrlToUse + "'),url('" + self._defaultFeedIcon + "')"
                });

                if (article.VideoUrl) {
                    if (article.VideoUrl.indexOf(".mp3") > 0) {
                        smallArticleImage.html("<img src=\"" + self._mp3Icon + "\" class=\"videoIcon\"/>");
                    }
                    else {
                        smallArticleImage.html("<img src=\"" + self._videoIcon + "\" class=\"videoIcon\"/>");
                    }
                }
                shortDesc.append(smallArticleImage);
            }
            else if (article.Author != null && article.Author.ProfilePictureUrl != null) {
                var smallArticleImage = $("<div/>", {
                    "class": "smallArticleImage"
                });
                if (article.VideoUrl) {
                    if (article.VideoUrl.indexOf(".mp3") > 0) {
                        smallArticleImage.html("<img src=\"" + self._mp3Icon + "\" class=\"videoIcon\"/>");
                    }
                    else {
                        smallArticleImage.html("<img src=\"" + self._videoIcon + "\" class=\"videoIcon\"/>");
                    }
                }
                smallArticleImage.append("<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"profile-picture\"></div>")
                shortDesc.append(smallArticleImage);
            }
            else {
                if (article.VideoUrl) {
                    var smallArticleImage = $("<div/>", {
                        "class": "smallArticleImage"
                    });
                    if (article.VideoUrl) {
                        if (article.VideoUrl.indexOf(".mp3") > 0) {
                            smallArticleImage.html("<img src=\"" + self._mp3Icon + "\" class=\"videoIcon\"/>");
                        }
                        else {
                            smallArticleImage.html("<img src=\"" + self._videoIcon + "\" class=\"videoIcon\"/>");
                        }
                    }
                    shortDesc.append(smallArticleImage);
                }
            }

            shortDesc.append("<p class=\"shortDescTitle grayText\">" + title + "</p>");
            // END SHORT DESC

            // UPPER BAR
            var upperBar = $("<div/>", { "class": "upperbar" });
            var sf = article.SourceFeed();
            if (article.SourceFeed().Type.CssClass === "rss") {
                var iconSources = [];

                if (article.FavIcon) { iconSources.push(article.FavIcon); }
                if (article.SourceFeed().Icon) { iconSources.push(article.SourceFeed().Icon); }
                iconSources.push(self._defaultFeedIcon);
                iconSources.push("https://s2.googleusercontent.com/s2/favicons?domain_url=" + encodeURIComponent(article.Url.replace("https:", "http:")) + "&alt=s&sz=16");

                if (iconSources.length > 0) {
                    var icon = $("<img/>", {
                        "class": "grayFilter",
                        "data-src-count": 1
                    });

                    icon.on("error", function () {
                        var counter = $(this).attr("data-src-count");
                        var src = $(this).attr("data-src-" + counter);
                        if (src.length) {
                            counter++;
                            $(this).attr("data-src-count", counter);
                            $(this).attr("src", src);
                        }
                        else {
                            $(this).remove();
                        }
                    });

                    for (var i = iconSources.length - 1; i >= 0; i--) {
                        if (i === 0) {
                            icon.attr("src", iconSources[i]);
                        }
                        else {
                            icon.attr("data-src-" + i, iconSources[i]);
                        }
                    }

                    upperBar.append(icon);
                }
            }
            else if (article.Author && article.Author.ProfilePictureUrl) {
                upperBar.append('<div class="img-circle profile-picture grayFilter" style="background-image:url(\'' + article.Author.ProfilePictureUrl + '\')"></div>');
            }


            upperBar.append("<p class=\"fullname grayText grayFilter\">" + self._getSourceName(article) + "</p>");

            upperBar.append("\u00a0/\u00a0");
            upperBar.append(self.$Indicator_Time(article, settings));
            upperBar.append("\u00a0");
            if (NewsTab.Modules.SectionsManager.CurrentSection !== null && NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.SOCIALTRENDING) {
                upperBar.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
            }
            else {
                upperBar.append(self.$Indicator_Trending(article, settings));
            }

            upperBar.append("\u00a0<span class=\"icon-markasread\"></span>\u00a0<span class=\"icon-pin\"></span>");

            if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                var indicators = $("<div/>", { "class": "indicators grayText" });
                indicators.append(self.$Indicator_Likes(article, settings));
                indicators.append("\u00a0");
                indicators.append(self.$Indicator_Retweets(article, settings));
                upperBar.append(indicators);
            }
            // END UPPER BAR

            // IMAGE CONTENT
            var $thumbnail = $("<div style=\"overflow: hidden;\"><div class=\"thumbnail\" style=\"width: 100%; max-height: " + (placeholder.width()) + "px; height: " + (placeholder.width() - 100) + "px; background-image: url('" + NewsTab.PlaceholderImage + "')\"></div></div>");
            // END IMAGE CONTENT

            // INFO
            var info = $("<div/>", { "class": "info" });

            //var d = summary !== undefined ? summary.replace(/<\/?[^>]+(>|$)/g, "") : "";
            var d = summary !== undefined ? summary.replace(/<(?!\s*\/?(i|b|strong)\b)[^>]+(>|$)/ig, "") : "";

            if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
                d = scEmoji(d);
            }

            if (d === undefined || d.length === 0) {
                d = "";
                showExpandBtn = false;
            }
            else {
                var desc = "<div class=\"desc grayText\">" + d + "</div>";
            }

            if (article.SourceFeed().Type.CssClass === "rss") {
                info.append(shortDesc);
                info.append(upperBar);
                info.append($thumbnail);
                info.append(desc);
            } else {
                info.append(upperBar);
                info.append(shortDesc);
                info.append($thumbnail);
            }

            // OPTIONS
            if (settings.ShowOptions) {
                var options = $("<div/>", { "class": "options" });

                options.append(self.$Generate_MarkAsReadButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                options.append(self.$Generate_PinButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                options.append(self.$Generate_ShareButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                options.append(self.$Generate_ViewOrginalButton(article, { cssClass: "float-left", tooltipPosition: "top" }));

                if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
                    options.append(self.$Generate_RetweetButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                    options.append(self.$Generate_LikeButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                }

                if (NewsTab.User.HasAccessToAdvancedPinned && NewsTab.Modules.SectionsManager.CurrentSection !== null && NewsTab.Modules.SectionsManager.CurrentSection.SectionTypeId === Section.TYPES.Pinned) {
                    options.append(self.$Generate_AdvancedPinSettingsButton(article, { cssClass: "float-left", tooltipPosition: "top" }));
                }

                if (showExpandBtn) {
                    options.append(self.$Generate_ExtensionExpandButton({ cssClass: "arrow floatRight", tooltipPosition: "top" }));
                }

                info.append(options);
            }
            // END OPTIONS
            // END INFO
            back.addClass("fadeInAnimation");
            back.append(info);

            self.Set_DefaultBackgroundImage(placeholder, article, settings);
        }
    },


    UpdatePreviewItemsSize: function () {
        var self = this;
        $(".socialTabItem", self.DEFAULT_SETTINGS.ContainerSelector).attr("style", "width: calc(" + (100 / self.DEFAULT_SETTINGS.ActiveBoxSize()) + "% - 26px) !important;");
        $(".socialTabItem .back .thumbnail", self.DEFAULT_SETTINGS.ContainerSelector).css({
            "max-height": $(".socialTabItem", self.DEFAULT_SETTINGS.ContainerSelector).width() + "px",
            "height": ($(".socialTabItem", self.DEFAULT_SETTINGS.ContainerSelector).width() - 100) + "px"
        });
    },

    // FULL IMAGE LOAD
    Set_DefaultBackgroundImage: function (placeholder, article, settings) {
        var self = this;

        if (placeholder === undefined || placeholder === null) {
            placeholder = $(".socialTabItem[data-article-id='" + article.Id + "']", settings.ContainerSelector);
        }

        // Add expand icon if requied (not added prior full html parsing)
        //if (placeholder.parent().hasClass("layout-cascading-grid")) {
        //    var showExpandBtn = true;
        //    var summary = self._getSummary(article);
        //    var d = summary !== undefined ? summary.replace(/<(?!\s*\/?(i|b|strong)\b)[^>]+(>|$)/ig, "") : "";

        //    if (d === undefined || d.length === 0) {
        //        showExpandBtn = false;
        //    }
        //    else {
        //        if ($(".info .desc", placeholder).length === 0) {
        //            var desc = "<div class=\"desc grayText\">" + d + "</div>";
        //            var $options = $(".options", placeholder);
        //            $options.before(desc);
        //        }
        //    }

        //    if (settings.ShowOptions && showExpandBtn) {
        //        var $options = $(".options", placeholder);
        //        if ($(".arrow", $options).length === 0) {
        //            $options.append(self.$Generate_ExtensionExpandButton({ cssClass: "arrow floatRight", tooltipPosition: "top" }));
        //        }
        //    }
        //}

        var mp3Icon = self._mp3Icon;
        var videoIcon = self._videoIcon;

        var $thumbnail;
        if (placeholder.parent().hasClass("layout-list")) {
            $thumbnail = placeholder.find(".list-layout-thumbnail");
            mp3Icon = self._mp3IconSmall;
            videoIcon = self._videoIconSmall;
        } else if (placeholder.parent().hasClass("layout-compact")) {
            $thumbnail = placeholder.find(".compact-layout-thumbnail");
            mp3Icon = self._mp3IconSmall;
            videoIcon = self._videoIconSmall;
        } else {
            $thumbnail = placeholder.find(".thumbnail");
        }

        if (article.VideoUrl && $thumbnail.find(".videoIcon").length === 0) {
            if (article.VideoUrl.indexOf(".mp3") > 0) {
                $thumbnail.append("<img src=\"" + mp3Icon + "\" class=\"videoIcon\"/>");
            }
            else {
                $thumbnail.append("<img src=\"" + videoIcon + "\" class=\"videoIcon\"/>");
            }
        }

        if (settings.FlipAutomaticlyOnImageLoad) {
            placeholder.find(".flipItem").addClass("flipped");
            placeholder.addClass("flippedItem");
        }


        if (!article.FullHtmlParsed && settings.FullHtmlParse === true) {
            self.API_ParseUnparsedArticle(article, settings);
        } else {
            if (article.ImageUrl) {
                var widthToCompare = placeholder.width() * 0.6;
                if (placeholder.parent().hasClass("layout-list")) {
                    widthToCompare = 120;
                } else if (placeholder.parent().hasClass("layout-compact")) {
                    widthToCompare = 120;
                }


                var tempImage = $("<img/>");
                tempImage.on("error", function () {

                    if (article.Thumbnail) {
                        var smallArticleImage = $(".smallArticleImage", placeholder);
                        smallArticleImage.attr("style", "background-image:url('" + article.Thumbnail + "'),url('" + self._defaultFeedIcon + "')");
                    }
                    self.Set_ThumbnailAsBackgroundImage(placeholder, article);
                    $(this).remove();
                });
                tempImage.on("load", function () {
                    if (tempImage[0].width >= widthToCompare || tempImage[0].naturalWidth >= widthToCompare) {
                        var background = "url(\"" + $(this).attr("src") + "\")";

                        $thumbnail.css("background-image", background);
                        $thumbnail.css("background-color", self._flipColorLight);
                        if (!NewsTab.IsExtension) {
                            $thumbnail.addClass("fadeInAnimation");
                        }
                        placeholder.removeClass("noContent");
                        if (NewsTab.IsExtension) {
                            NewsTab.Extension.UpdateArticleGridWidth();
                        }
                    } else {
                        self.Set_ThumbnailAsBackgroundImage(placeholder, article);
                    }

                    $(this).remove();
                });
                tempImage.attr("src", NewsTab.Helpers.HtmlEnDeCode.htmlDecode(article.ImageUrl));
            } else {
                self.Set_ThumbnailAsBackgroundImage(placeholder, article);
            }
        }
    },

    Set_ThumbnailAsBackgroundImage: function (placeholder, article) {
        var self = this;
        if (article.Thumbnail) {
            var $thumbnail;
            var widthToCompare = placeholder.width() * 0.6;
            if (placeholder.parent().hasClass("layout-list")) {
                $thumbnail = placeholder.find(".list-layout-thumbnail");
                widthToCompare = 120;
            } else if (placeholder.parent().hasClass("layout-compact")) {
                $thumbnail = placeholder.find(".compact-layout-thumbnail");
                widthToCompare = 120;
            } else {
                $thumbnail = placeholder.find(".thumbnail");
            }

            var tempImage = $("<img/>");
            tempImage.on("error", function () {
                self.Set_NoImage(placeholder, article);
                $(this).remove();
            });
            tempImage.on("load", function () {
                if (tempImage[0].width >= widthToCompare || tempImage[0].naturalWidth >= widthToCompare) {
                    var background = "url(\"" + $(this).attr("src") + "\")";
                    $thumbnail.css("background-image", background);
                    $thumbnail.css("background-color", self._flipColorLight);
                    if (!NewsTab.IsExtension) {
                        $thumbnail.addClass("fadeInAnimation");
                    }
                    placeholder.removeClass("noContent");
                    if (NewsTab.IsExtension) {
                        NewsTab.Extension.UpdateArticleGridWidth();
                    }
                } else {
                    self.Set_NoImage(placeholder, article);
                }
                $(this).remove();
            });
            tempImage.attr("src", NewsTab.Helpers.HtmlEnDeCode.htmlDecode(article.Thumbnail));
        }
        else {
            self.Set_NoImage(placeholder, article);
        }
    },

    Set_NoImage: function (placeholder, article) {
        var self = this;
        placeholder.addClass("noContent");

        if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id && article.Author.ProfilePictureUrl) {
            var t;
            if (placeholder.parent().hasClass("layout-list")) {
                t = placeholder.find(".list-layout-thumbnail");
            } else if (placeholder.parent().hasClass("layout-compact")) {
                t = placeholder.find(".compact-layout-thumbnail");
            } else {
                t = placeholder.find(".thumbnail");
            }
            var result = "<div style=\"background-image:url('" + article.Author.ProfilePictureUrl + "')\" class=\"img-circle profile-picture\"></div>";
            t.prepend(result);
        }
        else {
            if (article.VideoUrl) {
                var $smallArticleImage = placeholder.find(".smallArticleImage");
                if ($smallArticleImage.length === 0) {
                    $smallArticleImage = $("<div/>", {
                        "class": "smallArticleImage",
                        "style": "background-image:url('" + self._defaultFeedIcon + "');width: 100px;height: 100px;"
                    });
                    $(".info .shortdesc", placeholder).prepend($smallArticleImage);
                }

                if ($(".videoIcon", $smallArticleImage).length === 0) {
                    if (article.VideoUrl.indexOf(".mp3") > 0) {
                        $smallArticleImage.append("<img src=\"" + self._mp3IconSmall + "\" class=\"videoIcon\"/>");
                    }
                    else {
                        $smallArticleImage.append("<img src=\"" + self._videoIconSmall + "\" class=\"videoIcon\"/>");
                    }
                }
            }
        }

        if (NewsTab.IsExtension) {
            NewsTab.Extension.UpdateArticleGridWidth();
            setTimeout(function () {
                NewsTab.Extension.UpdateArticleGridWidth();
            }, 300);
        }
    },

    // EXPAND
    SetUp_ExpandViewById: function (id) {
        var article = $.grep(NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Items, function (e) { return e.Id === id.toString() });
        if (article.length > 0) {
            this.SetUp_ExpandView(article[0]);
        }
    },

    SetUp_ExpandView: function (article, instantShow) {
        var self = this;

        NewsTab.Modules.ArticleView.ListedArticles = $(".socialTabItem:not(.empty)");
        if (NewsTab.IsExtension) {
            NewsTab.TrackAnalyticsEvent("Article", "Simplified Article", NewsTab.Extension.NameId);
        }

        var expandSetting = localStorage.getItem("ST_" + NewsTab.User.Id + "_ExpandFormSettings");
        if (expandSetting) {
            self.ExpandFormSettings = JSON.parse(expandSetting);
        }

        $(".socialTabItem.flippedItem").removeClass("selected");

        var obj = $(".socialTabItem[data-article-id=" + article.Id + "]").first();
        obj.addClass("selected");

        var elementPos = NewsTab.Modules.ArticleView.ListedArticles.index(obj);
        var prev = elementPos - 1 >= 0 ? elementPos - 1 : null;
        var next = elementPos + 1 < NewsTab.Modules.ArticleView.ListedArticles.length ? elementPos + 1 : null;
        if (next === NewsTab.Modules.ArticleView.ListedArticles.length - 1 && !NewsTab.IsExtension) {
            SocialTab.LoadMore();
        }
        SocialTab.ExpandedArticle = article;
        self.API_SetAsRead(article, true);

        var screenWidth = 960;

        if (article.SourceFeed().Type === Feed.TYPES.TWITTER) {
            screenWidth = 750;
        }
        if ($(window).width() < screenWidth) {
            screenWidth = $(window).width() * 0.95;
        }

        var viewMode;
        if (NewsTab.Modules.SectionsManager.CurrentSection != null) {
            viewMode = localStorage.getItem("ST_" + NewsTab.User.Id + "_Mode_" + NewsTab.Modules.SectionsManager.CurrentSection.Id + "_" + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Id + article.SourceFeed().Type.Id);
        } else {
            viewMode = localStorage.getItem("ST_" + NewsTab.User.Id + "_Mode_" + article.SourceFeed().Type.Id);
        }

        var $form = $("<div/>", {
            "style": "height:100%"
        });


        // MENU
        var menu = $("<div/>", { "class": "menu" });
        var menu_icon = $("<a/>", {
            "class": "source gAbalyticsReport",
            "href": article.Url,
            "target": "_blank",
            "data-space": "true",
            "data-report-category": "Web View All (Including Preview)",
            "data-report-action": "Simplified Article - Header"
        })

        var iconSources = [];
        if (article.SourceFeed().Type.CssClass === "rss") {
            if (article.FavIcon) {
                iconSources.push(article.FavIcon);
            }
            if (article.SourceFeed().Icon) {
                iconSources.push(article.SourceFeed().Icon);
            }
            iconSources.push(this._defaultFeedIcon);

            if (iconSources.length > 0) {
                var icon = $("<img/>");
                icon.on("error", function () {
                    iconSources.shift();
                    if (iconSources.length > 0) {
                        icon.attr("src", iconSources[0])
                    }
                    else {
                        icon.remove();
                    }
                });
                icon.attr("src", iconSources[0]);
                menu_icon.append(icon);
            }
        }
        else {
            menu_icon.append("<span class=\"icon-" + article.SourceFeed().Type.CssClass + "\"></span>");
        }

        menu_icon.append((article.SourceFeed().Type.CssClass === "rss") ? (article.Author.Username ? article.Author.Username : (article.SourceFeed().SiteUrl ? article.SourceFeed().SiteUrl : article.SourceFeed().Name)) : article.SourceFeed().SiteUrl);
        menu.append(menu_icon);

        if (self.ForceParsButton) {
            menu.append(self.$Generate_ForceParseButton(article, { tooltipPosition: "bottom" }));
        }
        if (NewsTab.Modules.ArticleView.ListedArticles.length > 0) {
            var expandNavigation = $("<div/>", { "class": "expandNavigation" });

            var prevBtn = $("<button/>", {
                "type": "button",
                "class": "icon-angle-left",
                "title": "Previous",
                "data-tooltip": "tooltip",
                "data-placement": "bottom",
                "data-space": "true"
            });
            if (prev != null) {
                prevBtn.click(function (e) {
                    e.stopPropagation();
                    self.SetUp_ExpandViewById(NewsTab.Modules.ArticleView.ListedArticles.eq(prev).data("article-id"));
                });
            }
            else {
                prevBtn.addClass("disabled");
            }
            expandNavigation.append(prevBtn);

            var nextBtn = $("<button/>", {
                "type": "button",
                "class": "icon-angle-right",
                "title": "Next",
                "data-tooltip": "tooltip",
                "data-placement": "bottom",
                "data-space": "true"
            });
            if (next != null) {
                nextBtn.click(function (e) {
                    e.stopPropagation();
                    self.SetUp_ExpandViewById(NewsTab.Modules.ArticleView.ListedArticles.eq(next).data("article-id"));
                });
            }
            else {
                //nextBtn.prop("disabled", true);
                nextBtn.addClass("disabled");
            }
            expandNavigation.append(nextBtn);

            menu.append(expandNavigation);
        }

        if (article.SourceFeed().Type.CssClass === "rss" && (article.Title !== null || article.Summary !== null)) {
            var switchView = $("<div/>", { "class": "switch-view" });

            var simpView = $("<button/>", {
                "type": "button",
                "class": "icon-reader" + (viewMode === "simple" ? " active" : ""),
                "title": "Simplified Article",
                "data-tooltip": "tooltip",
                "data-placement": "bottom",
                "data-space": "true"
            });
            simpView.click(function (e) {
                e.stopPropagation();
                $(this).addClass("active");
                rssView.removeClass("active");
                self.SetUp_ExpandViewMode(article, $form, "simple");
            });
            switchView.append(simpView);

            var rssView = $("<button/>", {
                "type": "button",
                "class": "icon-rss" + (viewMode === "rss" ? " active" : ""),
                "title": "Rss",
                "data-tooltip": "tooltip",
                "data-placement": "bottom",
                "data-space": "true"
            });
            rssView.click(function (e) {
                e.stopPropagation();
                $(this).addClass("active");
                simpView.removeClass("active");
                self.SetUp_ExpandViewMode(article, $form, "rss");
            });
            switchView.append(rssView);

            menu.append(switchView);
        }
        // END MENU

        var innerLeftContainer = $("<div/>", { "class": "innerLeftContainer" });

        var vsBtn = $("<button/>", {
            "type": "button",
            "class": "icon-showsource",
            "title": "View\u00a0original",
            "data-placement": "right",
            "data-tooltip": "tooltip",
            "data-space": "true"
        });
        vsBtn.click(function (e) {
            e.stopPropagation();
            if (NewsTab.IsExtension) {
                NewsTab.TrackAnalyticsEvent("Web View All (Including Preview)", "Simplified Article - View Original (left)", NewsTab.Extension.NameId);
            }
            window.open(article.Url, "_blank");
        });
        innerLeftContainer.append(vsBtn);

        innerLeftContainer.append(self.$Generate_PinButton(article, { tooltipPosition: "right" }));

        var dbBtn = $("<button/>", {
            "id": "displaySettingsBtn",
            "type": "button",
            "title": "Display\u00a0settings",
            "data-module-group": "expandSettings",
            "data-tooltip": "tooltip",
            "data-placement": "right",
            "data-space": "true"
        });
        dbBtn.html("Aa");
        dbBtn.click(function (e) {
            e.stopPropagation();
            if (!$(this).hasClass("active")) {
                var modal = self.$Generate_DisplaySettiongs_Modal($form);
                var offsetTop = dbBtn.offset().top;
                $(this).after(modal);
                if (innerLeftContainer.height() === 40) {
                    var top = dbBtn.position().top;
                    if (offsetTop + modal.outerHeight() > $(window).height()) {
                        top -= ((offsetTop + modal.outerHeight()) - $(window).height());
                    }
                    modal.css("top", top);
                }
                modal.focus();
                modal.focusout(function (e) {
                    var self = $(this);
                    setTimeout(function () {
                        var focus = $(document.activeElement);
                        if (self.has(focus).length === 0 && !self.is(focus)) {
                            self.fadeOut(function () {
                                dbBtn.removeClass("active");
                                self.remove();
                            });
                        }
                    }, 100);

                });
            }
            $(this).toggleClass("active");
        });
        innerLeftContainer.append(dbBtn);

        if (article.SourceFeed().Type.Id === Feed.TYPES.TWITTER.Id) {
            innerLeftContainer.append(self.$Generate_LikeButton(article, { tooltipPosition: "right" }));
            innerLeftContainer.append(self.$Generate_RetweetButton(article, { tooltipPosition: "right" }));
        }

        innerLeftContainer.append("<span class=\"divider\"></span>");
        $form.append(innerLeftContainer);

        // INNERRIGHTCONTAINER
        var innerRightContainer = $("<div/>", { "class": "innerRightContainer " + self._getContentBlockClasses() });
        $form.append(innerRightContainer);
        // END INNERRIGHTCONTAINER

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Id: "socialTabItemForm",
            ShowTitle: true,
            Title: menu,
            Classes: [article.SourceFeed().Type.CssClass + " " + this._getStatusClasses(NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed, article, true)],
            Data: [
                { "Name": "feed-id", "Value": article.SourceFeedId },
                { "Name": "article-id", "Value": article.Id }
            ],
            Height: "100vh",
            Width: "100%",
            ScrollableText: false,
            CloseOnOutsideClick: true,
            MaxWidth: screenWidth,
            Text: $form,
            Buttons: [],
            OnShow: function ($modal) {
                self.SetUp_ExpandViewClasses($form);

                self.Update_InnerLeftContainer(article, innerLeftContainer);

                if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
                    self.SetUp_ExpandViewMode(article, $form, "social");
                    self.API_GetTwitterComments(article, $form);
                }
                else {
                    self.SetUp_ExpandViewMode(article, $form, viewMode);
                }
            }
        });
    },

    SetUp_ExpandViewMode: function (article, $form, viewMode) {
        var self = this;

        var settings = {};

        if (viewMode === null) {
            viewMode = "simple";
            if (article.SourceFeed().Type.CssClass === Feed.TYPES.TWITTER.CssClass) {
                viewMode = "social";
            }
        }

        if (NewsTab.Modules.SectionsManager.CurrentSection != null) {
            localStorage.setItem("ST_" + NewsTab.User.Id + "_Mode_" + NewsTab.Modules.SectionsManager.CurrentSection.Id + "_" + NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Id + article.SourceFeed().Type.Id, viewMode);
        }
        else {
            localStorage.setItem("ST_" + NewsTab.User.Id + "_Mode_" + article.SourceFeed().Type.Id, viewMode);
        }


        if (viewMode === "rss" && article.Title === null && article.Summary === null) {
            viewMode = "simple";
        }

        var title = this._getTitle(article);
        var isRtl = NewsTab.Helpers.ShouldBeRTL(title);
        var showRss = false;

        var $formContent = $("<div/>", { "class": "formContent" });

        if (viewMode === "simple") {
            $(".switch-simple-item", $form).addClass("active");
            $(".switch-rss-item", $form).removeClass("active");

            if (article.Text != null) {
                if (article.ImageUrl || article.VideoUrl) {
                    var view = $("<div class=\"image-back\" " + (isRtl ? "dir=\"rtl\"" : "") + "></div>");
                    view.append(self.$Generate_MediaView(article, $form, viewMode));
                    $formContent.append(view);
                }
                var $details = $("<div/>", { "class": "details-section" });
                if (isRtl) { $details.attr("dir", "rtl"); }

                $details.append($("<a/>", {
                    "class": "title gAbalyticsReport",
                    "href": article.Url,
                    "target": "_blank",
                    "data-report-category": "Web View All (Including Preview)",
                    "data-report-action": "Simplified Article - Title"
                }).html(article.HtmlTitle ? article.HtmlTitle : article.Title));

                if (article.HtmlSummary) {
                    $details.append("<span class=\"summary\">" + article.HtmlSummary + "</span>");
                }

                var $info = $("<span/>", { "class": "article-info" });
                if (article.Author.FullName || article.Author.Username || article.SourceFeed().Name) {
                    $info.append($("<a/>", {
                        "class": "gAbalyticsReport",
                        "href": article.Url,
                        "target": "_blank",
                        "data-report-category": "Web View All (Including Preview)",
                        "data-report-action": "Simplified Article - Author/Publisher"
                    }).html(article.Author.FullName ? article.Author.FullName : (article.Author.Username ? article.Author.Username : article.SourceFeed().Name)));
                    $info.append("\u00a0/\u00a0");
                }
                $info.append(self.$Indicator_Time(article, settings));
                $info.append("\u00a0");
                if (NewsTab.Modules.SectionsManager.CurrentSection != null &&
                    NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type !== undefined &&
                    NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type.Id === Feed.TYPES.SOCIALTRENDING.Id) {
                    $info.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
                }
                else {
                    $info.append(self.$Indicator_Trending(article, settings));
                }
                $details.append($info);

                if (article.Text !== undefined) {
                    $details.append("<div class=\"desc\">" + article.Text + "</div>");
                }
                $details.append($("<a/>", {
                    "class": "icon-showsource gAbalyticsReport",
                    "href": article.Url,
                    "target": "_blank",
                    "data-report-category": "Web View All (Including Preview)",
                    "data-report-action": "Simplified Article - View Original (bottom)"
                }).html(" View Original"));

                $formContent.append($details);
                if (article.Text.indexOf("twitter-tweet") !== -1) {
                    $formContent.append("<script async src=\"https://platform.twitter.com/widgets.js\" charset=\"utf-8\"></script>");
                }
                if (article.Text.indexOf("instagram-media") !== -1) {
                    $formContent.append("<script async defer src=\"https://platform.instagram.com/en_US/embeds.js\"></script>");
                }
            }
            else {
                showRss = true;
                $formContent.append("<p class=\"noSimplifiedMsg\">Simplified Article is not available for this story: <a class=\"gAbalyticsReport\"  data-report-category=\"Web View All (Including Preview)\" data-report-action=\"Simplified Article - No Content Label\" href=\"" + article.Url + "\" target=\"_blank\">View Original</a></p>");
            }
        }
        else if (viewMode === "social") {
            if (article.Thumbnail || article.VideoUrl || article.ImageUrl) {
                var view = $("<div class=\"image-back\" " + (isRtl ? "dir=\"rtl\"" : "") + "></div>");
                view.append(self.$Generate_MediaView(article, $form, viewMode));
                $formContent.append(view);
            }

            var $details = $("<div/>", { "class": "details-section" });
            if (isRtl) { $details.attr("dir", "rtl"); }

            var $contentDetails = $("<div/>", { "class": "content-details" });
            $contentDetails.append("<a href=\"" + article.SourceFeed().UserTagLinkBase + article.Author.Username + "\" class=\"profile-picture\" target=\"_blank\"><img src=\"" + article.Author.ProfilePictureUrl + "\" /></a>");

            var $indicators = $("<div/>", { "class": "indicatorsContainer" });
            $indicators.append("<a href=\"" + article.SourceFeed().UserTagLinkBase + article.Author.Username + "\" class=\"fullname\" target=\"_blank\">" + scEmoji(article.Author.FullName) + "</a>");
            $indicators.append("<a href=\"" + article.SourceFeed().UserTagLinkBase + article.Author.Username + "\" class=\"username\" target=\"_blank\">@" + scEmoji(article.Author.Username) + "</a>");
            $indicators.append("\u00a0/\u00a0");
            $indicators.append(self.$Indicator_Time(article, settings));
            var $location = self.$Indicator_Loaction(article, settings);
            if ($location != null) {
                $indicators.append("\u00a0/\u00a0");
                $indicators.append($location);
            }

            $contentDetails.append($indicators);
            $details.append($contentDetails);

            $details.append("<p class=\"desc\">" + (article.Summary ? scEmoji(article.Summary) : "") + "</p>");
            if (article.FullHtmlParsed && article.ExternalUrl && (article.HtmlTitle || article.Text)) {
                var $articlePreview = $("<div/>", { "class": "article-preview" });

                var $header = $("<div/>", { "class": "a-header" });
                if (article.HtmlTitle) {
                    $header.append("<h2>" + article.HtmlTitle + "</h2>");
                }
                var $switchView = $("<div/>", { "class": "switch-view" });
                $switchView.append("<a href=\"" + article.ExternalUrl + "\" target=\"_blank\" class=\"icon-showsource\" data-tooltip=\"tooltip\" title=\"View\u00a0orginal\u00a0in\u00a0new\u00a0tab\" data-placement=\"top\" data-space=\"true\"></a>");
                $switchView.append($("<button/>", {
                    "type": "button",
                    "class": "icon-reader",
                    "data-tooltip": "tooltip",
                    "title": "View\u00a0in\u00a0reader\u00a0mode",
                    "data-placement": "top",
                    "data-space": "true"
                }).click(function (e) {
                    e.stopPropagation();
                    $(this).toggleClass("active");
                    $articlePreview.toggleClass("expand");
                }));

                $header.append($switchView);

                $articlePreview.append($header);
                if (article.Text) {
                    $articlePreview.append("<div class=\"a-text\">" + article.Text + "</div>");
                }
                $details.append($articlePreview);
            }
            $formContent.append($details);
        }

        if (viewMode === "rss" || showRss) {
            if (!showRss) {
                $(".switch-simple-item").removeClass("active");
                $(".switch-rss-item").addClass("active");
            }
            var view = null;

            if (showRss && (article.ImageUrl || article.VideoUrl)) {
                view = $("<div class=\"image-back\" " + (isRtl ? "dir=\"rtl\"" : "") + "></div>");
                view.append(self.$Generate_MediaView(article, $form, "simple"));
                $formContent.append(view);
            }
            else if ((article.Thumbnail && article.Thumbnail.indexOf("SocialTab_Placeholder.png") < 0) || article.VideoUrl) {
                view = $("<div class=\"image-back\" " + (isRtl ? "dir=\"rtl\"" : "") + "></div>");
                view.append(self.$Generate_MediaView(article, $form, viewMode));
                $formContent.append(view);
            }

            var $details = $("<div/>", { "class": "details-section" });
            if (isRtl) { $details.attr("dir", "rtl"); }

            var title = null;
            if (showRss && article.HtmlTitle) {
                title = article.HtmlTitle;
            }
            else if (article.Title) {
                title = article.Title;
            }

            if (title !== null) {
                $details.append($("<a/>", {
                    "class": "title gAbalyticsReport",
                    "href": article.Url,
                    "target": "_blank",
                    "data-report-category": "Web View All (Including Preview)",
                    "data-report-action": "Simplified Article - Title (rss)"
                }).html(title));
            }

            var $info = $("<span/>", { "class": "article-info" });
            if (article.Author.FullName || article.Author.Username || article.SourceFeed().Name) {
                $info.append($("<a/>", {
                    "class": "gAbalyticsReport",
                    "href": article.Url,
                    "target": "_blank",
                    "data-report-category": "Web View All (Including Preview)",
                    "data-report-action": "Simplified Article - Author/Publisher (rss)"
                }).html(article.Author.FullName ? article.Author.FullName : (article.Author.Username ? article.Author.Username : article.SourceFeed().Name)));
                $info.append("\u00a0/\u00a0");
            }
            $info.append(self.$Indicator_Time(article, settings));
            $info.append("\u00a0");
            if (NewsTab.Modules.SectionsManager.CurrentSection != null &&
                NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type !== undefined &&
                NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type.Id === Feed.TYPES.SOCIALTRENDING.Id) {
                $info.append(self.$Indicator_SocialTrending(article, NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Name, settings));
            }
            else {
                $info.append(self.$Indicator_Trending(article, settings));
            }
            $details.append($info);

            if (article.Summary === undefined) {
                article.Summary = "";
            }
            $details.append("<div class=\"desc\">" + article.Summary + "</div>");

            $details.append($("<a/>", {
                "class": "icon-showsource gAbalyticsReport",
                "href": article.Url,
                "target": "_blank",
                "data-report-category": "Web View All (Including Preview)",
                "data-report-action": "Simplified Article - View Original (bottom - rss)"
            }).html(" View Original"));
            $formContent.append($details);
        }
        $(".desc a", $formContent).attr("target", "_blank");
        $(".innerRightContainer", $form).html("").append($formContent);
    },

    SetUp_ExpandViewClasses: function ($element) {
        var $form = $element.closest("#socialTabItemForm");
        localStorage.setItem("ST_" + NewsTab.User.Id + "_ExpandFormSettings", JSON.stringify(this.ExpandFormSettings));

        if ($form === undefined) {
            $form = $("#socialTabItemForm");
        }

        $form.removeClass($form.attr("data-brightness"));
        $form.attr("data-brightness", "brightness" + this.ExpandFormSettings.Brightness);
        $form.addClass("brightness" + this.ExpandFormSettings.Brightness);

        $form.removeClass($form.attr("data-backgroundColor"));
        $form.attr("data-backgroundColor", "backgroundColor" + this.ExpandFormSettings.Background);
        $form.addClass("backgroundColor" + this.ExpandFormSettings.Background);

        if (this.ExpandFormSettings.Background < 3) {
            $form.addClass("lightForm");
        }
        else {
            $form.removeClass("lightForm");
        }

        $(".innerRightContainer", $form).attr("class", "innerRightContainer " + this._getContentBlockClasses());
    },

    Update_InnerLeftContainer: function (article, $innerLeftContainer) {
        var self = this;

        if ($innerLeftContainer === undefined || $innerLeftContainer === null || $innerLeftContainer.length === 0) {
            $innerLeftContainer = $("#socialTabItemForm .innerLeftContainer");
        }

        var sharesToFit = 0, horizontal = false;

        var divider = $(".divider", $innerLeftContainer);
        divider.nextAll().remove();

        if ($innerLeftContainer.height() === 40) {
            horizontal = true;
            $("[data-placement]", $innerLeftContainer).attr("data-placement", "bottom").attr("data-viewport", "#socialTabItemForm");
            sharesToFit = Math.floor(($innerLeftContainer.width() - (divider.position().left + 40)) / 40);
        }
        else {
            sharesToFit = Math.floor(($innerLeftContainer.height() - (divider.position().top + 30)) / 70);
        }

        for (var i = 0; i < NewsTab.Modules.Sharing.TopShareProviders.length && i < sharesToFit; i++) {
            var cssClass = "";
            if (i === NewsTab.Modules.Sharing.TopShareProviders.length) {
                cssClass = " last";
            }

            $innerLeftContainer.append(NewsTab.Modules.Sharing.$Generate_ShareToXYButton(article, NewsTab.Modules.Sharing.ShareProviders[NewsTab.Modules.Sharing.TopShareProviders[i]], {
                cssClass: cssClass,
                mode: "last",
                tooltipPosition: horizontal ? "bottom" : "right",
                onClick: function (a, p) {
                    self.Update_InnerLeftContainer(a);
                }
            }));
        }
        var $allShares = $("<button/>", {
            "type": "button",
            "class": "icon-more small",
            "data-space": "true"
        });
        $allShares.click(function (e) {
            e.stopPropagation();
            if (!$(this).hasClass("active")) {
                var $modal = NewsTab.Modules.Sharing.$Generate_ShareModal(article, {
                    onShareClick: function (a, p) {
                        self.Update_InnerLeftContainer(a);
                    }
                });
                var offsetTop = $allShares.offset().top;
                $(this).after($modal);
                if (!horizontal) {
                    var top = $allShares.position().top;
                    if (offsetTop + $modal.outerHeight() > $(window).height()) {
                        top -= ((offsetTop + $modal.outerHeight()) - $(window).height());
                    }
                    $modal.css("top", top);
                }
                $modal.focus();
                $modal.focusout(function (e) {
                    var self = $(this);
                    setTimeout(function () {
                        var focus = $(document.activeElement);
                        if (self.has(focus).length === 0 && !self.is(focus)) {
                            self.fadeOut(function () {
                                $allShares.removeClass("active");
                                self.remove();
                            });
                        }
                    }, 100);

                });
            }
            $(this).toggleClass("active");
        });
        $innerLeftContainer.append($allShares);

    },

    $Generate_DisplaySettiongs_Modal: function ($form, options) {
        var self = this;

        var opt = $.extend({}, {
            onShareClick: null,
            onClose: null
        }, options);

        var $displaySettingsModal = $("<div/>", {
            "id": "displaySettingsModal",
            "tabindex": "-1"
        });
        $displaySettingsModal.append("<div>Display settings: </div>");

        var $closeBtn = $("<button/>", {
            "type": "type",
            "class": "icon-close",
            "data-space": "true"
        });
        $closeBtn.click(function (e) {
            if ($displaySettingsModal.hasClass("simplemodal-data")) {
                $.modal.impl.close();
            }
            else {
                $displaySettingsModal.remove();
            }
            if (opt.onClose) {
                opt.onClose();
            }
        });
        $("div", $displaySettingsModal).append($closeBtn);

        var $ul = $("<ul/>");

        // liOne
        var $liOne = $("<li/>");

        // fontSizeSetting
        var $fontSizeSetting = $("<div/>", {
            "class": "fontSizeSetting"
        });
        $fontSizeSetting.append("<span class=\"smallFont\">A</span>");
        $fontSizeSetting.append("<div id=\"expandFontSize\" class=\"range\" tabindex=\"0\"></div>");
        $fontSizeSetting.append("<span class=\"largeFont\">A</span>");

        noUiSlider.create($("#expandFontSize", $fontSizeSetting)[0], {
            start: [self.ExpandFormSettings.FontSize],
            step: 1,
            connect: "lower",
            animationDuration: 300,
            range: {
                'min': [0],
                'max': [4]
            }
        });

        $("#expandFontSize", $fontSizeSetting)[0].noUiSlider.on("change", function (value) {
            self.ExpandFormSettings.FontSize = parseInt(value);
            self.SetUp_ExpandViewClasses($form);
        });

        $liOne.append($fontSizeSetting);
        // END fontSizeSetting

        // fontTypeSetting
        var $fontTypeSetting = $("<div/>", {
            "class": "fontTypeSetting"
        });

        for (var fi = 0; fi < 3; fi++) {
            var $fontBtn = $("<button/>", {
                "type": "button",
                "class": "font" + fi + (self.ExpandFormSettings.Font == fi ? " selected" : ""),
                "data-value": fi,
                "data-space": "true"
            });
            $fontBtn.html(fi === 0 ? "Georgia" : (fi === 1 ? "Open Sans" : "Raleway"));
            $fontBtn.click(function (e) {
                e.stopPropagation();
                $("button", $fontTypeSetting).removeClass("selected");
                $(this).addClass("selected");
                self.ExpandFormSettings.Font = parseInt($(this).attr("data-value"));
                self.SetUp_ExpandViewClasses($form);
            });
            $fontTypeSetting.append($fontBtn);
        }

        $liOne.append($fontTypeSetting);
        // END fontTypeSetting

        // fontLineHeightSetting
        var $fontLineHeightSetting = $("<div/>", {
            "class": "fontLineHeightSetting"
        });
        $fontLineHeightSetting.append("<span class=\"smallFont icon-arrows-v\"></span>");
        $fontLineHeightSetting.append("<div id=\"expandLineHeight\" class=\"range\" tabindex=\"0\"></div>");
        $fontLineHeightSetting.append("<span class=\"largeFont icon-arrows-v\"></span>");

        noUiSlider.create($("#expandLineHeight", $fontLineHeightSetting)[0], {
            start: [self.ExpandFormSettings.LineHeight],
            step: 1,
            connect: "lower",
            animationDuration: 300,
            range: {
                'min': [0],
                'max': [8]
            }
        });
        $("#expandLineHeight", $fontLineHeightSetting)[0].noUiSlider.on("change", function (value) {
            self.ExpandFormSettings.LineHeight = parseInt(value);
            self.SetUp_ExpandViewClasses($form);
        });

        $liOne.append($fontLineHeightSetting);
        // END fontLineHeightSetting
        $ul.append($liOne);
        // END liOne

        // liTwo
        var $liTwo = $("<li/>");

        // backgroundSetting
        var $backgroundSetting = $("<div/>", { "class": "backgroundSetting" });

        for (var bi = 0; bi < 5; bi++) {
            var $backgroundColorBtn = $("<button/>", {
                "type": "button",
                "class": "backgroundColor" + bi + (self.ExpandFormSettings.Background == bi ? " selected" : ""),
                "data-value": bi,
                "data-space": "true"
            });
            $backgroundColorBtn.click(function (e) {
                e.stopPropagation();
                $("button", $backgroundSetting).removeClass("selected");
                $(this).addClass("selected");
                self.ExpandFormSettings.Background = parseInt($(this).attr("data-value"));
                self.SetUp_ExpandViewClasses($form);
            });
            $backgroundSetting.append($backgroundColorBtn);
        }

        $liTwo.append($backgroundSetting);
        // END backgroundSetting

        // brightnessSetting
        var $brightnessSetting = $("<div/>", {
            "class": "brightnessSetting"
        });
        $brightnessSetting.append("<span class=\"smallFont icon-sun-o\"></span>");
        $brightnessSetting.append("<div id=\"expandBrightness\" class=\"range\" tabindex=\"0\"></div>");
        $brightnessSetting.append("<span class=\"largeFont icon-sun-o\"></span>");

        noUiSlider.create($("#expandBrightness", $brightnessSetting)[0], {
            start: [self.ExpandFormSettings.Brightness],
            step: 1,
            connect: "lower",
            animationDuration: 300,
            range: {
                'min': [0],
                'max': [14]
            }
        });
        $("#expandBrightness", $brightnessSetting)[0].noUiSlider.on("change", function (value) {
            self.ExpandFormSettings.Brightness = parseInt(value);
            self.SetUp_ExpandViewClasses($form);
        });
        $liTwo.append($brightnessSetting);
        // END brightnessSetting
        // END backgroundSetting
        $ul.append($liTwo);
        // END liTwo

        $displaySettingsModal.append($ul);
        return $displaySettingsModal;
    },

    $Generate_MediaView: function (article, $form, viewMode) {
        /// <summary>Generates the HTML code for image or video view used in Expand Mode.</summary>  
        /// <param name='article' type='SocialTabItem' >The article for which to generate the image/video view</param>
        /// <param name='type' type='string' >The type ov view you want. It can be "simple","rss" or "social"</param>
        /// <returns name='html' type="jQueryObject">HTML code for the image/video view</returns>  

        var self = this;

        var imgToUse = null;
        switch (viewMode) {
            case "social":
                {
                    imgToUse = article.ImageUrl ? article.ImageUrl : article.Thumbnail;
                }
                break;
            case "rss":
                {
                    imgToUse = article.Thumbnail;
                }
                break;
            case "simple":
            default:
                {
                    imgToUse = article.ImageUrl;
                }
        }

        if (article.VideoUrl && viewMode !== "rss") {
            var $videoPlayers = $("<div/>", { "class": "video-players" });

            if (article.VideoUrl.indexOf("/video.php?v=") >= 0) { // Facebook
                return $videoPlayers.append("<div class=\"video-facebook\"><div class=\"fb-post\" data-href=\"" + article.VideoUrl + "\" data-width=\"100%\" data-height=\"100%\"></div></div>");
            }
            else if ((article.VideoUrl.indexOf(".mp4") > 0 && article.VideoUrl.indexOf(",.mp4") < 0) ||
                article.VideoUrl.indexOf(".ogg") > 0 ||
                article.VideoUrl.indexOf(".webm") > 0) { // .mp4/.ogg/.webm
                $videoPlayers.append("<video controls=\"controls\" poster=\"" + (imgToUse != null ? imgToUse : "") + "\" width=\"100%\" height=\"100%\" class=\"video-thumb\"></video>");
                if (article.VideoUrl.indexOf(".mp4") >= 0) {
                    $("video", $videoPlayers).append("<source src=\"" + article.VideoUrl + "\" type=\"video/mp4\" />");
                } else if (article.VideoUrl.indexOf(".ogg") >= 0) {
                    $("video", $videoPlayers).append("<source src=\"" + article.VideoUrl + "\" type=\"video/ogg\" />");
                } else if (article.VideoUrl.indexOf(".webm") >= 0) {
                    $("video", $videoPlayers).append("<source src=\"" + article.VideoUrl + "\" type=\"video/webm\" />");
                }
                if (imgToUse) {
                    $("video", $videoPlayers).append("<img alt=\"\" src=\"" + imgToUse + "\" title=\"No video playback capabilities, please download the video below\" />");
                }
                return $videoPlayers;
            }
            else if (article.VideoUrl.indexOf(".mp3") > 0) { // .mp3
                return $videoPlayers.append("<audio controls=\"controls\" poster=\"" + (imgToUse != null ? imgToUse : "") + "\" width=\"100%\" height=\"100%\" class=\"video-thumb\"><source src=\"" + article.VideoUrl + "\" /></audio>");
            }
            else { // iframe
                if (article.VideoUrl.indexOf("youtube.") >= 0 && article.VideoUrl.indexOf("/embed/") < 0) {
                    var start = article.VideoUrl.indexOf("?v=") + 3;
                    var yId = article.VideoUrl.substring(start);
                    article.VideoUrl = "https://www.youtube.com/embed/" + yId;
                }
                else if (article.VideoUrl.indexOf("youtu.") >= 0) {
                    var start = article.VideoUrl.lastIndexOf("/") + 1;
                    var yId = article.VideoUrl.substring(start);
                    article.VideoUrl = "https://www.youtube.com/embed/" + yId;
                }

                if (article.VideoUrl.indexOf(".youtube.") >= 0 ||
                    article.VideoUrl.indexOf("vimeo.") >= 0 ||
                    article.VideoUrl.indexOf("facebook.") >= 0 ||
                    article.VideoUrl.indexOf("soundcloud.") >= 0 ||
                    article.VideoUrl.indexOf("dailymotion.") >= 0 ||
                    article.VideoUrl.indexOf("twitter.") >= 0) // ||  article.VideoUrl.indexOf("/embed/") >= 0) 
                { // Play inside IFrame
                    $videoPlayers.addClass("showIFrame");
                    return $videoPlayers.append("<iframe style=\"background-color:none\" width=\"100%\" height=\"100%\" src=\"" + article.VideoUrl + "\" frameborder=\"0\" allowfullscreen></iframe>");
                }
                else { // Show Image with video icon leading to the orginal page
                    if (NewsTab.ApiUrl_Global.indexOf("staging") >= 0 || NewsTab.ApiUrl_Global.indexOf("localhost") >= 0) {
                        $videoPlayers.append($("<button/>", {
                            "type": "button",
                            "class": "tryPlayVideo icon-warning",
                            "title": "Try playing this video inside an iframe",
                            "data-tooltip": "tooltip",
                            "data-placement": "bottom",
                            "data-space": "true",
                            "data-viewport": ".image-back"
                        }).html(" Try Play").click(function (e) {
                            e.stopPropagation();
                            var button = $(this);
                            var iframe = button.parent().find("iframe");

                            button.parent().toggleClass("showIFrame")
                            if (button.parent().hasClass("showIFrame")) {
                                button.removeClass("icon-warning");
                                button.addClass("icon-arrow-left");
                                button.html("");
                                button.attr("data-original-title", "Back");
                                iframe.attr("src", iframe.attr("data-src"));
                            }
                            else {
                                button.addClass("icon-warning");
                                button.removeClass("icon-arrow-left");
                                button.html(" Try Play");
                                button.attr("data-original-title", "Try playing this video inside an iframe");
                                iframe.attr("src", null);
                            }
                        }));
                    }
                    var $iframeVideo = $("<a/>", {
                        "class": "iframeVideoIcon gAbalyticsReport",
                        "href": article.Url,
                        "target": "_blank",
                        "data-report-category": "Web View All (Including Preview)",
                        "data-report-action": "Simplified Article - View Video"
                    }).html("<img src=\"" + self._videoIcon + "\"/>");

                    if (imgToUse) {
                        $iframeVideo.attr("style", "background-image:url('" + imgToUse + "'); background-size:cover;background-position:center;");
                    }
                    $videoPlayers.append($iframeVideo);

                    if (NewsTab.ApiUrl_Global.indexOf("staging") >= 0 || NewsTab.ApiUrl_Global.indexOf("localhost")) {
                        $videoPlayers.append("<iframe style=\"background-color:none\" width=\"100%\" height=\"100%\" data-src=\"" + article.VideoUrl + "\" frameborder=\"0\" allowfullscreen></iframe>");
                    }
                    return $videoPlayers;
                }
            }
        }
        else {
            if (imgToUse) { // Image Only
                var $imgThumb = $("<div/>", { "class": "img-thumb" });

                var $tempImg = $("<img/>");
                $tempImg.on("error", function () {
                    $(this).remove();
                    $imgThumb.remove();
                });
                $tempImg.on("load", function () {
                    $imgThumb.css("background-image", "url(\"" + $tempImg.attr("src") + "\")")
                    var height = ($(".innerRightContainer", $form).height() - 60);
                    var width = ($(".innerRightContainer", $form).width() - 60);
                    var maxheight = height * 0.6;
                    if (this.width >= this.height) {
                        var ration = this.width / this.height;

                        if (this.width * 2.0 > width) {
                            var wr = width / ration;
                            $imgThumb.css("height", wr > maxheight ? maxheight : wr);
                        }
                        else {
                            var wr = (this.width * 2.0) / ration;
                            $imgThumb.css("height", wr > maxheight ? maxheight : wr);
                        }
                    }
                    else {
                        if (this.height * 2.0 > maxheight) {
                            $imgThumb.css("height", maxheight);
                        }
                        else {
                            $imgThumb.css("height", this.height * 2.0);
                        }
                    }

                });
                $tempImg.attr("src", imgToUse);
                return $imgThumb;
            }
        }
        return null;
    },

    // BUTTONS & INDICATORS
    $Generate_LikeButton: function (article, options) {
        if (article.Favorites) {
            var opt = $.extend({}, {
                cssClass: "",
                tooltipPosition: "bottom"
            }, options);

            var $button = $("<button/>", {
                "type": "button",
                "class": "icon-heart " + opt.cssClass,
                "title": "Like",
                "data-tooltip": "tooltip",
                "data-placement": opt.tooltipPosition,
                "data-space": "true",
                "data-likes": article.Favorites.Count
            });
            $button.html("<small>" + NewsTab.Helpers.GetShortCount(article.Favorites.Count) + "</small>");
            $button.on("click", function (e) {
                e.stopPropagation();
                if (NewsTab.Modules.SectionsManager.CurrentSection != null && NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.PINWALL) {
                    //if (article.IdOfOrginalArticle) {
                    //    SocialTab.PostFavorite(article.Id);
                    //}
                    //else {
                    SocialTab.OpenOriginalView(article.Id, article.Url);
                    //}
                }
                else {
                    SocialTab.PostFavorite(article.Id);
                }
            });
            return $button;
        }
        return null;
    },

    $Generate_RetweetButton: function (article, options) {
        if (article.Retweets) {
            var opt = $.extend({}, {
                cssClass: "",
                tooltipPosition: "bottom"
            }, options);

            var button = $("<button/>", {
                "type": "button",
                "class": "icon-retweet " + opt.cssClass,
                "title": "Retweet",
                "data-tooltip": "tooltip",
                "data-placement": opt.tooltipPosition,
                "data-space": "true",
                "data-retweets": article.Retweets.Count
            });
            button.html("<small>" + NewsTab.Helpers.GetShortCount(article.Retweets.Count) + "</small>");
            button.click(function (e) {
                e.stopPropagation();
                SocialTab.OpenOriginalView(article.Id, article.Url);
            });
            return button;
        }
        return null;
    },

    $Generate_MarkAsReadButton: function (article, options) {
        var self = this;
        if (NewsTab.Token !== null) {
            var opt = $.extend({}, {
                cssClass: "",
                tooltipPosition: "bottom"
            }, options);

            var button = $("<button/>", {
                "type": "button",
                "class": "icon-markasread " + opt.cssClass,
                "title": "Mark as read",
                "data-tooltip": "tooltip",
                "data-placement": opt.tooltipPosition,
                "data-space": "true"
            });
            var asread = $.grep(SocialTab.ReadItems, function (e) { return e === article.Alias; });
            if (asread.length) {
                button.attr("title", "Mark as unread");
            }
            button.click(function (e) {
                e.stopPropagation();
                self.API_SetAsRead(article);
            });
            return button;
        }
        return null;
    },

    $Generate_PinButton: function (article, options) {
        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom"
        }, options);

        if (NewsTab.Token) {
            var button = $("<button/>", {
                "type": "button",
                "class": "" + ((NewsTab.Modules.SectionsManager.CurrentSection != null && NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.PINWALL) ? "icon-bin " : "icon-pin ") + opt.cssClass,
                "title": article.Pinned ? (NewsTab.Modules.SectionsManager.CurrentSection != null && NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.PINWALL ? "Delete" : "Unpin") : "Pin",
                "data-tooltip": "tooltip",
                "data-placement": opt.tooltipPosition,
                "data-space": "true",
                "data-public-remove": "true"
            });
            button.click(function (e) {
                e.stopPropagation();

                if (NewsTab.Modules.SectionsManager.CurrentSection != null && NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Type === Feed.TYPES.PINWALL) {
                    SocialTab.Notification_SetUpHTML_OkCancel("Delete Item", "Are you sure you want to permanently delete this item from Pinned?", "Ok", "Cancel", function () {
                        $(".notification-modal").addClass("disabled");
                        $(".notification-modal .ok").html("<span class=\"icon\"><span class=\"spinner\"></span></span>Removing...");
                        return NewsTab.Modules.SpecialPinSystem.API_PinArticle(article, false, true);
                    });
                } else {
                    var item = $(this).closest(".socialTabItem, #socialTabItemForm");
                    var el = $(this);
                    if (item.hasClass("pinned")) {
                        if ($(this).parent().attr("data-original-title") === "Unpin") {
                            el = $(this).parent();
                        }
                        el.attr("data-original-title", "Pin");
                        item.find(".tooltip-inner").text("Pin");
                        NewsTab.Modules.SpecialPinSystem.API_PinArticle(article, false);
                    } else {
                        if ($(this).parent().attr("data-original-title") === "Pin") {
                            el = $(this).parent();
                        }
                        el.attr("data-original-title", "Unpin");
                        item.find(".tooltip-inner").text("Unpin");
                        NewsTab.Modules.SpecialPinSystem.API_PinArticle(article, true);
                    }
                }
            });
            return button;
        }
        return null;
    },

    $Generate_ViewOrginalButton: function (article, options) {
        var self = this;

        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom"
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-showsource " + opt.cssClass,
            "title": "View original",
            "data-tooltip": "tooltip",
            "data-placement": opt.tooltipPosition,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();
            if (NewsTab.IsExtension) {
                NewsTab.TrackAnalyticsEvent("Web View All (Including Preview)", "View Orginal Button on Preview Article", NewsTab.Extension.NameId);
                NewsTab.TrackAnalyticsEvent("Article", "Web View (Original)", NewsTab.Extension.NameId);
            }
            SocialTab.OpenOriginalView(article.Id, article.Url);
        });
        return button;
    },

    $Generate_ShareButton: function (article, options) {
        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom"
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-share " + opt.cssClass,
            "title": "Share",
            "data-tooltip": "tooltip",
            "data-placement": opt.tooltipPosition,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();
            NewsTab.Modules.Sharing.Open_ShareArticleModal(article);
        });
        return button;
    },

    $Generate_ForceParseButton: function (article, options) {
        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom"
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-fort-awesome " + opt.cssClass,
            "title": "Force Re-Parse",
            "data-tooltip": "tooltip",
            "data-placement": opt.tooltipPosition,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();
            button.addClass("disabled");
            NewsTab.Modules.ArticleView.API_ParseArticle(article, button);
        });
        return button;
    },

    $Generate_AdvancedPinSettingsButton: function (article, options) {
        var self = this;

        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom"
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-cog " + opt.cssClass,
            "title": "Advanced Pin",
            "data-tooltip": "tooltip",
            "data-placement": opt.tooltipPosition,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();
            NewsTab.Modules.SpecialPinSystem.HTML_Modal_PinUrlAdvanced(article);
        });

        return button;
    },

    $Generate_ArrowButton: function (options) {
        var self = this;

        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom",
            placeholder: null
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-angle-down " + opt.cssClass,
            "title": "Expand",
            "data-tooltip": "tooltip",
            "data-placement": opt.tooltipPosition,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();

            if (opt.placeholder === null) {
                opt.placeholder = $(this).closest(".socialTabItem");
            }
            if (!$(this).hasClass("active")) {
                $(".desc", opt.placeholder).hide();
                $(this).addClass("active");
                opt.placeholder.addClass("expandDetails");
                $(".desc", opt.placeholder).show();
            } else {
                $(".desc", opt.placeholder).hide();
                $(this).removeClass("active");
                opt.placeholder.removeClass("expandDetails");
            }
        });

        return button;
    },

    $Generate_MobileHoverButton: function (options) {
        var self = this;

        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom",
            placeholder: null
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-angle-down mobileHover " + opt.cssClass,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();
            if (!$(this).closest(".back").hasClass("jsHover")) {
                $(this).closest(".back").addClass("jsHover");
            } else {
                $(this).closest(".back").removeClass("jsHover");
            }
        });

        return button;
    },

    $Generate_ExtensionExpandButton: function (options) {
        var self = this;

        var opt = $.extend({}, {
            cssClass: "",
            tooltipPosition: "bottom",
            placeholder: null
        }, options);

        var button = $("<button/>", {
            "type": "button",
            "class": "icon-angle-down " + opt.cssClass,
            "data-space": "true"
        });
        button.click(function (e) {
            e.stopPropagation();

            var item = $(this).closest(".socialTabItem");
            var newHeight;
            if (item.hasClass("expand")) {
                newHeight = item.hasClass("noContent") ? item.height() - 198 : item.height() - 109;
                item.height(newHeight);
                item.removeClass("expand");
            } else {
                newHeight = item.hasClass("noContent") ? item.height() + 198 : item.height() + 109;
                item.height(newHeight);
                item.addClass("expand");
                if (NewsTab.IsExtension) {
                    NewsTab.TrackAnalyticsEvent("Article", "Expand Preview", NewsTab.Extension.NameId);
                }
            }
            NewsTab.Extension.UpdateArticleGridWidth();
            setTimeout(function () { NewsTab.Extension.UpdateArticleGridWidth(); }, 2000);

        });

        return button;
    },

    $Indicator_Likes: function (article, options) {
        if (article.Favorites && article.Favorites.Count > 0) {
            return $("<span/>", {
                "class": "icon-heart",
                "data-likes": article.Favorites.Count
            }).html("<small>" + NewsTab.Helpers.GetShortCount(article.Favorites.Count) + "</small>");
        }
        return null;
    },

    $Indicator_Retweets: function (article, options) {
        if (article.Retweets && article.Retweets.Count > 0) {
            return $("<span/>", {
                "class": "icon-retweet ",
                "data-retweets": article.Retweets.Count
            }).html("<small>" + NewsTab.Helpers.GetShortCount(article.Retweets.Count) + "</small>");
        }
        return null;
    },

    $Indicator_Time: function (article, options) {
        var placeholder = $(".socialTabItem[data-article-id=" + article.Id + "]", options.ContainerSelector);

        var appendDiagnostics = '\n';

        if (article.TrendingDiagnostics) {
            for (var name in article.TrendingDiagnostics) {
                if (article.TrendingDiagnostics.hasOwnProperty(name)) {
                    appendDiagnostics += (name + ": " + article.TrendingDiagnostics[name]).replace(/ /g, '\u00a0') + '\n';
                }
            }
        }

        var buildVersion = $("#buildVersion");

        if (buildVersion.length && (buildVersion.html().indexOf(".L") > -1 || buildVersion.html().indexOf(".S") > -1)) {
            return $("<span/>", {
                "class": "time",
                "data-tooltip": "tooltip",
                "data-placement": placeholder.parent().hasClass("layout-compact") || placeholder.parent().hasClass("layout-list") ? "left" : "bottom",
                "title": "Published:\u00a0" + new Date(article.PubDate).toUTCString().replace("Central European Daylight Time", "CEST").replace(/ /g, '\u00a0').substr(5) +
                    " Parsed:\u00a0" + new Date(article.DateCreated).toUTCString().replace("Central European Daylight Time", "CEST").replace(/ /g, '\u00a0').substr(5) +
                    appendDiagnostics
            }).html(NewsTab.Helpers.GetPasedTime(article.PubDate, false));
        } else {
            return $("<span/>", {
                "class": "time"
            }).html(NewsTab.Helpers.GetPasedTime(article.PubDate, false));
        }
    },

    $Indicator_TwitterAction: function (article, options) {
        if (article.Retweet != null) {
            return $("<span/>", {
                "class": "action"
            }).html("Retweeted\u00a0<a class=\"action_name\" href=\"" + article.SourceFeed().UserTagLinkBase + article.Retweet.Author.Username + "\" target=\"_blank\">@" + article.Retweet.Author.Username + "</a>");
        }
        return null;
    },

    $Indicator_Loaction: function (article, options) {
        if (article.Location) {
            if (article.Location.Name) {
                var locName = article.Location.Name;
                if (article.SourceFeed().LocationLinkBase) {
                    locName = "<a href=\"" + article.SourceFeed().LocationLinkBase + article.Location.Id + "\" target=\"_blank\">" + article.Location.Name + "</a>";
                }
                return $("<span/>", { "class": "icon-location location" }).html(locName);
            } else if (article.Location.Id) {
                return $("<span/>", { "class": "location" });
            }
        }
        return null;
    },

    $Indicator_Trending: function (article, options) {
        if (article.Rsr >= 3) {
            var placeholder = $(".socialTabItem[data-article-id=" + article.Id + "]", options.ContainerSelector);

            return $("<span/>", {
                "class": "trending-icons orderByAboutPop " + (article.Rsr >= 15 ? "icon-trending2" : (article.Rsr >= 10 && article.Rsr < 15 ? "icon-trending1" : "icon-trending0")),
                "data-tooltip": "tooltip",
                "data-placement": (placeholder.parent().hasClass("layout-compact") || placeholder.parent().hasClass("layout-list") ? "left" : "top"),
                "title": "Rank: " + Math.round(article.Rsr)
            });
        }
        return null;
    },

    $Indicator_SocialTrending: function (article, network, options) {
        if (article.Rsr >= 1) {
            var placeholder = $(".socialTabItem[data-article-id=" + article.Id + "]", options.ContainerSelector);

            return $("<span/>", {
                "class": "trending-icons orderByAboutPop " + (network === "Facebook" ? "icon-facebook" : (network === "Google+" ? "icon-google" : "icon-linkedin")),
                "data-tooltip": "tooltip",
                "data-placement": (placeholder.parent().hasClass("layout-compact") || placeholder.parent().hasClass("layout-list") ? "left" : "top"),
                "title": "Trending Rank: " + Math.round(article.Rsr + 0.5)
            });
        }
        return null;
    },

    // Twitter
    $Generate_Comment: function (article, comment) {
        var html = ["<li class=\"comment\" data-comment-id=\"" + comment.Id + "\">"];
        html.push("<a href=\"" + article.SourceFeed().UserTagLinkBase + comment.User.Username + "\" class=\"co-profile-picture\" target=\"_blank\">");
        html.push("<img src=\"" + comment.User.ProfilePictureUrl + "\" class=\"img-circle\" /></a>");
        html.push("<div class=\"co-info\"><a href=\"" + article.SourceFeed().UserTagLinkBase + comment.User.Username + "\" class=\"co-fullname\" target=\"_blank\">" + scEmoji(comment.User.FullName) + "</a>");
        html.push("\u00a0<a href=\"" + article.SourceFeed().UserTagLinkBase + comment.User.Username + "\" class=\"co-username\" target=\"_blank\">@" + scEmoji(comment.User.Username) + "</a>");
        html.push("\u00a0/\u00a0<span class=\"time\">" + NewsTab.Helpers.GetPasedTime(comment.CreatTime) + "</span></div>");
        html.push("<div class=\"co-text\">" + comment.Text + "</div>");
        html.push("</li>");

        return $(html.join(""));
    },

    SetUp_TwitterCommentList: function (article, $form) {
        if (article.Comments != null && article.Comments.Count > 0) {
            var self = this;

            var existingComents = true;
            var $commentsControl = $(".content-comments", $form);
            var $commentList = null;

            if ($commentsControl.length == 0) {
                existingComents = false;
                $commentsControl = $("<div/>", { "class": "content-comments" });
                $commentsControl.append("<h4>Comments:</h4>");

                $commentList = $("<ul/>", { "class": "comment-list" });

                $commentsControl.append($commentList);
            }
            else {
                $commentList = $(".comment-list", $commentsControl);
            }

            for (var i = 0, len = article.Comments.Count; i < len; i++) {
                if (!$(".comment[data-comment-id=" + article.Comments.Items[i].Id + "]", $commentList).length) {
                    $commentList.append(self.$Generate_Comment(article, article.Comments.Items[i]));
                }
            }
            if (!existingComents) {
                $(".details-section", $form).append($commentsControl);
            }
        }
    },

    // API
    API_GetTwitterComments: function (article, $form) {
        var self = this;
        var id = article.IdOfOrginalArticle ? article.IdOfOrginalArticle : article.Id;
        var callUrl = NewsTab.ApiUrl_Base + "api/twitter/get/comments/" + id;

        $.ajax({
            url: callUrl,
            headers: { Authorization: 'Bearer ' + NewsTab.Token },
            error: function (error) {
                console.error("ERROR - GetComments:");
                console.error(error);
            }
        }).then(function (data) {
            if (data) {
                article.Comments = data;
                self.SetUp_TwitterCommentList(article, $form);
            }
        });
    },

    API_ParseArticle: function (article, btn) {
        $.ajax({
            url: NewsTab.FeedApiUrl_Feed + "forcereadfullarticle?articleUrl=" + encodeURIComponent(article.Url) + "&feedUrl=" + encodeURIComponent(article.SourceFeed().Url),
            type: "GET",
            async: true,
            cache: true,
            timeout: 15000,
            error: function (x) {
                alert("ERROR - Forced parsing failed!");
                console.error("ERROR - API_ParseArticle:");
                console.error(x);
                if (btn) {
                    btn.removeClass("disabled");
                }
            }
        }).then(function (data) {
            if (data) {
                var feedItem = $(".socialTabItem[data-article-id=" + article.Id + "]");

                if (data.DefaultImageUrl) {
                    article.DefaultImageUrl = data.DefaultImageUrl;
                }

                if (data.HtmlTitle) {
                    article.HtmlTitle = data.HtmlTitle;
                }

                if (data.FullHtmlParsed) {
                    article.FullHtmlParsed = data.FullHtmlParsed;
                }

                if (data.XFrameOptions) {
                    article.XFrameOptions = data.XFrameOptions;
                }

                if (data.ImageUrl) {
                    article.ImageUrl = data.ImageUrl;
                }

                if (data.VideoUrl) {
                    article.VideoUrl = data.VideoUrl.replace(/&amp;/g, "&");
                }

                if (data.FavIcon) {
                    article.FavIcon = data.FavIcon;
                    feedItem.find(".profile-picture:not(.prof-pic2)").attr("src", article.FavIcon);
                }

                if (data.HtmlSummary) {
                    article.HtmlSummary = data.HtmlSummary;
                }

                if (data.LanguageCode) {
                    article.LanguageCode = data.LanguageCode;

                    if (NewsTab.Helpers.ShouldBeRTL(data.Title)) {
                        feedItem.attr("dir", "rtl");
                        //feedItem.find(".hoverInfo .time").html(NewsTab.Helpers.GetPasedTime(feed.PubDate, false));
                    }
                }

                if (data.Author && data.Author.Username) {
                    article.Author = data.Author;
                }

                if (data.Text) {
                    article.Text = data.Text;
                }

                var ff = $.grep(NewsTab.Modules.SectionsManager.ParsedFeeds, function (f) { return f.Id === NewsTab.Modules.SectionsManager.CurrentSection.CurrentFeed.Id });
                if (ff.length) {
                    var aa = $.grep(ff[0].Articles, function (x) { return x.Url === article.Url });
                    if (aa.length) {
                        aa = article;
                    }
                }

                NewsTab.Modules.ArticleView.SetUp_ExpandView(SocialTab.ExpandedArticle);
                alert("Re-Parsing Finished!");
                if (btn) {
                    btn.removeClass("disabled");
                }
            }
        });
    },

    API_ParseUnparsedArticle: function (article, options) {
        var self = this;

        // Prevent from ReParsing (try only once to parse a single article)
        var opt = $.extend({}, options, { FullHtmlParse: false });

        var articleUrl = (article.SourceFeed().Type.CssClass === "rss") ?
            encodeURIComponent(article.Url) :
            encodeURIComponent(article.ExternalUrl);

        var feedUrl = article.SourceFeed().Url;
        if (feedUrl != undefined) {
            feedUrl = encodeURIComponent(feedUrl);
        }

        var xhr_parseArticle = $.ajax({
            url: NewsTab.FeedApiUrl_Feed + "readfullarticle?articleUrl=" + articleUrl + "&feedUrl=" + feedUrl,
            type: "GET",
            async: true,
            cache: true,
            timeout: 15000,
            error: function (jqXHR, textStatus, errorThrown) {
                console.error("ERROR - API_ParseUnparsedArticle:", errorThrown);
            },
            success: function (response) {
                var $article = $(".socialTabItem[data-article-id=" + article.Id + "]");

                if (response.DefaultImageUrl) {
                    article.DefaultImageUrl = response.DefaultImageUrl;
                }

                if (response.HtmlTitle) {
                    article.HtmlTitle = response.HtmlTitle;
                }

                if (response.FullHtmlParsed) {
                    article.FullHtmlParsed = response.FullHtmlParsed;
                }

                if (response.XFrameOptions) {
                    article.XFrameOptions = response.XFrameOptions;
                }

                if (response.ImageUrl) {
                    article.ImageUrl = response.ImageUrl;
                }

                if (response.VideoUrl) {
                    article.VideoUrl = response.VideoUrl.replace(/&amp;/g, "&");
                }

                if (response.FavIcon) {
                    article.FavIcon = response.FavIcon;
                    $article.find(".upperBar > img").attr("src", article.FavIcon);
                }

                if (response.HtmlSummary) {
                    article.HtmlSummary = response.HtmlSummary;
                }

                if (response.LanguageCode) {
                    article.LanguageCode = response.LanguageCode;

                    if (NewsTab.Helpers.ShouldBeRTL(response.Title)) {
                        $article.attr("dir", "rtl");
                        //$article.find(".hoverInfo .time").html(NewsTab.Helpers.GetPasedTime(feed.PubDate, false));
                    }
                }

                if (response.Author && response.Author.Username) {
                    article.Author = response.Author;
                }

                if (response.Text) {
                    article.Text = response.Text;
                }

                if (NewsTab.IsExtension) {
                    if (NewsTab.Modules.SectionsManager.DEFAULT_FEED_PARSING_OPTIONS.StoreFeedItemsLocaly) {
                        var destFeed = NewsTab.Modules.SectionsManager.GetFeedById(article.DestinationFeedId, null, true);
                        if (destFeed !== null) {
                            NewsTab.Helpers.SetLocalStorageItem("FeedArticles_" + destFeed.Id, destFeed.Items.slice(0, NewsTab.Modules.SectionsManager.PARSED_ARTICLES_PAGE_SIZE));
                        }
                    }
                }
            },
            complete: function (jqXHR, textStatus) {
                //if (!opt.LoadInbackground) { //TO-DO: Gray Images and Feed load fix
                    var destFeed = NewsTab.Modules.SectionsManager.GetFeedById(article.DestinationFeedId, null, true);
                    //NewsTab.Modules.ArticleView.Set_DefaultBackgroundImage(null, article, opt);
                    switch (opt.Layout) {
                        case "layout-compact":
                            self.SetUp_CompactPreviewItem(destFeed, article, opt);
                            break;
                        case "layout-list":
                            self.SetUp_ListPreviewItem(destFeed, article, opt);
                            break;
                        case "layout-cascading-grid":
                            self.SetUp_CascadingGridPreviewItem(destFeed, article, opt);
                            break;
                        default:
                            self.SetUp_GridPreviewItem(destFeed, article, opt);
                    }
                //}

                if (opt.CompleteCallback) {
                    opt.CompleteCallback();
                }
            }
        });

        NewsTab.Modules.XhrManager.Xhr_ParseUnparsedArticles.push(xhr_parseArticle);
    },

    API_ReadHistory: function (options) {
        var self = this;

        var opt = $.extend({}, NewsTab.BASE_REQUEST_OPTIONS, options);

        if (NewsTab.Token) {
            var callUrl = NewsTab.ApiUrl_Global + "get/history";
            $.ajax({
                url: callUrl,
                type: "GET",
                headers: { Authorization: 'Bearer ' + NewsTab.Token },
                error: function (jqXHR, textStatus, errorThrown) {
                    console.error("ERROR - API_ReadHistory:", errorThrown);

                    if (opt.ErrorCallback) opt.ErrorCallback(jqXHR, textStatus, errorThrown);
                },
                success: function (response) {
                    self.ReadArticles = response;
                    localStorage.setItem("ST_" + NewsTab.User.Id + "_ReadItems", JSON.stringify(self.ReadArticles));

                    for (var i = 0, l = self.ReadArticles.length; i < l; i++) {
                        var obj = $(".socialTabItem[data-article-id=" + self.ReadArticles[i] + "]").first();
                        if (obj.length > 0) {
                            obj.addClass("readX");
                            obj.find(".icon-markasread").attr("data-original-title", "Mark as unread");
                            obj.find(".tooltip-inner").text("Mark as unread");
                        }
                    }

                    if (opt.SuccessCallback) opt.SuccessCallback(response);
                },
                complete: function (jqXHR, textStatus) {
                    if (opt.CompleteCallback) opt.CompleteCallback(jqXHR, textStatus);
                }
            });
        }
    },

    API_SetAsRead: function (article, setAsRead) {
        var self = this;

        var asread = $.grep(self.ReadArticles, function (e) { return e === article.Alias });

        var obj = $(".socialTabItem[data-article-id=" + article.Id + "]").first();

        if (asread.length > 0) {
            if (!setAsRead) {
                NewsTab.Helpers.RemoveItemFromArray(self.ReadArticles, article.Alias);
                obj.removeClass("readX");
                obj.find(".icon-markasread").attr("data-original-title", "Mark as read");
                obj.find(".tooltip-inner").text("Mark as read");
                if (NewsTab.Token && !NewsTab.Modules.Sharing.DisplayingSharedArticle) {
                    var callUrl = NewsTab.ApiUrl_Global + "delete/read/" + article.Alias;
                    $.ajax({
                        url: callUrl,
                        type: "DELETE",
                        contentType: "application/json",
                        dataType: "json",
                        headers: { Authorization: 'Bearer ' + NewsTab.Token },
                        error: function (error) {
                            console.error("ERROR - SetAsRead:");
                            console.error(error);
                        }
                    });
                }
            }
        } else {
            self.ReadArticles.push(article.Alias);
            obj.addClass("readX");
            obj.find(".icon-markasread").attr("data-original-title", "Mark as unread");
            obj.find(".tooltip-inner").text("Mark as unread");
            if (NewsTab.Token && !NewsTab.Modules.Sharing.DisplayingSharedArticle) {
                var callUrl = NewsTab.ApiUrl_Global + "post/read/" + article.Alias + "/" + article.SourceFeedId;
                $.ajax({
                    url: callUrl,
                    type: "POST",
                    contentType: "application/json",
                    dataType: "json",
                    headers: { Authorization: 'Bearer ' + NewsTab.Token },
                    error: function (error) {
                        console.error("ERROR - SetAsRead:");
                        console.error(error);
                    }
                }).then(function () {
                    SocialTab.UpdateUsageCount({ Id: article.SourceFeedId }, 0.5);
                });
            }
        }
        localStorage.setItem("ST_" + NewsTab.User.Id + "_ReadItems", JSON.stringify(this.ReadItems));
    }
};