﻿var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.ImageCollage = {

    TestImages: [
        "https://qzprod.files.wordpress.com/2018/03/screen-shot-2018-03-22-at-10-37-59-am-e1521729905319.png?w=1400&strip=all&quality=75",
        "https://cdn.vox-cdn.com/thumbor/_FOZ5xj7cRJvAcCN0M4CtauMFRk=/0x191:2040x1259/fit-in/1200x630/cdn.vox-cdn.com/uploads/chorus_asset/file/8048055/DSCF5006.0.jpg",
        "https://media.wired.com/photos/5ab3d86e0b4a6a6950f917c4/191:100/pass/instagramfeed.jpg",
        "https://techcrunch.com/wp-content/uploads/2018/02/samsung-galaxy-s9-1.jpg",
        "https://images.idgesg.net/images/article/2018/03/google-enemy-100752818-large.3x2.jpg",
        "https://cdn.vox-cdn.com/thumbor/S5do-TbnbC2Lwy9NtpVmCTBL63A=/0x62:2040x1130/fit-in/1200x630/cdn.vox-cdn.com/uploads/chorus_asset/file/10475595/arobertson_180321_2403_0002.jpg",
        "https://o.aolcdn.com/images/dims?thumbnail=1200%2C630&quality=80&image_uri=https%3A%2F%2Fs.aolcdn.com%2Fhss%2Fstorage%2Fmidas%2F386a199804c585716b8cec0be1ab32d2%2F206234094%2FSiren_Still_01.png&client=cbc79c14efcebee57402&signature=819df35c60d84f430677019801d4e0c7009430f2",
        "https://i.kinja-img.com/gawker-media/image/upload/s--2WZTeAcG--/c_fill,fl_progressive,g_center,h_450,q_80,w_800/z6gotuxqaypbiwyfpzdg.jpg",
        "https://static.gamespot.com/uploads/screen_kubrick/1578/15787979/3366875-8747973079-30807.jpg",
        "https://venturebeat.com/wp-content/uploads/2018/03/ubisoft-mumbai.jpg?fit=1714%2C1125&strip=all"
    ],

    Templates: [
        "/Media/Images/CollageTemplates/temp-3.png",
        "/Media/Images/CollageTemplates/temp-4.png",
        "/Media/Images/CollageTemplates/temp-5.png",
        "/Media/Images/CollageTemplates/temp-6.png",
        "/Media/Images/CollageTemplates/temp-7.png",
        "/Media/Images/CollageTemplates/temp-8.png",
        "/Media/Images/CollageTemplates/temp-9.png",
        "/Media/Images/CollageTemplates/temp-10.png"
    ],
    TemplatesFull: [
        "/Media/Images/CollageTemplates/temp-8-full.png",
        "/Media/Images/CollageTemplates/temp-14-full.png"
    ],
    TemplateScale: 4,
    TemplateCanvas: null,

    Init: function () {
        var self = this;
    },

    GenerateCollage: function (options) {
        var self = this;

        if (options.TemplateScale !== undefined) {
            self.TemplateScale = options.TemplateScale;
        }

        var hosts = [];

        for (var i = 0, l = options.Images.length; i < l; i++) {
            var link = options.Images[i];
            if (link.indexOf("http") === 0) {
                var uri = new URL(link);
                if (hosts.indexOf(uri.host) === -1) {
                    hosts.push(uri.host);
                }
            }
        }

        if (hosts.length > 0) {
            var origins=[];
            for (var j = 0, jl = hosts.length; j < jl; j++) {
                origins.push("*://" + hosts[j] + "/");
            }

            chrome.permissions.contains({ origins: origins }, function (contains) {
                if (contains) {
                    self.BuildCollage(options);
                } else {
                    self.AskPremissions(origins, options);
                }
            });
        }
        
    },

    BuildCollage: function (options) {
        var self = this;

        var template = new Image();
        template.crossOrigin = "anonymous";
        template.onload = function () {
            if (self.TemplateCanvas === null) {
                self.TemplateCanvas = document.createElement('canvas');
            }

            self.TemplateCanvas.width = this.width;
            self.TemplateCanvas.height = this.height;

            var ctx = self.TemplateCanvas.getContext("2d");
            ctx.drawImage(this, 0, 0, self.TemplateCanvas.width, self.TemplateCanvas.height);

            var imgPlaces = self.GetImagePlaceholders(ctx);

            if (imgPlaces.length > options.Images.length) {
                console.error("Not enough images!",
                    "Images sent:" + options.Images.length,
                    "Image placeholders on template:" + imgPlaces.length);
            }

            var sortedPlaceholders = self.SortAndGroupItems(imgPlaces);

            var canvas = document.createElement('canvas');
            canvas.width = self.TemplateCanvas.width * self.TemplateScale;
            canvas.height = self.TemplateCanvas.height * self.TemplateScale;

            ctx = canvas.getContext("2d");

            // NOTE: Draw background image or color here
            if (options.BackgroundColor !== undefined) {
                ctx.fillStyle = options.BackgroundColor;
                ctx.fillRect(0, 0, canvas.width, canvas.height);
            }

            var loadCount = 0;
            var loadedImages = [];
            for (var i = 0; i < options.Images.length; i++) {
                var img = new Image();
                img.crossOrigin = "anonymous";
                img.onerror = function () {
                    loadCount++;
                };
                img.onload = function () {
                    if ('naturalHeight' in this) {
                        if (this.naturalHeight + this.naturalWidth === 0) {
                            this.onerror();
                            return;
                        }
                    } else if (this.width + this.height === 0) {
                        this.onerror();
                        return;
                    }

                    loadedImages.push(this);
                    loadCount++;

                };
                img.src = options.Images[i];
            }

            var loadAllImages = setInterval(function () {
                if (loadCount >= options.Images.length) {
                    clearInterval(loadAllImages);

                    var sortedImages = self.SortAndGroupItems(loadedImages);
                    var i = 0, l = 0;
                    //First Draw (L to L and P to P)
                    for (i = 0, l = sortedPlaceholders.Landscape.length; i < l; i++) {
                        if (sortedImages.Landscape.length > 0 && sortedPlaceholders.Landscape[i].filled !== true) {
                            sortedPlaceholders.Landscape[i].filled =
                                true; // mark as filled so we don't try to fill it on the Second Draw
                            self.DrawImageOnPlaceHolder(ctx,
                                sortedImages.Landscape[0],
                                sortedPlaceholders.Landscape[i]);
                            sortedImages.Landscape.shift(); // remove the image so it's no reused
                        }
                    }
                    for (i = 0, l = sortedPlaceholders.Portrait.length; i < l; i++) {
                        if (sortedImages.Portrait.length > 0 && sortedPlaceholders.Portrait[i].filled !== true) {
                            sortedPlaceholders.Portrait[i].filled = true;
                            self.DrawImageOnPlaceHolder(ctx,
                                sortedImages.Portrait[0],
                                sortedPlaceholders.Portrait[i]);
                            sortedImages.Portrait.shift();
                        }
                    }
                    // Second Draw (All to All)
                    var unfilledPlaceholders = $.grep(
                        sortedPlaceholders.Landscape.concat(sortedPlaceholders.Portrait),
                        function (a) { return a.filled !== true; });
                    var remainingImages = sortedImages.Landscape.concat(sortedImages.Portrait);

                    for (i = 0, l = unfilledPlaceholders.length; i < l; i++) {
                        if (remainingImages.length > 0) {
                            self.DrawImageOnPlaceHolder(ctx, remainingImages[0], unfilledPlaceholders[i]);
                            remainingImages.shift(); // remove the image so it's no reused
                        }
                    }

                    if (options.Callback) {
                        // If we need full, first we will get normal image and ask for full page
                        if (options.GetFullPageImage && !options.LoadingFullPageImage) {
                            options.LoadingFullPageImage = true;
                            options.NormalImage = canvas.toDataURL("image/webp");
                            self.GenerateCollage(options);
                        } else if (!options.GetFullPageImage && !options.LoadingFullPageImage) {
                            options.Callback({
                                NormalImage: canvas.toDataURL("image/webp"),
                                FullPageImage: canvas.toDataURL("image/webp")
                            });
                        } else {
                            options.Callback({
                                NormalImage: options.NormalImage,
                                FullPageImage: canvas.toDataURL("image/webp")
                            });
                        }
                    }
                }
            }, 100);
        };

        if (options.LoadingFullPageImage) {
            template.src = self.TemplatesFull[options.TemplateIndexFullPage];
        } else {
            template.src = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";
        }
    },

    AskPremissions: function (origins, options) {
        var self = this;

        var html = '<p>To create an image collage, we need your pemission to fetch images from the following sites:</p>';
        html += '<ul>';
        origins.forEach(function (link) {
            html += '<li>' + link +'</li>';
        });
        html += '</ul>';
        html += '<p>We will ask for your premission each time there is a image from a news site. If you wish to give us the premission to fetch images from ALL future sites, you can do so by clicking "Allow All Futue Sites".</p>';

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: "<div class=\"title\"><h2>Premission to fetch images for collage</h2></div>",
            Height: "auto",
            MaxHeight: "285",
            Text: html,
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Allow Listed Sites",
                    Classes: ["blue", "allow"],
                    Action: function (e, $btn, $modal) {
                        NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                        chrome.permissions.request({ origins: origins }, function (granted) {
                            if (granted) {
                                self.BuildCollage(options);
                            } else {
                                alert("Premissions Rejected, The Image Collage failed to be created!");
                            }
                        });
                    }
                },
                {
                    Name: "Allow All Futue Sites",
                    Classes: ["blue", "allow-all"],
                    Action: function (e, $btn, $modal) {
                        NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                        chrome.permissions.request({ origins: "*://*/*" }, function (granted) {
                            if (granted) {
                                self.BuildCollage(options);
                            } else {
                                alert("Premissions Rejected, The Image Collage failed to be created!");
                            }
                        });
                    }
                }
            ]
        });
    },

    GeneratePromoLogo: function (options) {
        var self = this;

        if (options.TemplateScale !== undefined) {
            self.TemplateScale = options.TemplateScale;
        }

        var template = new Image();
        template.crossOrigin = "anonymous";
        template.onload = function () {
            if (self.TemplateCanvas === null) {
                self.TemplateCanvas = document.createElement('canvas');
            }

            self.TemplateCanvas.width = this.width;
            self.TemplateCanvas.height = this.height;

            var canvas = document.createElement('canvas');
            canvas.width = self.TemplateCanvas.width * self.TemplateScale;
            canvas.height = self.TemplateCanvas.height * self.TemplateScale;

            var ctx = canvas.getContext("2d");

            var bgImage = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";;
            var img = self.LoadImage(bgImage, self.OnloadCallback);
            var cimg = self.LoadImage(options.Image, self.OnloadCallback);

            // NOTE: Draw background image or color here
            if (options.BackgroundColor !== undefined) {
                ctx.fillStyle = options.BackgroundColor;
                ctx.fillRect(0, 0, canvas.width, canvas.height);
            }

            var loadAllImages = setInterval(function () {
                clearInterval(loadAllImages);

                ctx.drawImage(img, 0, 0);

                var cpos = canvas.width / 2;
                ctx.save();
                ctx.beginPath();
                ctx.arc(cpos, cpos + 1, cpos - 10, 0, Math.PI * 2, true);
                ctx.closePath();
                ctx.clip();

                ctx.drawImage(cimg, 0, 0, cpos * 2, cpos * 2);

                if (options.Callback) {
                    options.Callback({
                        Image: canvas.toDataURL("image/png")
                    });
                }
            }, 100);
        };

        template.src = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";
    },

    GenerateCsScreenshot: function (options) {
        var self = this;

        if (options.TemplateScale !== undefined) {
            self.TemplateScale = options.TemplateScale;
        }

        var template = new Image();
        template.crossOrigin = "anonymous";
        template.onload = function () {
            if (self.TemplateCanvas === null) {
                self.TemplateCanvas = document.createElement('canvas');
            }

            self.TemplateCanvas.width = this.width;
            self.TemplateCanvas.height = this.height;

            var canvas = document.createElement('canvas');
            canvas.width = self.TemplateCanvas.width * self.TemplateScale;
            canvas.height = self.TemplateCanvas.height * self.TemplateScale;

            var ctx = canvas.getContext("2d");

            var bgImage = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";
            var img = self.LoadImage(bgImage, self.OnloadCallback);
            var cimg = self.LoadImage(options.Image, self.OnloadCallback);

            // NOTE: Draw background image or color here
            if (options.BackgroundColor !== undefined) {
                ctx.fillStyle = options.BackgroundColor;
                ctx.fillRect(0, 0, canvas.width, canvas.height);
            }

            var loadAllImages = setInterval(function () {
                clearInterval(loadAllImages);

                ctx.drawImage(img, 0, 0, self.TemplateCanvas.width, self.TemplateCanvas.height);
                ctx.drawImage(cimg, 95, 76, 457, 253);

                if (options.Callback) {
                    options.Callback({
                        Image: canvas.toDataURL("image/png")
                    });
                }
            }, 100);
        };

        template.src = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";
    },

    GeneratePromoImage: function (options) {
        var self = this;

        if (options.TemplateScale !== undefined) {
            self.TemplateScale = options.TemplateScale;
        }

        var template = new Image();
        template.crossOrigin = "anonymous";
        template.onload = function () {
            if (self.TemplateCanvas === null) {
                self.TemplateCanvas = document.createElement('canvas');
            }

            self.TemplateCanvas.width = this.width;
            self.TemplateCanvas.height = this.height;

            var ctx = self.TemplateCanvas.getContext("2d");
            ctx.drawImage(this, 0, 0, self.TemplateCanvas.width, self.TemplateCanvas.height);

            var imgPlaces = self.GetImagePlaceholders(ctx);

            if (imgPlaces.length > options.Images.length) {
                console.error("Not enough images!",
                    "Images sent:" + options.Images.length,
                    "Image placeholders on template:" + imgPlaces.length);
            }

            var sortedPlaceholders = self.SortAndGroupItems(imgPlaces);

            var canvas = document.createElement('canvas');
            canvas.width = self.TemplateCanvas.width * self.TemplateScale;
            canvas.height = self.TemplateCanvas.height * self.TemplateScale;

            ctx = canvas.getContext("2d");

            // NOTE: Draw background image or color here
            if (options.BackgroundColor !== undefined) {
                ctx.fillStyle = options.BackgroundColor;
                ctx.fillRect(0, 0, canvas.width, canvas.height);
            }

            var loadCount = 0;
            var loadedImages = [];
            for (var i = 0; i < options.Images.length; i++) {
                var img = new Image();
                img.setAttribute("crossOrigin", "anonymous");
                img.onload = function () {
                    if ('naturalHeight' in this) {
                        if (this.naturalHeight + this.naturalWidth === 0) {
                            this.onerror();
                            return;
                        }
                    } else if (this.width + this.height === 0) {
                        this.onerror();
                        return;
                    }
                    loadedImages.push(this);
                    loadCount++;
                };
                img.onerror = function () {
                    loadCount++;
                };
                img.src = options.Images[i];
            }

            var loadAllImages = setInterval(function () {
                if (loadCount >= options.Images.length) {
                    clearInterval(loadAllImages);

                    var sortedImages = self.SortAndGroupItems(loadedImages);

                    var i = 0, l = 0;
                    //First Draw (L to L and P to P)
                    for (i = 0, l = sortedPlaceholders.Landscape.length; i < l; i++) {
                        if (sortedImages.Landscape.length > 0 && sortedPlaceholders.Landscape[i].filled !== true) {
                            sortedPlaceholders.Landscape[i].filled =
                                true; // mark as filled so we don't try to fill it on the Second Draw
                            self.DrawImageOnPlaceHolder(ctx,
                                sortedImages.Landscape[0],
                                sortedPlaceholders.Landscape[i]);
                            sortedImages.Landscape.shift(); // remove the image so it's no reused
                        }
                    }
                    for (i = 0, l = sortedPlaceholders.Portrait.length; i < l; i++) {
                        if (sortedImages.Portrait.length > 0 && sortedPlaceholders.Portrait[i].filled !== true) {
                            sortedPlaceholders.Portrait[i].filled = true;
                            self.DrawImageOnPlaceHolder(ctx,
                                sortedImages.Portrait[0],
                                sortedPlaceholders.Portrait[i]);
                            sortedImages.Portrait.shift();
                        }
                    }
                    // Second Draw (All to All)
                    var unfilledPlaceholders = $.grep(
                        sortedPlaceholders.Landscape.concat(sortedPlaceholders.Portrait),
                        function(a) { return a.filled !== true; });
                    var remainingImages = sortedImages.Landscape.concat(sortedImages.Portrait);

                    for (i = 0, l = unfilledPlaceholders.length; i < l; i++) {
                        if (remainingImages.length > 0) {
                            self.DrawImageOnPlaceHolder(ctx, remainingImages[0], unfilledPlaceholders[i]);
                            remainingImages.shift(); // remove the image so it's no reused
                        }
                    }

                    if (options.Box !== undefined) {
                        self.FillText(ctx, options.Box);
                    }

                    if (options.AddHDTriangle) {
                        self.GenerateHdTriangle(ctx, options);
                    }

                    self.DataURIToBlob(canvas.toDataURL("image/png"), function(blob) {
                        if (options.Callback) {
                            options.Callback({
                                Image: URL.createObjectURL(blob)
                            });
                        }
                    });
                }
            }, 100);
        }

        
        template.src = "/Media/Images/CollageTemplates/temp-" + options.TemplateIndexNormal + ".png";
    },

    DataURIToBlob: function (dataUri, callback) {
        var binStr = atob(dataUri.split(",")[1]),
            len = binStr.length,
            arr = new Uint8Array(len);

        for (var i = 0; i < len; i++) {
            arr[i] = binStr.charCodeAt(i);
        }

        callback(new Blob([arr], { type: "image/png" }));
    },

    GetImagePlaceholders: function (context2D) {
        var self = this;

        var placeholders = [];
        var cw = self.TemplateCanvas.width;
        var ch = self.TemplateCanvas.height;
        var pixelData = context2D.getImageData(0, 0, cw, ch).data;
        var x = 0, y = 0, startX = 0, endX = 0, startY = 0, endY = 0, pixelPostition = 0;

        for (var i = 0; i < pixelData.length; i += 4) {
            if (pixelData[i] === 0 && pixelData[i + 1] === 0 && pixelData[i + 2] === 0) {
                startX = x;
                startY = y;

                for (var xx = startX; xx < cw; xx++) {
                    pixelPostition = (xx * 4) + (startY * cw * 4);
                    if (pixelData[pixelPostition] > 0 ||
                        pixelData[pixelPostition + 1] > 0 ||
                        pixelData[pixelPostition + 2] > 0) {
                        endX = xx;
                        break;
                    }
                }

                for (var yy = startY; yy < ch; yy++) {
                    pixelPostition = (startX * 4) + (yy * cw * 4);
                    if (pixelData[pixelPostition] > 0 ||
                        pixelData[pixelPostition + 1] > 0 ||
                        pixelData[pixelPostition + 2] > 0) {
                        endY = yy;
                        break;
                    }
                }

                context2D.fillStyle = "#FFFFFF";
                context2D.fillRect(startX, startY, endX - startX, endY - startY);

                placeholders.push({
                    x: startX * self.TemplateScale,
                    y: startY * self.TemplateScale,
                    width: (endX - startX) * self.TemplateScale,
                    height: (endY - startY) * self.TemplateScale,
                    filled: false
                });
                pixelData = context2D.getImageData(0, 0, cw, ch).data;
            }

            x++;
            if (x >= cw) {
                x = 0;
                y++;
            }
        }

        return placeholders;
    },

    FillText: function (context, options) {
        var self = this;
        var words = options.Text.split(" ");
        var line = "";
        var boxWidth = options.Width - options.Margin;
        var tempHeight = options.TeplateHeight === undefined ? self.TemplateCanvas.height : options.Height;
        var tempScale = options.Scale === undefined ? self.TemplateScale : options.Scale;
        var boxBottomPos = tempHeight * tempScale - options.Margin - options.Height;

        if (options.BackgroundColor) {
            context.fillStyle = options.BackgroundColor;
        }
        context.fillRect(options.Margin, boxBottomPos, boxWidth, options.Height);

        context.fillStyle = options.TextColor;
        context.font = options.Font;

        for (var i = 0; i < words.length; i++) {
            var textLine = line + words[i] + " ";
            var metrics = context.measureText(textLine);
            var textWidth = metrics.width;
            if (textWidth > options.MaxWidth && i > 0) {
                context.fillText(line, options.Position.X, options.Position.Y + boxBottomPos);
                line = words[i] + " ";
                options.Position.Y += options.LineHeight;
            }
            else {
                line = textLine;
            }
        }
        context.fillText(line, options.Position.X, options.Position.Y + boxBottomPos);

        if (options.SubText !== undefined && options.SubText.Text !== undefined) {
            var subWords = options.SubText.Text.split(" ");
            var subLine = "";

            context.fillStyle = options.SubText.TextColor;
            context.font = options.SubText.Font;

            for (var j = 0; j < subWords.length; j++) {
                var subTextLine = subLine + subWords[j] + " ";
                var subMetrics = context.measureText(subTextLine);
                var subTextWidth = subMetrics.width;
                if (subTextWidth > options.MaxWidth && j > 0) {
                    context.fillText(subLine, options.SubText.Position.X, options.SubText.Position.Y + boxBottomPos);
                    subLine = subWords[j] + " ";
                    options.SubText.Position.Y += options.SubText.LineHeight;
                }
                else {
                    subLine = subTextLine;
                }
            }
            context.fillText(subLine, options.SubText.Position.X, options.SubText.Position.Y + boxBottomPos);
        }
    },

    GenerateHdTriangle: function (context, options) {
        var trglPosX = options.OrigionalScale;
        var trglPosY = options.OrigionalScale;
        var trglWidth = 25 * options.OrigionalScale;

        context.beginPath();
        context.moveTo(trglPosX, trglPosY);
        context.lineTo(trglWidth - trglPosX, trglPosX);
        context.lineTo(trglPosY, trglWidth - trglPosY);

        context.fillStyle = "#FF0000";
        context.fill();

        context.fillStyle = "#FFFFFF";
        context.font = "bold " + (8 * options.OrigionalScale) + "px Arial";
        context.rotate(-45 * Math.PI / 180);
        context.fillText("HD", (-6 * options.OrigionalScale), (15 * options.OrigionalScale));
    },

    SortAndGroupItems: function (items) {
        var result = { Landscape: [], Portrait: [] };

        // Split to landscape and portrait
        for (var i = 0; i < items.length; i++) {
            if (items[i].width >= items[i].height) {
                result.Landscape.push(items[i]);
            } else {
                result.Portrait.push(items[i]);
            }
        }

        // Sort by size and proportion
        result.Landscape = result.Landscape.sort(function (a, b) {
            var aValue = a.width * a.height * (a.width / a.height);
            var bValue = b.width * b.height * (b.width / b.height);

            return (bValue > aValue) ? 1 : ((aValue > bValue) ? -1 : 0);
        });

        result.Portrait = result.Portrait.sort(function (a, b) {
            var aValue = a.width * a.height * (a.height / a.width);
            var bValue = b.width * b.height * (b.height / b.width);

            return (bValue > aValue) ? 1 : ((aValue > bValue) ? -1 : 0);
        });

        return result;
    },

    DrawImageOnPlaceHolder: function (context2D, img, placeholder) {

        var iw = img.width,
            ih = img.height,
            r = Math.min(placeholder.width / iw, placeholder.height / ih),
            nw = iw * r,
            nh = ih * r,
            cx, cy, cw, ch, ar = 1;

        if (nw < placeholder.width) {
            ar = placeholder.width / nw;
        }

        if (Math.abs(ar - 1) < 1e-14 && nh < placeholder.height) {
            ar = placeholder.height / nh;
        }
        nw *= ar;
        nh *= ar;

        cw = iw / (nw / placeholder.width);
        ch = ih / (nh / placeholder.height);

        cx = (iw - cw) * 0.5;
        cy = (ih - ch) * 0.5;

        if (cx < 0) cx = 0;
        if (cy < 0) cy = 0;
        if (cw > iw) cw = iw;
        if (ch > ih) ch = ih;

        context2D.drawImage(img, cx, cy, cw, ch, placeholder.x, placeholder.y, placeholder.width, placeholder.height);
    },

    GetImageUrlsFormLoadedArticles: function () {
        var urls = [];
        var $thumbnails = $(".thumbnail", "#articles_grid");
        for (var i = 0, l = $thumbnails.length; i < l; i++) {
            var bi = $thumbnails.eq(i).css("background-image").replace("url(\"", "").replace("\")", "");
            if (bi.length > 0 && bi.indexOf("http") === 0) {
                urls.push(bi.replace(/,/g, "%2C"));
            }
        }
        return urls;
    },

    LoadImage: function (src, callback) {
        var img = new Image();
        img.onload = callback;
        img.setAttribute("crossOrigin", "anonymous");
        img.src = src;
        return img;
    },

    OnloadCallback: function () {
        if ("naturalHeight" in this) {
            if (this.naturalHeight + this.naturalWidth === 0) {
                this.onerror();
                return;
            }
        } else if (this.width + this.height === 0) {
            this.onerror();
            return;
        }
    }
};