﻿var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.RegionManager = {

    Countries: [],
    UsStates: [],

    MainEdition: "US",
    PreviousEdition: null,

    AddFeedEdition: "US",
    PreviousAddFeedEdition: null,

    TemporaryEdition: null,

    MainEditionGnValue: "US",
    AddFeedEditionGnValue: "US",

    Get_RegionCodeByCountyId: function (conuntyId) {
        var self = this;
        var county = $.grep(self.Countries, function (c) { return c.Id === conuntyId; });
        if (county.length) {
            return county[0].Code;
        }
        return "US";
    },

    $CountryDropeDownOption: function (item) {
        var $option = $("<option>", {
            "value": item.Name,
            "data-iconurl": NewsTab.MainCDN + "/Media/Images/Backgrounds/default_flag_bg.jpg",
            "data-src": item.FlagUrl,
            "data-countrycode": item.Code,
            "data-googlenewscode": item.GoogleNewsEditionCode,
            "data-isgooglenewsedition": item.IsGoogleNewsEdition
        });

        if (item.Selected === true) {
            $option.prop("selected", true);
        }
        $option.html(item.Name);

        return $option;
    },

    SetUp_CountryDropdown: function (selector, main, edition) {
        var self = this;

        if (self.Countries.length === 0) {
            self.Countries = NewsTab.Helpers.GetLocalStorageItem("Countries");
        }

        if (self.Countries === null || self.Countries.length === 0) {
            self.Countries = [];
            self.API_GetCountries(edition, function () { self.SetUp_CountryDropdown(selector, main, edition); });
        } else {
            $.each(self.Countries, function () {
                if (!main && !this.IsGoogleNewsEdition && edition === "") return;
                selector.append(self.$CountryDropeDownOption({
                    Selected: ((main && this.Code.toUpperCase() === self.MainEdition.toUpperCase()) ||
                        (!main && (edition == undefined || edition === "") && this.GoogleNewsEditionCode != undefined && this.GoogleNewsEditionCode.toUpperCase() === self.AddFeedEditionGnValue.toUpperCase()) ||
                        (edition != undefined && edition !== "" && (this.Code.toUpperCase() === edition.toUpperCase() || this.Code.toUpperCase().indexOf(edition.toUpperCase()) === 0)))
                        ? true : false,
                    Name: this.Name,
                    Code: this.Code.toUpperCase(),
                    FlagUrl: this.FlagUrl,
                    GoogleNewsEditionCode: this.GoogleNewsEditionCode != null ? this.GoogleNewsEditionCode.toUpperCase() : "",
                    IsGoogleNewsEdition: this.IsGoogleNewsEdition
                }));
            });
            selector.selectBoxIt({
                similarSearch: true,
                downArrowIcon: "icon-angle-down"
            });
            selector.parent().find(".selectboxit-option-icon.selectboxit-option-icon-url").lazyload({
                effect: "fadeIn",
                container: selector.parent().find(".selectboxit-options.selectboxit-list")
            });
        }
    },

    Modal_ChangeUserRegion: function (sender) {
        var self = this;

        var name = sender.closest(".regionBox").find(".selectboxit-option.selectboxit-focus").attr("data-val");
        var code = sender.closest(".regionBox").find(".selectboxit-option.selectboxit-focus").attr("data-countrycode");
        var gnCode = sender.closest(".regionBox").find(".selectboxit-option.selectboxit-focus").attr("data-googlenewscode");

        var message = gnCode !== "" ? "Changing the region will change your default Google News feeds to <strong>" + name + "</strong>. Please note that you can find more local feeds for <strong>" + name + "</strong> in the New Feed menu, under <a href=\"#\" class=\"addContentItem textbutton\" data-tab-select=\"#tabFeeds\" data-tab-param=\"tabRegionPublishers\">Local Publishers</a>."
            : "Changing the region will change your default Google News feeds to <strong>United States</strong>, as local Google News feeds for " + name + " are not available. Please note that you can find local feeds for <strong>" + name + "</strong> in the New Feed menu under <a  href=\"#\" class=\"addContentItem textbutton\" data-tab-select=\"#tabFeeds\" data-tab-param=\"tabRegionPublishers\">Local Publishers</a>.";

        var $div = $("<div/>", { "class": "text-center" });
        $div.append("<h3>Change region to <strong>" + name + "</strong></h3>");
        $div.append("<p>" + message + "</p>");

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Text: $div,
            Height: "auto",
            Buttons: [
                {
                    Name: "Cancel",
                    Classes: ["gray", "closeNotification", "float-left"],
                    Action: function (e, $btn) {
                        self.SetUp_CountryDropdown(sender, true, self.MainEdition);
                    }
                },
                {
                    Name: "Change",
                    Action: function (e, $btn) {
                        var $modal = $btn.closest(".notificationModal");
                        $modal.addClass("disabled");
                        $btn.html("<span class=\"spinner\"></span> Changing...");

                        self.MainEdition = code;
                        self.AddFeedEdition = self.AddFeedEdition;
                        self.MainEditionGnValue = gnCode !== "" ? gnCode : "US";
                        self.AddFeedEditionGnValue = self.MainEditionGnValue;

                        var county = $.grep(self.Countries, function (c) { return c.Name === name; });

                        self.API_SaveUserRegion(county[0].Id);
                    }
                }
            ],
            FullscreenOption: false
        });
    },
    /* API */
    API_GetCountries: function (regionCode, callback) {
        var self = this;

        $.ajax({
            type: "GET",
            url: NewsTab.ApiUrl_Global_Cdn + "get/countries?code=" + regionCode,
            dataType: "json",
            cache: true,
            success: function (response) {
                self.Countries = response.Countries;
                NewsTab.Helpers.SetLocalStorageItem("Countries", self.Countries);
                if (NewsTab.User && NewsTab.User.CountryId) {
                    self.MainEdition = self.Get_RegionCodeByCountyId(NewsTab.User.CountryId);
                }

                if (callback) {
                    callback();
                }
            }
        });
    },

    API_GetUsStates: function (options) {
        var self = this;

        if (NewsTab.Modules.XhrManager.Xhr_GetUsState && NewsTab.Modules.XhrManager.Xhr_GetUsState.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_GetUsState.abort();
        }

        var opt = $.extend(NewsTab.BASE_REQUEST_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "get/usstates";
        NewsTab.Modules.XhrManager.Xhr_GetUsState = $.ajax({
            url: callUrl,
            timeout: 10000,
            error: function (error) {
                console.error("ERROR - API_GetUsStates:", error);
                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
                //NewsTab.Modules.AddNewContent.DisplayErrorScreen($("#regionPublishersTabSection"), true);
                //$("#contentFeeds .errorScreen .feedRetry").on("click", function () {
                //    NewsTab.Modules.SectionsManager.API_GetUsStates();
                //});
            },
            beforeSend: function () {
                if (opt.BeforeSendCallback) {
                    opt.BeforeSendCallback();
                }
            }
        });

        NewsTab.Modules.XhrManager.Xhr_GetUsState.then(function (data) {
            if (data) {
                self.UsStates = [];
                data.States.forEach(function (d) {
                    if (d != null) {
                        self.UsStates.push({
                            Id: d.Id,
                            Name: d.Name,
                            Code: d.Code,
                            IconUrl: d.FlagUrl,
                            NumberOfPublishers: d.NumberOfPublishers
                        });
                    }
                });
                if (opt.SuccessCallback) {
                    opt.SuccessCallback();
                }
            }
        });
    },

    API_SaveUserRegion: function (countyId) {
        var self = this;

        var callUrl = NewsTab.ApiUrl_Global + "post/userregion?userId=" + NewsTab.User.Id + "&countryId=" + countyId;
        $.ajax({
            url: callUrl,
            headers: { Authorization: "Bearer " + NewsTab.Token },
            type: "POST",
            contentType: "application/json; charset=utf-8",
            cache: false,
            success: function () {
                self.API_GetSectionsbyregion();
            }
        });
    },

    API_GetSectionsbyregion: function () {
        var self = this;

        $.ajax({
            url: NewsTab.ApiUrl_Global + "get/sectionsbyregion/" + self.MainEditionGnValue + "/true",
            headers: { Authorization: "Bearer " + NewsTab.Token },
            type: "GET",
            contentType: "application/json; charset=utf-8",
            cache: false,
            success: function () {
                location.reload();
            }
        });
    }
};