﻿var NewsTab = NewsTab || {};
NewsTab.Modules = NewsTab.Modules || {};

NewsTab.Modules.SectionsManager = {
    DEFAULT_SECTION_COLOR: "#00b8f5",
    DEFAULT_FEED_PARSING_OPTIONS: {
        Refresh: false,
        NewFeed: false,
        LoadInbackground: false,
        StoreFeedItemsLocaly: false
    },

    MAX_NUMBER_OF_FEEDS_PER_REQUEST: 20,
    PARSED_ARTICLES_PAGE_SIZE: 20,

    FEED_LOCAL_CACHE_TIME: 15 * 60, // 15 minutes -> 900 sec

    STORE_CURRENT_SECTION_FEED_ID: false,

    SysSections: [],
    AppSections: [],
    ExtensionSections: [],
    Sections: [],
    PreviewSection: new Section().Init({ Id: Section.TYPES.Default, Name: "Preview" }),
    CurrentSection: null,
    CurrentLayout: "layout-grid",

    LocalStoredFeedData: null,

    LastActiveSectionId: null,
    LastActiveFeedId: null,

    LastActiveSections: [],
    LastActiveFeeds: [],

    NumberOfVisibleSections: 5,
    NumberOfVisibleFeeds: 7,

    RegionPublishers: [],
    PopularPublishers: [],
    PopularTopics: [],

    ParsedFeeds: [], // TO-DO: Remove

    PreviewFeed: null,

    SlowNetTimer: null,

    Init: function () {
        var self = this;

        self.API_GetPopularPublishersTopics();

        $(document).ready(function () {
            self.DocumentReadyInit();
        });
    },

    DocumentReadyInit: function () {
        var self = this;

        // GLOBAL EVENTS
        $(document.body).on("click", ".openFeed", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            $('.openFeed').removeClass("active");
            $(this).addClass("active");

            var sId = $(this).closest("[data-section-id]").data("section-id");
            var fId = $(this).data("feed-id");

            self.ChangeFeed(sId, fId);
        });
    },


    GetRandomColorForCurrentSection: function () {
        /// <summary>
        /// Gets a variation of the section color or DEFAULT_SECTION_COLOR in case of CurrentSection not defined
        /// </summary>
        /// <returns type="String">Hex Color Code (#000000)</returns>
        var self = this;
        if (self.CurrentSection && self.CurrentSection.Id) {
            if (self.CurrentSection.GetRandomColor) {
                return self.CurrentSection.GetRandomColor();
            }
        }
        return self.DEFAULT_SECTION_COLOR;
    },

    GetNewExtensionFeeds: function () {
        var self = this;
        var feed = null;
        var feeds = [];

        self.ExtensionSections.forEach(function (es) {
            es.Feeds.forEach(function (esf) {
                feed = self.GetFeedByUrl(esf.Url, null);
                if (feed === null) {
                    feeds.push(esf);
                }
            });
        });

        return feeds;
    },

    GetFeedById: function (id, defValue, includeExtensionSections) {
        /// <summary>
        /// Search all sections for the first feed with matching id
        /// </summary>
        /// <param name="id" type="Guid">Feed Id to search for</param>
        /// <param name="defValue" type="Object">
        /// In case that no matching feed is found, defValue will be returned.
        /// Value: a Feed.js Object or null
        /// </param>
        /// <param name="includeExtensionSections" type="Boolean">Search in Extension Sections for the feed as well. Default: false</param>
        /// <returns type="Object"></returns>
        var self = this;
        var i, j, len1, len2;

        for (i = 0, len1 = self.Sections.length; i < len1; i++) {
            for (j = 0, len2 = self.Sections[i].Feeds.length; j < len2; j++) {
                if (self.Sections[i].Feeds[j].Id === id) {
                    return self.Sections[i].Feeds[j];
                }
            }
        }
        if (includeExtensionSections === true) {
            for (i = 0, len1 = self.ExtensionSections.length; i < len1; i++) {
                for (j = 0, len2 = self.ExtensionSections[i].Feeds.length; j < len2; j++) {
                    if (self.ExtensionSections[i].Feeds[j].Id === id) {
                        return self.ExtensionSections[i].Feeds[j];
                    }
                }
            }
        }

        if (self.PreviewFeed !== null && self.PreviewFeed.Id === id) {
            return self.PreviewFeed;
        }

        return defValue;
    },

    GetFeedByUrl: function (url, defValue, includeExtensionSections) {
        /// <summary>
        /// Search all sections for the first feed with matching url
        /// </summary>
        /// <param name="url" type="String">Feed Url to search for</param>
        /// <param name="defValue" type="Object">
        /// In case that no matching feed is found, defValue will be returned.
        /// Value: a Feed.js Object or null
        /// </param>
        /// <param name="includeExtensionSections" type="Boolean">Search in Extension Sections for the feed as well. Default: false</param>
        /// <returns type="Object"></returns>
        var self = this;
        var i, j, len1, len2;

        for (i = 0, len1 = self.Sections.length; i < len1; i++) {
            for (j = 0, len2 = self.Sections[i].Feeds.length; j < len2; j++) {
                if (self.Sections[i].Feeds[j].Url === url) {
                    return self.Sections[i].Feeds[j];
                }
            }
        }
        if (includeExtensionSections === true) {
            for (i = 0, len1 = self.ExtensionSections.length; i < len1; i++) {
                for (j = 0, len2 = self.ExtensionSections[i].Feeds.length; j < len2; j++) {
                    if (self.ExtensionSections[i].Feeds[j].Url === url) {
                        return self.ExtensionSections[i].Feeds[j];
                    }
                }
            }
        }

        if (self.PreviewFeed !== null && self.PreviewFeed.Url === url) {
            return self.PreviewFeed;
        }

        return defValue;
    },

    GetSectionById: function (id, defValue) {
        /// <summary>
        /// Search all sections (user and special like extension) for the first with matching id
        /// </summary>
        /// <param name="id" type="Guid">Section Id to search for</param>
        /// <param name="defValue" type="Object">
        /// In case that no matching section is found, defValue will be returned.
        /// Value: a Section.js Object or null
        /// </param>
        /// <returns type="Object"></returns>
        var self = this;
        var i, len;

        for (i = 0, len = self.Sections.length; i < len; i++) {
            if (self.Sections[i].Id === id) {
                return self.Sections[i];
            }
        }

        for (i = 0, len = self.ExtensionSections.length; i < len; i++) {
            if (self.ExtensionSections[i].Id === id) {
                return self.ExtensionSections[i];
            }
        }

        if (self.PreviewSection.Id === id) {
            return self.PreviewSection;
        }

        return defValue;
    },

    SetCurrentSectionAndFeed: function (sectionId, feedId) {
        /// <summary>
        /// Set the CurrentSection of SectionManager and the CurrentFeed of that section.
        /// In case of missing section or feed with the provided Id(s) a fallback section/feed will be used if posible.
        /// Return: true if successful or false if there wasn't a fallback section/feed.
        /// </summary>
        /// <param name="sectionId" type="Guid"></param>
        /// <param name="feedId" type="Guid"></param>
        /// <returns type="Boolean"></returns>
        var self = this;


        // Search for the matching section or use a fallback section
        var section = $.grep(self.ExtensionSections, function (x) { return x.Id === sectionId; });
        if (section.length === 0) {
            section = $.grep(self.Sections, function (x) { return x.Id === sectionId; });
        }

        if (section.length) {
            self.CurrentSection = section[0];
        }
        else {
            // Fallback Section for Extension and Web
            if (NewsTab.IsExtension && NewsTab.Token === null) {
                if (self.ExtensionSections[1].Feeds.length > 0) {
                    self.CurrentSection = self.ExtensionSections[1];
                }
                else {
                    self.CurrentSection = self.ExtensionSections[0];
                }
            } else {
                self.CurrentSection = self.Sections[0];
            }
        }

        if (self.CurrentSection !== undefined) {
            // Search for the matching feed or use a fallback feed
            var feed = $.grep(self.CurrentSection.Feeds, function (x) { return x.Id === feedId; });
            if (feed.length) {
                self.CurrentSection.CurrentFeed = feed[0];
            }
            else {
                // Fallback Feed
                feed = $.grep(self.CurrentSection.Feeds, function (x) { return x.ShowInList === true; });
                if (feed.length) {
                    self.CurrentSection.CurrentFeed = feed[0];
                }
                else if (self.CurrentSection.Feeds.length > 0) {
                    self.CurrentSection.CurrentFeed = self.CurrentSection.Feeds[0];
                }
            }

            // Return true if both are set and store in localStorage if STORE_CURRENT_SECTION_FEED_ID is true
            if (self.CurrentSection !== null && self.CurrentSection.CurrentFeed !== null) {
                if (self.STORE_CURRENT_SECTION_FEED_ID) {
                    NewsTab.Helpers.SetLocalStorageItem("CurrentSectionId", self.CurrentSection.Id);
                    NewsTab.Helpers.SetLocalStorageItem("CurrentFeedId", self.CurrentSection.CurrentFeed.Id);
                }
                return true;
            }
        }
        return false;
    },

    SetFeedIsLoading: function (feed, isLoading) {
        var self = this;
        if (feed.IsLoading !== isLoading) {
            feed.IsLoading = isLoading;
            if (feed.IsLoading) {
                self.LoadingAnimation_Start();
            }
            else {
                self.LoadingAnimation_Stop();
            }
        }
    },

    AddSection: function (section) {
        var self = this;

        self.Sections.unshift(section);
        self.LastActiveSections.unshift(String(section.Id));

        for (var i = 0; i < self.LastActiveSections.length; i++) {
            var s = $.grep(self.Sections, function (t) { return t.Id === self.LastActiveSections[i] });
            if (s.length) {
                s = s[0];
                s.OrderNumber = i;
            }
        }

        self.Sections.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
            Title: "Add New Section",
            Text: "The section \"" + section.Name + "\" has been added to your account.",
            Color: "green"
        });

        var feeds = $.grep(section.Feeds, function (s) { return s.ShowInList === true; });
        if (feeds.length > 0) {
            self.ChangeFeed(section.Id, feeds[0].Id);
        }

        $(document.body).trigger("setion:added", [section]);
    },

    StoreFeedItemsLocaly: function (feed, articles) {
        /// <summary>
        /// Stores PARSED_ARTICLES_PAGE_SIZE number of articles into the localStorage and cleans up previous stored data
        /// </summary>
        /// <param name="feed" type="Object">A Feed.js Object</param>
        /// <param name="articles" type="Array">List of articles to store</param>
        NewsTab.Helpers.ClearLocalStorageByKey("FeedArticles_");
        NewsTab.Helpers.SetLocalStorageItem("FeedArticles_" + feed.Id, articles.slice(0, this.PARSED_ARTICLES_PAGE_SIZE));
    },

    UpdateExtensionSection: function (feeds) {
        /// <summary>
        /// Updates the ExtensionSections with the new feed list returned by API_GetFeedsByRegion 
        /// and joins it with the feeds from Extension Confing file
        /// </summary>
        /// <param name="feeds" type="Array" elementType="Object">Array of feeds as returned by API_GetFeedsByRegion</param>
        var self = this;
        self.ExtensionSections[0].Feeds = [];
        self.ExtensionSections[1].Feeds = [];

        var includedFeedsSectionIndex = 1;
        if (NewsTab.Extension.Config.MergeIncludedFeeds === true) {
            includedFeedsSectionIndex = 0;
        }

        if (NewsTab.Extension.Config.IncludedFeeds) {
            $.each(NewsTab.Extension.Config.IncludedFeeds, function (index, feed) {
                var name = feed.Title.replace(" - Google News", "").replace(" (Google News)", "");
                self.ExtensionSections[includedFeedsSectionIndex].Feeds.push(new Feed().Init({
                    Id: feed.FeedId,
                    Name: name,
                    Url: feed.FeedUrl,
                    SiteUrl: feed.Domain,
                    Icon: feed.IconUrl,
                    Type: Feed.TYPES.RSS,
                    Language: "en-US",
                    Count: Feed.DEFAULT_COUNT,
                    Layout: "layout-grid",
                    LayoutType: 1,
                    ShowInList: true,
                    SectionId: self.ExtensionSections[includedFeedsSectionIndex].Id
                }));
            });
        }

        $.each(feeds, function (index, feed) {
            var name = feed.Title.replace(" - Google News", "").replace(" (Google News)", "");
            var feed0 = $.grep(self.ExtensionSections[0].Feeds, function (x) { return x.Name === name; });
            var feed1 = $.grep(self.ExtensionSections[1].Feeds, function (x) { return x.Name === name; });
            if (feed0.length === 0 && feed1.length === 0) {
                self.ExtensionSections[0].Feeds.push(new Feed().Init({
                    Id: feed.FeedId,
                    Name: name,
                    Url: feed.FeedUrl,
                    SiteUrl: feed.Domain,
                    Icon: feed.IconUrl,
                    Type: Feed.TYPES.RSS,
                    Language: "en-US",
                    Count: Feed.DEFAULT_COUNT,
                    Layout: "layout-grid",
                    LayoutType: 1,
                    ShowInList: true,
                    SectionId: self.ExtensionSections[0].Id
                }));
            }
        });
        NewsTab.Helpers.SetLocalStorageItem("ExtensionSections", self.ExtensionSections);
    },


    LoadFeed: function (feed, options) {
        var self = this;

        if ($.grep(self.CurrentSection.Feeds, function (x) { return x.ShowInList === true; }).length === 0) {
            NewsTab.Modules.ArticleView.DisplayFeedEmpty(self.CurrentSection.CurrentFeed);
            return;
        }

        if (self.SlowNetTimer !== null) {
            clearTimeout(self.SlowNetTimer);
            self.SlowNetTimer = null;
        }
        if (feed.Layout === "null" || feed.Layout === undefined) {
            feed.Layout = NewsTab.IsExtension ? "layout-cascading-grid" : "layout-grid";
            feed.LayoutType = 1;
        } else if (feed.Layout === "layout-grid" && NewsTab.IsExtension) {
            feed.Layout = "layout-cascading-grid";
            feed.LayoutType = 1;
        } else if (feed.Layout === "layout-cascading-grid" && !NewsTab.IsExtension) {
            feed.Layout = "layout-grid";
            feed.LayoutType = 1;
        }



        var opt = $.extend({},
            NewsTab.Modules.ArticleView.DEFAULT_SETTINGS,
            self.DEFAULT_FEED_PARSING_OPTIONS,
            options, { Layout: feed.Layout });

        if (opt.IsPreview === true) {
            self.PreviewFeed = feed;
        }

        if ($(".socialTabItem.empty:first", opt.ContainerSelector).length === 0 && !opt.LoadInbackground) {
            NewsTab.Modules.ArticleView.SetUp_PreviewItem_Placeholders(self.PARSED_ARTICLES_PAGE_SIZE, opt);
        }

        $("#show_hide .refresh").removeClass("newArticles");

        if (opt.NewFeed) {
            Feed.Reset(feed);
            NewsTab.Modules.ArticleView.ResetArticleView(opt);
        }

        if (!opt.Refresh || opt.LoadInbackground) {
            var sec = Math.floor(Date.now() / 1000);
            if (feed.Items.length >= feed.PageNbr * self.PARSED_ARTICLES_PAGE_SIZE &&
                feed.LastUpdate > sec - self.FEED_LOCAL_CACHE_TIME) {
                NewsTab.Log("Load Feed Data from memory", feed);
                NewsTab.Modules.ArticleView.DisplayFeedArticles(feed, opt);
                return;
            }
            else {
                var articles = NewsTab.Helpers.GetLocalStorageItem("FeedArticles_" + feed.Id);
                if (articles !== null && articles.length > 0) {

                    feed.Items = [];
                    for (var i = 0, len = articles.length; i < len; i++) {
                        feed.Items.push(new Article().Init(articles[i]));
                    }

                    NewsTab.Log("Load Feed Data from localStorage", feed);
                    NewsTab.Modules.ArticleView.DisplayFeedArticles(feed, opt);
                    opt = $.extend({}, opt, { LoadInbackground: true });
                    //opt.LoadInbackground = true; //TO-DO: Gray images fix - check for potential issues with this commented out
                }
            }
        }
        else {
            Feed.Reset(feed);
            NewsTab.Modules.ArticleView.ResetArticleView(opt);
        }

        if (opt.LoadInbackground !== true) {
            self.SlowNetTimer = setTimeout(function () {
                clearTimeout(self.SlowNetTimer);
                self.SlowNetTimer = null;
                NewsTab.Modules.ArticleView.DisplaySlowNet(opt);
            }, 3000);
        }

        switch (feed.Type) {
            case Feed.TYPES.TWITTER:
                if (Feed.Linked(feed.Type)) {
                    self.API_ParseTwitterFeed(feed, opt);
                } else if (!opt.LoadInbackground) {
                    NewsTab.Modules.ArticleView.DisplayFeedEmpty(feed, opt);
                }
                break;
            case Feed.TYPES.PINWALL:
                self.API_ParsePinnedFeed(feed, opt);
                break;
            case Feed.TYPES.SOCIALTRENDING:
                SocialTab.LoadSocialTrendingFeed(feed, opt);
                break;
            case Feed.TYPES.PUBLICPINWALL:
                NewsTab.Modules.PublicDashboard.Load_PublicPinnedFeed(feed, opt);
                break;
            default:
                self.API_ParseFeed(feed, opt);
                break;
        }
    },

    LoadCurrentFeed: function (options) {
        var self = this;

        var opt = $.extend({},
            NewsTab.Modules.ArticleView.DEFAULT_SETTINGS,
            self.DEFAULT_FEED_PARSING_OPTIONS,
            options);

        if (self.CurrentSection !== null && self.CurrentSection.CurrentFeed !== null) {
            self.LoadFeed(self.CurrentSection.CurrentFeed, opt);
        }
    },

    ChangeFeed: function (sectionId, feedId) {
        var self = this;

        if (self.CurrentSection === null ||
            self.CurrentSection.Id !== sectionId ||
            self.CurrentSection.CurrentFeed === null ||
            self.CurrentSection.CurrentFeed.Id !== feedId) {

            var options = $.extend({},
                NewsTab.Modules.ArticleView.DEFAULT_SETTINGS,
                self.DEFAULT_FEED_PARSING_OPTIONS,
                { NewFeed: true });

            if (self.SetCurrentSectionAndFeed(sectionId, feedId)) {

                if (NewsTab.IsExtension) {
                    NewsTab.Extension.SetUp_NewTabFeedTitle();
                    NewsTab.TrackAnalyticsEvent("Change Feed", self.CurrentSection.CurrentFeed.Name, NewsTab.Extension.NameId);
                }
                self.LoadCurrentFeed(options);

            }

            SocialTab.HTML_SetUpFeedList();
            SocialTab.Calculate_Section_ItemNumber(false);
        }
    },


    ChangeFeedVisibility: function (section, feed, show, update) {
        var self = this;

        var visibleFeeds = $.grep(section.Feeds, function (t) { return (t.ShowInList == true) });

        if (show || !update || (visibleFeeds.length > 1 && !show)) {
            if (feed.Name != "Timeline" || feed.IsList == true || feed.Type === Feed.TYPES.SOCIALTRENDING) {
                if (show) {
                    feed.ShowInList = true;
                    NewsTab.Helpers.RemoveItemFromArray(section.HiddenLists, feed.Id)
                    localStorage.setItem("ST_" + NewsTab.User.Id + "_" + section.Name + "_HiddenLists", JSON.stringify(section.HiddenLists));
                }
                else {
                    feed.ShowInList = false;
                    section.HiddenLists.push(feed.Id);
                    localStorage.setItem("ST_" + NewsTab.User.Id + "_" + section.Name + "_HiddenLists", JSON.stringify(section.HiddenLists));
                }

                if (update) {
                    if (self.CurrentSection.CurrentFeed.Id === feed.Id) {
                        var visibleFeed = $.grep(self.CurrentSection.Feeds, function (t) { return (t.ShowInList === true) });
                        if (visibleFeed.length > 0) {
                            self.ChangeFeed(self.CurrentSection.Id, visibleFeed[0].Id);
                        }
                        else {
                            var defaultFeed = $.grep(self.CurrentSection.Feeds, function (t) { return (t.Name === "Timeline") });
                            if (defaultFeed) {
                                defaultFeed = defaultFeed[0];
                                defaultFeed.ShowInList = true;
                                NewsTab.Helpers.RemoveItemFromArray(self.CurrentSection.HiddenLists, defaultFeed.Id);
                                localStorage.setItem("ST_" + NewsTab.User.Id + "_" + self.CurrentSection.Name + "_HiddenLists", JSON.stringify(self.CurrentSection.HiddenLists));
                                self.ChangeFeed(self.CurrentSection.Id, defaultFeed.Id);
                            }

                        }
                    }
                }
            }
        }
        else {
            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Twitter Preferences",
                Color: "red",
                Text: "At least one feed needs to be enabled"
            });
        }
    },
    /* UI */
    DisplayErrorScreen: function (container, show) {
        $("#contentFeeds .errorScreen").remove();
        if (show) {
            container.find(".searchItems").html("");
            container.find(".popularItems").html("");
            var result = "<div class=\"errorScreen\">";
            result += "<div class=\"table\">";
            result += "<div class=\"noIframe content\">";
            result += "<img src=\"/Media/DashboardImages/sorry_penguin.png\" />";
            result += "<h2>Oops...</h2>";
            result += "<p>Something went wrong!. Please check<br/> your internet connection and try again!</p>";
            result += "<p><span class=\"feedRetry btn blue\" style=\"width: 300px;height: 40px;line-height: 2;\" tabindex=\"1010\">Retry</span></p>";
            result += "</div>";
            result += "</div>";
            result += "</div>";

            $("#contentFeeds").find(container).append(result);
        }
    },

    LoadingAnimation_Start: function () {
        $(".feedsPage").find(".loadingAnimation").addClass("loading");
    },

    LoadingAnimation_Stop: function () {
        $(".feedsPage").find(".loadingAnimation").removeClass("loading");
    },

    _$copyMoveSectionItem: function (section, action) {
        var self = this;

        var $item = $("<li/>", {
            "data-section-id": section.Id,
            "class": "item",
            "tabindex": 0
        });



        var html = [];
        html.push('<img src="' + NewsTab.PlaceholderImage + '"/>');
        html.push('<div style="background-image: url(\'/Media/Images/scratch_back.png\');background-color:' + section.Color + '">');
        html.push("<i class=\"icon-statusIcon\"></i>");

        if (section.ClassName) {
            html.push("<i class=\"iconName icon-" + section.ClassName + "\"></i>");
        } else {
            html.push("<span class=\"iconName\">" + section.Name.substring(0, 2) + "</span>");
        }

        html.push("<h3><span class=\"name\">" + section.Name + "</span></h3>");
        html.push("</div>");

        $item.html(html.join(""));

        $item.on("click", function (e) {
            if (action === "COPY") {
                $item.toggleClass("selected");
            }
            else {
                if ($item.hasClass("selected")) {
                    $item.removeClass("selected");
                }
                else {
                    $("li", $item.parent()).removeClass("selected");
                    $item.addClass("selected");
                }
            }

            if ($("li.selected", $item.parent()).length === 0) {
                $item.closest(".copyMoveModal").find(".save").addClass("disabled");
            }
            else {
                $item.closest(".copyMoveModal").find(".save").removeClass("disabled");
            }
        });

        return $item;
    },

    /* UI - Context Menu */
    ContextMenu_Section: function (section, $destination, type) {
        var self = this;
        $(".dropdown").removeClass("open");
        $(".customDropdown").remove();

        var feeds = $.grep(section.Feeds, function (x) { return x.ShowInList === true });
        var $customDropdown = $("<div/>", {
            "class": "customDropdown",
            "style": "background-color:" + section.Color
        });

        var html = [];

        html.push('<ul id="sectionMenuOptions">');

        if (section.SectionTypeId === Section.TYPES.Normal) {
            if (type === 1) {
                html.push('<li class="sectionMoveAllOption">Move all feeds</li>');
                html.push('<li class="sectionCopyAllOption">Copy all feeds</li>');
                html.push('<li class="sectionRemoveAllOption">Delete all feeds</li>');
            } else {
                html.push('<li class="sectionEditOption">Edit</li>');
                html.push('<li class="sectionRemoveOption">Remove</li>');
            }
        }
        else if (feeds.length > 0 && section.SectionTypeId === Section.TYPES.Home) {
            html.push('<li class="openTopFeeds">Top Feeds</li>');
        } else if (section.SectionTypeId === Section.TYPES.Social) {
            if (Feed.Linked(Feed.TYPES.TWITTER)) {
                var acc = Feed.Account(Feed.TYPES.TWITTER);

                html.push('<li class="twitterProfile">');
                if (acc != null && acc.ProfilePictureUrl != null) {
                    html.push('<img class="stProfilePicture" src="' + acc.ProfilePictureUrl + '" alt="">');
                }
                html.push('<div class="stUsername">' + (acc ? acc.Username : "") + '</div>');
                html.push('</li>');

                html.push('<li class="twitterDisconnectOption">Disconnect</li>');
                html.push('<li class="twitterPreferencesOption">Preferences</li>');
            }
            else {
                html.push('<li class="twitterConnectOption">Connect to ' + section.Name + '</li>');
            }
            html.push('<li class="twitterRemoveOption">Remove</li>');
        }

        html.push('</ul>');
        $customDropdown.html(html.join(""));


        $(".sectionEditOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_EditSection(section);
        });

        $(".sectionRemoveOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_RemoveSection(section);
        });


        $(".sectionCopyAllOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_CopyMoveFeed(section, null, "COPY_ALL");
        });

        $(".sectionMoveAllOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_CopyMoveFeed(section, null, "MOVE_ALL");
        });

        $(".sectionRemoveAllOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_RemoveSection(section);
        });


        $(".openTopFeeds", $customDropdown).on("click", function (w) {
            w.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            NewsTab.Modules.TopFeeds.Modal_TopFeeds();
        });


        $(".twitterConnectOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            NewsTab.Modules.TwitterManager.Connect();
        });

        $(".twitterDisconnectOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_DisconnectTwitterSection(section);
        });

        $(".twitterPreferencesOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_TwitterPreferences(section);
        });

        $(".twitterRemoveOption", $customDropdown).on("click", function (e) {
            e.stopPropagation();
            $(".dropdown").removeClass("open");
            $(".customDropdown").remove();

            self.Modal_RemoveTwitterSection(section);
        });


        $destination.append($customDropdown);
    },

    ContextMenu_Feed: function (section, feed, $destination, type) {
        var self = this;
        $(".dropdown").removeClass("open");
        $(".customDropdown").remove();

        var $customDropdown = $("<div/>", {
            "class": "customDropdown"
        });

        var html = "<ul id=\"feedMenuOptions\">";

        if (section.SectionTypeId === Section.TYPES.Normal) {
            html += "<li class=\"feedEditOption\">Edit</li>";
            html += "<li class=\"feedRemoveOption\">Remove</li>";

            var normalTypeSections = $.grep(NewsTab.Modules.SectionsManager.Sections, function (e) { return e.SectionTypeId === Section.TYPES.Normal; });
            if (normalTypeSections.length > 1) {
                html += "<li class=\"feedMoveOption\">Move to</li>";
                html += "<li class=\"feedCopyOption\">Copy to</li>";
            }

            html += "<li class=\"feedAboutOption\">About</li>";
        } else if (section.SectionTypeId === Section.TYPES.Social) {
            html += "<li class=\"feedHideOption\">Hide</li>";
        }
        html += "</ul>";
        $customDropdown.html(html);

        $(".feedEditOption", $customDropdown).on("click", function (e) {
            self.Modal_EditFeed(section, feed);
        });

        $(".feedRemoveOption", $customDropdown).on("click", function (e) {
            self.Modal_RemoveFeed(section, feed);
        });

        $(".feedAboutOption", $customDropdown).on("click", function (e) {
            self.Modal_AboutFeed(feed);
        });

        $(".feedMoveOption", $customDropdown).on("click", function (e) {
            self.Modal_CopyMoveFeed(section, feed, "MOVE");
        });

        $(".feedCopyOption", $customDropdown).on("click", function (e) {
            self.Modal_CopyMoveFeed(section, feed, "COPY");
        });

        $(".feedHideOption", $customDropdown).on("click", function (e) {
            self.Modal_TwitterFeedVisibility(section, feed);
        });

        $destination.append($customDropdown);
    },

    /* UI - Modals */
    Modal_EditSection: function (section) {
        var self = this;

        var $editSection = $("<div/>", {
            "class": "editSectionModal newItem"
        });

        var html = "";
        html += "<div class=\"inputContainer\">";
        html += "<input type=\"text\" class=\"newSectionName clearable\" placeholder=\"Section Name\" value=\"" + section.Name.trim() + "\" />";

        if (section.ClassName) {
            html += "<span class=\"iconPreview\" style=\"background-color:" + section.Color + "\"><i class=\"icon-" + section.ClassName + "\"></i></span>";
        } else {
            html += "<span class=\"iconPreview\" style=\"background-color:" + section.Color + "\">" + section.Name.substring(0, 2) + "</span>";
        }
        html += "</div>";
        html += "<ul class=\"colors\">";
        html += "</ul>";

        $editSection.html(html);

        NewsTab.Setup_SectionColors($editSection, section.Color);

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: "<div class=\"title\"><h2>Edit section</h2></div>",
            Height: "auto",
            MaxHeight: "285",
            Text: $editSection,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["gray", "float-left"]
                },
                {
                    Name: "Save",
                    Action: function (e, $sender, $modal) {
                        var name = $(".newSectionName", $editSection).val().trim();
                        var color = $(".colors li.selected", $editSection).data("color");

                        $modal.addClass("disabled");
                        $(".save", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Saving...");

                        self.API_EditSection(section, name, color, {
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".save", $modal).html("Save");
                            }
                        });

                    },
                    Classes: ["blue", "save"]
                }],
            OnShow: function ($modal) {
                $(".newSectionName", $modal).on("keyup paste", function (event) {
                    var sName = $(this).val();

                    if (section.ClassName) {
                        if (sName.length > 1 && sName.length <= 50) {
                            $(".save", $modal).removeClass("disabled");
                        } else {
                            $(".save", $modal).addClass("disabled");
                        }
                    }
                    else {
                        if (sName.length > 1 && sName.length <= 50) {
                            $(".iconPreview", $modal).html(sName.substring(0, 2));
                            $(".save", $modal).removeClass("disabled");
                        } else {
                            $(".iconPreview", $modal).html("");
                            $(".save", $modal).addClass("disabled");
                        }
                    }
                });
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_RemoveSection: function (section) {
        var self = this;

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: $('<h2>Remove "<span style="color:' + section.Color + '">' + section.Name + '</span>" Section</h2><p>You can always add it back from the "+ Add news feeds".</p>'),
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Remove",
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Removing...");
                        self.API_RemoveSection(section, {
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".remove", $modal).html("Remove");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Remove Section",
                                    Text: "Failed to remove " + section.Name + " section!",
                                    Color: "red"
                                });
                            }
                        });
                    },
                    Classes: ["red", "remove"]
                }

            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_RemoveAllSectionFeeds: function (section) {
        var self = this;

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: $('<h2>Remove all Feeds from "<span style="color:' + section.Color + '">' + section.Name + '</span>"</h2><p>You can always add them back from the "+ Add news feeds".</p>'),
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Remove",
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Removing...");
                        self.API_RemoveSection(section, {
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".remove", $modal).html("Remove");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Remove Section",
                                    Text: "Failed to remove " + section.Name + " section!",
                                    Color: "red"
                                });
                            }
                        });
                    },
                    Classes: ["red", "remove"]
                }

            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_EditFeed: function (section, feed) {
        var self = this;

        var $content = $("<div/>", {
            "class": "editFeedModal newItem"
        });

        var html = "";
        html += "<div class=\"inputContainer\">";
        html += "<input type=\"text\" class=\"feedName clearable\" placeholder=\"Feed Name\" value=\"" + feed.Name.trim() + "\" />";
        html += "<img class=\"iconPreview\" src=\"" + (feed.Icon ? feed.Icon : "/Media/ExtensionImages/rss.png") + "\" onError=\"this.onerror=null;this.src='/Media/ExtensionImages/rss.png';\"/>"
        html += "<label class=\"errorMessage\"></lebel>";
        html += "</div>";

        $content.html(html);

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: "<div class=\"title\"><h2>Edit feed</h2></div>",
            Height: "auto",
            MaxHeight: "200",
            Text: $content,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["gray", "float-left"]
                },
                {
                    Name: "Save",
                    Action: function (e, $sender, $modal) {
                        var name = $(".feedName", $modal).val().trim();

                        $modal.addClass("disabled");
                        $(".save", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Saving...");

                        self.API_EditFeed(section.Id, feed.Id, name, feed.TopFeed, {
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".save", $modal).html("Save");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Edit Feed",
                                    Text: "Failed to save changes!",
                                    Color: "red"
                                });
                            }
                        });

                    },
                    Classes: ["blue", "save"]
                }],
            OnShow: function ($modal) {
                $(".feedName", $modal).on("keyup paste", function (event) {
                    var name = $(this).val();

                    if (name.length > 1 && name.length <= 50) {
                        $(".save", $modal).removeClass("disabled");
                    } else {
                        $(".save", $modal).addClass("disabled");
                    }
                });
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_RemoveFeed: function (section, feed) {
        var self = this;

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: $('<h2>Remove "' + feed.Name + '" from <span style="color:' + section.Color + '">' + section.Name + '<span></h2><p>You can always add it back from the "+ Add news feeds".</p>'),
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Remove",
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Removing...");
                        self.API_RemoveFeed(section, feed, {
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".remove", $modal).html("Remove");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Remove Feed",
                                    Text: "Failed to remove " + section.Name + " section!",
                                    Color: "red"
                                });
                            }
                        });
                    },
                    Classes: ["red", "remove"]
                }
            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_AboutFeed: function (feed) {
        var self = this;

        var $content = $("<div/>", {
            "class": "feedAbout"
        });

        var height = 220;


        var html = [];
        html.push('<div><p class="feedAbout-title">Feed Displayed Name: </p><p class="feedAbout-body-ellipsis">' + feed.Name + '</p></div>');

        if (feed.OriginalFeedName && feed.OriginalFeedName.replace("(Google News)", "").trim() !== feed.Name.replace("(Google News)", "").trim()) {
            html.push('<div><p class="feedAbout-title">Original Feed Name: </p><p class="feedAbout-body-ellipsis">' + feed.OriginalFeedName + '</p></div>');
            height += 30;
        }

        html.push('<div style="margin-top: 10px;margin-bottom: 10px;"><p class="feedAbout-title">Feed Url: </p><a href="' + feed.Url + '" class="feedAbout-body-ellipsis feedAbout-body-link" target="_blank">' + feed.Url + '</a></div>');
        html.push('<div><p class="feedAbout-title">Feed Domain: </p><p class="feedAbout-body-ellipsis">' + feed.SiteUrl + '</p></div>');

        if (feed.Description) {
            html.push('<div><p class="feedAbout-title">Description: </p><p class="feedAbout-body-ellipsis">' + feed.Description + '</p></div>');
            height += 30;
        }

        if (feed.Url.indexOf("news.google.com") > -1) {
            var searchTerm = NewsTab.Helpers.GetParameterFromUrl("q", feed.Url);
            var location = NewsTab.Helpers.GetParameterFromUrl("geo", feed.Url);
            if (location) {
                html.push('<div><p class="feedAbout-title">Location: </p><p class="feedAbout-body-ellipsis">' + location + '</p></div>');
                height += 30;
            } else if (searchTerm) {
                html.push('<div><p class="feedAbout-title">Search Term: </p><p class="feedAbout-body-ellipsis">' + searchTerm + '</p></div>');
                height += 30;
            }
        }
        if (feed.TotalSubscribers > 0) {
            html.push('<div><p class="feedAbout-title">Popularity: </p>');
            html.push('<div class="popularity-star" style="margin-left: 92px;">');
            html.push('<span class="popularity-star-full ' + SocialTab.HTML_PopularityRating(feed.TotalSubscribers) + '"></span>');
            html.push('</div>');
            html.push('</div>');
            height += 30;
        }

        $content.html(html.join(""));

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: "<div class=\"title\"><h2>About</h2></div>",
            Height: "auto",
            Width: "100%",
            MaxHeight: height,
            MaxWidth: "700",
            Text: $content,
            FullscreenOption: true,
            Buttons: [{ Name: "Ok", Action: "Close" }]
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_CopyMoveFeed: function (section, feed, action) {
        var self = this;

        var title = "", actionButton = "";

        switch (action) {
            case "COPY": {
                title = "<div class=\"title\"><h2>Copy feed to:</h2></div>";
                actionButton = "Copy";
                break;
            }
            case "MOVE": {
                title = "<div class=\"title\"><h2>Move feed to:</h2></div>";
                actionButton = "Move";
                break;
            }
            case "COPY_ALL": {
                title = "<div class=\"title\"><h2>Copy all feeds to:</h2></div>";
                actionButton = "Copy All";
                break;
            }
            case "MOVE_ALL": {
                title = "<div class=\"title\"><h2>Move all feeds to:</h2></div>";
                actionButton = "Move All";
                break;
            }
        }

        var $modal = $("<div/>", { "class": "copyMoveModal" });

        var html = [];
        html.push('<ul class="gridList sections"></ul>');
        $modal.html(html.join(""));

        var $sections = $(".sections", $modal);

        self.Sections.forEach(function (s) {
            if (s.ShowInSourceList && s.SectionTypeId !== Section.TYPES.Social && s.Id !== section.Id) {
                $sections.append(self._$copyMoveSectionItem(s, action.replace("_ALL", "")));
            }
        });

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Title: title,
            Width: "100%",
            Height: "auto",
            MaxWidth: "700",
            MaxHeight: "570",
            Text: $modal,
            FullscreenOption: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: actionButton,
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i>");

                        switch (action) {
                            case "COPY": {
                                SocialTab.CopyFeed(feed.Id, feed.Name, feed.Url, feed.SiteUrl.replace(/(\r\n|\n|\r)/gm, ""), feed.Icon);
                                break;
                            }
                            case "MOVE": {
                                SocialTab.MoveFeed(feed.Id, feed.Name, section.Id, feed.Url, feed.SiteUrl.replace(/(\r\n|\n|\r)/gm, ""), feed.Icon);
                                break;
                            }
                            case "COPY_ALL": {
                                SocialTab.CopyAllFeed(section.Id);
                                break;
                            }
                            case "MOVE_ALL": {
                                SocialTab.MoveAllFeed(section.Id);
                                break;
                            }
                        }

                        //self.API_RemoveFeed(section, feed, {
                        //    SuccessCallback: function () {
                        //        NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                        //    },
                        //    ErrorCallback: function () {
                        //        $modal.removeClass("disabled");
                        //        $(".remove", $modal).html(actionButton);

                        //        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        //            Title: "Remove Feed",
                        //            Text: "Failed to remove " + section.Name + " section!",
                        //            Color: "red"
                        //        });
                        //    }
                        //});
                    },
                    Classes: ["blue", "save"]
                }
            ],
            OnResize: function ($modal) {
                $(".gridList li", $modal).css("font-size", ($(".gridList li", $modal).first().width() / 10) + "px");
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },


    Modal_DisconnectTwitterSection: function (section) {
        var self = this;
        var acc = Feed.Account(Feed.TYPES.TWITTER);

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: $('<h2>Disconnect "<span style="color:' + section.Color + '">' + section.Name + '</span>"</h2><p>Disconnect your ' + section.Name + ' account (' + (acc != null ? acc.Username : 'none') + ') from NewsTab.</p>'),
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Disconnect",
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Disconnecting...");
                        NewsTab.Modules.TwitterManager.Disconnect({
                            RemoveSection: false,
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".remove", $modal).html("Disconnect");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Disconnect Twitter",
                                    Text: "Failed to disconnect " + section.Name + "!",
                                    Color: "red"
                                });
                            }
                        });
                    },
                    Classes: ["red", "remove"]
                }

            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_RemoveTwitterSection: function (section) {
        var self = this;
        var acc = Feed.Account(Feed.TYPES.TWITTER);

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: $('<h2>Remove "<span style="color:' + section.Color + '">' + section.Name + '</span>"</h2><p>Disconnect your ' + section.Name + ' account (' + (acc != null ? acc.Username : 'none') + ') from NewsTab and remove the ' + section.Name + ' section.</p>'),
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: "Remove",
                    Action: function (e, $sender, $modal) {
                        $modal.addClass("disabled");
                        $(".remove", $modal).html("<i class=\"icon\"><span class=\"spinner\"></span></i> Removing...");
                        NewsTab.Modules.TwitterManager.Disconnect({
                            RemoveSection: true,
                            SuccessCallback: function () {
                                NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                            },
                            ErrorCallback: function () {
                                $modal.removeClass("disabled");
                                $(".remove", $modal).html("Remove");

                                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                    Title: "Remove Twitter",
                                    Text: "Failed to remove " + section.Name + "!",
                                    Color: "red"
                                });
                            }
                        });
                    },
                    Classes: ["red", "remove"]
                }

            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_TwitterPreferences: function (section) {
        var self = this;

        var screenWidth = 950;

        if ($(window).width() < screenWidth) {
            screenWidth = $(window).width() * 0.95;
        }

        var $content = $("<div/>", {});

        var html = [];

        html.push('<h3>Twitter Feeds<br/><span>Select which feed will be visible on your dashboard</span></h3>');
        var feeds = $.grep(section.Feeds, function (c) { return (c.Name !== "None"); });
        if (feeds.length > 0) {

            html.push('<ul class="twitterFeeds">');
            for (var i = 0, l = feeds.length; i < l; i++) {
                html.push('<li data-feed-id="' + feeds[i].Id + '">');
                if (feeds[i].Name != "Timeline") {
                    html.push('<div class="checkContainer">');
                    html.push('<input type="checkbox" id="twitterfeed_' + feeds[i].Id + '" ' + (feeds[i].ShowInList ? 'checked=""' : '') + '/><label for="twitterfeed_' + feeds[i].Id + '">' + feeds[i].Name + (feeds[i].IsList ? ' [List]' : '') + '</label>');
                    html.push('</div>');
                }
                else {
                    html.push('<span>' + feeds[i].Name + '</span>');
                }
                html.push('</li>');
            }
            html.push('</ul>');
        }

        $content.html(html.join(""));

        NewsTab.Modules.NotificationsManager.Open_Modal({
            Id: "twitterListPreferences",
            Title: '<div class="title"><span class="icon-twitter"></span> Twitter <span class="icon-angle-right"></span> Preferences</div>',
            Height: "100vh",
            Width: "100%",
            MaxWidth: screenWidth,
            Text: $content,
            FullscreenOption: true,
            Buttons: [{ Name: "Cancel", Action: "Close", Classes: ["cancel", "gray"] }, {
                Name: "Apply", Action: function (e, $sender, $modal) {
                    var $feeds = $(".twitterFeeds li", $modal);
                    var visibleFeeds = $(".twitterFeeds li input:checked", $modal);

                    if (visibleFeeds.length == 0) {
                        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                            Title: "Twitter Preferences",
                            Color: "red",
                            Text: "At least one feed needs to be enabled"
                        });
                    }
                    else {
                        for (var i = 0, l = $feeds.length; i < l; i++) {
                            var enabled = $feeds.eq(i).find("input").is(":checked");
                            var id = $feeds.eq(i).data("feed-id");
                            var feed = $.grep(section.Feeds, function (t) { return (t.Id === id) });
                            self.ChangeFeedVisibility(section, feed[0], enabled, false);
                        }

                        if (self.CurrentSection.CurrentFeed.ShowInList == false) {
                            var visibleFeed = $.grep(self.CurrentSection.Feeds, function (t) { return (t.ShowInList === true) });
                            if (visibleFeed.length > 0) {
                                self.ChangeFeed(self.CurrentSection.Id, visibleFeed[0].Id);
                            }
                            else {
                                self.ChangeFeed(self.CurrentSection.Id, self.CurrentSection.Feeds[0].Id);
                            }
                        }

                        $(document.body).trigger("section:updated", [section]);
                    }
                }
            }]
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },

    Modal_TwitterFeedVisibility: function (section, feed) {
        var self = this;

        var title = '<h2>' + (feed.ShowInList ? 'Hide ' : 'Show ') + '<span style="color:' + section.Color + '">' + feed.Name + '<span></h2>';
        var message = '<p>' + (feed.ShowInList ?
            "You can add it back from the Preferences menu in your Twitter section or in Organize by selecting the Twitter Section." :
            "You can hide it again from the Preferences menu in your Twitter section or in Organize by selecting the Twitter Section.") + '</p>';

        NewsTab.Modules.NotificationsManager.Open_Modal({
            ShowTitle: false,
            Height: "auto",
            MaxHeight: "285",
            Text: title + message,
            CenterText: true,
            Buttons: [
                {
                    Name: "Cancel",
                    Action: "Close",
                    Classes: ["cancel", "gray", "float-left"]
                },
                {
                    Name: feed.ShowInList ? "Hide" : "Show",
                    Action: function (e, $sender, $modal) {
                        self.ChangeFeedVisibility(section, feed, !feed.ShowInList, true);
                        NewsTab.Modules.NotificationsManager.Close_Modal($modal);
                    },
                    Classes: ["red"]
                }
            ],
            OnShow: function ($modal) {
                $(".cancel", $modal).focus();
            }
        });

        if (NewsTab.Modules.Context !== undefined) {
            NewsTab.Modules.Context.inputFocusKeys();
        }
    },
    /* API */
    API_GetAutocompleteData: function (release) {
        var search = this.value, items = [];
        if (NewsTab.Modules.XhrManager.Xhr_GetAutocompleteData && NewsTab.Modules.XhrManager.Xhr_GetAutocompleteData.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_GetAutocompleteData.abort();
        }

        if (search[0] === "\"" || search[0] === "'") {
            items.push(
                {
                    Query: search,
                    TopicSize: 0,
                    SiteSize: 0,
                    Id: -1,
                    IconUrl: "https://upload.wikimedia.org/wikipedia/commons/0/09/Google_News_Logo.png",
                    NumberOfFeeds: -1,
                    Name: search,
                    Type: "GOOGLENEWS",
                    Value: search
                });
            release(items);
        } else {
            var callUrl = NewsTab.ApiUrl_Feeds + "suggestions?query=" + encodeURIComponent(search);
            $("#contentFeeds").removeClass("showSideSections");

            NewsTab.Modules.XhrManager.Xhr_GetAutocompleteData = $.ajax({
                url: callUrl,
                timeout: 25000,
                error: function (error) {
                    if (error.statusText != "abort") {
                        console.error("ERROR - Autocomplete:");
                        console.error(error);
                    }
                }
            });

            NewsTab.Modules.XhrManager.Xhr_GetAutocompleteData.then(function (response) {
                items.push({
                    Query: search
                });

                if (search.indexOf("http://") === -1 && search.indexOf("https://") === -1 && search.indexOf("www.") === -1) {
                    items.push({
                        Id: -1,
                        IconUrl: "http://cdn-storage-staging-nt.newstab.com/cdnstaging/Media/Images/googleNewsIcon_Keyword.png",
                        NumberOfFeeds: -1,
                        Name: search,
                        Type: "GOOGLENEWS",
                        Value: search
                    });
                }

                response.forEach(function (item) {
                    items.push(item);
                });

                release(items);

                SocialTab.MarkSelectText(search);
            });
        }
    },

    API_GetPublishersByRegion: function (options) {
        var self = this;

        if (NewsTab.Modules.XhrManager.Xhr_GetPublishersByRegion && NewsTab.Modules.XhrManager.Xhr_GetPublishersByRegion.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_GetPublishersByRegion.abort();
        }

        var opt = $.extend(NewsTab.BASE_REQUEST_OPTIONS, { Region: "US" }, options);

        if (opt.Region.indexOf("US") < 0) {
            opt.Region = opt.Region.split("_")[0];
        }
        var callUrl = NewsTab.ApiUrl_Feeds + "GetPublisherByRegion?region=" + opt.Region;

        NewsTab.Modules.XhrManager.Xhr_GetPublishersByRegion = $.ajax({
            url: callUrl,
            timeout: 10000,
            beforeSend: function () {
                if (opt.BeforeSendCallback) {
                    opt.BeforeSendCallback();
                }
            },
            error: function (error) {
                console.error("ERROR - API_GetPublishersByRegion:", error);

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            },
            success: function (data) {
                if (data) {
                    self.RegionPublishers = [];
                    data.forEach(function (d) {
                        self.RegionPublishers.push({
                            Id: d.Id,
                            PublisherName: d.PublisherName !== "" ? d.PublisherName : d.Name,
                            Color: d.Color,
                            IconUrl: "http://cdn-storage-staging-nt.newstab.com/cdnstaging/publishersscreenshots/" + d.Name + ".png",
                            SiteUrl: d.Name,
                            WebSiteUrl: d.Name,
                            NumberOfFeeds: d.NumberOfFeeds,
                            IsActive: true,
                            DateCreated: new Date().toLocaleString(),
                            PopularityIndex: d.PopularityIndex,
                            Type: d.Type,
                            Region: d.Region,
                            TotalRank: d.TotalRank,
                            SubscribersScore: d.SubscribersScore,
                            Domain: d.Domain,
                            IconUrlWidth: d.IconUrlWidth,
                            IconUrlHeight: d.IconUrlHeight,
                            DisplayType: "REGION"
                        });
                    });
                }

                if (opt.SuccessCallback) {
                    opt.SuccessCallback(data);
                }
            }
        });
    },

    API_GetPopularPublishersTopics: function (beforeSendCallback, successCallback, errorCallback) {
        if (NewsTab.Modules.XhrManager.Xhr_GetPopular && NewsTab.Modules.XhrManager.Xhr_GetPopular.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_GetPopular.abort();
        }

        var callUrl = NewsTab.ApiUrl_Feeds + "getpopular?maxTopics=33&maxSites=60";
        NewsTab.Modules.XhrManager.Xhr_GetPopular = $.ajax({
            url: callUrl,
            timeout: 10000,
            beforeSend: function () {
                if (beforeSendCallback) {
                    beforeSendCallback();
                }
            },
            success: function (data) {
                if (data) {
                    data.forEach(function (d) {
                        if (d.Type === "SITE") {
                            NewsTab.Modules.SectionsManager.PopularPublishers.push({
                                IconUrl: "http://cdn-storage-staging-nt.newstab.com/cdnstaging/popularpublisherslogos/" + d.Name + ".png",
                                //IconUrl: d.IconUrl,
                                Id: d.Id,
                                SiteUrl: d.Name,
                                WebSiteUrl: d.WebSiteUrl,
                                PublisherName: d.PublisherName,
                                IsActive: true,
                                NumberOfFeeds: d.NumberOfFeeds,
                                DateCreated: new Date().toLocaleString(),
                                Type: "rss",
                                RssUrl: d.RssUrl,
                                Color: d.Color,
                                PopularityIndex: d.PopularityIndex,
                                IconUrlWidth: d.IconUrlWidth,
                                IconUrlHeight: d.IconUrlHeight,
                                DisplayType: d.Type
                            });
                        } else if (d.Type === "TOPIC") {
                            NewsTab.Modules.SectionsManager.PopularTopics.push({
                                IconUrl: d.IconUrl,
                                Id: d.Id,
                                SiteUrl: d.SiteUrl,
                                WebSiteUrl: d.WebSiteUrl,
                                Name: d.Name,
                                IsActive: true,
                                NumberOfFeeds: d.NumberOfFeeds,
                                DateCreated: new Date().toLocaleString(),
                                Type: "rss",
                                RssUrl: d.RssUrl,
                                Color: d.Color,
                                DisplayType: d.Type
                            });
                        }
                    });

                    if (successCallback) {
                        successCallback(data);
                    }
                }
            },
            error: function (error) {
                console.error("ERROR - API_GetPopularPublishersTopics:", error);

                if (errorCallback) {
                    errorCallback(error);
                }
            }
        });
    },

    API_SearchFeed: function (options) {
        var opt = $.extend({
            Url: "",
            SkipRekords: 0,
            TakeRecords: 10,
            SearchType: "SITE",
            BeforeSendCallback: null,
            ErrorCallback: null,
            SuccessCallback: null
        }, options);

        if (NewsTab.Modules.XhrManager.Xhr_SearchFeedsXhr && NewsTab.Modules.XhrManager.Xhr_SearchFeedsXhr.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_SearchFeedsXhr.abort();
        }

        var callUrl = NewsTab.ApiUrl_Feeds + "search?query=" + encodeURIComponent(opt.Url) + "&searchType=" + opt.SearchType + "&skipRecords=" + opt.SkipRekords + "&takeRecords=" + opt.TakeRecords;
        NewsTab.Modules.XhrManager.Xhr_SearchFeedsXhr = $.ajax({
            url: callUrl,
            timeout: 10000,
            crossDomain: true,
            cache: false,
            beforeSend: function () {
                if (opt.BeforeSendCallback) {
                    opt.BeforeSendCallback();
                }
            },
            error: function (error) {
                console.error("ERROR - API_SearchFeed:", error);
                if (opt.ErrorCallback) {
                    opt.ErrorCallback(error);
                }
            },
            success: function (data) {
                if (opt.SuccessCallback) {
                    opt.SuccessCallback(data);
                }
            }
        });
    },

    API_GetFeedsByRegion: function (regionCode, callback) {
        var self = this;
        if (regionCode === undefined || regionCode === null || regionCode === "") {
            regionCode = "US";
        }

        $.ajax({
            url: NewsTab.ApiUrl_Global + "get/defaultfeedsbyregion/" + regionCode,
            type: "GET",
            contentType: "application/json; charset=utf-8",
            cache: false,
            success: function (data) {
                if (data.length > 0) {
                    self.UpdateExtensionSection(data);
                    if (callback) {
                        callback();
                    }
                }
            }
        });
    },

    API_GetGoogleNewsCategories: function (options) {
        var opt = $.extend({
            Url: "",
            SkipRekords: 0,
            TakeRecords: 10,
            BeforeSendCallback: null,
            ErrorCallback: null,
            SuccessCallback: null
        }, options);

        if (NewsTab.Modules.XhrManager.Xhr_SearchGoogleNews && NewsTab.Modules.XhrManager.Xhr_SearchGoogleNews.readystate !== 4) {
            NewsTab.Modules.XhrManager.Xhr_SearchGoogleNews.abort();
        }

        var callUrl = NewsTab.ApiUrl_Global + "get/googlenewscategories?subCategories=true&edition=" + NewsTab.Modules.RegionManager.AddFeedEditionGnValue +
            "&skipRecords=" + opt.SkipRekords + "&takeRecords=" + opt.TakeRecords;
        NewsTab.Modules.XhrManager.Xhr_SearchGoogleNews = $.ajax({
            url: callUrl,
            headers: { Authorization: "Bearer " + NewsTab.Token },
            timeout: 16000,
            crossDomain: true,
            beforeSend: function () {
                if (opt.BeforeSendCallback) {
                    opt.BeforeSendCallback();
                }
            },
            error: function (error) {
                console.error("ERROR - API_GetGoogleNewsCategories:", error);
                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            },
            success: function (data) {
                if (opt.SuccessCallback) {
                    opt.SuccessCallback(data);
                }
            }
        });
    },
    /**/
    API_GetTwitterTags: function (query) {
        var self = this;

        $(".searchTagResult").remove();
        var callUrl = NewsTab.ApiUrl_Base + "api/twitter/get/getsearchtags/" + encodeURIComponent(query);

        $.ajax({
            url: callUrl,
            cache: false,
            headers: { Authorization: 'Bearer ' + NewsTab.Token },
            error: function (error) {
                if (self.ShowLoadErrors) {
                    console.error("ERROR - GetSearchTags:");
                    console.error(error);
                }

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Twitter",
                    Color: "red",
                    Text: "Please connect a Twitter Account first!"
                });
            }
        }).then(function (data) {
            if (data) {
                var twitterSection = $.grep(self.Sections, function (e) { return e.Name === "Twitter" });
                $(".searchTagResult").remove();
                var height = $("#addNewContent").height() - 180;

                var hasUsers = false, hasHashs = false;
                var html = "<div style=\"height:" + height + "px\" class=\"searchTagResult twoColumns\">";
                html += "<div class=\"searchTagResultStates\"><span class=\"icon-twitter\"></span> Results for <strong>" + query + "</strong> (" + data.length + " feeds):</div>";
                var htmlHash = "<ul class=\"searchTagResultHash\">";
                var htmlUser = "<ul class=\"searchTagResultUser\">";
                var tabIndex = 1010;
                for (var i = 0, l = data.length; i < l; i++) {
                    if (data[i].ProfilePictureUrl != null && data[i].ProfilePictureUrl[0] == "#") {
                        var feed = $.grep(twitterSection[0].Feeds, function (e) { return e.Name === "#" + data[i].Username });
                        hasHashs = true;
                        htmlHash += "<li data-type=\"hashtag\" data-value=\"" + data[i].Username + "\" " + (feed.length > 0 ? "class=\"disabled\"" : "") + " tabindex=\"" + tabIndex + "\">";
                        htmlHash += "<div class=\"imageContainer\"><span class=\"hash\">#</span><div class=\"previewBtn\"><a href=\"#\" class=\"btn blue preview\" tabindex=\"" + (++tabIndex) + "\">Preview</a></div></div>";
                        htmlHash += "<i class=\"icon-add button\"></i><div class=\"filterDesc\">#" + data[i].Username + "</div>";
                        htmlHash += "<div class=\"feedPreview\"></div>";
                        htmlHash += "</li>";
                    }
                    else {
                        var feed = $.grep(twitterSection[0].Feeds, function (e) { return e.Name === "@" + data[i].Username });
                        hasUsers = true;
                        htmlUser += "<li data-type=\"usertag\" data-value=\"" + data[i].Username + "\" " + (feed.length > 0 ? "class=\"disabled\"" : "") + " tabindex=\"" + tabIndex + "\">";
                        htmlUser += "<div class=\"imageContainer\" style=\"background-image:url('" + data[i].ProfilePictureUrl + "');\"><div class=\"previewBtn\"><a href=\"#\" class=\"btn blue preview\" tabindex=\"" + (++tabIndex) + "\">Preview</a></div></div>";
                        htmlUser += "<i class=\"icon-add button\"></i><div class=\"filterDesc\">@" + data[i].Username + "<span>" + data[i].FullName + "</span></div>";
                        htmlUser += "<div class=\"feedPreview\"></div>";
                        htmlUser += "</li>";
                    }
                    tabIndex++;
                }
                if (!hasHashs) {
                    htmlHash += "<div class=\"table\"><div class=\"content\"><div class=\"twitterFeedForm\"><span class=\"icon-twitter\"></span><br/>No matching twitter hashtags found!</div></div></div>";
                }
                if (!hasUsers) {
                    htmlUser += "<div class=\"table\"><div class=\"content\"><div class=\"twitterFeedForm\"><span class=\"icon-twitter\"></span><br/>No matching twitter users found!</div></div></div>";
                }
                htmlHash += "</ul>";
                htmlUser += "</ul>";
                html += htmlUser;
                html += htmlHash;
                html += "</div>";
                $("#contentTwitterFeed .table").before(html);

                $("li", "#contentTwitterFeed").on("click", function (e) {
                    $(this).toggleClass("selected");
                    var nbrOfSel = $("li.selected", "#contentTwitterFeed .searchTagResult").length;
                    if (nbrOfSel > 0) {
                        $(".searchTagResult", "#contentTwitterFeed").css("height", "calc(100% - 50px)");
                        $("#addTwitterFeedBtnField").show();
                    }
                    else {
                        $(".searchTagResult", "#contentTwitterFeed").css("height", "100%");
                        $("#addTwitterFeedBtnField").hide();
                    }
                });

                $("li .preview", "#contentTwitterFeed").on("click", function (e) {
                    NewsTab.Modules.AddNewContent.Open_TwitterFeedPreview($(this).closest("li"));
                });
            }
        });
    },

    API_AddTwitterFeed: function (filter, setupMenuList) {
        var self = this;

        var section = $.grep(self.Sections, function (x) { return x.Name === "Twitter" });
        if (section.length > 0 && filter.length < 30) {
            section = section[0];
            var filterValue = filter.substring(1);
            var callUrl;
            if (filter[0] === "#") {
                callUrl = NewsTab.ApiUrl_Base + "api/" + section.ClassName + "/post/specialfeed/hashtag/" + section.Id + "/" + filterValue;
            } else
                callUrl = NewsTab.ApiUrl_Base + "api/" + section.ClassName + "/post/specialfeed/usertag/" + section.Id + "/" + filterValue;
        }

        $.ajax({
            url: callUrl,
            type: "POST",
            contentType: "application/json",
            headers: {
                Authorization: "Bearer " + NewsTab.Token
            },
            error: function (error) {
                console.error("ERROR - API_AddTwitterFeed:");
                console.error(error);
            }
        }).then(function (data) {
            if (data != null) {
                var feed = new Feed().NewInit(data);
                feed.SectionId = section.Id;

                section.Feeds.push(feed);

                if (setupMenuList) {
                    self.ChangeFeed(section.Id, section.Feeds[section.Feeds.length - 1].Id);

                    if (filter[0] === "#") {
                        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                            Title: "New Twitter feed",
                            Color: "green",
                            Text: "New Hashtag Feed added to Twitter!"
                        });
                    }
                    else {
                        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                            Title: "New Twitter feed",
                            Color: "green",
                            Text: "New User Feed added to Twitter!"
                        });
                    }

                    $(document.body).trigger("newcontent");
                }
            }
        });
    },
    /**/
    API_AddSection: function (id, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "post/section/" + id;

        $.ajax({
            url: callUrl,
            headers: {
                Authorization: 'Bearer ' + NewsTab.Token,
                Layout: self.CurrentLayout
            },
            type: "POST",
            contentType: "application/json",
            dataType: "json",
            processdata: true,
            error: function (error) {
                console.error("ERROR - AddSection:", error);

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
                if (opt.CompleteCallback) {
                    opt.CompleteCallback();
                }
            }
        }).then(function (response) {
            if (response) {
                if (response.Id !== NewsTab.EMPTY_GUID) {
                    var section = new Section().Init(response);
                    self.AddSection(section);

                    if (opt.SuccessCallback) {
                        opt.SuccessCallback(response);
                    }
                    if (opt.CompleteCallback) {
                        opt.CompleteCallback();
                    }

                    return true;
                }
            }

            if (opt.ErrorCallback) {
                opt.ErrorCallback();
            }
            if (opt.CompleteCallback) {
                opt.CompleteCallback();
            }
        });
    },

    API_AddNewSection: function (section, options) {
        var self = this;

        var opt = $.extend({}, NewsTab.BASE_REQUEST_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "post/section";

        $.ajax({
            url: callUrl,
            type: "POST",
            headers: {
                Authorization: "Bearer " + NewsTab.Token,
                Layout: self.CurrentLayout
            },
            contentType: "application/json; charset=utf-8",
            data: JSON.stringify(section),
            error: function (error) {
                console.error("ERROR - API_AddNewSection:", error);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Add New Section",
                    Text: "Failed to add new section!",
                    Color: "red"
                });

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
                if (opt.CompleteCallback) {
                    opt.CompleteCallback();
                }
            }
        }).then(function (response) {
            if (response) {
                if (response.Id !== NewsTab.EMPTY_GUID) {
                    var section = new Section().Init(response);
                    self.AddSection(section);

                    if (opt.SuccessCallback) {
                        opt.SuccessCallback(response);
                    }
                    if (opt.CompleteCallback) {
                        opt.CompleteCallback();
                    }
                    return true;
                }
            }

            if (opt.ErrorCallback) {
                opt.ErrorCallback();
            }
            if (opt.CompleteCallback) {
                opt.CompleteCallback();
            }
        });
    },

    API_AddChromeSection: function (options) {
        var self = this;

        var opt = $.extend({}, NewsTab.BASE_REQUEST_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "post/chromesection/" + NewsTab.Extension.NId;

        $.ajax({
            url: callUrl,
            type: "POST",
            headers: {
                Authorization: "Bearer " + NewsTab.Token,
                Layout: self.CurrentLayout
            },
            contentType: "application/json; charset=utf-8",
            error: function (jqXHR, textStatus, errorThrown) {
                console.error("ERROR - API_AddChromeSection:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Add New Section",
                    Text: "Failed to add new section!",
                    Color: "red"
                });

                if (opt.ErrorCallback) opt.ErrorCallback(jqXHR, textStatus, errorThrown);
            },
            success: function (response) {
                if (opt.SuccessCallback) opt.SuccessCallback(response);
            },
            complete: function (jqXHR, textStatus) {
                if (opt.CompleteCallback) opt.CompleteCallback(jqXHR, textStatus);
            }
        });
    },

    API_EditSection: function (section, name, color, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "post/section/" + section.Id + "/" + encodeURIComponent(name) + "/" + color.slice(1);

        $.ajax({
            url: callUrl,
            type: "GET",
            headers: { Authorization: "Bearer " + NewsTab.Token },
            beforeSend: function () {
                if (opt.BeforeSendCallback) {
                    opt.BeforeSendCallback();
                }
            },
            error: function (xhr, status, errorThrown) {
                console.error("ERROR - API_EditSection:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Edit " + section.Name + " Section",
                    Text: xhr.responseText,
                    Color: "red"
                });

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            },
            success: function (response) {
                if (response.Id !== NewsTab.EMPTY_GUID && response.ErrorCode === 0) {
                    section.Name = response.Name;
                    section.Color = response.Color;

                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Edit " + section.Name + " Section",
                        Text: response.Message,
                        Color: "green"
                    });

                    $(document.body).trigger("setion:updated", [section]);

                    if (opt.SuccessCallback) {
                        opt.SuccessCallback(response);
                    }
                }
                else {
                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Edit " + section.Name + " Section",
                        Text: response.Message,
                        Color: "red"
                    });

                    if (opt.ErrorCallback) {
                        opt.ErrorCallback();
                    }
                }
            }
        });
    },

    API_RemoveSection: function (section, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Global + "delete/section/" + section.Id;

        $.ajax({
            url: callUrl,
            headers: { Authorization: "Bearer " + NewsTab.Token },
            type: "DELETE",
            contentType: "application/json",
            dataType: "json",
            processdata: true,
            error: function (xhr, status, errorThrown) {
                console.error("ERROR - RemoveSection:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Remove " + section.Name + " Section",
                    Text: xhr.responseText,
                    Color: "red"
                });

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            }
        }).then(function (response) {
            if (response) {
                if (response.Id !== NewsTab.EMPTY_GUID) {
                    NewsTab.Helpers.RemoveItemFromArray(self.Sections, section);
                    NewsTab.Helpers.RemoveItemFromArray(self.LastActiveSections, section.Id);


                    if (self.LastActiveSections.length < self.NumberOfVisibleSections &&
                        self.Sections.length > self.NumberOfVisibleSections) {

                        var added = false;
                        $.grep(self.Sections, function (s) {
                            if (!added && jQuery.inArray(s.Id, self.LastActiveSections) === -1 && s.ShowInSourceList) {
                                self.LastActiveSections.push(s.Id);
                                added = true;
                            }
                        });
                    }

                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Remove Section",
                        Text: response.Message,
                        Color: "green"
                    });

                    if (self.CurrentSection.Id === section.Id) {
                        self.ChangeFeed(self.LastActiveSections[0], null);
                    }

                    $(document.body).trigger("setion:removed", [section]);

                    if (opt.SuccessCallback) {
                        opt.SuccessCallback();
                    }

                    return true;
                }
            }

            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Remove Section",
                Text: response.Message,
                Color: "red"
            });

            if (opt.ErrorCallback) {
                opt.ErrorCallback();
            }
        });
    },

    API_RemoveSectionById: function (id, options) {
        var self = this;

        var section = $.grep(self.Sections, function (s) { return s.Id === id; });
        if (section.length > 0) {
            self.API_RemoveSection(section[0], options);
        } else {
            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Remove Section",
                Text: "Failed to find section with " + id + " Id!",
                Color: "red"
            });
        }
    },

    API_RemoveAllSectionFeeds: function (section, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Rss + "RemoveAllFeedsFromSection?sectionId=" + section.Id;

        $.ajax({
            url: callUrl,
            headers: { Authorization: "Bearer " + NewsTab.Token },
            type: "DELETE",
            dataType: "json",
            error: function (xhr, status, errorThrown) {
                console.error("ERROR - API_RemoveAllSectionFeeds:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Remove Feeds from " + section.Name,
                    Text: xhr.responseText,
                    Color: "red"
                });

                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            }
        }).then(function (response) {
            if (response) {
                var feedsToRemove = [];
                for (var i = 0, l = section.Feeds.length; i < l; i++) {
                    if (section.Feeds[i].Type.Id === Feed.TYPES.RSS.Id) {
                        feedsToRemove.push(section.Feeds[i]);
                    }
                }

                for (var i = 0, l = feedsToRemove.length; i < l; i++) {
                    NewsTab.Helpers.RemoveItemFromArray(section.Feeds, feedsToRemove[i]);
                }

                if (section.Feeds.length > 1) {
                    section.DefaultFeed = section.Feeds[1];
                } else {
                    section.DefaultFeed = section.Feeds[0];
                }

                if (self.CurrentSection.Id === section.Id) {
                    if (feedsToRemove.indexOf(section.CurrentFeed.Id) > -1) {
                        if (section.Feeds.length > 1) {
                            self.ChangeFeed(section.Id, section.Feeds[1].Id);
                        } else {
                            self.ChangeFeed(section.Id, section.DefaultFeed.Id);
                        }
                    }
                }

                section.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                $(document.body).trigger("setion:feedsremoved", [section]);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Remove Feeds from " + section.Name,
                    Text: "Successfully removed " + feedsToRemove.length + " feeds.",
                    Color: "green"
                });

                if (opt.SuccessCallback) {
                    opt.SuccessCallback();
                }

                return true;
            }

            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Remove Feeds from " + section.Name,
                Text: "Failed to remove any feeds.",
                Color: "red"
            });

            if (opt.ErrorCallback) {
                opt.ErrorCallback();
            }
        });
    },
    /**/
    API_EditFeed: function (sectionId, feedId, name, top, options) {
        var self = this;

        if (name.length > 350) {
            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Rename Feed",
                Color: "red",
                Text: "Feed name is too long."
            });
        } else if (name.length === 0) {
            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Rename Feed",
                Color: "red",
                Text: "Feed name is too short."
            });
        } else {
            var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

            var callUrl = NewsTab.ApiUrl_Rss + "RenameFeed?sectionId=" + sectionId + "&feedId=" + feedId + "&name=" + encodeURIComponent(name.trim()) + "&top=" + top;

            $.ajax({
                url: callUrl,
                contentType: "application/x-www-form-urlencoded;charset=ISO-8859-15",
                type: "POST",
                headers: {
                    Authorization: 'Bearer ' + NewsTab.Token
                },
                error: function (error) {
                    console.error("ERROR - RenameFeed:", error);
                    if (opt.ErrorCallback) {
                        opt.ErrorCallback();
                    }
                }
            }).then(function (response) {
                if (response.Success) {
                    var section = $.grep(self.Sections, function (e) { return e.Id === sectionId });
                    if (section.length > 0) {
                        section = section[0];

                        var feed = $.grep(section.Feeds, function (e) { return e.Id === feedId });
                        if (feed.length > 0) {
                            feed = feed[0];
                            feed.Name = name;

                            $(document.body).trigger("feed:updated", [section, feed]);
                        }
                    }
                    if (opt.SuccessCallback) {
                        opt.SuccessCallback(response);
                    }
                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Edit Feed",
                        Text: response.Message,
                        Color: "green"
                    });
                }
                else {
                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Edit Feed",
                        Text: response.Message,
                        Color: "red"
                    });
                    if (opt.ErrorCallback) {
                        opt.ErrorCallback();
                    }
                }
            });
        }
    },

    API_RemoveFeed: function (section, feed, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var callUrl = NewsTab.ApiUrl_Rss + "RemoveFeedFromSection";

        $.ajax({
            url: callUrl,
            contentType: "charset=utf-8",
            type: "POST",
            headers: { Authorization: 'Bearer ' + NewsTab.Token, 'SectionId': section.Id, 'FeedId': feed.Id },
            error: function (error) {
                console.error("ERROR - RemoveFeedFromSection:", error);
                if (opt.ErrorCallback) {
                    opt.ErrorCallback();
                }
            }
        }).then(function (response) {
            if (response === true) {
                NewsTab.Helpers.RemoveItemFromArray(section.Feeds, feed);

                if (section.DefaultFeed.Id === feed.Id) {
                    if (section.Feeds.length > 1) {
                        section.DefaultFeed = section.Feeds[1];
                    } else {
                        section.DefaultFeed = section.Feeds[0];
                    }
                }

                section.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                var j = 0;
                for (var i = 0, len = section.Feeds.length; i < len; i++) {
                    var order = section.Feeds[i].OrderNumber;
                    if (parseFloat(order) < 0) {
                        section.Feeds[i].OrderNumber = order;
                    } else {
                        section.Feeds[i].OrderNumber = j;
                        j++;
                    }
                }

                $(document.body).trigger("feed:removed", [section, feed]);

                if (self.CurrentSection.Id === section.Id) {
                    if (section.CurrentFeed.Id === feed.Id) {
                        if (section.Feeds.length > 1) {
                            self.ChangeFeed(section.Id, section.Feeds[1].Id);
                        } else {
                            self.ChangeFeed(section.Id, section.DefaultFeed.Id);
                        }
                    }
                }

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Remove " + feed.Name + " Feed",
                    Text: "Feed removed successfull",
                    Color: "green"
                });

                if (opt.SuccessCallback) {
                    opt.SuccessCallback(response);
                }
                return;
            }


            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                Title: "Remove " + feed.Name + " Feed",
                Text: "Failed to remove feed",
                Color: "red"
            });

            if (opt.ErrorCallback) {
                opt.ErrorCallback();
            }
        });
    },
    /**/
    API_ParseFeed: function (feed, options) {
        /// <summary>
        /// Get Articles from the API of a feed (standar rss feed or a special feed like "All Feed", "All" and "Best" of Home Section
        /// </summary>
        /// <param name="feed" type="Object">Feed object to be parsed. It can be a standard RSS feed or a special feed which agragets multiple RSS feeds, like "All" and "Best" of the Home Section, and the "All Feeds".</param>
        /// <param name="refresh" type="Boolean" optional="true">Is this a refresh? A refresh will reset the feed "Page Number" and empty the previously parsed articles. Default: false</param>
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        // Get a reference to the section this feed belongs to
        var section = self.GetSectionById(feed.SectionId, null);

        if (section !== null) {
            var tempFeedList = [];
            var i, j, len, len2;

            // Populate Array with feeds to parse
            switch (feed.Type) {
                case Feed.TYPES.HOME:  // HOME ALL
                case Feed.TYPES.TRENDING: // HOME BEST
                    for (i = 0, len = self.Sections.length; i < len; i++) {
                        for (j = 0, len2 = self.Sections[i].Feeds.length; j < len2; j++) {
                            if (self.Sections[i].Feeds[j].Type.Id === Feed.TYPES.RSS.Id) {
                                tempFeedList.push(self.Sections[i].Feeds[j]);
                            }
                        }
                    }
                    break;
                case Feed.TYPES.ALL: // ALL FEEDS
                    if (section.Feeds.length === 2) {
                        //SocialTab.ClearPlaceholders(false);
                        return;
                    }

                    for (i = 0, len = section.Feeds.length; i < len && i < 10; i++) {
                        if (section.Feeds[i].Type.Id === Feed.TYPES.RSS.Id) {
                            tempFeedList.push(section.Feeds[i]);
                        }
                    }
                    break;
                default: // STANDARD RSS FEED
                    tempFeedList.push(feed);
                    break;
            }

            if (tempFeedList.length > 0) {
                // Order feeds by "UsageCount"
                tempFeedList.sort(NewsTab.Helpers.Sort("UsageCount", true, parseInt));

                var feedtoparse = [];

                // Limit to MAX_NUMBER_OF_FEEDS_PER_REQUEST and populate a new feed array with the minimal data needed for the API call
                for (i = 0, len = tempFeedList.length; i < self.MAX_NUMBER_OF_FEEDS_PER_REQUEST && i < len; i++) {
                    feedtoparse.push({
                        FeedId: tempFeedList[i].Id,
                        FeedUrl: tempFeedList[i].Url,
                        IncludeInParsedArticlesList: true,
                        FeedUrlHash: tempFeedList[i].FeedUrlHash,
                        FeedName: tempFeedList[i].Name
                    });
                }

                var request = { "ParsedArticlesPageSize": self.PARSED_ARTICLES_PAGE_SIZE, "FeedRequests": feedtoparse, "CurrentPage": opt.Refresh ? 1 : feed.PageNbr };

                var invokerParams = "";
                var feedUrlHashes = "";
                var customOrderBy = false;

                if (feed.Type === Feed.TYPES.TRENDING) {
                    // Home BEST Feed
                    $.each(request.FeedRequests, function (index, feed) {
                        feedUrlHashes = feedUrlHashes + feed.FeedUrlHash + ",";
                    });
                    feedUrlHashes.substr(0, feedUrlHashes.length - 1);
                    invokerParams = "gettrendings?feedUrlHashes=" + feedUrlHashes;
                    customOrderBy = NewsTab.Helpers.GetLocalStorageItem("OrderTrending_" + NewsTab.User.Id) !== 1;
                } else if (request.FeedRequests && request.FeedRequests.length === 1) {
                    // Single Feed Request (Standard RSS Feed or Home ALL with just 1 feed)
                    invokerParams = "readsinglefeed?pageSize=" + request.ParsedArticlesPageSize +
                        "&currentPage=" + request.CurrentPage +
                        "&feedId=" + encodeURIComponent(request.FeedRequests[0].FeedId) +
                        "&feedUrl=" + encodeURIComponent(request.FeedRequests[0].FeedUrl);

                    if (section.SectionTypeId !== Section.TYPES.Home) {
                        customOrderBy = request.FeedRequests[0].FeedUrl.indexOf("news.google.com") > -1;
                    }

                    invokerParams += "&feedName=" + request.FeedRequests[0].FeedName;
                } else {
                    // Multiple Feeds (HOME ALL with more than 1 feed)
                    $.each(request.FeedRequests, function (index, feed) {
                        feedUrlHashes = feedUrlHashes + feed.FeedUrlHash + ",";
                    });
                    feedUrlHashes.substr(0, feedUrlHashes.length - 1);
                    invokerParams = "readmultiplefeeds?pageSize=" + request.ParsedArticlesPageSize +
                        "&currentPage=" + request.CurrentPage +
                        "&feedUrlHashes=" + feedUrlHashes;
                }

                if (NewsTab.IsExtension && NewsTab.Extension) {
                    invokerParams += "&nid=" + NewsTab.Extension.NId;
                }
                if (NewsTab.User) {
                    invokerParams += "&uid=" + NewsTab.User.Id;
                }

                // Mark the feed as "IsLoading" to prevent multiple request
                self.SetFeedIsLoading(feed, true);

                $.ajax({
                    url: NewsTab.FeedApiUrl_Feed + invokerParams,
                    type: "GET",
                    cache: !opt.Refresh,
                    error: function (error) {
                        console.error("ERROR - Parser filed for the following request:", request);
                        console.error("Error message is:", error);

                        feed.IsAtEnd = true;
                        self.SetFeedIsLoading(feed, false);

                        opt.ErrorCode = error.status;
                        if (!opt.LoadInbackground) {
                            NewsTab.Modules.ArticleView.DisplayFeedError(feed, opt);
                        }

                        if (opt.CompleteCallback) {
                            opt.CompleteCallback();
                        }
                    },
                    success: function (data) {
                        // Prepare and store the articles from the response
                        if (data && data.Result != undefined) {
                            var newArticles = [], backgroundArticles = [], article = null, sourceFeed = null;

                            for (var i = 0, len = data.Result.length; i < len; i++) {
                                // Get the source feed in case of agregated feed pars request, 
                                // this is requed in cases like the special feeds BEST and ALL of Home section
                                sourceFeed = self.GetFeedById(data.Result[i].FeedId, feed);

                                for (var j = 0, len1 = data.Result[i].FeedArticles.length; j < len1; j++) {
                                    article = new Article().Init(data.Result[i].FeedArticles[j]);
                                    article._sourceFeed = sourceFeed;
                                    if (article._sourceFeed) {
                                        article.SourceFeedId = article._sourceFeed.Id;
                                    }
                                    // Prevent duplicated articles to be added to the list by looking for a article with matching Alias
                                    var articleindex = feed.Items.map(function (x) { return x.Alias; }).indexOf(article.Alias);
                                    if (articleindex === -1) {
                                        // Limit the processed article number to PARSED_ARTICLES_PAGE_SIZE in case that the API returned more
                                        if (newArticles.length < self.PARSED_ARTICLES_PAGE_SIZE) {
                                            newArticles.push(article);
                                        }
                                    }
                                }
                            }

                            if (newArticles.length > 0) {
                                if (customOrderBy === true) {
                                    if (feed.Type === Feed.TYPES.TRENDING) {
                                        newArticles.sort(function (a, b) { return b.Rsr - a.Rsr; });
                                    } else {
                                        newArticles.sort(function (a, b) { return parseInt(b.CustomOrder) - parseInt(a.CustomOrder); });
                                    }
                                } else {
                                    newArticles.sort(function (a, b) { return new Date(b.PubDate) - new Date(a.PubDate); });
                                }

                                for (i = 0, len = newArticles.length; i < len; i++) {
                                    // Is article pinned
                                    if (NewsTab.Modules.ArticleView.PinnedItemAliases.indexOf(newArticles[i].Alias) > -1) {
                                        newArticles[i].Pinned = true;
                                    }

                                    newArticles[i].DestinationFeedId = feed.Id;

                                    if (!opt.LoadInbackground || opt.Refresh) {
                                        feed.Items.push(newArticles[i]);

                                        // Does this article need to be parsed 
                                        if (!newArticles[i].FullHtmlParsed) {
                                            NewsTab.Modules.ArticleView.UnparsedArticles.push(newArticles[i]);
                                        }
                                    }
                                    else {
                                        backgroundArticles = newArticles.concat(feed.Items);
                                    }
                                }


                                if (!opt.LoadInbackground) {
                                    if (feed.MinId === "X") {
                                        feed.MinId = feed.Items[0].Id;
                                    }
                                    feed.MaxId = feed.Items[feed.Items.length - 1].Id;

                                    NewsTab.Modules.ArticleView.DisplayFeedArticles(feed, opt);

                                    if (opt.StoreFeedItemsLocaly) {
                                        self.StoreFeedItemsLocaly(feed, feed.Items);
                                    }
                                }
                                else {
                                    if (opt.StoreFeedItemsLocaly) {
                                        self.StoreFeedItemsLocaly(feed, backgroundArticles);
                                    }
                                    $("#show_hide .refresh").addClass("newArticles");
                                }
                            } else if (feed.Items.length === 0 && !opt.LoadInbackground) {
                                NewsTab.Modules.ArticleView.DisplayFeedEmpty(feed);
                            }

                            if (!opt.LoadInbackground) {
                                feed.IsAtEnd = !data.MoreArticlesAvailable;
                                if (feed.IsAtEnd) {
                                    NewsTab.Modules.ArticleView.ClearPlaceholders(opt);
                                }

                                feed.LastUpdate = Math.floor(Date.now() / 1000); // seconds elapsed since January 1, 1970 00:00:00 UTC
                            }

                            //var expandForm = $("div:not(#itemFormHolder) > #socialTabItemForm");
                            //if (expandForm.length > 0) {
                            //    NewsTab.Modules.ArticleView.SetUp_ExpandViewById(expandForm.attr("data-article-id"));
                            //}

                            //if ($(window).scrollTop() + $(window).height() >= $(document).height() - 10) {
                            //    SocialTab.LoadMore();
                            //}
                        }
                        self.SetFeedIsLoading(feed, false);

                        if (opt.CompleteCallback) {
                            opt.CompleteCallback();
                        }
                        //SocialTab.ResetArticleHover();
                    }
                });
                //$("body").css({ height: "100%" });
            } else if (!opt.LoadInbackground) {
                NewsTab.Modules.ArticleView.DisplayFeedEmpty(feed);
            }
        }
    },

    API_ParseTwitterFeed: function (feed, options) {
        var self = this;
        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var section = self.GetSectionById(feed.SectionId, null);

        if (section !== null) {

            self.SetFeedIsLoading(feed, true);

            var tusername = "";
            var tu = $.grep(NewsTab.User.SocialNetworks, function (a) { return a.SourceId == 2 });
            if (tu.length) {
                tusername = tu[0].Username;
            }
            var callUrl = feed.ApiUrl.replace("{count}", feed.Count).replace("{min_id}", "X").replace("{max_id}", feed.MaxId) + "?_cache=" + tusername + feed.Id;
            if (callUrl.indexOf("top20tweets") >= 0) {
                feed.IsAtEnd = true;
            }

            if (feed.SpecialParameter) {
                callUrl = callUrl.replace("{special_parameter}", feed.SpecialParameter)
            }

            $.ajax({
                url: callUrl,
                cache: true,
                headers: { Authorization: "Bearer " + NewsTab.Token },
                error: function (error) {
                    console.error("ERROR - " + feed.Name + ":", error);
                    self.SetFeedIsLoading(feed, false);

                    if (opt.CompleteCallback) {
                        opt.CompleteCallback();
                    }
                }
            }).then(function (data) {
                var newArticles = [], backgroundArticles = [], article = null;

                if (data.Error == "Rate limit exceeded!") {
                    feed.RateLimit = true;
                }
                else {
                    feed.RateLimit = false;
                }

                for (var j = 0, len2 = data.Items.length; j < len2; j++) {
                    article = new Article().Init(data.Items[j]);

                    article._sourceFeed = feed;
                    if (article.SourceFeed().TextLinker) {
                        if (article.Title) {
                            article.Title = article.SourceFeed().TextLinker.link(article.Title);
                        }
                        if (article.Summary) {
                            article.Summary = article.SourceFeed().TextLinker.link(article.Summary);
                        }
                    }

                    var articleindex = feed.Items.map(function (x) { return x.Alias; }).indexOf(article.Alias);
                    if (articleindex === -1) {
                        newArticles.push(article);
                    } else {
                        //feed.Items[articleindex] = article;
                    }
                }

                if (newArticles.length > 0) {
                    for (i = 0, len = newArticles.length; i < len; i++) {
                        // Is article pinned
                        if (NewsTab.Modules.ArticleView.PinnedItemAliases.indexOf(newArticles[i].Alias) > -1) {
                            newArticles[i].Pinned = true;
                        }

                        newArticles[i].DestinationFeedId = feed.Id;

                        if (!opt.LoadInbackground) {
                            feed.Items.push(newArticles[i]);

                            // Does this article need to be parsed 
                            if (!newArticles[i].FullHtmlParsed && newArticles[i].ExternalUrl) {
                                NewsTab.Modules.ArticleView.UnparsedArticles.push(newArticles[i]);
                            }
                        }
                        else {
                            backgroundArticles = newArticles.concat(feed.Items);
                        }
                    }


                    if (!opt.LoadInbackground) {
                        if (feed.MinId === "X") {
                            feed.MinId = feed.Items[0].Id;
                        }
                        feed.MaxId = feed.Items[feed.Items.length - 1].Id;

                        NewsTab.Modules.ArticleView.DisplayFeedArticles(feed, opt);

                        if (opt.StoreFeedItemsLocaly) {
                            self.StoreFeedItemsLocaly(feed, feed.Items);
                        }
                    }
                    else {
                        if (opt.StoreFeedItemsLocaly) {
                            self.StoreFeedItemsLocaly(feed, backgroundArticles);
                        }
                        $("#show_hide .refresh").addClass("newArticles");
                    }
                }

                if (!opt.LoadInbackground) {
                    feed.IsAtEnd = !data.MoreArticlesAvailable;
                    if (feed.IsAtEnd) {
                        //SocialTab.ClearPlaceholders();
                    }

                    feed.LastUpdate = Math.floor(Date.now() / 1000); // seconds elapsed since January 1, 1970 00:00:00 UTC
                }

                self.SetFeedIsLoading(feed, false);

                if (opt.CompleteCallback) {
                    opt.CompleteCallback();
                }
            });
        }
    },

    API_ParsePinnedFeed: function (feed, options) {
        var self = this;

        var opt = $.extend({}, self.DEFAULT_FEED_PARSING_OPTIONS, options);

        var section = self.GetSectionById(feed.SectionId, null);

        if (section !== null) {

            self.SetFeedIsLoading(feed, true);
            var callUrl = NewsTab.ApiUrl_Global + "get/pinned/" + feed.Count + "/X/" + encodeURIComponent(feed.MaxId);

            $.ajax({
                url: callUrl,
                headers: { Authorization: "Bearer " + NewsTab.Token },
                cache: false,
                error: function (error) {
                    console.error("ERROR - " + feed.Name + ":", error);
                    self.SetFeedIsLoading(feed, false);

                    if (opt.CompleteCallback) {
                        opt.CompleteCallback();
                    }
                }
            }).then(function (data) {
                var newArticles = [], backgroundArticles = [], article = null;

                $.each(data.Items, function (index, a) {
                    if (NewsTab.Modules.ArticleView.PinnedItemAliases.indexOf(a.Alias) === -1) {
                        NewsTab.Modules.ArticleView.PinnedItemAliases.push(a.Alias);

                        var obj = $(".socialTabItem[feed-alias-id='" + a.FeedId + "_NT_" + a.Alias + "']");
                        if (!obj.hasClass("pinned")) {
                            obj.addClass("pinned");
                        }
                    }
                });


                for (var j = 0, len2 = data.Items.length; j < len2; j++) {
                    article = new Article().Init(data.Items[j]);

                    article.Pinned = true;
                    article._sourceFeed = self.GetFeedById(article.FeedId, feed);

                    if (article.SourceFeed() == null || article.SourceFeed().Type === Feed.TYPES.PINWALL) {
                        var otherParams = article.OtherParams.split("_NT_");
                        var type, publisher, siteurl = null;

                        if (otherParams.length >= 1) {
                            type = otherParams[0];
                        }
                        if (otherParams.length >= 2) {
                            publisher = otherParams[1];
                        }
                        if (otherParams.length >= 3) {
                            siteurl = otherParams[2];
                        }

                        var sourceFeed = new Feed().Init(
                            {
                                Id: 0,
                                Name: publisher,
                                SiteUrl: siteurl,
                                Type: type == "2" ? Feed.TYPES.TWITTER : Feed.TYPES.RSS,
                                Count: Feed.DEFAULT_COUNT,
                                OrderNumber: 0,
                                Layout: feed.Layout,
                                LayoutType: feed.LayoutType,
                                ShowInList: false
                            });
                        article._sourceFeed = sourceFeed;
                    }

                    var articleindex = feed.Items.map(function (x) { return x.Alias; }).indexOf(article.Alias);
                    if (articleindex === -1) {
                        newArticles.push(article);
                    }
                }

                if (newArticles.length > 0) {
                    for (i = 0, len = newArticles.length; i < len; i++) {

                        newArticles[i].DestinationFeedId = feed.Id;

                        if (!opt.LoadInbackground) {
                            feed.Items.push(newArticles[i]);

                            // Does this article need to be parsed 
                            if (!newArticles[i].FullHtmlParsed) {
                                NewsTab.Modules.ArticleView.UnparsedArticles.push(newArticles[i]);
                            }
                        }
                        else {
                            backgroundArticles = newArticles.concat(feed.Items);
                        }
                    }


                    if (!opt.LoadInbackground) {
                        if (feed.MinId === "X") {
                            feed.MinId = new Date(feed.Items[0].DateCreated).getTime();
                        }
                        feed.MaxId = new Date(feed.Items[feed.Items.length - 1].DateCreated).getTime();

                        NewsTab.Modules.ArticleView.DisplayFeedArticles(feed, opt);

                        if (opt.StoreFeedItemsLocaly) {
                            self.StoreFeedItemsLocaly(feed, feed.Items);
                        }
                    }
                    else {
                        if (opt.StoreFeedItemsLocaly) {
                            self.StoreFeedItemsLocaly(feed, backgroundArticles);
                        }
                        $("#show_hide .refresh").addClass("newArticles");
                    }
                } else if (feed.Items.length === 0) {
                    NewsTab.Modules.ArticleView.DisplayFeedEmpty(feed);
                }

                if (!opt.LoadInbackground) {
                    feed.IsAtEnd = !data.MoreArticlesAvailable;
                    if (feed.IsAtEnd) {
                        //SocialTab.ClearPlaceholders();
                    }

                    feed.LastUpdate = Math.floor(Date.now() / 1000); // seconds elapsed since January 1, 1970 00:00:00 UTC
                }

                self.SetFeedIsLoading(feed, false);

                if (opt.CompleteCallback) {
                    opt.CompleteCallback();
                }
            });
        }
    },
    /**/
    API_AddFeedToSection: function (feed, options) {
        var self = this;
        var opt = $.extend({}, { Type: null, OldSectionId: null, DontOpenFeed: false, ShowToastNotification: true }, options);

        var callUrl = NewsTab.ApiUrl_Rss + "AddFeedToSection";

        var url = feed.RssUrl !== undefined ? feed.RssUrl.trim() : feed.Url.trim();

        $.ajax({
            url: callUrl,
            type: "POST",
            contentType: "charset=utf-8",
            headers: {
                Authorization: "Bearer " + NewsTab.Token,
                'SectionId': feed.SectionId,
                'FeedId': feed.Id,
                'Name': encodeURIComponent(feed.Name),
                'SiteUrl': encodeURIComponent(feed.SiteUrl),
                'RssUrl': encodeURIComponent(url.replace(/\s/g, "%20")),
                'IconUrl': feed.IconUrl
            },
            error: function (jqXHR, textStatus, errorThrown) {
                console.error("ERROR - API_AddFeedToSection:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Feed manager",
                    Color: "red",
                    Text: "Failed to add feed."
                });

                if (opt.ErrorCallback) opt.ErrorCallback(jqXHR, textStatus, errorThrown);
            },
            success: function (response) {
                if (response && response.Id !== NewsTab.EMPTY_GUID) {
                    var section = $.grep(self.Sections, function (e) { return e.Id === feed.SectionId });
                    if (section.length > 0) {
                        section = section[0];

                        var newfeed = new Feed().NewInit(response)
                        newfeed.SectionId = section.Id;
                        newfeed.Layout = NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.Layout;
                        section.Feeds.unshift(newfeed);

                        section.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                        NewsTab.Modules.SectionsManager.ChangeFeed(section.Id, feed.Id);

                        if (opt.ShowToastNotification) {
                            NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                                Title: "Feed manager",
                                Color: "green",
                                Text: response.Message
                            });
                        }

                        $(document.body).trigger("newcontent");

                        if (opt.SuccessCallback) {
                            opt.SuccessCallback();
                        }
                    }

                    //var section = self.GetSectionById(feed.SectionId, null);
                    //if (section !== null) {
                    //    if (data.Id !== SocialTab.NewId) {
                    //        var newfeed = new Feed().NewInit(data);
                    //        newfeed.SectionId = section.Id;
                    //        newfeed.Layout = NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.Layout;
                    //        section.Feeds.unshift(newfeed);

                    //        if (opt.Type === "MOVEFEED") {
                    //            data.Message = data.Message.replace("added", "moved");
                    //            SocialTab.RemoveFeedFromSection(opt.OldSectionId, feed.Id, opt.DontOpenFeed);
                    //        }
                    //        else if (opt.Type === "COPY") {
                    //            data.Message = data.Message.replace("added", "copied");
                    //        }

                    //        var currentFeed = $.grep(section.Feeds, function (t) { return t.Id === data.Id });
                    //        if (currentFeed.length) {
                    //            currentFeed = currentFeed[0];
                    //            section.CurrentFeed = currentFeed;
                    //        }

                    //        SocialTab.LastActiveFeeds = [];
                    //        for (var i = 0; i < section.Feeds.length; i++) {
                    //            if (section.Feeds[i].ShowInList) {
                    //                SocialTab.LastActiveFeeds.push(section.Feeds[i].Id);
                    //            }
                    //        }

                    //        section.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                    //        if (!opt.DontOpenFeed) {
                    //            NewsTab.Modules.SectionsManager.ChangeFeed(section.Id, data.Id);
                    //        }

                    //        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    //            Title: "Feed manager",
                    //            Color: "green",
                    //            Text: data.Message
                    //        });
                    //        SocialTab.Menu_SetUpHTML_SubmenuList();
                    //    } else {
                    //        if (!opt.DontOpenFeed) {
                    //            var selectFeed = $.grep(section.Feeds, function (e) { return e.Url === feed.RssUrl });
                    //            if (selectFeed.length) {
                    //                NewsTab.Modules.SectionsManager.ChangeFeed(section.Id, selectFeed.Id);
                    //            }
                    //        }
                    //        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    //            Title: "Feed manager",
                    //            Color: "red",
                    //            Text: data.Message
                    //        });
                    //    }
                    //}
                } else {
                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Feed manager",
                        Color: "reed",
                        Text: response.Message
                    });
                }

                if (opt.SuccessCallback) opt.SuccessCallback(response);
            },
            complete: function (jqXHR, textStatus) {
                if (opt.CompleteCallback) opt.CompleteCallback(jqXHR, textStatus);
            }
        });
    },

    API_AddFeedToNewSection: function (feed, section, options) {
        var self = this;

        var opt = $.extend({}, NewsTab.BASE_REQUEST_OPTIONS, options);
        var callUrl = NewsTab.ApiUrl_Rss + "AddFeedToNewSection";

        $.ajax({
            url: callUrl,
            contentType: "charset=utf-8",
            type: "POST",
            timeout: 25000,
            headers: {
                Authorization: 'Bearer ' + NewsTab.Token,
                'FeedId': feed.Id,
                'Name': encodeURIComponent(feed.Name),
                'SectionName': encodeURIComponent(section.Name),
                'SectionColor': section.Color != undefined ? section.Color : "#e91e63",
                'SiteUrl': encodeURIComponent(feed.SiteUrl),
                'RssUrl': encodeURIComponent(feed.RssUrl.trim().replace(/\s/g, "%20")),
                'IconUrl': feed.IconUrl,
                'Layout': localStorage.getItem("ST_" + NewsTab.User.Id + "_Settings_DefaultLayoutType")
            },
            error: function (jqXHR, textStatus, errorThrown) {
                console.error("ERROR - API_AddFeedToNewSection:", errorThrown);

                NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                    Title: "Feed manager",
                    Color: "red",
                    Text: "Failed to add feed."
                });

                if (opt.ErrorCallback) opt.ErrorCallback(jqXHR, textStatus, errorThrown);
            },
            success: function (response) {
                if (response && response.Id !== NewsTab.EMPTY_GUID) {
                    var section = $.grep(self.Sections, function (e) { return e.Id === response.Id });
                    if (section.length > 0) {
                        section = section[0];
                        var newfeed = new Feed().NewInit(response.Feeds[0])
                        newfeed.SectionId = section.Id;
                        newfeed.Layout = NewsTab.Modules.ArticleView.DEFAULT_SETTINGS.Layout;
                        section.Feeds.unshift(newfeed);

                        var currentFeed = $.grep(section.Feeds, function (t) { return t.Id === response.Feeds[0].Id });
                        if (currentFeed.length) {
                            currentFeed = currentFeed[0];
                            section.CurrentFeed = currentFeed;
                        }

                        self.LastActiveFeeds = [];
                        for (var i = 0; i < section.Feeds.length; i++) {
                            if (section.Feeds[i].ShowInList) {
                                self.LastActiveFeeds.push(section.Feeds[i].Id);
                            }
                        }

                        section.Feeds.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                            Title: "Feed manager",
                            Color: "green",
                            Text: response.Feeds[0].Message
                        });
                    } else {
                        section = new Section().Init(response);

                        self.Sections.push(section);
                        self.LastActiveSections.unshift(String(section.Id));

                        for (var j = 0; j < self.LastActiveSections.length; j++) {
                            var s = $.grep(self.Sections, function (t) { return t.Id === self.LastActiveSections[j] });
                            if (s.length) {
                                s = s[0];
                                s.OrderNumber = j;
                            }
                        }

                        self.Sections.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));

                        NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                            Title: "Feed manager",
                            Color: "green",
                            Text: response.Feeds[0].Message
                        });
                    }

                    NewsTab.Modules.SectionsManager.ChangeFeed(section.Id, feed.Id);

                    $(document.body).trigger("newcontent");
                } else {
                    NewsTab.Modules.NotificationsManager.Open_ToastMessage({
                        Title: "Feed manager",
                        Color: "red",
                        Text: response.Message
                    });
                    $("#contentFeeds").find(".searchItems li.added.new").removeClass("added").removeClass("new");
                }

                if (opt.SuccessCallback) opt.SuccessCallback(response);
            },
            complete: function (jqXHR, textStatus) {
                if (opt.CompleteCallback) opt.CompleteCallback(jqXHR, textStatus);
            }
        });
    },

    API_AddExtensonSectionsToUser: function (options) {
        var self = this;

        var nuberOfTasks = 0;
        var feedCounter = 0;
        var newFeeds = [];

        self.ExtensionSections.forEach(function (s, i) {
            s.Feeds.forEach(function (f, j) {
                if (self.GetFeedByUrl(f.Url, null) === null) {
                    newFeeds.unshift(f);
                }
            });
        });

        if (newFeeds.length > 0) {
            var chromeSection = $.grep(self.Sections, function (x) { return x.SectionTypeId === Section.TYPES.Chrome; });
            nuberOfTasks++;

            if (chromeSection.length === 0) {
                self.API_AddChromeSection(
                    {
                        BeforeSendCallback: function () {
                            $("#syncInfo").html("Adding new Section: My Extensions Feeds...");
                        },
                        SuccessCallback: function (response) {
                            $("#syncInfo").html("Adding new Feeds: 0 out of " + newFeeds.length);
                            var section = new Section().Init(response);
                            self.AddSection(section);

                            newFeeds.forEach(function (f, j) {
                                nuberOfTasks++;
                                f.SectionId = section.Id;
                                self.API_AddFeedToSection(f, {
                                    CompleteCallback: function () {
                                        feedCounter++;
                                        $("#syncInfo").html("Adding new Feeds to My Extensions Feeds: " + feedCounter + " out of " + newFeeds.length);
                                        nuberOfTasks--;
                                    },
                                    ShowToastNotification: false
                                });
                            });
                        },
                        CompleteCallback: function () {
                            nuberOfTasks--;
                        }
                    });
            } else {
                self.API_AddChromeSection(
                    {
                        BeforeSendCallback: function () {
                            $("#syncInfo").html("Syncing Section: My Extensions Feeds...");
                        },
                        SuccessCallback: function (response) {
                            newFeeds.forEach(function (f, j) {
                                nuberOfTasks++;
                                f.SectionId = response.Id;
                                self.API_AddFeedToSection(f, {
                                    CompleteCallback: function () {
                                        feedCounter++;
                                        $("#syncInfo").html("Adding new Feeds to My Extensions Feeds: " + feedCounter + " out of " + newFeeds.length);
                                        nuberOfTasks--;
                                    },
                                    ShowToastNotification: false
                                });
                            });
                        },
                        CompleteCallback: function () {
                            nuberOfTasks--;
                        }
                    });
            }
        }

        var setupExtension = setInterval(function () {
            if (nuberOfTasks === 0) {
                clearInterval(setupExtension);
                if (options.SuccessCallback) {
                    options.SuccessCallback();
                }
            }

        }, 100);
    }
};