﻿var NewsTab = {
    DEFAULT_COLOR: "#03a9f4",

    EMPTY_GUID: "00000000-0000-0000-0000-000000000000",

    GOOGLE_ANALYTICS_ID_EXTENSION: "UA-81715685-4",
    GOOGLE_ANALYTICS_ID_WEB: "UA-81715685-2",

    BASE_REQUEST_OPTIONS: {
        BeforeSendCallback: null,
        ErrorCallback: null,
        SuccessCallback: null,
        CompleteCallback: null
    },

    DeveloperMode: false,
    IsExtension: false,
    IsLightTheme: false,

    PlaceholderImage: "/Media/Images/Backgrounds/square.png",
    DefaultProfileImage: "/Media/Images/profile_pic_default.png",

    User: {
        Id: "0",
        Username: "PublicProfile"
    },

    ProfileImages: [],

    LocalAccountAvailable: false,

    // Base
    Colors: [],
    Tags: [],
    Languages: [],


    // API
    GoogleAnalyticsId: null,

    Token: null,

    EnviromentUrl: window.newsTabWebPath !== undefined ? window.newsTabWebPath : chrome.runtime.getManifest().homepage_url,

    MainCDN: "https://cdn.newstab.com",

    ApiUrl_Base: null,
    ApiUrl_Feeds: null,
    ApiUrl_Global: null,

    ApiUrl_Rss: null,
    FeedApiUrl_Feed: null,

    ApiUrl_Global_Cdn: null,

    PlayStoreUrl: null,
    AppStoreUrl: null,
    ChromeExtensionId: null,
    ChromeExtensionUrl: null,

    Init: function () {
        this.GoogleAnalyticsId = window.NewsTab_GoogleAnalytics !== undefined ? window.NewsTab_GoogleAnalytics : this.GOOGLE_ANALYTICS_ID_EXTENSION;

        if (window.newsTabCdnPath !== undefined) {
            this.MainCDN = window.newsTabCdnPath;
        }

        this.ApiUrl_Base = this.EnviromentUrl + "/api/";

        this.ApiUrl_Global = this.EnviromentUrl + "/api/global/";
        this.ApiUrl_Feeds = this.EnviromentUrl + "/api/feeds/";
        this.ApiUrl_Rss = this.EnviromentUrl + "/api/apirss/";

        this.FeedApiUrl_Feed = this.EnviromentUrl + "/feedapi/feed/";

        this.ApiUrl_Global_Cdn = this.MainCDN + "/api/global/";

        this.PlayStoreUrl = window.NewsTab_PlayStoreUrl;
        this.AppStoreUrl = window.NewsTab_AppStoreUrl;
        this.ChromeExtensionId = window.NewsTab_ChromeExtensionId;
        this.ChromeExtensionUrl = "https://chrome.google.com/webstore/detail/" + window.NewsTab_ChromeExtensionId;
    },

    Log: this.DeveloperMode ? console.log : function () { },

    SetDeveloperMode: function (mode) {
        if (mode !== true) {
            this.DeveloperMode = false;
        }
        else {
            this.DeveloperMode = true;
        }
        this.Log = this.DeveloperMode ? console.log : function () { };
        NewsTab.Helpers.SetLocalStorageItem("DeveloperMode", this.DeveloperMode);
    },

    Setup_DeveloperMode: function () {
        if (this.DeveloperMode) {
            var $dm = $("<a/>", {
                "id": "developer-links",
                "class": "extension-link ",
                "html": "Developer"
            });
            $dm.on("click", function () {
                var $dmlist = $("#developer-links-popup");
                if ($dmlist.length === 1) {
                    $dmlist.remove();
                } else {
                    $dmlist = $("<div/>", {
                        "id": "developer-links-popup",
                        "class": "extension-link-popup developer-sublinks",
                        "html": "<ul></ul>"
                    });

                    var $localStorageState = $("<li/>", { "html": "<a href=\"#\">Local Storage Stats</a>" });
                    $localStorageState.on("click", function () {

                        var _lsTotal = 0, _keyLen = 0, _key = "", _entries = [], _size = 0;
                        var i = 0, il = 0;
                        for (i = 0, il = localStorage.length; i < il; i++) {
                            _key = localStorage.key(i);
                            _keyLen = ((localStorage[_key].length + _key.length) * 2);
                            _size = (_keyLen / 1024).toFixed(2);
                            _entries.push({ "size": _size, "message": "<p><b>" + _key + "</b> = " + _size + " KB</p>" });
                            _lsTotal += _keyLen;
                        }
                        _entries.sort(function (a, b) { return b.size - a.size });

                        var html = [];
                        html.push("<p><b>Total Used</b> = <b>" + (_lsTotal / 1024).toFixed(2) + " KB</b></p>");
                        html.push("<p><b>Remaining</b> = <b><span class=\"remaininglocalsize\">0</span> KB</b> <button>Calculate remaining (will freez browser)</button></p><hr>");
                        for (i = 0, il = _entries.length; i < il; i++) {
                            html.push(_entries[i].message);
                        }

                        var $text = $("<div/>", { "class": "localStorageStats" }).html(html.join(""));
                        $("button", $text).on("click", function (e) {
                            $(".remaininglocalsize", $text).html(((NewsTab.Helpers.GetLocalStorageMaxSize(10) * 2) / 1024).toFixed(2));
                        });

                        var options = {
                            Title: "Local Storage Stats",
                            Text: $text,
                            Buttons: [{ Name: "Ok", Action: "Close" }]
                        };

                        NewsTab.Modules.NotificationsManager.Open_Modal(options);
                    });

                    $("ul", $dmlist).append($localStorageState);
                    $("ul", $dmlist).append($imageCollage);
                    $dm.after($dmlist);
                }
            });
            $(".extension-links:first").append($dm);
        } else {
            $("#developer-links").remove();
        }
    },


    Setup_SectionColors: function ($element, selectedColor) {
        var lastIndex = $(".front", $element).attr("tabindex");
        var index = lastIndex !== undefined ? parseInt(lastIndex) + 1 : 0;
        var colorList = $(".colors", $element);
        colorList.html("");

        NewsTab.Colors.forEach(function (color) {
            var $item = $("<li/>", {
                "class": color === selectedColor ? "selected" : "",
                "data-color": color,
                "style": "background-color:" + color,
                "data-space": "true"
            });
            if (index !== -1) {
                $item.attr("tabindex", index)
                index++;
            }
            else {
                $item.attr("tabindex", 0)
            }
            $item.on("click", function () {
                var btn = $(this);
                btn.siblings().removeClass("selected");
                btn.addClass("selected");

                $(".iconPreview", $element).attr("style", "background-color:" + btn.data("color"));
            });
            colorList.append($item);
        });
    },

    Setup_SectionTags: function () {
        var list = $(".topicTags");
        list.html("");

        list.append("<li class=\"active\" tag-id=\"0\" >All Categories</li>");
        NewsTab.Tags.forEach(function (tag) {
            var $item = $("<li/>", { "tag-id": tag.Id });
            $item.html(NewsTab.Helpers.FirstLetterCapitalize(tag.Name));
            $item.on("click", function () {
                SocialTab.HTML_Event_SectionTag_Click($(this));
            });
            list.append($item);
        });
        list.find("li:first-child").trigger("click");
    },

    Setup_Languages: function () {
        var list1 = $(".languageList");
        list1.html("");

        list1.append("<option value=\"0\">All Languages</option>");
        this.Languages.forEach(function (language) {
            list1.append("<option value=\"" + language.Id + "\">" + language.Name + "</option>");
        });
    },


    API_GetInitialDataForPublicExtension: function () {
        this.API_GetColors();
        this.API_GetTags();
        this.API_GetLanguages();
    },

    API_GetColors: function () {
        var callUrl = this.ApiUrl_Global + "get/colors";
        $.ajax({
            url: callUrl,
            timeout: 16000,
            error: function (error) {
                console.error("ERROR - API_GetColors:");
                console.error(error);
            }
        }).then(function (data) {
            if (data.length) {
                NewsTab.Colors = data;
                NewsTab.Setup_SectionColors();
            }
        });
    },

    API_GetTags: function () {
        var callUrl = this.ApiUrl_Global + "get/tags";
        $.ajax({
            url: callUrl,
            timeout: 16000,
            error: function (error) {
                console.error("ERROR - API_GetTags:");
                console.error(error);
            }
        }).then(function (data) {
            if (data.length) {
                NewsTab.Tags = data;
                NewsTab.Setup_SectionTags();
            }
        });
    },

    API_GetLanguages: function () {
        var callUrl = this.ApiUrl_Global + "get/languages";
        $.ajax({
            url: callUrl,
            timeout: 16000,
            error: function (error) {
                console.error("ERROR - API_GetLanguages:");
                console.error(error);
            }
        }).then(function (data) {
            if (data.length) {
                NewsTab.Languages = data;
                NewsTab.Setup_Languages();
            }
        });
    },

    API_GetUserData: function () {
        if (NewsTab.Token) {
            var callUrl = NewsTab.ApiUrl_Global + "get/initialdata";
            $.ajax({
                type: "GET",
                url: callUrl,
                headers: { Authorization: "Bearer " + NewsTab.Token }, // SETING UP THE TOKEN FOR "Authorize" ACCESS
                cache: false,
                error: function (error) {
                    NewsTab.Helpers.CenterContent($(".confirmationMessage"), false);
                    console.error("ERROR - UserData_GetFromAPI:", error);
                    var failsNumber = localStorage.getItem("Fails");
                    failsNumber = failsNumber != undefined ? parseInt(failsNumber) : 0;

                    if (error.status === 401 || error.status === 403 || failsNumber > 0) {
                        localStorage.setItem("Fails", 0);
                    } else {
                        var message = "Something went wrong! Please try again!";
                        var button = {
                            btnClass: "btn blue",
                            action: "window.location.reload()",
                            btnName: "Try again"
                        };
                        SocialTab.ShowConfirmationMessage(message, button);
                        localStorage.setItem("Fails", ++failsNumber);
                    }
                }
            }).then(function (data) {
                if (data) {
                    NewsTab.Helpers.SetLocalStorageItem("UserData", data);
                    NewsTab.UseUserData(data);
                }
            });
        } else {
            console.log("UserData_GetFromAPI: No Token Key");
        }
    },

    API_GetLightUserData: function () {
        if (NewsTab.Token) {
            var callUrl = NewsTab.ApiUrl_Global + "get/lightinitialdata";
            $.ajax({
                type: "GET",
                url: callUrl,
                headers: { Authorization: "Bearer " + NewsTab.Token }, // SETING UP THE TOKEN FOR "Authorize" ACCESS
                cache: false,
                error: function (error) {
                    NewsTab.Helpers.CenterContent($(".confirmationMessage"), false);
                    console.error("ERROR - UserData_GetFromAPI:");
                    console.error(error);
                    var failsNumber = localStorage.getItem("Fails");
                    failsNumber = failsNumber != undefined ? parseInt(failsNumber) : 0;

                    if (error.status === 401 || error.status === 403 || failsNumber > 0) {
                        localStorage.setItem("Fails", 0);
                    } else {
                        var message = "Something went wrong! Please try again!";
                        var button = {
                            btnClass: "btn blue",
                            action: "window.location.reload()",
                            btnName: "Try again"
                        };
                        SocialTab.ShowConfirmationMessage(message, button);
                        localStorage.setItem("Fails", ++failsNumber);
                    }
                }
            }).then(function (data) {
                if (data) {
                    NewsTab.Helpers.SetLocalStorageItem("UserData", data);
                    NewsTab.UseUserData(data);

                    var currentSectionId = NewsTab.Helpers.GetLocalStorageItem("CurrentSectionId");
                    var currentFeedId = NewsTab.Helpers.GetLocalStorageItem("CurrentFeedId");

                    if (NewsTab.Modules.SectionsManager.SetCurrentSectionAndFeed(currentSectionId, currentFeedId) === true && NewsTab.Extension.IsBackgroundScript !== true) {
                        NewsTab.Modules.SectionsManager.LoadCurrentFeed();
                        NewsTab.Extension.SetUp_NewTabFeedTitle();
                    }
                }
            });
        } else {
            console.log("UserData_GetFromAPI: No Token Key");
        }
    },

    UseUserData: function (data) {
        var self = this;

        self.ProfileImages = data.ProfileImages;
        self.User = data.User;
        self.MaxTopFeeds = data.User.MaxTopFeeds;
        self.LocalAccountAvailable = data.LocalAccountAvailable;

        if (data.User.SocialNetworks) {
            for (var i = 0, len = data.User.SocialNetworks.length; i < len; i++) {
                if (data.User.SocialNetworks[i].SourceId === 2) {
                    Feed.TYPES.TWITTER.Account = data.User.SocialNetworks[i];
                }
            }
        }

        if (!self.User.ProfilePictureUrl) {
            self.User.ProfilePictureUrl = self.DefaultProfileImage;
        }

        if (NewsTab.Modules.RegionManager.Countries.length > 0) {
            if (self.User.CountryId) {
                var county = $.grep(NewsTab.Modules.RegionManager.Countries, function (c) { return c.Id === self.User.CountryId; });
                if (county.length > 0) {
                    county = county[0];
                    NewsTab.Modules.RegionManager.MainEdition = county.Code;
                    NewsTab.Modules.RegionManager.AddFeedEdition = NewsTab.Modules.RegionManager.MainEdition;
                    NewsTab.Modules.RegionManager.MainEditionGnValue = (county.GoogleNewsEditionCode != null && county.GoogleNewsEditionCode !== "") ? county.GoogleNewsEditionCode : "US";
                    NewsTab.Modules.RegionManager.AddFeedEditionGnValue = NewsTab.Modules.RegionManager.MainEditionGnValue;
                } else {
                    NewsTab.Modules.RegionManager.MainEdition = "US";
                    NewsTab.Modules.RegionManager.AddFeedEdition = NewsTab.Modules.RegionManager.MainEdition;
                    NewsTab.Modules.RegionManager.MainEditionGnValue = NewsTab.Modules.RegionManager.MainEdition;
                    NewsTab.Modules.RegionManager.AddFeedEditionGnValue = NewsTab.Modules.RegionManager.MainEdition;
                }
            }
        }

        NewsTab.Modules.SectionsManager.SysSections = data.Sections;

        NewsTab.Modules.SectionsManager.LastActiveSectionId = data.User.LastActiveSection;
        NewsTab.Modules.SectionsManager.LastActiveFeedId = data.User.LastActiveFeed;

        //if (NewsTab.Modules.RegionManager !== undefined) {
        //    var county;
        //    if (self.User.CountryId) {
        //        county = $.grep(NewsTab.Modules.RegionManager.Countries, function (c) { return c.Id === self.User.CountryId; });
        //        if (county.length > 0) {
        //            SocialTab.SelectedCountry = county[0];
        //            NewsTab.Modules.RegionManager.MainEdition = SocialTab.SelectedCountry.Code;
        //            NewsTab.Modules.RegionManager.AddFeedEdition = NewsTab.Modules.RegionManager.MainEdition;
        //            NewsTab.Modules.RegionManager.MainEditionGnValue = (county[0].GoogleNewsEditionCode != null && county[0].GoogleNewsEditionCode !== "")
        //                ? county[0].GoogleNewsEditionCode : "US";
        //            NewsTab.Modules.RegionManager.AddFeedEditionGnValue = NewsTab.Modules.RegionManager.MainEditionGnValue;
        //        } else {
        //            NewsTab.Modules.RegionManager.MainEdition = "US";
        //            NewsTab.Modules.RegionManager.AddFeedEdition = NewsTab.Modules.RegionManager.MainEdition;
        //            NewsTab.Modules.RegionManager.MainEditionGnValue = NewsTab.Modules.RegionManager.MainEdition;
        //            NewsTab.Modules.RegionManager.AddFeedEditionGnValue = NewsTab.Modules.RegionManager.MainEdition;
        //        }
        //    }
        //}



        // Init Sections
        NewsTab.Modules.SectionsManager.Sections = [];
        if (self.User.Sections !== null) {
            self.User.Sections.forEach(function (s) {
                if (NewsTab.IsExtension) {
                    s.Layout = "layout-cascading-grid";
                }
                NewsTab.Modules.SectionsManager.Sections.push(new Section().Init(s));
            });
        }
        NewsTab.Modules.SectionsManager.Sections.sort(NewsTab.Helpers.Sort("OrderNumber", false, parseInt));
    },

    //Event Category: Search
    //Event Action: NewsTab / Google / Omnibox
    //Event Label: ID (e.g. Boxing_153)
    TrackAnalyticsEvent: function (category, action, label) {
        NewsTab.Log("TrackAnalyticsEvent", category, action, label);
        if (!NewsTab.DeveloperMode) {
            _gaq.push(['_trackEvent', category, action, label]);
        }
    }
};

NewsTab.Init();

var _gaq = _gaq || [];
_gaq.push(['_setAccount', NewsTab.GoogleAnalyticsId]);
_gaq.push(['_trackPageview']);

(function () {
    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
    ga.src = 'https://ssl.google-analytics.com/ga.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
})();

